@@ -1,20 +1,25 @@
 /*
-    Copyright (C) 2001, 2006 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2007-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2014-2017 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2016 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cmath>
 #include <cassert>
@@ -24,8 +29,6 @@
 
 #include <gtkmm2ext/gtk_ui.h>
 
-#include "pbd/stacktrace.h"
-
 #include "ardour/audioregion.h"
 #include "ardour/audiofilesource.h"
 #include "ardour/audio_track.h"
@@ -39,7 +42,6 @@
 
 #include "audio_streamview.h"
 #include "audio_region_view.h"
-#include "tape_region_view.h"
 #include "audio_time_axis.h"
 #include "region_selection.h"
 #include "region_gain_line.h"
@@ -59,8 +61,13 @@ using namespace Editing;
 AudioStreamView::AudioStreamView (AudioTimeAxisView& tv)
 	: StreamView (tv)
 {
-	color_handler ();
+	/* use a dedicated group for Audio regions (on top of the grid) */
+	_region_group = new ArdourCanvas::Container (_canvas_group);
+	_region_group->raise_to_top ();
+	_region_group->set_render_with_alpha (UIConfiguration::instance().modifier ("region alpha").a());
+
 	_amplitude_above_axis = 1.0;
+	color_handler ();
 }
 
 int
@@ -84,10 +91,10 @@ AudioStreamView::set_amplitude_above_axis (gdouble app)
 }
 
 RegionView*
-AudioStreamView::create_region_view (boost::shared_ptr<Region> r, bool wait_for_waves, bool recording)
+AudioStreamView::create_region_view (std::shared_ptr<Region> r, bool wait_for_waves, bool recording)
 {
 	AudioRegionView *region_view = 0;
-	boost::shared_ptr<AudioRegion> region = boost::dynamic_pointer_cast<AudioRegion> (r);
+	std::shared_ptr<AudioRegion> region = std::dynamic_pointer_cast<AudioRegion> (r);
 
 	if (region == 0) {
 		return 0;
@@ -98,21 +105,18 @@ AudioStreamView::create_region_view (boost::shared_ptr<Region> r, bool wait_for_
 	case NonLayered:
 	case Normal:
 		if (recording) {
-			region_view = new AudioRegionView (_canvas_group, _trackview, region,
+			region_view = new AudioRegionView (_region_group, _trackview, region,
 							   _samples_per_pixel, region_color, recording, TimeAxisViewItem::Visibility(
 								   TimeAxisViewItem::ShowFrame |
 								   TimeAxisViewItem::HideFrameRight |
 								   TimeAxisViewItem::HideFrameLeft |
 								   TimeAxisViewItem::HideFrameTB));
 		} else {
-			region_view = new AudioRegionView (_canvas_group, _trackview, region,
+			region_view = new AudioRegionView (_region_group, _trackview, region,
 					_samples_per_pixel, region_color);
 		}
 		break;
-	case Destructive:
-		region_view = new TapeAudioRegionView (_canvas_group, _trackview, region,
-		                                       _samples_per_pixel, region_color);
-		break;
+
 	default:
 		fatal << string_compose (_("programming error: %1"), "illegal track mode in ::create_region_view()") << endmsg;
 		abort(); /*NOTREACHED*/
@@ -127,7 +131,7 @@ AudioStreamView::create_region_view (boost::shared_ptr<Region> r, bool wait_for_
 	   insensitive to events
 	*/
 
-	if (region->length() == 1) {
+	if (region->length_samples() == 1) {
 		region_view->set_sensitive (false);
 	}
 
@@ -135,7 +139,7 @@ AudioStreamView::create_region_view (boost::shared_ptr<Region> r, bool wait_for_
 }
 
 RegionView*
-AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_waves, bool recording)
+AudioStreamView::add_region_view_internal (std::shared_ptr<Region> r, bool wait_for_waves, bool recording)
 {
 	RegionView *region_view = create_region_view (r, wait_for_waves, recording);
 
@@ -147,13 +151,23 @@ AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wai
 
 	/* catch region going away */
 
-	r->DropReferences.connect (*this, invalidator (*this), boost::bind (&AudioStreamView::remove_region_view, this, boost::weak_ptr<Region> (r)), gui_context());
+	r->DropReferences.connect (*this, invalidator (*this), boost::bind (&AudioStreamView::remove_region_view, this, std::weak_ptr<Region> (r)), gui_context());
 
 	RegionViewAdded (region_view);
 
 	return region_view;
 }
 
+void
+AudioStreamView::set_layer_display (LayerDisplay d)
+{
+	StreamView::set_layer_display (d);
+
+	for (auto& rv : region_views) {
+		rv->set_frame_color ();
+	}
+}
+
 void
 AudioStreamView::redisplay_track ()
 {
@@ -162,26 +176,38 @@ AudioStreamView::redisplay_track ()
 	// Flag region views as invalid and disable drawing
 	for (i = region_views.begin(); i != region_views.end(); ++i) {
 		(*i)->set_valid (false);
-		(*i)->enable_display (false);
 	}
 
 	// Add and display views, and flag them as valid
 	if (_trackview.is_audio_track()) {
-		_trackview.track()->playlist()->foreach_region(
-			sigc::hide_return (sigc::mem_fun (*this, &StreamView::add_region_view))
-			);
+		_trackview.track()->playlist()->foreach_region (sigc::hide_return (sigc::mem_fun (*this, &StreamView::add_region_view)));
 	}
 
 	// Stack regions by layer, and remove invalid regions
 	layer_regions();
 }
 
+void
+AudioStreamView::reload_waves ()
+{
+	list<RegionView *>::iterator i;
+	for (i = region_views.begin(); i != region_views.end(); ++i) {
+		AudioRegionView* arv = dynamic_cast<AudioRegionView*> (*i);
+		if (!arv) {
+			continue;
+		}
+		arv->delete_waves();
+		arv->create_waves();
+	}
+}
+
 void
 AudioStreamView::setup_rec_box ()
 {
 	//cerr << _trackview.name() << " streamview SRB region_views.size() = " << region_views.size() << endl;
 
-	if (!_trackview.session()->transport_stopped()) {
+	if (!_trackview.session()->transport_stopped_or_stopping() &&
+	    (_trackview.session()->transport_rolling() || _trackview.session()->get_record_enabled())) {
 
 		// cerr << "\trolling\n";
 
@@ -195,49 +221,48 @@ AudioStreamView::setup_rec_box ()
 				SourceList sources;
 
 				rec_data_ready_connections.drop_connections ();
-				boost::shared_ptr<AudioTrack> tr = _trackview.audio_track();
+				std::shared_ptr<AudioTrack> tr = _trackview.audio_track();
 
 				for (uint32_t n = 0; n < tr->n_channels().n_audio(); ++n) {
-					boost::shared_ptr<AudioFileSource> src = tr->write_source (n);
+					std::shared_ptr<AudioFileSource> src = tr->write_source (n);
 					if (src) {
 						sources.push_back (src);
 						src->PeakRangeReady.connect (rec_data_ready_connections,
 						                             invalidator (*this),
-						                             boost::bind (&AudioStreamView::rec_peak_range_ready, this, _1, _2, boost::weak_ptr<Source>(src)),
+						                             boost::bind (&AudioStreamView::rec_peak_range_ready, this, _1, _2, std::weak_ptr<Source>(src)),
 						                             gui_context());
 					}
 				}
 
 				// handle multi
 
-				framepos_t start = 0;
+				samplepos_t start = 0;
 				if (rec_regions.size() > 0) {
-					start = rec_regions.back().first->start()
-							+ _trackview.track()->get_captured_frames(rec_regions.size()-1);
+					start = rec_regions.back().first->start_sample()
+							+ _trackview.track()->get_captured_samples(rec_regions.size()-1);
 				}
 
 				PropertyList plist;
 
 				plist.add (Properties::start, start);
-				plist.add (Properties::length, 1);
+				plist.add (Properties::length, timepos_t (1));
 				plist.add (Properties::name, string());
 				plist.add (Properties::layer, 0);
 
-				boost::shared_ptr<AudioRegion> region (
-					boost::dynamic_pointer_cast<AudioRegion>(RegionFactory::create (sources, plist, false)));
+				std::shared_ptr<AudioRegion> region (
+					std::dynamic_pointer_cast<AudioRegion>(RegionFactory::create (sources, plist, false)));
 
 				assert(region);
-				region->set_position (_trackview.session()->transport_frame());
+				region->set_position (timepos_t (_trackview.session()->transport_sample()));
 				rec_regions.push_back (make_pair(region, (RegionView*) 0));
 			}
 
 			/* start a new rec box */
 
-			boost::shared_ptr<AudioTrack> at = _trackview.audio_track();
-			framepos_t const frame_pos = at->current_capture_start ();
-			double     const width     = ((at->mode() == Destructive) ? 2 : 0);
+			std::shared_ptr<AudioTrack> at = _trackview.audio_track();
+			samplepos_t const sample_pos = at->current_capture_start ();
 
-			create_rec_box(frame_pos, width);
+			create_rec_box(sample_pos, 0);
 
 		} else if (rec_active &&
 		           (_trackview.session()->record_status() != Session::Recording ||
@@ -261,8 +286,8 @@ AudioStreamView::setup_rec_box ()
 
 			/* remove temp regions */
 
-			for (list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); ) {
-				list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator tmp;
+			for (list<pair<std::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); ) {
+				list<pair<std::shared_ptr<Region>,RegionView*> >::iterator tmp;
 
 				tmp = iter;
 				++tmp;
@@ -289,11 +314,11 @@ AudioStreamView::setup_rec_box ()
 }
 
 void
-AudioStreamView::rec_peak_range_ready (framepos_t start, framecnt_t cnt, boost::weak_ptr<Source> weak_src)
+AudioStreamView::rec_peak_range_ready (samplepos_t start, samplecnt_t cnt, std::weak_ptr<Source> weak_src)
 {
 	ENSURE_GUI_THREAD (*this, &AudioStreamView::rec_peak_range_ready, start, cnt, weak_src)
 
-	boost::shared_ptr<Source> src (weak_src.lock());
+	std::shared_ptr<Source> src (weak_src.lock());
 
 	if (!src) {
 		return;
@@ -301,8 +326,8 @@ AudioStreamView::rec_peak_range_ready (framepos_t start, framecnt_t cnt, boost::
 
 	// this is called from the peak building thread
 
-	if (rec_data_ready_map.size() == 0 || start + cnt > last_rec_data_frame) {
-		last_rec_data_frame = start + cnt;
+	if (rec_data_ready_map.size() == 0 || start + cnt > last_rec_data_sample) {
+		last_rec_data_sample = start + cnt;
 	}
 
 	rec_data_ready_map[src] = true;
@@ -314,7 +339,7 @@ AudioStreamView::rec_peak_range_ready (framepos_t start, framecnt_t cnt, boost::
 }
 
 void
-AudioStreamView::update_rec_regions (framepos_t start, framecnt_t cnt)
+AudioStreamView::update_rec_regions (samplepos_t start, samplecnt_t cnt)
 {
 	if (!UIConfiguration::instance().get_show_waveforms_while_recording ()) {
 		return;
@@ -322,9 +347,9 @@ AudioStreamView::update_rec_regions (framepos_t start, framecnt_t cnt)
 
 	uint32_t n = 0;
 
-	for (list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); n++) {
+	for (list<pair<std::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); n++) {
 
-		list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator tmp = iter;
+		list<pair<std::shared_ptr<Region>,RegionView*> >::iterator tmp = iter;
 		++tmp;
 
 		assert (n < rec_rects.size());
@@ -335,27 +360,27 @@ AudioStreamView::update_rec_regions (framepos_t start, framecnt_t cnt)
 			continue;
 		}
 
-		boost::shared_ptr<AudioRegion> region = boost::dynamic_pointer_cast<AudioRegion>(iter->first);
+		std::shared_ptr<AudioRegion> region = std::dynamic_pointer_cast<AudioRegion>(iter->first);
 
 		if (!region) {
 			iter = tmp;
 			continue;
 		}
 
-		framecnt_t origlen = region->length();
+		samplecnt_t origlen = region->length_samples();
 
 		if (region == rec_regions.back().first && rec_active) {
 
-			if (last_rec_data_frame > region->start()) {
+			if (last_rec_data_sample > region->start_sample()) {
 
-				framecnt_t nlen = last_rec_data_frame - region->start();
+				samplecnt_t nlen = last_rec_data_sample - region->start_sample();
 
-				if (nlen != region->length()) {
+				if (nlen != region->length_samples()) {
 
 					region->suspend_property_changes ();
 					/* set non-musical position / length */
-					region->set_position (_trackview.track()->get_capture_start_frame(n));
-					region->set_length (nlen, 0);
+					region->set_position (timepos_t (_trackview.track()->get_capture_start_sample(n)));
+					region->set_length (timecnt_t (nlen));
 					region->resume_property_changes ();
 
 					if (origlen == 1) {
@@ -364,25 +389,25 @@ AudioStreamView::update_rec_regions (framepos_t start, framecnt_t cnt)
 						setup_new_rec_layer_time (region);
 					}
 
-					check_record_layers (region, (region->position() - region->start() + start + cnt));
+					check_record_layers (region, (region->position_sample() - region->start_sample() + start + cnt));
 
 					/* also update rect */
 					ArdourCanvas::Rectangle * rect = rec_rects[n].rectangle;
-					gdouble xend = _trackview.editor().sample_to_pixel (region->position() + region->length());
+					gdouble xend = _trackview.editor().sample_to_pixel (region->position_sample() + region->length_samples());
 					rect->set_x1 (xend);
 				}
 
 			} else {
 
-				framecnt_t nlen = _trackview.track()->get_captured_frames(n);
+				samplecnt_t nlen = _trackview.track()->get_captured_samples(n);
 
-				if (nlen != region->length()) {
+				if (nlen != region->length_samples()) {
 
-					if (region->source_length(0) >= region->start() + nlen) {
+					if (region->source_length(0) >= region->start_sample() + nlen) {
 
 						region->suspend_property_changes ();
-						region->set_position (_trackview.track()->get_capture_start_frame(n));
-						region->set_length (nlen, 0);
+						region->set_position (timepos_t (_trackview.track()->get_capture_start_sample(n)));
+						region->set_length (timecnt_t (nlen));
 						region->resume_property_changes ();
 
 						if (origlen == 1) {
@@ -430,7 +455,7 @@ AudioStreamView::hide_all_fades ()
  *  second is the AudioRegionViews that end xfades were hidden for.
  */
 pair<list<AudioRegionView*>, list<AudioRegionView*> >
-AudioStreamView::hide_xfades_with (boost::shared_ptr<AudioRegion> ar)
+AudioStreamView::hide_xfades_with (std::shared_ptr<AudioRegion> ar)
 {
 	list<AudioRegionView*> start_hidden;
 	list<AudioRegionView*> end_hidden;
@@ -438,8 +463,8 @@ AudioStreamView::hide_xfades_with (boost::shared_ptr<AudioRegion> ar)
 	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
 		AudioRegionView* const arv = dynamic_cast<AudioRegionView*>(*i);
 		if (arv) {
-			switch (arv->region()->coverage (ar->position(), ar->last_frame())) {
-			case Evoral::OverlapNone:
+			switch (arv->region()->coverage (ar->position(), ar->nt_last())) {
+			case Temporal::OverlapNone:
 				break;
 			default:
 				if (arv->start_xfade_visible ()) {
@@ -460,6 +485,8 @@ AudioStreamView::hide_xfades_with (boost::shared_ptr<AudioRegion> ar)
 void
 AudioStreamView::color_handler ()
 {
+	_region_group->set_render_with_alpha (UIConfiguration::instance().modifier ("region alpha").a());
+
 	//case cAudioTrackBase:
 	if (_trackview.is_track()) {
 		canvas_rect->set_fill_color (UIConfiguration::instance().color_mod ("audio track base", "audio track base"));
