@@ -1,21 +1,27 @@
 /*
-    Copyright (C) 2002 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Karsten Wiese <fzuuzf@googlemail.com>
+ * Copyright (C) 2005 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006 Hans Fugal <hans@fugal.net>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2007-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2014-2017 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_automation_line_h__
 #define __ardour_automation_line_h__
@@ -27,8 +33,6 @@
 
 #include <sigc++/signal.h>
 
-#include "evoral/TimeConverter.hpp"
-
 #include "pbd/undo.h"
 #include "pbd/statefuldestructible.h"
 #include "pbd/memento_command.h"
@@ -64,19 +68,21 @@ public:
 	AutomationLine (const std::string&                                 name,
 	                TimeAxisView&                                      tv,
 	                ArdourCanvas::Item&                                parent,
-	                boost::shared_ptr<ARDOUR::AutomationList>          al,
-	                const ARDOUR::ParameterDescriptor&                 desc,
-	                Evoral::TimeConverter<double, ARDOUR::framepos_t>* converter = 0);
+	                std::shared_ptr<ARDOUR::AutomationList>          al,
+	                const ARDOUR::ParameterDescriptor&                 desc);
+
 
 	virtual ~AutomationLine ();
 
+	virtual Temporal::timepos_t get_origin () const;
+
 	void queue_reset ();
 	void reset ();
 	void clear ();
 	void set_fill (bool f) { _fill = f; } // owner needs to call set_height
 
 	void set_selected_points (PointSelection const &);
-	void get_selectables (ARDOUR::framepos_t, ARDOUR::framepos_t, double, double, std::list<Selectable*>&);
+	void get_selectables (Temporal::timepos_t const &, Temporal::timepos_t const &, double, double, std::list<Selectable*>&);
 	void get_inverted_selectables (Selection&, std::list<Selectable*>& results);
 
 	virtual void remove_point (ControlPoint&);
@@ -86,7 +92,7 @@ public:
 	virtual void start_drag_single (ControlPoint*, double, float);
 	virtual void start_drag_line (uint32_t, uint32_t, float);
 	virtual void start_drag_multiple (std::list<ControlPoint*>, float, XMLNode *);
-	virtual std::pair<float, float> drag_motion (double, float, bool, bool with_push, uint32_t& final_index);
+	virtual std::pair<float, float> drag_motion (Temporal::timecnt_t const &, float, bool, bool with_push, uint32_t& final_index);
 	virtual void end_drag (bool with_push, uint32_t final_index);
 
 	ControlPoint* nth (uint32_t);
@@ -108,6 +114,7 @@ public:
 	void set_height (guint32);
 
 	bool get_uses_gain_mapping () const;
+	void tempo_map_changed ();
 
 	TimeAxisView& trackview;
 
@@ -120,16 +127,17 @@ public:
 	std::string fraction_to_string (double) const;
 	std::string delta_to_string (double) const;
 	double string_to_fraction (std::string const &) const;
-	void   view_to_model_coord (double& x, double& y) const;
+
 	void   view_to_model_coord_y (double &) const;
-	void   model_to_view_coord (double& x, double& y) const;
-	void   model_to_view_coord_y (double &) const;
+
+	double              model_to_view_coord_y (double) const;
+	Temporal::timecnt_t model_to_view_coord_x (Temporal::timepos_t const &) const;
 
 	double compute_delta (double from, double to) const;
 	void   apply_delta (double& val, double delta) const;
 
-	void set_list(boost::shared_ptr<ARDOUR::AutomationList> list);
-	boost::shared_ptr<ARDOUR::AutomationList> the_list() const { return alist; }
+	void set_list(std::shared_ptr<ARDOUR::AutomationList> list);
+	std::shared_ptr<ARDOUR::AutomationList> the_list() const { return alist; }
 
 	void track_entered();
 	void track_exited();
@@ -137,7 +145,7 @@ public:
 	bool is_last_point (ControlPoint &);
 	bool is_first_point (ControlPoint &);
 
-	XMLNode& get_state (void);
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int version);
 	void set_colors();
 
@@ -145,39 +153,35 @@ public:
 
 	virtual MementoCommandBinder<ARDOUR::AutomationList>* memento_command_binder ();
 
-	const Evoral::TimeConverter<double, ARDOUR::framepos_t>& time_converter () const {
-		return *_time_converter;
-	}
+	std::pair<Temporal::timepos_t, Temporal::timepos_t> get_point_x_range () const;
 
-	std::pair<ARDOUR::framepos_t, ARDOUR::framepos_t> get_point_x_range () const;
-
-	void set_maximum_time (ARDOUR::framecnt_t);
-	ARDOUR::framecnt_t maximum_time () const {
+	void set_maximum_time (Temporal::timepos_t const &);
+	Temporal::timepos_t maximum_time () const {
 		return _maximum_time;
 	}
 
-	void set_offset (ARDOUR::framecnt_t);
-	ARDOUR::framecnt_t offset () { return _offset; }
-	void set_width (ARDOUR::framecnt_t);
+	void set_offset (Temporal::timepos_t const &);
+	Temporal::timepos_t offset () { return _offset; }
+	void set_width (Temporal::timecnt_t const &);
 
-	framepos_t session_position (ARDOUR::AutomationList::const_iterator) const;
+	Temporal::timepos_t session_position (Temporal::timepos_t const &) const;
+	void dump (std::ostream&) const;
+
+	double dt_to_dx (Temporal::timepos_t const &, Temporal::timecnt_t const &);
 
 protected:
 
 	std::string    _name;
 	guint32        _height;
 	uint32_t       _line_color;
-
-	boost::shared_ptr<ARDOUR::AutomationList> alist;
-	Evoral::TimeConverter<double, ARDOUR::framepos_t>* _time_converter;
-	/** true if _time_converter belongs to us (ie we should delete it on destruction) */
-	bool _our_time_converter;
+	uint32_t       _view_index_offset;
+	std::shared_ptr<ARDOUR::AutomationList> alist;
 
 	VisibleAspects _visible;
 
 	bool    terminal_points_can_slide;
 	bool    update_pending;
-	bool    have_timeout;
+	bool    have_reset_timeout;
 	bool    no_draw;
 	bool    _is_boolean;
 	/** true if we did a push at any point during the current drag */
@@ -192,18 +196,18 @@ protected:
 	class ContiguousControlPoints : public std::list<ControlPoint*> {
 public:
 		ContiguousControlPoints (AutomationLine& al);
-		double clamp_dx (double dx);
-		void move (double dx, double dvalue);
+		Temporal::timecnt_t clamp_dt (Temporal::timecnt_t const & dx, Temporal::timepos_t const & region_limit);
+		void move (Temporal::timecnt_t const &, double dvalue);
 		void compute_x_bounds (PublicEditor& e);
 private:
 		AutomationLine& line;
-		double before_x;
-		double after_x;
+		Temporal::timepos_t before_x;
+		Temporal::timepos_t after_x;
 	};
 
 	friend class ContiguousControlPoints;
 
-	typedef boost::shared_ptr<ContiguousControlPoints> CCP;
+	typedef std::shared_ptr<ContiguousControlPoints> CCP;
 	std::vector<CCP> contiguous_points;
 
 	bool sync_model_with_view_point (ControlPoint&);
@@ -219,13 +223,11 @@ private:
 	std::list<ControlPoint*> _drag_points; ///< points we are dragging
 	std::list<ControlPoint*> _push_points; ///< additional points we are dragging if "push" is enabled
 	bool _drag_had_movement; ///< true if the drag has seen movement, otherwise false
-	double _drag_x; ///< last x position of the drag, in units
-	double _drag_distance; ///< total x movement of the drag, in canvas units
 	double _last_drag_fraction; ///< last y position of the drag, as a fraction
 	/** offset from the start of the automation list to the start of the line, so that
 	 *  a +ve offset means that the 0 on the line is at _offset in the list
 	 */
-	ARDOUR::framecnt_t _offset;
+	Temporal::timepos_t _offset;
 
 	bool is_stepped() const;
 	void update_visibility ();
@@ -238,7 +240,7 @@ private:
 	PBD::ScopedConnectionList _list_connections;
 
 	/** maximum time that a point on this line can be at, relative to the position of its region or start of its track */
-	ARDOUR::framecnt_t _maximum_time;
+	Temporal::timepos_t _maximum_time;
 
 	bool _fill;
 
