@@ -1,21 +1,27 @@
 /*
-    Copyright (C) 2005 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2006-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2017 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2014-2019 Ben Loftis <ben@harrisonconsoles.com>
+ * Copyright (C) 2015-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifdef WAF_BUILD
 #include "gtk2ardour-config.h"
@@ -29,6 +35,7 @@
 
 #include "pbd/error.h"
 
+#include "canvas/arc.h"
 #include "canvas/canvas.h"
 #include "canvas/rectangle.h"
 #include "canvas/pixbuf.h"
@@ -52,6 +59,7 @@
 #include "editor_cursors.h"
 #include "mouse_cursors.h"
 #include "note_base.h"
+#include "region_peak_cursor.h"
 #include "ui_config.h"
 #include "verbose_cursor.h"
 
@@ -73,7 +81,7 @@ Editor::initialize_canvas ()
 	_track_canvas = _track_canvas_viewport->canvas ();
 
 	_track_canvas->set_background_color (UIConfiguration::instance().color ("arrange base"));
-	_track_canvas->use_nsglview ();
+	_track_canvas->use_nsglview (UIConfiguration::instance().get_nsgl_view_mode () == NSGLHiRes);
 
 	/* scroll group for items that should not automatically scroll
 	 *  (e.g verbose cursor). It shares the canvas coordinate space.
@@ -99,6 +107,7 @@ Editor::initialize_canvas ()
 	_track_canvas->add_scroller (*cg);
 
 	_verbose_cursor = new VerboseCursor (this);
+	_region_peak_cursor = new RegionPeakCursor (get_noscroll_group ());
 
 	/*a group to hold global rects like punch/loop indicators */
 	global_rect_group = new ArdourCanvas::Container (hv_scroll_group);
@@ -134,6 +143,12 @@ Editor::initialize_canvas ()
 	_time_markers_group = new ArdourCanvas::Container (h_scroll_group);
 	CANVAS_DEBUG_NAME (_time_markers_group, "time bars");
 
+
+	/* Note that because of ascending-y-axis coordinates, this order is
+	 * bottom-to-top. But further note that the actual order is set in
+	 * ::update_ruler_visibility()
+	 */
+
 	cd_marker_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, 0.0));
 	CANVAS_DEBUG_NAME (cd_marker_group, "cd marker group");
 	/* the vide is temporarily placed a the same location as the
@@ -147,34 +162,66 @@ Editor::initialize_canvas ()
 	CANVAS_DEBUG_NAME (transport_marker_group, "transport marker group");
 	range_marker_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, (timebar_height * 3.0) + 1.0));
 	CANVAS_DEBUG_NAME (range_marker_group, "range marker group");
-	tempo_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, (timebar_height * 4.0) + 1.0));
+	tempo_meta_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, (timebar_height * 4.0) + 1.0));
+	CANVAS_DEBUG_NAME (tempo_meta_group, "tempo meta group");
+	tempo_group = new ArdourCanvas::Container (tempo_meta_group, ArdourCanvas::Duple (0.0, 0.0));
 	CANVAS_DEBUG_NAME (tempo_group, "tempo group");
+	mapping_group = new ArdourCanvas::Container (tempo_meta_group, ArdourCanvas::Duple (0.0, 0.0));
+	CANVAS_DEBUG_NAME (mapping_group, "mapping group");
 	meter_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, (timebar_height * 5.0) + 1.0));
 	CANVAS_DEBUG_NAME (meter_group, "meter group");
 
-	meter_bar = new ArdourCanvas::Rectangle (meter_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	float timebar_thickness = timebar_height; //was 4
+	float timebar_top = (timebar_height - timebar_thickness)/2;
+	float timebar_btm = timebar_height - timebar_top;
+
+	meter_bar = new ArdourCanvas::Rectangle (meter_group, ArdourCanvas::Rect (0.0, 0., ArdourCanvas::COORD_MAX, timebar_height));
 	CANVAS_DEBUG_NAME (meter_bar, "meter Bar");
-	meter_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
+	meter_bar->set_outline(false);
 
 	tempo_bar = new ArdourCanvas::Rectangle (tempo_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
-	CANVAS_DEBUG_NAME (tempo_bar, "Tempo  Bar");
-	tempo_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
+	CANVAS_DEBUG_NAME (tempo_bar, "Tempo Bar");
+	tempo_bar->set_fill(true);
+	tempo_bar->set_outline(false);
+	tempo_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
 
-	range_marker_bar = new ArdourCanvas::Rectangle (range_marker_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	mapping_bar = new ArdourCanvas::Rectangle (mapping_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	CANVAS_DEBUG_NAME (mapping_bar, "Mapping Bar");
+	mapping_bar->set_fill(true);
+	mapping_bar->set_outline(false);
+	mapping_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
+
+	switch (UIConfiguration::instance().get_tempo_edit_behavior()) {
+	case Editing::TempoMapping:
+		tempo_group->hide ();
+		break;
+	case Editing::TempoChanging:
+		mapping_group->hide ();
+		break;
+	}
+
+	range_marker_bar = new ArdourCanvas::Rectangle (range_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
 	CANVAS_DEBUG_NAME (range_marker_bar, "Range Marker Bar");
-	range_marker_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
 
-	transport_marker_bar = new ArdourCanvas::Rectangle (transport_marker_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	transport_marker_bar = new ArdourCanvas::Rectangle (transport_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
 	CANVAS_DEBUG_NAME (transport_marker_bar, "transport Marker Bar");
-	transport_marker_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
 
-	marker_bar = new ArdourCanvas::Rectangle (marker_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	marker_bar = new ArdourCanvas::Rectangle (marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
 	CANVAS_DEBUG_NAME (marker_bar, "Marker Bar");
-	marker_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
 
-	cd_marker_bar = new ArdourCanvas::Rectangle (cd_marker_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, timebar_height));
+	cd_marker_bar = new ArdourCanvas::Rectangle (cd_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
 	CANVAS_DEBUG_NAME (cd_marker_bar, "CD Marker Bar");
-	cd_marker_bar->set_outline_what (ArdourCanvas::Rectangle::BOTTOM);
+
+	cue_marker_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, 0.0));
+	cue_marker_bar = new ArdourCanvas::Rectangle (cue_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
+	CANVAS_DEBUG_NAME (cue_marker_bar, "Cue Marker Bar");
+
+	ruler_separator = new ArdourCanvas::Line(_time_markers_group);
+	CANVAS_DEBUG_NAME (ruler_separator, "separator between ruler and main canvas");
+	ruler_separator->set (ArdourCanvas::Duple(0.0, 0.0), ArdourCanvas::Duple(ArdourCanvas::COORD_MAX, 0.0));
+	ruler_separator->set_outline_color(Gtkmm2ext::rgba_to_color (0, 0, 0, 1.0));
+	ruler_separator->set_outline_width(1.0);
+	ruler_separator->show();
 
 	ARDOUR_UI::instance()->video_timeline = new VideoTimeLine(this, videotl_group, (timebar_height * videotl_bar_height));
 
@@ -183,6 +230,11 @@ Editor::initialize_canvas ()
 	cd_marker_bar_drag_rect->set_outline (false);
 	cd_marker_bar_drag_rect->hide ();
 
+	cue_marker_bar_drag_rect = new ArdourCanvas::Rectangle (cue_marker_group, ArdourCanvas::Rect (0.0, 0.0, 100, timebar_height));
+	CANVAS_DEBUG_NAME (cd_marker_bar_drag_rect, "cd marker drag");
+	cue_marker_bar_drag_rect->set_outline (false);
+	cue_marker_bar_drag_rect->hide ();
+
 	range_bar_drag_rect = new ArdourCanvas::Rectangle (range_marker_group, ArdourCanvas::Rect (0.0, 0.0, 100, timebar_height));
 	CANVAS_DEBUG_NAME (range_bar_drag_rect, "range drag");
 	range_bar_drag_rect->set_outline (false);
@@ -207,15 +259,19 @@ Editor::initialize_canvas ()
 	transport_punchout_line->set_y1 (ArdourCanvas::COORD_MAX);
 	transport_punchout_line->hide();
 
-	tempo_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_tempo_bar_event), tempo_bar));
-	meter_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_meter_bar_event), meter_bar));
-	marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_marker_bar_event), marker_bar));
-	cd_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_cd_marker_bar_event), cd_marker_bar));
+	tempo_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), tempo_bar, TempoBarItem, "tempo bar"));
+	mapping_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), mapping_bar, MappingBarItem, "mapping bar"));
+	meter_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), meter_bar, MeterBarItem, "meter bar"));
+	marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), marker_bar, MarkerBarItem, "marker bar"));
+	cd_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), cd_marker_bar, CdMarkerBarItem, "cd marker bar"));
+	cue_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), cue_marker_bar, CueMarkerBarItem, "cd marker bar"));
 	videotl_group->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_videotl_bar_event), videotl_group));
-	range_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_range_marker_bar_event), range_marker_bar));
-	transport_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_transport_marker_bar_event), transport_marker_bar));
+	range_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), range_marker_bar, RangeMarkerBarItem, "range marker bar"));
+	transport_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), transport_marker_bar, TransportMarkerBarItem, "transport marker bar"));
 
-	playhead_cursor = new EditorCursor (*this, &Editor::canvas_playhead_cursor_event);
+	_playhead_cursor = new EditorCursor (*this, &Editor::canvas_playhead_cursor_event, X_("playhead"));
+
+	_snapped_cursor = new EditorCursor (*this, X_("snapped"));
 
 	_canvas_drop_zone = new ArdourCanvas::Rectangle (hv_scroll_group, ArdourCanvas::Rect (0.0, 0.0, ArdourCanvas::COORD_MAX, 0.0));
 	/* this thing is transparent */
@@ -239,7 +295,7 @@ Editor::initialize_canvas ()
 	_track_canvas->add_events (Gdk::POINTER_MOTION_HINT_MASK | Gdk::SCROLL_MASK | Gdk::KEY_PRESS_MASK | Gdk::KEY_RELEASE_MASK);
 	_track_canvas->signal_leave_notify_event().connect (sigc::mem_fun(*this, &Editor::left_track_canvas), false);
 	_track_canvas->signal_enter_notify_event().connect (sigc::mem_fun(*this, &Editor::entered_track_canvas), false);
-	_track_canvas->set_flags (CAN_FOCUS);
+	_track_canvas->set_can_focus ();
 
 	_track_canvas->PreRender.connect (sigc::mem_fun(*this, &Editor::pre_render));
 
@@ -247,11 +303,9 @@ Editor::initialize_canvas ()
 
 	vector<TargetEntry> target_table;
 
-	// Drag-N-Drop from the region list can generate this target
-	target_table.push_back (TargetEntry ("regions"));
-
-	target_table.push_back (TargetEntry ("text/plain"));
+	target_table.push_back (TargetEntry ("x-ardour/region.pbdid", TARGET_SAME_APP));
 	target_table.push_back (TargetEntry ("text/uri-list"));
+	target_table.push_back (TargetEntry ("text/plain"));
 	target_table.push_back (TargetEntry ("application/x-rootwin-drop"));
 
 	_track_canvas->drag_dest_set (target_table);
@@ -287,10 +341,6 @@ Editor::track_canvas_viewport_size_allocated ()
 
 	if (height_changed) {
 
-		for (LocationMarkerMap::iterator i = location_markers.begin(); i != location_markers.end(); ++i) {
-			i->second->canvas_height_set (_visible_canvas_height);
-		}
-
 		vertical_adjustment.set_page_size (_visible_canvas_height);
 		if ((vertical_adjustment.get_value() + _visible_canvas_height) >= vertical_adjustment.get_upper()) {
 			/*
@@ -304,7 +354,8 @@ Editor::track_canvas_viewport_size_allocated ()
 	}
 
 	update_fixed_rulers();
-	redisplay_tempo (false);
+	update_tempo_based_rulers ();
+	redisplay_grid (false);
 	_summary->set_overlays_dirty ();
 }
 
@@ -314,11 +365,11 @@ Editor::reset_controls_layout_width ()
 	GtkRequisition req = { 0, 0 };
 	gint w;
 
-	edit_controls_vbox.size_request (req);
+	req = edit_controls_vbox.size_request ();
 	w = req.width;
 
-	if (_group_tabs->is_visible()) {
-		_group_tabs->size_request (req);
+	if (_group_tabs->get_visible()) {
+		req = _group_tabs->size_request ();
 		w += req.width;
 	}
 
@@ -371,7 +422,10 @@ Editor::track_canvas_drag_data_received (const RefPtr<Gdk::DragContext>& context
 					 const SelectionData& data,
 					 guint info, guint time)
 {
-	if (data.get_target() == "regions") {
+	if (!ARDOUR_UI_UTILS::engine_is_running ()) {
+		return;
+	}
+	if (data.get_target() == "x-ardour/region.pbdid") {
 		drop_regions (context, x, y, data, info, time);
 	} else {
 		drop_paths (context, x, y, data, info, time);
@@ -379,16 +433,17 @@ Editor::track_canvas_drag_data_received (const RefPtr<Gdk::DragContext>& context
 }
 
 bool
-Editor::idle_drop_paths (vector<string> paths, framepos_t frame, double ypos, bool copy)
+Editor::idle_drop_paths (vector<string> paths, timepos_t pos, double ypos, bool copy)
 {
-	drop_paths_part_two (paths, frame, ypos, copy);
+	drop_paths_part_two (paths, pos, ypos, copy);
 	return false;
 }
 
 void
-Editor::drop_paths_part_two (const vector<string>& paths, framepos_t frame, double ypos, bool copy)
+Editor::drop_paths_part_two (const vector<string>& paths, timepos_t const & p, double ypos, bool copy)
 {
 	RouteTimeAxisView* tv;
+	timepos_t pos (p);
 
 	/* MIDI files must always be imported, because we consider them
 	 * writable. So split paths into two vectors, and follow the import
@@ -406,21 +461,19 @@ Editor::drop_paths_part_two (const vector<string>& paths, framepos_t frame, doub
 		}
 	}
 
-
 	std::pair<TimeAxisView*, int> const tvp = trackview_by_y_position (ypos, false);
 	if (tvp.first == 0) {
 
 		/* drop onto canvas background: create new tracks */
 
-		frame = 0;
-		InstrumentSelector is; // instantiation builds instrument-list and sets default.
-		do_import (midi_paths, Editing::ImportDistinctFiles, ImportAsTrack, SrcBest, SMFTrackName, SMFTempoIgnore, frame, is.selected_instrument());
+		InstrumentSelector is(InstrumentSelector::ForTrackDefault); // instantiation builds instrument-list and sets default.
+	        do_import (midi_paths, Editing::ImportDistinctFiles, ImportAsTrack, SrcBest, SMFTrackNumber, SMFTempoIgnore, pos, is.selected_instrument());
 
 		if (UIConfiguration::instance().get_only_copy_imported_files() || copy) {
 			do_import (audio_paths, Editing::ImportDistinctFiles, Editing::ImportAsTrack,
-			           SrcBest, SMFTrackName, SMFTempoIgnore, frame);
+			           SrcBest, SMFTrackName, SMFTempoIgnore, pos);
 		} else {
-			do_embed (audio_paths, Editing::ImportDistinctFiles, ImportAsTrack, frame);
+			do_embed (audio_paths, Editing::ImportDistinctFiles, ImportAsTrack, pos);
 		}
 
 	} else if ((tv = dynamic_cast<RouteTimeAxisView*> (tvp.first)) != 0) {
@@ -428,17 +481,14 @@ Editor::drop_paths_part_two (const vector<string>& paths, framepos_t frame, doub
 		/* check that its a track, not a bus */
 
 		if (tv->track()) {
-			/* select the track, then embed/import */
-			selection->set (tv);
-
 			do_import (midi_paths, Editing::ImportSerializeFiles, ImportToTrack,
-				   SrcBest, SMFTrackName, SMFTempoIgnore, frame);
+				   SrcBest, SMFTrackNumber, SMFTempoIgnore, pos, std::shared_ptr<ARDOUR::PluginInfo>(), tv->track ());
 
 			if (UIConfiguration::instance().get_only_copy_imported_files() || copy) {
 				do_import (audio_paths, Editing::ImportSerializeFiles, Editing::ImportToTrack,
-					   SrcBest, SMFTrackName, SMFTempoIgnore, frame);
+					   SrcBest, SMFTrackName, SMFTempoIgnore, pos, std::shared_ptr<PluginInfo>(), tv->track ());
 			} else {
-				do_embed (audio_paths, Editing::ImportSerializeFiles, ImportToTrack, frame);
+				do_embed (audio_paths, Editing::ImportSerializeFiles, ImportToTrack, pos, std::shared_ptr<ARDOUR::PluginInfo>(), tv->track ());
 			}
 		}
 	}
@@ -454,16 +504,15 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 	GdkEvent ev;
 	double cy;
 
-	if (convert_drop_to_paths (paths, context, x, y, data, info, time) == 0) {
+	if (_session && convert_drop_to_paths (paths, data)) {
 
-		/* D-n-D coordinates are window-relative, so convert to canvas coordinates
-		 */
+		/* D-n-D coordinates are window-relative, so convert to canvas coordinates */
 
 		ev.type = GDK_BUTTON_RELEASE;
 		ev.button.x = x;
 		ev.button.y = y;
 
-		MusicFrame when (window_event_sample (&ev, 0, &cy), 0);
+		timepos_t when (window_event_sample (&ev, 0, &cy));
 		snap_to (when);
 
 		bool copy = ((context->get_actions() & (Gdk::ACTION_COPY | Gdk::ACTION_LINK | Gdk::ACTION_MOVE)) == Gdk::ACTION_COPY);
@@ -472,9 +521,9 @@ Editor::drop_paths (const RefPtr<Gdk::DragContext>& context,
 		   the main event loop with GTK/Quartz. Since import/embed wants
 		   to push up a progress dialog, defer all this till we go idle.
 		*/
-		Glib::signal_idle().connect (sigc::bind (sigc::mem_fun (*this, &Editor::idle_drop_paths), paths, when.frame, cy, copy));
+		Glib::signal_idle().connect (sigc::bind (sigc::mem_fun (*this, &Editor::idle_drop_paths), paths, when, cy, copy));
 #else
-		drop_paths_part_two (paths, when.frame, cy, copy);
+		drop_paths_part_two (paths, when, cy, copy);
 #endif
 	}
 
@@ -520,7 +569,7 @@ Editor::maybe_autoscroll (bool allow_horiz, bool allow_vert, bool from_headers)
 
 		controls_layout.get_parent()->translate_coordinates (*toplevel,
 		                                                     alloc.get_x(), alloc.get_y(),
-		        		                             wx, wy);
+		                                                     wx, wy);
 
 		scrolling_boundary = ArdourCanvas::Rect (wx, wy, wx + alloc.get_width(), wy + alloc.get_height());
 
@@ -569,69 +618,82 @@ Editor::maybe_autoscroll (bool allow_horiz, bool allow_vert, bool from_headers)
 
 	toplevel->get_window()->get_pointer (x, y, mask);
 
-	if ((allow_horiz && ((x < scrolling_boundary.x0 && leftmost_frame > 0) || x >= scrolling_boundary.x1)) ||
+	if ((allow_horiz && ((x < scrolling_boundary.x0 && _leftmost_sample > 0) || x >= scrolling_boundary.x1)) ||
 	    (allow_vert && ((y < scrolling_boundary.y0 && vertical_adjustment.get_value() > 0)|| y >= scrolling_boundary.y1))) {
 		start_canvas_autoscroll (allow_horiz, allow_vert, scrolling_boundary);
 	}
 }
 
+bool
+Editor::drag_active () const
+{
+	return _drags->active();
+}
+
+bool
+Editor::preview_video_drag_active () const
+{
+	return _drags->preview_video ();
+}
+
 bool
 Editor::autoscroll_active () const
 {
 	return autoscroll_connection.connected ();
 }
 
-std::pair <framepos_t,framepos_t>
-Editor::session_gui_extents ( bool use_extra ) const
+std::pair <timepos_t,timepos_t>
+Editor::session_gui_extents (bool use_extra) const
 {
 	if (!_session) {
-		return std::pair <framepos_t,framepos_t>(max_framepos,0);
+		return std::make_pair (timepos_t::max (Temporal::AudioTime), timepos_t (Temporal::AudioTime));
 	}
-	
-	framecnt_t session_extent_start = _session->current_start_frame();
-	framecnt_t session_extent_end = _session->current_end_frame();
 
-	//calculate the extents of all regions in every playlist
-	//NOTE:  we should listen to playlists, and cache these values so we don't calculate them every time.
+	timepos_t session_extent_start (_session->current_start_sample());
+	timepos_t session_extent_end (_session->current_end_sample());
+
+	/* calculate the extents of all regions in every playlist
+	 * NOTE: we should listen to playlists, and cache these values so we don't calculate them every time.
+	 */
 	{
-		boost::shared_ptr<RouteList> rl = _session->get_routes();
-		for (RouteList::iterator r = rl->begin(); r != rl->end(); ++r) {
-			boost::shared_ptr<Track> tr = boost::dynamic_pointer_cast<Track> (*r);
-			if (tr) {
-				boost::shared_ptr<Playlist> pl = tr->playlist();
-				if ( pl && !pl->all_regions_empty() ) {
-					pair<framepos_t, framepos_t> e;
-					e = pl->get_extent();
-					if (e.first < session_extent_start) {
-						session_extent_start = e.first;
-					}
-					if (e.second > session_extent_end) {
-						session_extent_end = e.second;
-					}
-				}
+		std::shared_ptr<RouteList const> rl = _session->get_routes();
+		for (auto const& r : *rl) {
+			std::shared_ptr<Track> tr = std::dynamic_pointer_cast<Track> (r);
+
+			if (!tr) {
+				continue;
 			}
+			if (tr->presentation_info ().hidden ()) {
+				continue;
+			}
+			pair<timepos_t, timepos_t> e = tr->playlist()->get_extent ();
+			if (e.first == e.second) {
+				/* no regions present */
+				continue;
+			}
+			session_extent_start = std::min (session_extent_start, e.first);
+			session_extent_end   = std::max (session_extent_end, e.second);
 		}
 	}
 
-	//ToDo: also incorporate automation regions (in case the session has no audio/midi but is just used for automating plugins or the like)
+	/* ToDo: also incorporate automation regions (in case the session has no audio/midi but is just used for automating plugins or the like) */
 
-	//add additional time to the ui extents ( user-defined in config )
+	/* add additional time to the ui extents (user-defined in config) */
 	if (use_extra) {
-		framecnt_t const extra = UIConfiguration::instance().get_extra_ui_extents_time() * 60 * _session->nominal_frame_rate();
-		session_extent_end += extra;
-		session_extent_start -= extra;
+		timecnt_t const extra ((samplepos_t) (UIConfiguration::instance().get_extra_ui_extents_time() * 60 * _session->nominal_sample_rate()));
+		session_extent_end += timepos_t (extra);
+		session_extent_start.shift_earlier (extra);
 	}
-			
-	//range-check
-	if (session_extent_end > max_framepos) {
-		session_extent_end = max_framepos;
+
+	/* range-check */
+	if (session_extent_end >= timepos_t::max (Temporal::AudioTime)) {
+		session_extent_end = timepos_t::max (Temporal::AudioTime);
 	}
-	if (session_extent_start < 0) {
-		session_extent_start = 0;
+	if (session_extent_start.is_negative()) {
+		session_extent_start = timepos_t (0);
 	}
-	
-	std::pair <framepos_t,framepos_t> ret (session_extent_start, session_extent_end);
-	return ret;
+
+	return std::make_pair (session_extent_start, session_extent_end);
 }
 
 bool
@@ -639,7 +701,7 @@ Editor::autoscroll_canvas ()
 {
 	int x, y;
 	Gdk::ModifierType mask;
-	frameoffset_t dx = 0;
+	sampleoffset_t dx = 0;
 	bool no_stop = false;
 	Gtk::Window* toplevel = dynamic_cast<Gtk::Window*>(contents().get_toplevel());
 
@@ -654,7 +716,7 @@ Editor::autoscroll_canvas ()
 
 	if (autoscroll_horizontal_allowed) {
 
-		framepos_t new_frame = leftmost_frame;
+		samplepos_t new_sample = _leftmost_sample;
 
 		/* horizontal */
 
@@ -665,13 +727,13 @@ Editor::autoscroll_canvas ()
 			dx += 10 + (2 * (autoscroll_cnt/2));
 
 			dx = pixel_to_sample (dx);
-			
+
 			dx *= UIConfiguration::instance().get_draggable_playhead_speed();
 
-			if (leftmost_frame < max_framepos - dx) {
-				new_frame = leftmost_frame + dx;
+			if (_leftmost_sample < max_samplepos - dx) {
+				new_sample = _leftmost_sample + dx;
 			} else {
-				new_frame = max_framepos;
+				new_sample = max_samplepos;
 			}
 
 			no_stop = true;
@@ -685,17 +747,17 @@ Editor::autoscroll_canvas ()
 
 			dx *= UIConfiguration::instance().get_draggable_playhead_speed();
 
-			if (leftmost_frame >= dx) {
-				new_frame = leftmost_frame - dx;
+			if (_leftmost_sample >= dx) {
+				new_sample = _leftmost_sample - dx;
 			} else {
-				new_frame = 0;
+				new_sample = 0;
 			}
 
 			no_stop = true;
 		}
 
-		if (new_frame != leftmost_frame) {
-			vc.time_origin = new_frame;
+		if (new_sample != _leftmost_sample) {
+			vc.time_origin = new_sample;
 			vc.add (VisualChange::TimeOrigin);
 		}
 	}
@@ -898,6 +960,15 @@ Editor::entered_track_canvas (GdkEventCrossing* ev)
 	reset_canvas_action_sensitivity (true);
 
 	if (!was_within) {
+
+		if (internal_editing()) {
+			/* ensure that key events go here because there are
+			   internal editing bindings associated only with the
+			   canvas. if the focus is elsewhere, we cannot find them.
+			*/
+			_track_canvas->grab_focus ();
+		}
+
 		if (ev->detail == GDK_NOTIFY_NONLINEAR ||
 		    ev->detail == GDK_NOTIFY_NONLINEAR_VIRTUAL) {
 			/* context menu or something similar */
@@ -917,6 +988,9 @@ Editor::ensure_time_axis_view_is_visible (TimeAxisView const & track, bool at_to
 		return;
 	}
 
+	/* apply any pending [height] changes */
+	(void) process_redisplay_track_views ();
+
 	/* compute visible area of trackview group, as offsets from top of
 	 * trackview group.
 	 */
@@ -960,6 +1034,7 @@ void
 Editor::tie_vertical_scrolling ()
 {
 	if (pending_visual_change.idle_handler_id < 0) {
+		_region_peak_cursor->hide ();
 		_summary->set_overlays_dirty ();
 	}
 }
@@ -969,7 +1044,7 @@ Editor::set_horizontal_position (double p)
 {
 	horizontal_adjustment.set_value (p);
 
-	leftmost_frame = (framepos_t) floor (p * samples_per_pixel);
+	_leftmost_sample = (samplepos_t) floor (p * samples_per_pixel);
 }
 
 void
@@ -986,13 +1061,14 @@ Editor::color_handler()
 	bbt_ruler->set_fill_color (base);
 	bbt_ruler->set_outline_color (text);
 
-	playhead_cursor->set_color (UIConfiguration::instance().color ("play head"));
+	_playhead_cursor->set_color (UIConfiguration::instance().color ("play head"));
 
 	meter_bar->set_fill_color (UIConfiguration::instance().color_mod ("meter bar", "marker bar"));
 	meter_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
 
 	tempo_bar->set_fill_color (UIConfiguration::instance().color_mod ("tempo bar", "marker bar"));
-	tempo_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
+
+	mapping_bar->set_fill_color (UIConfiguration::instance().color_mod ("mapping bar", "marker bar"));
 
 	marker_bar->set_fill_color (UIConfiguration::instance().color_mod ("marker bar", "marker bar"));
 	marker_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
@@ -1000,6 +1076,9 @@ Editor::color_handler()
 	cd_marker_bar->set_fill_color (UIConfiguration::instance().color_mod ("cd marker bar", "marker bar"));
 	cd_marker_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
 
+	cue_marker_bar->set_fill_color (UIConfiguration::instance().color_mod ("cd marker bar", "marker bar"));
+	cue_marker_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
+
 	range_marker_bar->set_fill_color (UIConfiguration::instance().color_mod ("range marker bar", "marker bar"));
 	range_marker_bar->set_outline_color (UIConfiguration::instance().color ("marker bar separator"));
 
@@ -1042,7 +1121,7 @@ Editor::color_handler()
 	_track_canvas->queue_draw ();
 
 /*
-	redisplay_tempo (true);
+	redisplay_grid (true);
 
 	if (_session)
 	      _session->tempo_map().apply_with_metrics (*this, &Editor::draw_metric_marks); // redraw metric markers
@@ -1052,7 +1131,7 @@ Editor::color_handler()
 double
 Editor::horizontal_position () const
 {
-	return sample_to_pixel (leftmost_frame);
+	return sample_to_pixel (_leftmost_sample);
 }
 
 bool
@@ -1146,26 +1225,6 @@ Editor::pop_canvas_cursor ()
 	}
 }
 
-Gdk::Cursor*
-Editor::which_grabber_cursor () const
-{
-	Gdk::Cursor* c = _cursors->grabber;
-
-	switch (_edit_point) {
-	case EditAtMouse:
-		c = _cursors->grabber_edit_point;
-		break;
-	default:
-		boost::shared_ptr<Movable> m = _movable.lock();
-		if (m && m->locked()) {
-			c = _cursors->speaker;
-		}
-		break;
-	}
-
-	return c;
-}
-
 Gdk::Cursor*
 Editor::which_trim_cursor (bool left) const
 {
@@ -1176,7 +1235,6 @@ Editor::which_trim_cursor (bool left) const
 	Trimmable::CanTrim ct = entered_regionview->region()->can_trim ();
 
 	if (left) {
-
 		if (ct & Trimmable::FrontTrimEarlier) {
 			return _cursors->left_side_trim;
 		} else {
@@ -1265,7 +1323,7 @@ Editor::which_track_cursor () const
 	switch (_join_object_range_state) {
 	case JOIN_OBJECT_RANGE_NONE:
 	case JOIN_OBJECT_RANGE_OBJECT:
-		cursor = which_grabber_cursor ();
+		cursor = _cursors->grabber;
 		break;
 	case JOIN_OBJECT_RANGE_RANGE:
 		cursor = _cursors->selector;
@@ -1297,29 +1355,22 @@ Editor::which_canvas_cursor(ItemType type) const
 	    mouse_mode == MouseContent) {
 
 		/* find correct cursor to use in object/smart mode */
-
 		switch (type) {
 		case RegionItem:
 		/* We don't choose a cursor for these items on top of a region view,
 		   because this would push a new context on the enter stack which
 		   means switching the region context for things like smart mode
-		   won't actualy change the cursor. */
-		// case RegionViewNameHighlight:
-		// case RegionViewName:
+		   won't actually change the cursor. */
 		// case WaveItem:
 		case StreamItem:
 		case AutomationTrackItem:
 			cursor = which_track_cursor ();
 			break;
+		case MappingBarItem:
+			cursor = _cursors->trimmer;
+			break;
 		case PlayheadCursorItem:
-			switch (_edit_point) {
-			case EditAtMouse:
-				cursor = _cursors->grabber_edit_point;
-				break;
-			default:
-				cursor = _cursors->grabber;
-				break;
-			}
+			cursor = _cursors->grabber;
 			break;
 		case SelectionItem:
 			cursor = _cursors->selector;
@@ -1361,17 +1412,31 @@ Editor::which_canvas_cursor(ItemType type) const
 			cursor = _cursors->cross_hair;
 			break;
 		case LeftFrameHandle:
-			if ( effective_mouse_mode() == MouseObject )  // (smart mode): if the user is in the btm half, show the trim cursor
+			if (effective_mouse_mode() == MouseObject) // (smart mode): if the user is in the btm half, show the trim cursor
 				cursor = which_trim_cursor (true);
 			else
-				cursor = _cursors->selector;  // (smart mode): in the top half, just show the selection (range) cursor
+				cursor = _cursors->selector; // (smart mode): in the top half, just show the selection (range) cursor
 			break;
 		case RightFrameHandle:
-			if ( effective_mouse_mode() == MouseObject )  //see above
+			if (effective_mouse_mode() == MouseObject) // see above
 				cursor = which_trim_cursor (false);
 			else
 				cursor = _cursors->selector;
 			break;
+		case RegionViewName:
+		case RegionViewNameHighlight:
+			/* the trim bar is used for trimming, but we have to determine if we are on the left or right side of the region */
+			cursor = MouseCursors::invalid_cursor ();
+			if (entered_regionview) {
+				samplepos_t where;
+				bool in_track_canvas;
+				if (mouse_sample (where, in_track_canvas)) {
+					samplepos_t start = entered_regionview->region()->first_sample();
+					samplepos_t end = entered_regionview->region()->last_sample();
+					cursor = which_trim_cursor ((where - start) < (end - where));
+				}
+			}
+			break;
 		case StartCrossFadeItem:
 			cursor = _cursors->fade_in;
 			break;
@@ -1419,6 +1484,7 @@ Editor::which_canvas_cursor(ItemType type) const
 
 		/* These items use the grabber cursor at all times */
 	case MeterMarkerItem:
+	case BBTMarkerItem:
 	case TempoMarkerItem:
 	case MeterBarItem:
 	case TempoBarItem:
@@ -1426,10 +1492,11 @@ Editor::which_canvas_cursor(ItemType type) const
 	case MarkerBarItem:
 	case RangeMarkerBarItem:
 	case CdMarkerBarItem:
+	case CueMarkerBarItem:
 	case VideoBarItem:
 	case TransportMarkerBarItem:
 	case DropZoneItem:
-		cursor = which_grabber_cursor();
+		cursor = _cursors->grabber;
 		break;
 
 	default:
@@ -1458,8 +1525,8 @@ Editor::choose_canvas_cursor_on_entry (ItemType type)
 void
 Editor::update_all_enter_cursors ()
 {
-	for (std::vector<EnterContext>::iterator i = _enter_stack.begin(); i != _enter_stack.end(); ++i) {
-		i->cursor_ctx->change(which_canvas_cursor(i->item_type));
+	for (auto & ec : _enter_stack) {
+		ec.cursor_ctx->change(which_canvas_cursor (ec.item_type));
 	}
 }
 
