@@ -1,34 +1,39 @@
 /*
-    Copyright (C) 2000 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2018 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2008 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2015-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2015-2019 Ben Loftis <ben@harrisonconsoles.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cstdlib>
 #include <cmath>
 #include <algorithm>
 #include <typeinfo>
 
-#include "pbd/stacktrace.h"
-
 #include "ardour/audio_track.h"
 #include "ardour/midi_track.h"
 #include "ardour/midi_region.h"
-#include "ardour/region_factory.h"
 #include "ardour/profile.h"
+#include "ardour/region_factory.h"
 
 #include "canvas/canvas.h"
 #include "canvas/text.h"
@@ -48,6 +53,7 @@
 #include "editor_drag.h"
 #include "midi_time_axis.h"
 #include "editor_regions.h"
+#include "editor_sources.h"
 #include "ui_config.h"
 #include "verbose_cursor.h"
 
@@ -75,11 +81,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 	switch (direction) {
 	case GDK_SCROLL_UP:
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::ScrollZoomHorizontalModifier)) {
-			if (UIConfiguration::instance().get_use_mouse_position_as_zoom_focus_on_scroll()) {
-				temporal_zoom_step_mouse_focus (false);
-			} else {
-				temporal_zoom_step (false);
-			}
+			temporal_zoom_step_mouse_focus (false);
 			return true;
 		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::ScrollHorizontalModifier)) {
 			scroll_left_step ();
@@ -104,11 +106,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 
 	case GDK_SCROLL_DOWN:
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::ScrollZoomHorizontalModifier)) {
-			if (UIConfiguration::instance().get_use_mouse_position_as_zoom_focus_on_scroll()) {
-				temporal_zoom_step_mouse_focus (true);
-			} else {
-				temporal_zoom_step (true);
-			}
+			temporal_zoom_step_mouse_focus (true);
 			return true;
 		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::ScrollHorizontalModifier)) {
 			scroll_right_step ();
@@ -167,7 +165,7 @@ bool
 Editor::track_canvas_button_press_event (GdkEventButton *event)
 {
 	_track_canvas->grab_focus();
-	if (!Keyboard::is_context_menu_event (event)) {
+	if (!internal_editing() && !Keyboard::is_context_menu_event (event)) {
 		begin_reversible_selection_op (X_("Clear Selection Click (track canvas)"));
 		selection->clear ();
 		commit_reversible_selection_op();
@@ -180,7 +178,15 @@ Editor::track_canvas_button_release_event (GdkEventButton *event)
 {
 	if (!Keyboard::is_context_menu_event (event)) {
 		if (_drags->active ()) {
-			_drags->end_grab ((GdkEvent*) event);
+
+			GdkEvent copy = *((GdkEvent*) event);
+			Duple winpos = Duple (event->x, event->y);
+			Duple where = _track_canvas->window_to_canvas (winpos);
+
+			copy.button.x = where.x;
+			copy.button.y = where.y;
+
+			_drags->end_grab (&copy);
 		}
 	}
 	return false;
@@ -641,7 +647,7 @@ Editor::canvas_fade_out_handle_event (GdkEvent *event, ArdourCanvas::Item* item,
 }
 
 struct DescendingRegionLayerSorter {
-	bool operator()(boost::shared_ptr<Region> a, boost::shared_ptr<Region> b) {
+	bool operator()(std::shared_ptr<Region> a, std::shared_ptr<Region> b) {
 		return a->layer() > b->layer();
 	}
 };
@@ -658,13 +664,6 @@ Editor::canvas_control_point_event (GdkEvent *event, ArdourCanvas::Item* item, C
 		clicked_routeview = dynamic_cast<RouteTimeAxisView*>(clicked_axisview);
 		clicked_regionview = 0;
 		break;
-
-	case GDK_SCROLL_UP:
-		break;
-
-	case GDK_SCROLL_DOWN:
-		break;
-
 	default:
 		break;
 	}
@@ -877,7 +876,7 @@ Editor::canvas_region_view_name_highlight_event (GdkEvent* event, ArdourCanvas::
 		break;
 	case GDK_MOTION_NOTIFY:
 		motion_handler (item, event);
-		ret = true; // force this to avoid progagating the event into the regionview
+		ret = true; // force this to avoid propagating the event into the regionview
 		break;
 	case GDK_ENTER_NOTIFY:
 		ret = enter_handler (item, event, RegionViewNameHighlight);
@@ -979,30 +978,6 @@ Editor::canvas_marker_event (GdkEvent *event, ArdourCanvas::Item* item, ArdourMa
 	return typed_event (item, event, MarkerItem);
 }
 
-bool
-Editor::canvas_marker_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
-{
-	return typed_event (item, event, MarkerBarItem);
-}
-
-bool
-Editor::canvas_range_marker_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
-{
-	return typed_event (item, event, RangeMarkerBarItem);
-}
-
-bool
-Editor::canvas_transport_marker_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
-{
-	return typed_event (item, event, TransportMarkerBarItem);
-}
-
-bool
-Editor::canvas_cd_marker_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
-{
-	return typed_event (item, event, CdMarkerBarItem);
-}
-
 bool
 Editor::canvas_videotl_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
 {
@@ -1027,6 +1002,12 @@ Editor::canvas_meter_marker_event (GdkEvent *event, ArdourCanvas::Item* item, Me
 	return typed_event (item, event, MeterMarkerItem);
 }
 
+bool
+Editor::canvas_bbt_marker_event (GdkEvent *event, ArdourCanvas::Item* item, BBTMarker* /*marker*/)
+{
+	return typed_event (item, event, BBTMarkerItem);
+}
+
 bool
 Editor::canvas_ruler_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType type)
 {
@@ -1040,11 +1021,12 @@ Editor::canvas_ruler_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType
 
 		switch (event->scroll.direction) {
 		case GDK_SCROLL_UP:
-			if (Keyboard::modifier_state_equals(event->scroll.state,
-			                                    Keyboard::ScrollHorizontalModifier)) {
+			if (Keyboard::modifier_state_equals (event->scroll.state, Keyboard::ScrollHorizontalModifier)) {
 				scroll_left_step ();
 			} else if (UIConfiguration::instance().get_use_mouse_position_as_zoom_focus_on_scroll()) {
 				temporal_zoom_step_mouse_focus (false);
+			} else if (Keyboard::modifier_state_equals (event->scroll.state, Keyboard::PrimaryModifier)) {
+				temporal_zoom_step_mouse_focus (false);
 			} else {
 				temporal_zoom_step (false);
 			}
@@ -1052,11 +1034,12 @@ Editor::canvas_ruler_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType
 			break;
 
 		case GDK_SCROLL_DOWN:
-			if (Keyboard::modifier_state_equals(event->scroll.state,
-			                                    Keyboard::ScrollHorizontalModifier)) {
+			if (Keyboard::modifier_state_equals (event->scroll.state, Keyboard::ScrollHorizontalModifier)) {
 				scroll_right_step ();
 			} else if (UIConfiguration::instance().get_use_mouse_position_as_zoom_focus_on_scroll()) {
 				temporal_zoom_step_mouse_focus (true);
+			} else if (Keyboard::modifier_state_equals (event->scroll.state, Keyboard::PrimaryModifier)) {
+				temporal_zoom_step_mouse_focus (true);
 			} else {
 				temporal_zoom_step (true);
 			}
@@ -1084,15 +1067,26 @@ Editor::canvas_ruler_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType
 }
 
 bool
-Editor::canvas_tempo_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
+Editor::canvas_ruler_bar_event (GdkEvent *event, ArdourCanvas::Item* item, ItemType type, std::string const& color_name)
 {
-	return typed_event (item, event, TempoBarItem);
-}
+	/* XXX consolidate with Editor::canvas_ruler_event ? */
 
-bool
-Editor::canvas_meter_bar_event (GdkEvent *event, ArdourCanvas::Item* item)
-{
-	return typed_event (item, event, MeterBarItem);
+	switch (event->type) {
+	case GDK_ENTER_NOTIFY:
+		if (event->crossing.detail != GDK_NOTIFY_INFERIOR) {
+			item->set_fill_color (Gtkmm2ext::HSV(UIConfiguration::instance().color_mod (color_name, "marker bar")).lighter(0.20).color());
+		}
+		break;
+	case GDK_LEAVE_NOTIFY:
+		if (event->crossing.detail != GDK_NOTIFY_INFERIOR) {
+			item->set_fill_color (UIConfiguration::instance().color_mod (color_name, "marker bar"));
+		}
+		break;
+	default:
+		break;
+	};
+
+	return typed_event (item, event, type);
 }
 
 bool
@@ -1144,6 +1138,10 @@ Editor::canvas_drop_zone_event (GdkEvent* event)
 	case GDK_LEAVE_NOTIFY:
 		return typed_event (_canvas_drop_zone, event, DropZoneItem);
 
+	case GDK_MOTION_NOTIFY:
+		return motion_handler (_canvas_drop_zone, event);
+		break;
+
 	default:
 		break;
 	}
@@ -1154,8 +1152,8 @@ Editor::canvas_drop_zone_event (GdkEvent* event)
 bool
 Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context, int x, int y, guint time)
 {
-	boost::shared_ptr<Region> region;
-	boost::shared_ptr<Region> region_copy;
+	std::shared_ptr<Region> region;
+	std::shared_ptr<Region> region_copy;
 	RouteTimeAxisView* rtav;
 	GdkEvent event;
 	double px;
@@ -1196,33 +1194,18 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 	}
 
 	if (can_drop) {
-		region = _regions->get_dragged_region ();
 
-		if (region) {
-
-			if (tv.first == 0
-			    && (
-				boost::dynamic_pointer_cast<AudioRegion> (region) != 0 ||
-				boost::dynamic_pointer_cast<MidiRegion> (region) != 0
-			       )
-			   )
-			{
+		if (target == "x-ardour/region.pbdid") {
+			if (tv.first == 0 && pbdid_dragged_dt != DataType::NIL) {
 				/* drop to drop-zone */
-				context->drag_status (context->get_suggested_action(), time);
+				context->drag_status (Gdk::ACTION_COPY, time);
 				return true;
 			}
 
-			if ((boost::dynamic_pointer_cast<AudioRegion> (region) != 0 &&
-			     dynamic_cast<AudioTimeAxisView*> (tv.first) != 0) ||
-			    (boost::dynamic_pointer_cast<MidiRegion> (region) != 0 &&
-			     dynamic_cast<MidiTimeAxisView*> (tv.first) != 0)) {
-
-				/* audio to audio
-				   OR
-				   midi to midi
-				*/
-
-				context->drag_status (context->get_suggested_action(), time);
+			if ((pbdid_dragged_dt == DataType::AUDIO && dynamic_cast<AudioTimeAxisView*> (tv.first) != 0) ||
+			    (pbdid_dragged_dt == DataType::MIDI  && dynamic_cast<MidiTimeAxisView*> (tv.first) != 0)) {
+				/* audio to audio OR MIDI to MIDI */
+				context->drag_status (Gdk::ACTION_COPY, time);
 				return true;
 			}
 		} else {
@@ -1252,7 +1235,7 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 void
 Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
                       int x, int y,
-                      const SelectionData& /*data*/,
+                      const SelectionData& data,
                       guint /*info*/, guint /*time*/)
 {
 	GdkEvent event;
@@ -1264,9 +1247,11 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 	event.button.y = y;
 	/* assume we're dragging with button 1 */
 	event.motion.state = Gdk::BUTTON1_MASK;
-	framepos_t const pos = window_event_sample (&event, &px, &py);
+	samplepos_t const pos = window_event_sample (&event, &px, &py);
+
+	PBD::ID rid (data.get_data_as_string ());
+	std::shared_ptr<Region> region = RegionFactory::region_by_id (rid);
 
-	boost::shared_ptr<Region> region = _regions->get_dragged_region ();
 	if (!region) { return; }
 
 	RouteTimeAxisView* rtav = 0;
@@ -1276,22 +1261,22 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 		rtav = dynamic_cast<RouteTimeAxisView*> (tv.first);
 	} else {
 		try {
-			if (boost::dynamic_pointer_cast<AudioRegion> (region)) {
-				uint32_t output_chan = region->n_channels();
+			if (std::dynamic_pointer_cast<AudioRegion> (region)) {
+				uint32_t output_chan = region->sources().size();
 				if ((Config->get_output_auto_connect() & AutoConnectMaster) && session()->master_out()) {
 					output_chan =  session()->master_out()->n_inputs().n_audio();
 				}
-				list<boost::shared_ptr<AudioTrack> > audio_tracks;
-				audio_tracks = session()->new_audio_track (region->n_channels(), output_chan, 0, 1, region->name(), PresentationInfo::max_order);
+				list<std::shared_ptr<AudioTrack> > audio_tracks;
+				audio_tracks = session()->new_audio_track (region->sources().size(), output_chan, 0, 1, region->name(), PresentationInfo::max_order);
 				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (audio_tracks.front()));
-			} else if (boost::dynamic_pointer_cast<MidiRegion> (region)) {
+			} else if (std::dynamic_pointer_cast<MidiRegion> (region)) {
 				ChanCount one_midi_port (DataType::MIDI, 1);
-				list<boost::shared_ptr<MidiTrack> > midi_tracks;
+				list<std::shared_ptr<MidiTrack> > midi_tracks;
 				midi_tracks = session()->new_midi_track (one_midi_port, one_midi_port,
 				                                         Config->get_strict_io () || Profile->get_mixbus (),
-				                                         boost::shared_ptr<ARDOUR::PluginInfo>(),
+				                                         std::shared_ptr<ARDOUR::PluginInfo>(),
 				                                         (ARDOUR::Plugin::PresetRecord*) 0,
-				                                         (ARDOUR::RouteGroup*) 0, 1, region->name(), PresentationInfo::max_order);
+				                                         (ARDOUR::RouteGroup*) 0, 1, region->name(), PresentationInfo::max_order, Normal, true);
 				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (midi_tracks.front()));
 			} else {
 				return;
@@ -1303,11 +1288,11 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 	}
 
 	if (rtav != 0 && rtav->is_track ()) {
-		boost::shared_ptr<Region> region_copy = RegionFactory::create (region, true);
+		std::shared_ptr<Region> region_copy = RegionFactory::create (region, true);
 
-		if ((boost::dynamic_pointer_cast<AudioRegion> (region_copy) != 0 && dynamic_cast<AudioTimeAxisView*> (rtav) != 0) ||
-		    (boost::dynamic_pointer_cast<MidiRegion> (region_copy) != 0 && dynamic_cast<MidiTimeAxisView*> (rtav) != 0)) {
-			_drags->set (new RegionInsertDrag (this, region_copy, rtav, pos), &event);
+		if ((std::dynamic_pointer_cast<AudioRegion> (region_copy) != 0 && dynamic_cast<AudioTimeAxisView*> (rtav) != 0) ||
+		    (std::dynamic_pointer_cast<MidiRegion> (region_copy) != 0 && dynamic_cast<MidiTimeAxisView*> (rtav) != 0)) {
+			_drags->set (new RegionInsertDrag (this, region_copy, rtav, timepos_t (pos), drag_time_domain (region_copy.get())), &event);
 			_drags->end_grab (&event);
 		}
 	}
@@ -1354,4 +1339,3 @@ Editor::key_release_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemType
 
 	return handled;
 }
-
