@@ -1,26 +1,33 @@
 /*
-    Copyright (C) 2009 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2009-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2009-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2013-2014 Colin Fletcher <colin.m.fletcher@googlemail.com>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2015-2018 Ben Loftis <ben@harrisonconsoles.com>
+ * Copyright (C) 2016 Tim Mayberry <mojofunk@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __gtk2_ardour_editor_drag_h_
 #define __gtk2_ardour_editor_drag_h_
 
 #include <list>
+#include <vector>
 
 #include <gdk/gdk.h>
 #include <stdint.h>
@@ -30,14 +37,18 @@
 
 #include "canvas/types.h"
 
+#include "gtkmm2ext/bindings.h"
+
 #include "cursor_context.h"
 #include "editor_items.h"
 #include "mouse_cursors.h"
 #include "editing.h"
 #include "track_selection.h"
+#include "region_view.h"
 
 namespace ARDOUR {
 	class Location;
+	class Region;
 	class TempoSection;
 }
 
@@ -58,7 +69,6 @@ class TimeAxisView;
 class MidiTimeAxisView;
 class Drag;
 class NoteBase;
-class RegionView;
 class TimeAxisView;
 class RouteTimeAxisView;
 class RegionSelection;
@@ -66,6 +76,7 @@ class MidiRegionView;
 class MeterMarker;
 class ArdourMarker;
 class TempoMarker;
+class TempoCurve;
 class ControlPoint;
 class AudioRegionView;
 class AutomationLine;
@@ -109,18 +120,26 @@ public:
 		return _current_pointer_y;
 	}
 
-	/** @return current pointer frame */
-	ARDOUR::framepos_t current_pointer_frame () const {
-		return _current_pointer_frame;
+	/** @return current pointer sample */
+	ARDOUR::samplepos_t current_pointer_sample () const {
+		return _current_pointer_time.samples();
 	}
 
+	/** @return current pointer time */
+	Temporal::timepos_t current_pointer_time () const {
+		return _current_pointer_time;
+	}
+
+	/** return drag-motion displays video-frame of drag-location */
+	bool preview_video () const;
+
 private:
 	Editor* _editor;
 	std::list<Drag*> _drags;
 	bool _ending; ///< true if end_grab or abort is in progress, otherwise false
 	double _current_pointer_x; ///< canvas-coordinate space x of the current pointer
 	double _current_pointer_y; ///< canvas-coordinate space y of the current pointer
-	ARDOUR::framepos_t _current_pointer_frame; ///< frame that the pointer is now at
+	Temporal::timepos_t _current_pointer_time; ///< time that the pointer is now at
 	bool _old_follow_playhead; ///< state of Editor::follow_playhead() before the drags started
 };
 
@@ -128,7 +147,7 @@ private:
 class Drag
 {
 public:
-	Drag (Editor *, ArdourCanvas::Item *, bool trackview_only = true);
+	Drag (Editor *, ArdourCanvas::Item *, Temporal::TimeDomain td, bool trackview_only = true);
 	virtual ~Drag () {}
 
 	void set_manager (DragManager* m) {
@@ -140,16 +159,21 @@ public:
 		return _item;
 	}
 
+	Temporal::TimeDomain time_domain() const { return _time_domain; }
+	void set_time_domain (Temporal::TimeDomain);
+
 	void swap_grab (ArdourCanvas::Item *, Gdk::Cursor *, uint32_t);
 	bool motion_handler (GdkEvent*, bool);
 	void abort ();
 
-	ARDOUR::MusicFrame adjusted_frame (ARDOUR::framepos_t, GdkEvent const *, bool snap = true) const;
-	ARDOUR::framepos_t adjusted_current_frame (GdkEvent const *, bool snap = true) const;
+	Temporal::timepos_t adjusted_time (Temporal::timepos_t const &, GdkEvent const *, bool snap = true) const;
+	Temporal::timepos_t adjusted_current_time (GdkEvent const *, bool snap = true) const;
 
 	bool was_double_click() const { return _was_double_click; }
 	void set_double_click (bool yn) { _was_double_click = yn; }
 
+	void set_grab_button_anyway (GdkEvent*);
+
 	/** Called to start a grab of an item.
 	 *  @param e Event that caused the grab to start.
 	 *  @param c Cursor to use, or 0.
@@ -183,9 +207,13 @@ public:
 		return true;
 	}
 
-	/** @return minimum number of frames (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (1, 1);
+	bool preview_video () const {
+		return _preview_video;
+	}
+
+	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (1, Temporal::AudioTime), 1);
 	}
 
 	virtual bool allow_vertical_autoscroll () const {
@@ -206,11 +234,20 @@ public:
 		return _initially_vertical;
 	}
 
-	/** Set up the _pointer_frame_offset */
-	virtual void setup_pointer_frame_offset () {
-		_pointer_frame_offset = 0;
+	/** Set up the _pointer_offset */
+	virtual void setup_pointer_offset () {
+		_pointer_offset = Temporal::timecnt_t();
 	}
 
+	/** Set up the _video_offset - relative to _current_pointer_time */
+	virtual void setup_video_offset () {
+		/* video offset is always in audio time */
+		_video_offset = Temporal::timecnt_t (Temporal::AudioTime);
+		_preview_video = false;
+	}
+
+	int grab_button() const { return _grab_button; }
+
 protected:
 
 	double grab_x () const {
@@ -221,12 +258,18 @@ protected:
 		return _grab_y;
 	}
 
-	ARDOUR::framepos_t raw_grab_frame () const {
-		return _raw_grab_frame;
+	Temporal::timepos_t pixel_to_time (double x) const;
+
+	Temporal::timepos_t raw_grab_time () const {
+		return _raw_grab_time;
 	}
 
-	ARDOUR::framepos_t grab_frame () const {
-		return _grab_frame;
+	ARDOUR::samplepos_t grab_sample () const {
+		return _grab_time.samples();
+	}
+
+	Temporal::timepos_t grab_time () const {
+		return _grab_time;
 	}
 
 	double last_pointer_x () const {
@@ -237,33 +280,40 @@ protected:
 		return _last_pointer_y;
 	}
 
-	ARDOUR::framepos_t last_pointer_frame () const {
-		return _last_pointer_frame;
+	ARDOUR::samplepos_t last_pointer_sample () const {
+		return _last_pointer_time.samples();
 	}
 
-	ARDOUR::frameoffset_t snap_delta (guint const) const;
-	double  snap_delta_music (guint const) const;
+	Temporal::timepos_t last_pointer_time () const {
+		return _last_pointer_time;
+	}
+
+	Temporal::timecnt_t snap_delta (guint const) const;
 
 	double current_pointer_x () const;
 	double current_pointer_y () const;
 
 	/* sets snap delta from unsnapped pos */
-	void setup_snap_delta (ARDOUR::MusicFrame pos);
+	void setup_snap_delta (Temporal::timepos_t const & pos);
 
-	boost::shared_ptr<ARDOUR::Region> add_midi_region (MidiTimeAxisView*, bool commit);
+	std::shared_ptr<ARDOUR::Region> add_midi_region (MidiTimeAxisView*, bool commit);
 
-	void show_verbose_cursor_time (framepos_t);
-	void show_verbose_cursor_duration (framepos_t, framepos_t, double xoffset = 0);
+	void show_verbose_cursor_time (Temporal::timepos_t const &);
+	void show_verbose_cursor_duration (Temporal::timepos_t const & , Temporal::timepos_t const & , double xoffset = 0);
 	void show_verbose_cursor_text (std::string const &);
+	void show_view_preview (Temporal::timepos_t const &);
 
 	Editor* _editor; ///< our editor
 	DragManager* _drags;
 	ArdourCanvas::Item* _item; ///< our item
 	/** Offset from the mouse's position for the drag to the start of the thing that is being dragged */
-	ARDOUR::framecnt_t _pointer_frame_offset;
+	Temporal::timecnt_t _pointer_offset;
+	Temporal::timecnt_t _video_offset;
+	bool _preview_video;
 	bool _x_constrained; ///< true if x motion is constrained, otherwise false
 	bool _y_constrained; ///< true if y motion is constrained, otherwise false
 	bool _was_rolling; ///< true if the session was rolling before the drag started, otherwise false
+	ARDOUR::timepos_t _earliest_time_limit; ///< time we cannot drag before (defaults to 0, indicating no such limit)
 
 private:
 	bool _trackview_only; ///< true if pointer y value should always be relative to the top of the trackview group
@@ -275,17 +325,20 @@ private:
 	double _grab_y; ///< y of the grab start position, possibly adjusted if _trackview_only is true
 	double _last_pointer_x; ///< trackview x of the pointer last time a motion occurred
 	double _last_pointer_y; ///< trackview y of the pointer last time a motion occurred
-	ARDOUR::framepos_t _raw_grab_frame; ///< unsnapped frame that the mouse was at when start_grab was called, or 0
-	ARDOUR::framepos_t _grab_frame; ///< adjusted_frame that the mouse was at when start_grab was called, or 0
-	ARDOUR::framepos_t _last_pointer_frame; ///< adjusted_frame the last time a motion occurred
+	Temporal::timepos_t _raw_grab_time; ///< unsnapped time that the mouse was at when start_grab was called, or 0
+	Temporal::timepos_t _grab_time; ///< adjusted_time that the mouse was at when start_grab was called, or 0
+	Temporal::timepos_t _last_pointer_time; ///< adjusted_time the last time a motion occurred
+	Temporal::TimeDomain _time_domain;
 
 	/* difference between some key position's snapped and unsnapped
-	 *  framepos. used for relative snap.
+	 *  samplepos. used for relative snap.
 	 */
-	framepos_t _snap_delta;
-	double     _snap_delta_music;
+	Temporal::timecnt_t _snap_delta;
 	CursorContext::Handle _cursor_ctx; ///< cursor change context
 	bool _constraint_pressed; ///< if the keyboard indicated constraint modifier was pressed on start_grab()
+	int _grab_button;
+
+	Gtkmm2ext::Bindings::DragsBlockBindings binding_blocker;
 };
 
 class RegionDrag;
@@ -307,10 +360,10 @@ public:
 	*/
 	double layer;
 	double initial_y; ///< the initial y position of the view before any reparenting
-	framepos_t initial_position; ///< initial position of the region
-	framepos_t initial_end; ///< initial end position of the region
-	framepos_t anchored_fade_length; ///< fade_length when anchored during drag
-	boost::shared_ptr<ARDOUR::Playlist> initial_playlist;
+	Temporal::timepos_t initial_position; ///< initial position of the region
+	Temporal::timepos_t initial_end; ///< initial end position of the region
+	samplepos_t anchored_fade_length; ///< fade_length when anchored during drag
+	std::shared_ptr<ARDOUR::Playlist> initial_playlist;
 	TimeAxisView* initial_time_axis_view;
 };
 
@@ -318,7 +371,7 @@ public:
 class RegionDrag : public Drag, public sigc::trackable
 {
 public:
-	RegionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	RegionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain);
 	virtual ~RegionDrag () {}
 
 protected:
@@ -335,21 +388,55 @@ protected:
 	int _visible_y_high;
 	uint32_t _ntracks;
 
+	void setup_video_offset ();
+
 	friend class DraggingView;
 
+protected:
+	void add_stateful_diff_commands_for_playlists (ARDOUR::PlaylistSet const &);
+
 private:
 
 	void region_going_away (RegionView *);
 	PBD::ScopedConnection death_connection;
 };
 
+/** Drag the Contents rather than the bounds of a region (i.e. Slip) */
+class RegionSlipContentsDrag : public RegionDrag
+{
+public:
+
+	RegionSlipContentsDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain td);
+	virtual ~RegionSlipContentsDrag () {}
+
+	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
+	virtual void motion (GdkEvent *, bool);
+	virtual void finished (GdkEvent *, bool);
+	virtual void aborted (bool);
+};
+
+/** Brush on a region to repeat it */
+class RegionBrushDrag : public RegionDrag
+{
+public:
+	RegionBrushDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain td);
+	virtual ~RegionBrushDrag () {}
+
+	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
+	virtual void motion (GdkEvent *, bool);
+	virtual void finished (GdkEvent *, bool);
+	virtual void aborted (bool);
+private:
+	typedef std::set<Temporal::timepos_t> TimePositionSet;
+	TimePositionSet _already_pasted;
+};
 
 /** Drags involving region motion from somewhere */
 class RegionMotionDrag : public RegionDrag
 {
 public:
 
-	RegionMotionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool);
+	RegionMotionDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain td);
 	virtual ~RegionMotionDrag () {}
 
 	virtual void start_grab (GdkEvent *, Gdk::Cursor *);
@@ -364,15 +451,17 @@ public:
 
 protected:
 
-	double compute_x_delta (GdkEvent const *, ARDOUR::MusicFrame *);
+	double compute_x_delta (GdkEvent const *, Temporal::timepos_t &);
 	virtual bool y_movement_allowed (int, double, int skip_invisible = 0) const;
+	void collect_ripple_views ();
 
-	bool _brushing;
 	bool _ignore_video_lock;
-	ARDOUR::MusicFrame _last_position; ///< last position of the thing being dragged
+	Temporal::timepos_t _last_position; ///< last position of the thing being dragged
 	double _total_x_delta;
 	int _last_pointer_time_axis_view;
 	double _last_pointer_layer;
+	std::vector<ArdourMarker*> ripple_markers;
+
 private:
 	uint32_t _ndropzone;
 	uint32_t _pdropzone;
@@ -386,61 +475,57 @@ private:
 class RegionMoveDrag : public RegionMotionDrag
 {
 public:
-	RegionMoveDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool, bool);
+	RegionMoveDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, bool, Temporal::TimeDomain);
 	virtual ~RegionMoveDrag () {}
 
 	void motion (GdkEvent *, bool);
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	void clear_draggingview_list ();
+
 	bool regions_came_from_canvas () const {
 		return true;
 	}
 
-	std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (4, 4);
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (4, Temporal::AudioTime), 4);
 	}
 
-	void setup_pointer_frame_offset ();
-
-protected:
-	typedef std::set<boost::shared_ptr<ARDOUR::Playlist> > PlaylistSet;
-	void add_stateful_diff_commands_for_playlists (PlaylistSet const &);
+	void setup_pointer_offset ();
 
 private:
 	void finished_no_copy (
 		bool const,
 		bool const,
-		ARDOUR::MusicFrame,
+		Temporal::timepos_t const &,
 		int32_t const ev_state
 		);
 
 	void finished_copy (
 		bool const,
 		bool const,
-		ARDOUR::MusicFrame,
+		Temporal::timepos_t const &,
 		int32_t const ev_state
 		);
 
 	RegionView* insert_region_into_playlist (
-		boost::shared_ptr<ARDOUR::Region>,
+		std::shared_ptr<ARDOUR::Region>,
 		RouteTimeAxisView*,
 		ARDOUR::layer_t,
-		ARDOUR::MusicFrame,
-		double quarter_note,
-		PlaylistSet&,
-		bool for_music = false
+		Temporal::timepos_t const &,
+		ARDOUR::PlaylistSet&
 		);
 
 	void remove_region_from_playlist (
-		boost::shared_ptr<ARDOUR::Region>,
-		boost::shared_ptr<ARDOUR::Playlist>,
-		PlaylistSet& modified_playlists
+		std::shared_ptr<ARDOUR::Region>,
+		std::shared_ptr<ARDOUR::Playlist>,
+		ARDOUR::PlaylistSet& modified_playlists
 		);
 
 
 	void collect_new_region_view (RegionView *);
-	RouteTimeAxisView* create_destination_time_axis (boost::shared_ptr<ARDOUR::Region>, TimeAxisView* original);
+	RouteTimeAxisView* create_destination_time_axis (std::shared_ptr<ARDOUR::Region>, TimeAxisView* original);
 
 	bool _copy;
 	RegionView* _new_region_view;
@@ -450,7 +535,7 @@ private:
 class RegionInsertDrag : public RegionMotionDrag
 {
 public:
-	RegionInsertDrag (Editor *, boost::shared_ptr<ARDOUR::Region>, RouteTimeAxisView*, ARDOUR::framepos_t);
+	RegionInsertDrag (Editor *, std::shared_ptr<ARDOUR::Region>, RouteTimeAxisView*, Temporal::timepos_t const &, Temporal::TimeDomain);
 
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
@@ -460,49 +545,11 @@ public:
 	}
 };
 
-/** Region drag in splice mode */
-class RegionSpliceDrag : public RegionMoveDrag
-{
-public:
-	RegionSpliceDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
-
-	void motion (GdkEvent *, bool);
-	void finished (GdkEvent *, bool);
-	void aborted (bool);
-};
-
-/** Region drag in ripple mode */
-
-class RegionRippleDrag : public RegionMoveDrag
-{
-public:
-	RegionRippleDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
-	~RegionRippleDrag () { delete exclude; }
-
-	void motion (GdkEvent *, bool);
-	void finished (GdkEvent *, bool);
-	void aborted (bool);
-protected:
-	bool y_movement_allowed (int delta_track, double delta_layer, int skip_invisible = 0) const;
-
-private:
-	TimeAxisView *prev_tav;		// where regions were most recently dragged from
-	TimeAxisView *orig_tav;		// where drag started
-	ARDOUR::framecnt_t prev_amount;
-	ARDOUR::framepos_t prev_position;
-	ARDOUR::framecnt_t selection_length;
-	bool allow_moves_across_tracks; // only if all selected regions are on one track
-	ARDOUR::RegionList *exclude;
-	void add_all_after_to_views (TimeAxisView *tav, ARDOUR::framepos_t where, const RegionSelection &exclude, bool drag_in_progress);
-	void remove_unselected_from_views (ARDOUR::framecnt_t amount, bool move_regions);
-
-};
-
 /** "Drag" to cut a region (action only on button release) */
 class RegionCutDrag : public Drag
 {
 public:
-	RegionCutDrag (Editor*, ArdourCanvas::Item*, framepos_t);
+	RegionCutDrag (Editor*, ArdourCanvas::Item*, samplepos_t);
 	~RegionCutDrag ();
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
@@ -511,7 +558,6 @@ public:
 	void aborted (bool);
 
 private:
-	EditorCursor* line;
 };
 
 /** Drags to create regions */
@@ -526,7 +572,7 @@ public:
 
 private:
 	MidiTimeAxisView* _view;
-	boost::shared_ptr<ARDOUR::Region> _region;
+	std::shared_ptr<ARDOUR::Region> _region;
 };
 
 /** Drags to resize MIDI notes */
@@ -540,6 +586,10 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 	MidiRegionView*     region;
 	bool                relative;
@@ -559,17 +609,22 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 
-	double total_dx (GdkEvent * event) const; // total movement in quarter notes
+	Temporal::timecnt_t total_dx (GdkEvent * event) const; // total movement in quarter notes
 	int8_t total_dy () const;
 
 	MidiRegionView* _region;
 	NoteBase* _primary;
-	double _cumulative_dx;
+	Temporal::timecnt_t _cumulative_dx;
 	double _cumulative_dy;
-	double _earliest; // earliest quarter note in note selection
+	Temporal::timepos_t  _earliest; // earliest note in note selection
 	bool   _was_selected;
 	double _note_height;
 	bool   _copy;
@@ -590,22 +645,26 @@ public:
 		return mode == Editing::MouseDraw || mode == Editing::MouseContent;
 	}
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 	bool y_movement_matters () const {
 		return false;
 	}
 
 private:
 	double y_to_region (double) const;
-	ARDOUR::framecnt_t grid_frames (framepos_t) const;
+	Temporal::Beats round_to_grid (Temporal::timepos_t const & pos, GdkEvent const * event) const;
 
-	/** @return minimum number of frames (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (0, 0);
+	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (0, Temporal::AudioTime), 0);
 	}
 
 	MidiRegionView* _region_view;
 	ArdourCanvas::Rectangle* _drag_rect;
-	framepos_t _note[2];
+	Temporal::timepos_t _note[2];
 };
 
 class HitCreateDrag : public Drag
@@ -615,9 +674,9 @@ public:
 	~HitCreateDrag ();
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
-	void motion (GdkEvent *, bool);
+	void motion (GdkEvent *, bool) {}
 	void finished (GdkEvent *, bool);
-	void aborted (bool);
+	void aborted (bool) {}
 
 	bool active (Editing::MouseMode mode) {
 		return mode == Editing::MouseDraw || mode == Editing::MouseContent;
@@ -629,15 +688,14 @@ public:
 
 private:
 	double y_to_region (double) const;
-	ARDOUR::framecnt_t grid_frames (framepos_t) const;
 
-	/** @return minimum number of frames (in x) and pixels (in y) that should be considered a movement */
-	virtual std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (0, 0);
+	/** @return minimum number of samples (in x) and pixels (in y) that should be considered a movement */
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t::zero (Temporal::AudioTime), 0);
 	}
 
 	MidiRegionView* _region_view;
-	framepos_t      _last_pos;
+	Temporal::timepos_t _last_pos;
 	double          _y;
 
 };
@@ -656,7 +714,11 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
 
 private:
 	MidiRegionView* _region_view;
@@ -671,7 +733,7 @@ public:
 	AVDraggingView (RegionView *);
 
 	RegionView* view; ///< the view
-	framepos_t initial_position; ///< initial position of the region
+	samplepos_t initial_position; ///< initial position of the region
 };
 
 /** Drag of video offset */
@@ -698,8 +760,8 @@ protected:
 	std::list<AVDraggingView> _views; ///< information about all audio that are being dragged along
 
 private:
-	ARDOUR::frameoffset_t _startdrag_video_offset;
-	ARDOUR::frameoffset_t _max_backwards_drag;
+	ARDOUR::sampleoffset_t _startdrag_video_offset;
+	ARDOUR::sampleoffset_t _max_backwards_drag;
 	bool _stuck;
 };
 
@@ -709,11 +771,10 @@ class TrimDrag : public RegionDrag
 public:
 	enum Operation {
 		StartTrim,
-		EndTrim,
-		ContentsTrim,
+		EndTrim
 	};
 
-	TrimDrag (Editor *, ArdourCanvas::Item *, RegionView*, std::list<RegionView*> const &, bool preserve_fade_anchor = false);
+	TrimDrag (Editor *, ArdourCanvas::Item *, RegionView*, std::list<RegionView*> const &, Temporal::TimeDomain td, bool preserve_fade_anchor = false);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -724,7 +785,11 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
 
 private:
 
@@ -732,6 +797,8 @@ private:
 
 	bool _preserve_fade_anchor;
 	bool _jump_position_when_done;
+
+	std::vector<RegionView::DisplaySuspender> suspenders;
 };
 
 /** Meter marker drag */
@@ -753,23 +820,42 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
 	MeterMarker* _marker;
-	ARDOUR::MeterSection* _real_section;
+	Temporal::TempoMap::WritableSharedPtr map;
+	Temporal::superclock_t initial_sclock;
 
-	bool _copy;
-	Editing::SnapType _old_snap_type;
+	bool _movable;
+	Editing::GridType _old_grid_type;
 	Editing::SnapMode _old_snap_mode;
 	XMLNode* before_state;
 };
 
+/** Tempo curve drag */
+class TempoCurveDrag : public Drag
+{
+public:
+	TempoCurveDrag (Editor*, ArdourCanvas::Item*);
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+private:
+	Temporal::TempoPoint* point;
+	double initial_bpm;
+	Temporal::TempoMap::WritableSharedPtr map;
+	XMLNode* _before_state;
+};
+
 /** Tempo marker drag */
 class TempoMarkerDrag : public Drag
 {
 public:
-	TempoMarkerDrag (Editor *, ArdourCanvas::Item *, bool);
+	TempoMarkerDrag (Editor *, ArdourCanvas::Item *);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -784,24 +870,24 @@ public:
 		return true;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
 	TempoMarker* _marker;
-	ARDOUR::TempoSection* _real_section;
+	Temporal::TempoPoint const * _real_section;
+	Temporal::TempoMap::WritableSharedPtr map;
 
-	bool _copy;
 	bool _movable;
-	ARDOUR::Tempo _grab_bpm;
-	double _grab_qn;
+	double _grab_bpm;
+	Temporal::Beats _grab_qn;
 	XMLNode* _before_state;
 };
 
-/** BBT Ruler drag */
-class BBTRulerDrag : public Drag
+/** Tempo marker drag */
+class BBTMarkerDrag : public Drag
 {
 public:
-	BBTRulerDrag (Editor *, ArdourCanvas::Item *);
+	BBTMarkerDrag (Editor *, ArdourCanvas::Item *);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -816,15 +902,86 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
-	double _grab_qn;
-	ARDOUR::TempoSection* _tempo;
+	BBTMarker* _marker;
+	Temporal::MusicTimePoint const * _point;
+	Temporal::TempoMap::WritableSharedPtr map;
+
+	XMLNode* _before_state;
+};
+
+class MappingLinearDrag : public Drag
+{
+public:
+	MappingLinearDrag (Editor *, ArdourCanvas::Item *, Temporal::TempoMap::WritableSharedPtr&, Temporal::TempoPoint&, Temporal::TempoPoint& after, XMLNode& before_state);
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	bool y_movement_matters () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
+
+private:
+	Temporal::TempoPoint& _tempo;
+	Temporal::TempoPoint& _after;
+	double _grab_bpm;
+	Temporal::TempoMap::WritableSharedPtr map;
+
 	XMLNode* _before_state;
 	bool     _drag_valid;
 };
 
+class MappingTwistDrag : public Drag
+{
+public:
+	MappingTwistDrag (Editor *, ArdourCanvas::Item *, Temporal::TempoMap::WritableSharedPtr&,
+	                  Temporal::TempoPoint& prev,
+	                  Temporal::TempoPoint& focus,
+	                  Temporal::TempoPoint& next,
+	                  XMLNode&);
+
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	bool y_movement_matters () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
+
+private:
+	Temporal::TempoPoint& prev;
+	Temporal::TempoPoint& focus;
+	Temporal::TempoPoint& next;
+	double _grab_bpm;
+	Temporal::TempoMap::WritableSharedPtr map;
+
+	double direction;
+	double delta;
+	double initial_npm;
+
+	XMLNode* _before_state;
+	bool     _drag_valid;
+};
+
+
 /** tempo curve twist drag */
 class TempoTwistDrag : public Drag
 {
@@ -844,18 +1001,18 @@ public:
 		return true;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
-	double _grab_qn;
-	ARDOUR::Tempo  _grab_tempo;
-	ARDOUR::TempoSection* _tempo;
-	ARDOUR::TempoSection* _next_tempo;
+	Temporal::Beats _grab_qn;
+	Temporal::TempoPoint* _tempo;
+	Temporal::TempoPoint*  _grab_tempo;
+	Temporal::TempoPoint const * _next_tempo;
+	Temporal::TempoMap::WritableSharedPtr map;
 	bool _drag_valid;
 	XMLNode* _before_state;
 };
 
-
 /** tempo curve twist drag */
 class TempoEndDrag : public Drag
 {
@@ -875,11 +1032,14 @@ public:
 		return true;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
-	double _grab_qn;
-	ARDOUR::TempoSection* _tempo;
+	Temporal::Beats _grab_qn;
+	Temporal::TempoPoint * _tempo;
+	Temporal::TempoPoint * previous_tempo;
+	Temporal::TempoMap::WritableSharedPtr map;
+
 	XMLNode* _before_state;
 	bool _drag_valid;
 };
@@ -904,11 +1064,11 @@ public:
 	}
 
 private:
-	void fake_locate (framepos_t);
+	void fake_locate (samplepos_t);
 
 	EditorCursor& _cursor;
 	bool _stop; ///< true to stop the transport on starting the drag, otherwise false
-	double _grab_zoom; ///< editor frames per unit when our grab started
+	double _grab_zoom; ///< editor samples per unit when our grab started
 
 	//used for zooming
 	int _last_mx;
@@ -921,7 +1081,7 @@ private:
 class FadeInDrag : public RegionDrag
 {
 public:
-	FadeInDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	FadeInDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -932,14 +1092,18 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
 };
 
 /** Region fade-out drag */
 class FadeOutDrag : public RegionDrag
 {
 public:
-	FadeOutDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	FadeOutDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain td);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -950,7 +1114,11 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	void setup_pointer_offset ();
 };
 
 /** Marker drag */
@@ -973,7 +1141,8 @@ public:
 		return false;
 	}
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
+	void setup_video_offset ();
 
 private:
 	void update_item (ARDOUR::Location *);
@@ -1005,6 +1174,11 @@ public:
 
 	bool active (Editing::MouseMode m);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+
 private:
 
 	ControlPoint* _point;
@@ -1015,6 +1189,8 @@ private:
 	bool     _pushing;
         uint32_t _final_index;
 	static double _zero_gain_fraction;
+
+	Temporal::timecnt_t total_dt (GdkEvent*) const;
 };
 
 /** Gain or automation line drag */
@@ -1022,12 +1198,17 @@ class LineDrag : public Drag
 {
 public:
 	LineDrag (Editor *e, ArdourCanvas::Item *i);
+	~LineDrag ();
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
 private:
 
 	AutomationLine* _line;
@@ -1036,6 +1217,7 @@ private:
 	double _cumulative_y_drag;
 	uint32_t _before;
 	uint32_t _after;
+	bool    have_command;
 };
 
 /** Transient feature line drags*/
@@ -1072,21 +1254,21 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (8, 1);
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (8, Temporal::AudioTime), 1);
 	}
 
 	void do_select_things (GdkEvent *, bool);
 
 	/** Select some things within a rectangle.
 	 *  @param button_state The button state from the GdkEvent.
-	 *  @param x1 The left-hand side of the rectangle in session frames.
-	 *  @param x2 The right-hand side of the rectangle in session frames.
+	 *  @param x1 The left-hand side of the rectangle as a timepos_t
+	 *  @param x2 The right-hand side of the rectangle as a timepos_t.
 	 *  @param y1 The top of the rectangle in trackview coordinates.
 	 *  @param y2 The bottom of the rectangle in trackview coordinates.
 	 *  @param drag_in_progress true if the drag is currently happening.
 	 */
-	virtual void select_things (int button_state, framepos_t x1, framepos_t x2, double y1, double y2, bool drag_in_progress) = 0;
+	virtual void select_things (int button_state, Temporal::timepos_t const & x1, Temporal::timepos_t const & x2, double y1, double y2, bool drag_in_progress) = 0;
 
 	virtual void deselect_things () = 0;
 
@@ -1100,7 +1282,7 @@ class EditorRubberbandSelectDrag : public RubberbandSelectDrag
 public:
 	EditorRubberbandSelectDrag (Editor *, ArdourCanvas::Item *);
 
-	void select_things (int, framepos_t, framepos_t, double, double, bool);
+	void select_things (int, Temporal::timepos_t const &, Temporal::timepos_t const &, double, double, bool);
 	void deselect_things ();
 };
 
@@ -1110,20 +1292,20 @@ class MidiRubberbandSelectDrag : public RubberbandSelectDrag
 public:
 	MidiRubberbandSelectDrag (Editor *, MidiRegionView *);
 
-	void select_things (int, framepos_t, framepos_t, double, double, bool);
+	void select_things (int, Temporal::timepos_t const &, Temporal::timepos_t const &, double, double, bool);
 	void deselect_things ();
 
 private:
 	MidiRegionView* _region_view;
 };
 
-/** A RubberbandSelectDrag for selecting MIDI notes but with no horizonal component */
+/** A RubberbandSelectDrag for selecting MIDI notes but with no horizontal component */
 class MidiVerticalSelectDrag : public RubberbandSelectDrag
 {
 public:
 	MidiVerticalSelectDrag (Editor *, MidiRegionView *);
 
-	void select_things (int, framepos_t, framepos_t, double, double, bool);
+	void select_things (int, Temporal::timepos_t const &, Temporal::timepos_t const &, double, double, bool);
 	void deselect_things ();
 
 private:
@@ -1134,7 +1316,7 @@ private:
 class TimeFXDrag : public RegionDrag
 {
 public:
-	TimeFXDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &);
+	TimeFXDrag (Editor *, ArdourCanvas::Item *, RegionView *, std::list<RegionView*> const &, Temporal::TimeDomain td);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -1173,15 +1355,15 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	void setup_pointer_frame_offset ();
+	void setup_pointer_offset ();
 
 private:
 	Operation _operation;
 	bool _add;
 	TrackSelection _track_selection_at_start;
 	bool _time_selection_at_start;
-	framepos_t start_at_start;
-	framepos_t end_at_start;
+	Temporal::timepos_t start_at_start;
+	Temporal::timepos_t end_at_start;
 };
 
 /** Range marker drag */
@@ -1230,8 +1412,8 @@ public:
 	void finished (GdkEvent *, bool);
 	void aborted (bool);
 
-	std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (4, 4);
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (4, Temporal::AudioTime), 4);
 	}
 
 private:
@@ -1244,8 +1426,8 @@ private:
 class AutomationRangeDrag : public Drag
 {
 public:
-	AutomationRangeDrag (Editor *, AutomationTimeAxisView *, std::list<ARDOUR::AudioRange> const &);
-	AutomationRangeDrag (Editor *, RegionView *, std::list<ARDOUR::AudioRange> const &);
+	AutomationRangeDrag (Editor *, AutomationTimeAxisView *, float initial_value, std::list<ARDOUR::TimelineRange> const &);
+	AutomationRangeDrag (Editor *, std::list<RegionView*> const &, std::list<ARDOUR::TimelineRange> const &, double y_origin, double y_height);
 
 	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
 	void motion (GdkEvent *, bool);
@@ -1257,25 +1439,26 @@ public:
 	}
 
 private:
-	void setup (std::list<boost::shared_ptr<AutomationLine> > const &);
-	double y_fraction (boost::shared_ptr<AutomationLine>, double global_y_position) const;
-	double value (boost::shared_ptr<ARDOUR::AutomationList> list, double x) const;
+	void setup (std::list<std::shared_ptr<AutomationLine> > const &);
+	double y_fraction (double global_y_position) const;
+	double value (std::shared_ptr<ARDOUR::AutomationList> list, Temporal::timepos_t const & x) const;
 
-	std::list<ARDOUR::AudioRange> _ranges;
+	std::list<ARDOUR::TimelineRange> _ranges;
 
 	/** A line that is part of the drag */
 	struct Line {
-		boost::shared_ptr<AutomationLine> line; ///< the line
+		std::shared_ptr<AutomationLine> line; ///< the line
 		std::list<ControlPoint*> points; ///< points to drag on the line
-		std::pair<ARDOUR::framepos_t, ARDOUR::framepos_t> range; ///< the range of all points on the line, in session frames
+		std::pair<Temporal::timepos_t, Temporal::timepos_t> range; ///< the range of all points on the line, in session time
 		XMLNode* state; ///< the XML state node before the drag
-		double original_fraction; ///< initial y-fraction before the drag
 	};
 
 	std::list<Line> _lines;
 	double          _y_origin;
+	double          _y_height;
 	bool            _nothing_to_drag;
 	bool            _integral;
+	float_t			_initial_value;
 };
 
 /** Drag of one edge of an xfade
@@ -1294,8 +1477,8 @@ public:
 		return false;
 	}
 
-	virtual std::pair<ARDOUR::framecnt_t, int> move_threshold () const {
-		return std::make_pair (4, 4);
+	std::pair<Temporal::timecnt_t,int> move_threshold () const {
+		return std::make_pair (Temporal::timecnt_t (4, Temporal::AudioTime), 4);
 	}
 
 private:
@@ -1303,5 +1486,32 @@ private:
 	bool start;
 };
 
-#endif /* __gtk2_ardour_editor_drag_h_ */
+class RegionMarkerDrag : public Drag
+{
+  public:
+	RegionMarkerDrag (Editor*, RegionView*, ArdourCanvas::Item*);
+	~RegionMarkerDrag ();
 
+	void start_grab (GdkEvent *, Gdk::Cursor* c = 0);
+	void motion (GdkEvent *, bool);
+	void finished (GdkEvent *, bool);
+	void aborted (bool);
+
+	bool allow_vertical_autoscroll () const {
+		return false;
+	}
+
+	bool y_movement_matters () const {
+		return false;
+	}
+
+	void setup_pointer_sample_offset ();
+
+  private:
+	RegionView* rv;
+	ArdourMarker* view;
+	ARDOUR::CueMarker model;
+	ARDOUR::CueMarker dragging_model;
+};
+
+#endif /* __gtk2_ardour_editor_drag_h_ */
