@@ -1,21 +1,26 @@
 /*
-    Copyright (C) 2000 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006 Hans Fugal <hans@fugal.net>
+ * Copyright (C) 2008-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2008-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2015-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <iostream>
 #include <cstdlib>
@@ -33,6 +38,7 @@
 #include "ardour/midi_stretch.h"
 #include "ardour/pitch.h"
 #include "ardour/region.h"
+#include "ardour/region_factory.h"
 #include "ardour/session.h"
 #include "ardour/stretch.h"
 
@@ -41,6 +47,7 @@
 #include "audio_region_view.h"
 #include "audio_time_axis.h"
 #include "editor.h"
+#include "editor_regions.h"
 #include "region_selection.h"
 #include "time_fx_dialog.h"
 
@@ -59,11 +66,10 @@ using namespace Gtkmm2ext;
 
 /** @return -1 in case of error, 1 if operation was cancelled by the user, 0 if everything went ok */
 int
-Editor::time_stretch (RegionSelection& regions, float fraction)
+Editor::time_stretch (RegionSelection& regions, Temporal::ratio_t const & ratio)
 {
 	RegionList audio;
 	RegionList midi;
-	int aret;
 
 	begin_reversible_command (_("stretch/shrink"));
 
@@ -75,15 +81,16 @@ Editor::time_stretch (RegionSelection& regions, float fraction)
 		}
 	}
 
-	if ((aret = time_fx (audio, fraction, false)) != 0) {
-		commit_reversible_command ();
+	int aret = time_fx (audio, ratio, false);
+	if (aret < 0) {
+		abort_reversible_command ();
 		return aret;
 	}
 
-	set<boost::shared_ptr<Playlist> > midi_playlists_affected;
+	PlaylistSet midi_playlists_affected;
 
 	for (RegionList::iterator i = midi.begin(); i != midi.end(); ++i) {
-		boost::shared_ptr<Playlist> playlist = (*i)->playlist();
+		std::shared_ptr<Playlist> playlist = (*i)->playlist();
 
 		if (playlist) {
 			playlist->clear_changes ();
@@ -92,10 +99,10 @@ Editor::time_stretch (RegionSelection& regions, float fraction)
 	}
 
 	ARDOUR::TimeFXRequest request;
-	request.time_fraction = fraction;
+	request.time_fraction = ratio;
 
 	for (RegionList::iterator i = midi.begin(); i != midi.end(); ++i) {
-		boost::shared_ptr<Playlist> playlist = (*i)->playlist();
+		std::shared_ptr<Playlist> playlist = (*i)->playlist();
 
 		if (!playlist) {
 			continue;
@@ -109,11 +116,19 @@ Editor::time_stretch (RegionSelection& regions, float fraction)
 		midi_playlists_affected.insert (playlist);
 	}
 
-	for (set<boost::shared_ptr<Playlist> >::iterator p = midi_playlists_affected.begin(); p != midi_playlists_affected.end(); ++p) {
-		_session->add_command (new StatefulDiffCommand (*p));
+	for (PlaylistSet::iterator p = midi_playlists_affected.begin(); p != midi_playlists_affected.end(); ++p) {
+		PBD::StatefulDiffCommand* cmd = new StatefulDiffCommand (*p);
+		_session->add_command (cmd);
+		if (!cmd->empty ()) {
+			++aret;
+		}
 	}
 
-	commit_reversible_command ();
+	if (aret > 0) {
+		commit_reversible_command ();
+	} else {
+		abort_reversible_command ();
+	}
 
 	return 0;
 }
@@ -131,31 +146,33 @@ Editor::pitch_shift (RegionSelection& regions, float fraction)
 
 	int ret = time_fx (rl, fraction, true);
 
-	if (ret == 0) {
+	if (ret > 0) {
 		commit_reversible_command ();
 	} else {
 		abort_reversible_command ();
 	}
 
-	return ret;
+	return ret < 0 ? -1 : 0;
 }
 
 /** @param val Percentage to time stretch by; ignored if pitch-shifting.
  *  @param pitching true to pitch shift, false to time stretch.
- *  @return -1 in case of error, 1 if operation was cancelled by the user, 0 if everything went ok */
+ *  @return -1 in case of error, otherwise number of regions processed */
 int
-Editor::time_fx (RegionList& regions, float val, bool pitching)
+Editor::time_fx (RegionList& regions, Temporal::ratio_t ratio, bool pitching)
 {
+	delete current_timefx;
+
 	if (regions.empty()) {
+		current_timefx = 0;
 		return 0;
 	}
 
-	const framecnt_t oldlen = (framecnt_t) (regions.front()->length());
-	const framecnt_t newlen = (framecnt_t) (regions.front()->length() * val);
-	const framecnt_t pos = regions.front()->position ();
+	const timecnt_t oldlen = regions.front()->length();
+	const timecnt_t newlen = regions.front()->length().scale (ratio);
+	const timepos_t pos = regions.front()->position ();
 
-	delete current_timefx;
-	current_timefx = new TimeFXDialog (*this, pitching, oldlen, newlen, pos);
+	current_timefx = new TimeFXDialog (*this, pitching, oldlen, newlen, ratio, pos);
 	current_timefx->regions = regions;
 
 	switch (current_timefx->run ()) {
@@ -163,14 +180,14 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 		break;
 	default:
 		current_timefx->hide ();
-		return 1;
+		return -1;
 	}
 
 	current_timefx->status = 0;
 	current_timefx->request.time_fraction = current_timefx->get_time_fraction ();
 	current_timefx->request.pitch_fraction = current_timefx->get_pitch_fraction ();
 
-	if (current_timefx->request.time_fraction == 1.0 &&
+	if (current_timefx->request.time_fraction.to_double() == 1.0 &&
 	    current_timefx->request.pitch_fraction == 1.0) {
 		/* nothing to do */
 		current_timefx->hide ();
@@ -201,14 +218,38 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 
 	txt = current_timefx->stretch_opts_selector.get_active_text ();
 
-	for (int i = 0; i <= 6; i++) {
+	for (size_t i = 0; i < rb_opt_strings.size(); i++) {
 		if (txt == rb_opt_strings[i]) {
 			rb_current_opt = i;
 			break;
 		}
 	}
 
-	switch (rb_current_opt) {
+	int rb_mode = rb_current_opt;
+
+	if (pitching /*&& rb_current_opt == 6*/) {
+		/* The timefx dialog does not show the "stretch_opts_selector"
+		 * when pitch-shifting.  So the most recently used option from
+		 * "Time Stretch" would be used (if any). That may even be
+		 * "resample without preserving pitch", which would be invalid.
+		 *
+		 * TODO: also show stretch_opts_selector when pitching (except the option
+		 * to not preserve pitch) and use separate  rb_current_opt when pitching.
+		 *
+		 * Actually overhaul this the dialog and processing opts below and use rubberband's
+		 * "Crispness" levels:
+		 *   -c 0   equivalent to --no-transients --no-lamination --window-long
+		 *   -c 1   equivalent to --detector-soft --no-lamination --window-long (for piano)
+		 *   -c 2   equivalent to --no-transients --no-lamination
+		 *   -c 3   equivalent to --no-transients
+		 *   -c 4   equivalent to --bl-transients
+		 *   -c 5   default processing options
+		 *   -c 6   equivalent to --no-lamination --window-short (may be good for drums)
+		 */
+		rb_mode = 4;
+	}
+
+	switch (rb_mode) {
 		case 0:
 			transients = NoTransients; peaklock = false; longwin = true; shortwin = false;
 			break;
@@ -228,10 +269,15 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 			transients = NoTransients;
 			precise = true;
 			preserve_formants = false;
-			current_timefx->request.pitch_fraction = 1/val;
+			current_timefx->request.pitch_fraction = 1.0 / current_timefx->request.time_fraction.to_double();
 			shortwin = true;
 			// peaklock = false;
 			break;
+	#ifdef HAVE_SOUNDTOUCH
+		case 7:
+			current_timefx->request.use_soundtouch = true;
+			break;
+	#endif
 		default:
 			/* default/4 */
 			transients = Transients; peaklock = true; longwin = false; shortwin = false;
@@ -245,6 +291,8 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 	if (longwin)           options |= RubberBandStretcher::OptionWindowLong;
 	if (shortwin)          options |= RubberBandStretcher::OptionWindowShort;
 
+	if (pitching)          options |= RubberBandStretcher::OptionPitchHighQuality;
+
 	switch (transients) {
 	case NoTransients:
 		options |= RubberBandStretcher::OptionTransientsSmooth;
@@ -257,6 +305,10 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 		break;
 	}
 
+#ifdef HAVE_RUBBERBAND_3_0_0
+	options |= RubberBandStretcher::OptionEngineFiner;
+#endif
+
 	current_timefx->request.opts = (int) options;
 #else
 	current_timefx->request.quick_seek = current_timefx->quick_button.get_active();
@@ -275,7 +327,6 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 	current_timefx->first_delete = current_timefx->signal_delete_event().connect
 		(sigc::mem_fun (current_timefx, &TimeFXDialog::delete_in_progress));
 
-	current_timefx->start_updates ();
 
 	if (pthread_create_and_store ("timefx", &current_timefx->request.thread, timefx_thread, current_timefx)) {
 		current_timefx->hide ();
@@ -283,7 +334,7 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 		return -1;
 	}
 
-	pthread_detach (current_timefx->request.thread);
+	current_timefx->start_updates ();
 
 	while (!current_timefx->request.done && !current_timefx->request.cancel) {
 		gtk_main_iteration ();
@@ -292,39 +343,36 @@ Editor::time_fx (RegionList& regions, float val, bool pitching)
 	pthread_join (current_timefx->request.thread, 0);
 
 	current_timefx->hide ();
+
+	if (current_timefx->status < 0) {
+		/* processing was cancelled, some regions may have
+		 * been created and removed via RegionFactory::map_remove()
+		 * The region-list does not update itself when a region is removed.
+		 */
+		_regions->redisplay ();
+	}
 	return current_timefx->status;
 }
 
 void
 Editor::do_timefx ()
 {
-	boost::shared_ptr<Playlist> playlist;
-	boost::shared_ptr<Region>   new_region;
-	set<boost::shared_ptr<Playlist> > playlists_affected;
+	typedef std::map<std::shared_ptr<Region>, std::shared_ptr<Region> > ResultMap;
+	ResultMap results;
 
 	uint32_t const N = current_timefx->regions.size ();
 
-	for (RegionList::iterator i = current_timefx->regions.begin(); i != current_timefx->regions.end(); ++i) {
-		boost::shared_ptr<Playlist> playlist = (*i)->playlist();
+	for (RegionList::const_iterator i = current_timefx->regions.begin(); i != current_timefx->regions.end(); ++i) {
 
-		if (playlist) {
-			playlist->clear_changes ();
-		}
-	}
-
-	for (RegionList::iterator i = current_timefx->regions.begin(); i != current_timefx->regions.end(); ++i) {
-
-		boost::shared_ptr<AudioRegion> region = boost::dynamic_pointer_cast<AudioRegion> (*i);
+		std::shared_ptr<AudioRegion> region = std::dynamic_pointer_cast<AudioRegion> (*i);
+		std::shared_ptr<Playlist> playlist;
 
 		if (!region || (playlist = region->playlist()) == 0) {
 			continue;
 		}
 
 		if (current_timefx->request.cancel) {
-			/* we were cancelled */
-			/* XXX what to do about playlists already affected ? */
-			current_timefx->status = 1;
-			return;
+			break;
 		}
 
 		Filter* fx;
@@ -333,7 +381,15 @@ Editor::do_timefx ()
 			fx = new Pitch (*_session, current_timefx->request);
 		} else {
 #ifdef USE_RUBBERBAND
+		#ifdef HAVE_SOUNDTOUCH
+			if (current_timefx->request.use_soundtouch) {
+				fx = new STStretch (*_session, current_timefx->request);
+			} else {
+				fx = new RBStretch (*_session, current_timefx->request);
+			}
+		#else
 			fx = new RBStretch (*_session, current_timefx->request);
+		#endif
 #else
 			fx = new STStretch (*_session, current_timefx->request);
 #endif
@@ -342,28 +398,44 @@ Editor::do_timefx ()
 		current_timefx->descend (1.0 / N);
 
 		if (fx->run (region, current_timefx)) {
-			current_timefx->status = -1;
-			current_timefx->request.done = true;
+			current_timefx->request.cancel = true;
 			delete fx;
-			return;
+			break;
 		}
 
 		if (!fx->results.empty()) {
-			new_region = fx->results.front();
-
-			playlist->replace_region (region, new_region, region->position());
-			playlists_affected.insert (playlist);
+			results[region] = fx->results.front();
 		}
 
 		current_timefx->ascend ();
 		delete fx;
 	}
 
-	for (set<boost::shared_ptr<Playlist> >::iterator p = playlists_affected.begin(); p != playlists_affected.end(); ++p) {
-		_session->add_command (new StatefulDiffCommand (*p));
+	pthread_setcancelstate (PTHREAD_CANCEL_DISABLE, NULL);
+	if (current_timefx->request.cancel) {
+		current_timefx->status = -1;
+		for (ResultMap::const_iterator i = results.begin(); i != results.end(); ++i) {
+			std::weak_ptr<Region> w = i->second;
+			RegionFactory::map_remove (w);
+		}
+	} else {
+		current_timefx->status = 0;
+		for (ResultMap::const_iterator i = results.begin(); i != results.end(); ++i) {
+			std::shared_ptr<Region> region = i->first;
+			std::shared_ptr<Region> new_region = i->second;
+			std::shared_ptr<Playlist> playlist = region->playlist();
+
+			playlist->clear_changes ();
+			playlist->replace_region (region, new_region, region->position());
+			PBD::StatefulDiffCommand* cmd = new StatefulDiffCommand (playlist);
+			_session->add_command (cmd);
+			if (!cmd->empty ()) {
+				++current_timefx->status;
+			}
+		}
 	}
+	pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
 
-	current_timefx->status = 0;
 	current_timefx->request.done = true;
 }
 
@@ -371,6 +443,7 @@ void*
 Editor::timefx_thread (void *arg)
 {
 	SessionEvent::create_per_thread_pool ("timefx events", 64);
+	Temporal::TempoMap::fetch ();
 
 	TimeFXDialog* tsd = static_cast<TimeFXDialog*>(arg);
 
@@ -382,12 +455,6 @@ Editor::timefx_thread (void *arg)
 	   event loop doesn't die before any changes we made are processed
 	   by the GUI ...
 	*/
-
-#ifdef PLATFORM_WINDOWS
-	Glib::usleep(2 * G_USEC_PER_SEC);
-#else
-	struct timespec t = { 2, 0 };
-	nanosleep (&t, 0);
-#endif
+	Glib::usleep(G_USEC_PER_SEC / 5);
 	return 0;
 }
