@@ -1,21 +1,29 @@
 /*
-  Copyright (C) 2002 Paul Davis
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2006 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2005-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2014-2015 Ben Loftis <ben@harrisonconsoles.com>
+ * Copyright (C) 2016 Julien "_FrnchFrgg_" RIVAUD <frnchfrgg@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <limits.h>
 
@@ -38,7 +46,6 @@
 #include "widgets/tooltips.h"
 
 #include "pbd/fastlog.h"
-#include "pbd/stacktrace.h"
 
 #include "gain_meter.h"
 #include "gui_thread.h"
@@ -98,6 +105,8 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	, meter_point_button (_("pre"))
 	, gain_astate_propagate (false)
 	, _data_type (DataType::AUDIO)
+	, _clear_meters (true)
+	, _meter_peaked (false)
 {
 	using namespace Menu_Helpers;
 
@@ -112,9 +121,11 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	fader_girth = rint (fader_girth * UIConfiguration::instance().get_ui_scale());
 
 	if (horizontal) {
-		gain_slider = manage (new HSliderController (&gain_adjustment, boost::shared_ptr<PBD::Controllable>(), fader_length, fader_girth));
+		gain_slider = manage (new HSliderController (&gain_adjustment, std::shared_ptr<PBD::Controllable>(), fader_length, fader_girth));
+		gain_slider->set_tweaks (ArdourFader::Tweaks(ArdourFader::NoButtonForward | ArdourFader::NoVerticalScroll));
 	} else {
-		gain_slider = manage (new VSliderController (&gain_adjustment, boost::shared_ptr<PBD::Controllable>(), fader_length, fader_girth));
+		gain_slider = manage (new VSliderController (&gain_adjustment, std::shared_ptr<PBD::Controllable>(), fader_length, fader_girth));
+		gain_slider->set_tweaks (ArdourFader::NoButtonForward);
 	}
 
 	level_meter = new LevelMeterHBox(_session);
@@ -123,7 +134,6 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	meter_metric_area.signal_button_press_event().connect (sigc::mem_fun (*this, &GainMeterBase::level_meter_button_press));
 	meter_metric_area.add_events (Gdk::BUTTON_PRESS_MASK);
 
-	gain_slider->set_tweaks (ArdourFader::Tweaks(ArdourFader::NoButtonForward | ArdourFader::NoVerticalScroll));
 	gain_slider->StartGesture.connect (sigc::mem_fun (*this, &GainMeter::amp_start_touch));
 	gain_slider->StopGesture.connect (sigc::mem_fun (*this, &GainMeter::amp_stop_touch));
 	gain_slider->set_name ("GainFader");
@@ -147,14 +157,15 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	peak_display.set_events (peak_display.get_events() & ~(Gdk::EventMask (Gdk::LEAVE_NOTIFY_MASK|Gdk::ENTER_NOTIFY_MASK|Gdk::POINTER_MOTION_MASK)));
 	peak_display.signal_map().connect (sigc::bind (sigc::ptr_fun (reset_cursor_to_default), &peak_display));
 	peak_display.signal_state_changed().connect (sigc::bind (sigc::ptr_fun (reset_cursor_to_default_state), &peak_display));
-	peak_display.unset_flags (Gtk::CAN_FOCUS);
+	peak_display.set_can_focus (false);
 	peak_display.set_editable (false);
 
 	gain_automation_state_button.set_name ("mixer strip button");
 
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
+	set_tooltip (peak_display, _("dBFS - Digital Peak Hold. Click to reset."));
 
-	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
+	gain_automation_state_button.set_can_focus (false);
 
 	gain_automation_state_button.set_size_request(15, 15);
 
@@ -165,7 +176,7 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 
 	set_tooltip (&meter_point_button, _("Metering point"));
 
-	meter_point_button.unset_flags (Gtk::CAN_FOCUS);
+	meter_point_button.set_can_focus (false);
 
 	meter_point_button.set_size_request(15, 15);
 
@@ -173,21 +184,22 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	meter_point_menu.set_reserve_toggle_size(false);
 
 	meter_point_menu.items().clear ();
-	meter_point_menu.items().push_back (MenuElem(_("Input"),
+	meter_point_menu.items().push_back (MenuElem(meterpt_string(MeterInput),
 	 sigc::bind (sigc::mem_fun (*this,
 	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterInput)));
-	meter_point_menu.items().push_back (MenuElem(_("Pre Fader"),
+	meter_point_menu.items().push_back (MenuElem(meterpt_string(MeterPreFader),
 	 sigc::bind (sigc::mem_fun (*this,
 	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterPreFader)));
-	meter_point_menu.items().push_back (MenuElem(_("Post Fader"),
+	meter_point_menu.items().push_back (MenuElem(meterpt_string (MeterPostFader),
 	 sigc::bind (sigc::mem_fun (*this,
 	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterPostFader)));
-	meter_point_menu.items().push_back (MenuElem(_("Output"),
+	meter_point_menu.items().push_back (MenuElem(meterpt_string (MeterOutput),
 	 sigc::bind (sigc::mem_fun (*this,
 	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterOutput)));
-	meter_point_menu.items().push_back (MenuElem(_("Custom"),
+	meter_point_menu.items().push_back (MenuElem(meterpt_string (MeterCustom),
 	 sigc::bind (sigc::mem_fun (*this,
 	 &GainMeterBase::meter_point_clicked), (MeterPoint) MeterCustom)));
+
 	meter_point_button.signal_button_press_event().connect (sigc::mem_fun (*this, &GainMeter::meter_press), false);
 
 	gain_adjustment.signal_value_changed().connect (sigc::mem_fun(*this, &GainMeterBase::fader_moved));
@@ -212,10 +224,10 @@ GainMeterBase::~GainMeterBase ()
 }
 
 void
-GainMeterBase::set_controls (boost::shared_ptr<Route> r,
-			     boost::shared_ptr<PeakMeter> pm,
-                             boost::shared_ptr<Amp> amp,
-                             boost::shared_ptr<GainControl> control)
+GainMeterBase::set_controls (std::shared_ptr<Route> r,
+			     std::shared_ptr<PeakMeter> pm,
+                             std::shared_ptr<Amp> amp,
+                             std::shared_ptr<GainControl> control)
 {
 	connections.clear ();
 	model_connections.drop_connections ();
@@ -224,7 +236,7 @@ GainMeterBase::set_controls (boost::shared_ptr<Route> r,
 
 	if (!pm && !control) {
 		level_meter->set_meter (0);
-		gain_slider->set_controllable (boost::shared_ptr<PBD::Controllable>());
+		gain_slider->set_controllable (std::shared_ptr<PBD::Controllable>());
 		_meter.reset ();
 		_amp.reset ();
 		_route.reset ();
@@ -254,14 +266,16 @@ GainMeterBase::set_controls (boost::shared_ptr<Route> r,
 
 		gain_astate_menu.items().clear ();
 
-		gain_astate_menu.items().push_back (MenuElem (S_("Automation|Manual"),
+		gain_astate_menu.items().push_back (MenuElem (astate_string (ARDOUR::Off),
 							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Off)));
-		gain_astate_menu.items().push_back (MenuElem (_("Play"),
+		gain_astate_menu.items().push_back (MenuElem (astate_string (ARDOUR::Play),
 							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Play)));
-		gain_astate_menu.items().push_back (MenuElem (_("Write"),
+		gain_astate_menu.items().push_back (MenuElem (astate_string (ARDOUR::Write),
 							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Write)));
-		gain_astate_menu.items().push_back (MenuElem (_("Touch"),
+		gain_astate_menu.items().push_back (MenuElem (astate_string (ARDOUR::Touch),
 							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Touch)));
+		gain_astate_menu.items().push_back (MenuElem (astate_string (ARDOUR::Latch),
+							      sigc::bind (sigc::mem_fun (*this, &GainMeterBase::set_gain_astate), (AutoState) ARDOUR::Latch)));
 
 		connections.push_back (gain_automation_state_button.signal_button_press_event().connect (sigc::mem_fun(*this, &GainMeterBase::gain_automation_state_button_event), false));
 		connections.push_back (ChangeGainAutomationState.connect (sigc::mem_fun(*this, &GainMeterBase::set_gain_astate)));
@@ -292,9 +306,7 @@ GainMeterBase::set_gain_astate (AutoState as)
 		ChangeGainAutomationState (as);
 		return;
 	}
-	if (_amp) {
-		_amp->set_parameter_automation_state (Evoral::Parameter (GainAutomation), as);
-	} else if (_control) {
+	if (_control) {
 		_control->set_automation_state (as);
 		_session->set_dirty ();
 	}
@@ -380,12 +392,6 @@ GainMeterBase::setup_meters (int len)
 	level_meter->setup_meters(len, meter_width);
 }
 
-void
-GainMeterBase::set_type (MeterType t)
-{
-	level_meter->set_meter_type(t);
-}
-
 void
 GainMeter::setup_meters (int len)
 {
@@ -408,12 +414,6 @@ GainMeter::setup_meters (int len)
 	GainMeterBase::setup_meters (len);
 }
 
-void
-GainMeter::set_type (MeterType t)
-{
-	GainMeterBase::set_type (t);
-}
-
 bool
 GainMeterBase::gain_key_press (GdkEventKey* ev)
 {
@@ -457,10 +457,7 @@ GainMeterBase::reset_peak_display ()
 		return;
 	}
 	_meter->reset_max();
-	level_meter->clear_meters();
-	max_peak = minus_infinity ();
-	peak_display.set_text (_("-inf"));
-	peak_display.set_name ("MixerStripPeakDisplay");
+	_clear_meters = true;
 }
 
 void
@@ -586,11 +583,12 @@ GainMeterBase::fader_moved ()
 			value = gain_adjustment.get_value();
 		}
 
-		// XXX hack allow to override group
-		// (this breaks group'ed  shift+click reset)
-		if (Keyboard::the_keyboard().key_is_down (GDK_Shift_R)
-				|| Keyboard::the_keyboard().key_is_down (GDK_Shift_L)) {
-			_control->set_value (value, Controllable::InverseGroup);
+		if (Keyboard::the_keyboard().modifier_state() == Keyboard::group_override_modifier ()) {
+			if (Config->get_group_override_inverts ()) {
+				_control->set_value (value, Controllable::InverseGroup);
+			} else {
+				_control->set_value (value, Controllable::NoGroup);
+			}
 		} else {
 			_control->set_value (value, Controllable::UseGroup);
 		}
@@ -647,6 +645,30 @@ GainMeterBase::set_fader_name (const char * name)
 	gain_slider->set_name (name);
 }
 
+void
+GainMeterBase::set_fader_fg (uint32_t c)
+{
+	gain_slider->set_fg (c);
+}
+
+void
+GainMeterBase::set_fader_bg (uint32_t c)
+{
+	gain_slider->set_bg (c);
+}
+
+void
+GainMeterBase::unset_fader_fg ()
+{
+	gain_slider->unset_fg ();
+}
+
+void
+GainMeterBase::unset_fader_bg ()
+{
+	gain_slider->unset_bg ();
+}
+
 void
 GainMeterBase::update_gain_sensitive ()
 {
@@ -682,8 +704,9 @@ GainMeterBase::meter_press(GdkEventButton* ev)
 					meter_point_change_target = MeterPointChangeSingle;
 				}
 				Gtkmm2ext::anchored_menu_popup(&meter_point_menu,
-			  	                             &meter_point_button,
-			    	                           "", 1, ev->time);
+				                               &meter_point_button,
+				                               meterpt_string (_route->meter_point()),
+				                               1, ev->time);
 				break;
 			default:
 				break;
@@ -704,7 +727,7 @@ GainMeterBase::set_route_group_meter_point (Route& route, MeterPoint mp)
 	RouteGroup* route_group;
 
 	if ((route_group = route.route_group ()) != 0) {
-		route_group->foreach_route (boost::bind (&Route::set_meter_point, _1, mp, false));
+		route_group->foreach_route (boost::bind (&Route::set_meter_point, _1, mp));
 	} else {
 		route.set_meter_point (mp);
 	}
@@ -731,13 +754,13 @@ GainMeterBase::meter_point_clicked (MeterPoint mp)
 void
 GainMeterBase::amp_start_touch ()
 {
-	_control->start_touch (_control->session().transport_frame());
+	_control->start_touch (timepos_t (_control->session().transport_sample()));
 }
 
 void
 GainMeterBase::amp_stop_touch ()
 {
-	_control->stop_touch (_control->session().transport_frame());
+	_control->stop_touch (timepos_t (_control->session().transport_sample()));
 	effective_gain_display ();
 }
 
@@ -753,7 +776,8 @@ GainMeterBase::gain_automation_state_button_event (GdkEventButton *ev)
 			gain_astate_propagate = Keyboard::modifier_state_contains (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier | Keyboard::TertiaryModifier));
 			Gtkmm2ext::anchored_menu_popup(&gain_astate_menu,
 			                               &gain_automation_state_button,
-			                               "", 1, ev->time);
+			                               astate_string(_control->alist()->automation_state()),
+			                               1, ev->time);
 			break;
 		default:
 			break;
@@ -778,39 +802,46 @@ GainMeterBase::short_astate_string (AutoState state)
 string
 GainMeterBase::_astate_string (AutoState state, bool shrt)
 {
-	string sstr;
-
 	switch (state) {
-	case ARDOUR::Off:
-		sstr = (shrt ? "M" : _("M"));
-		break;
-	case Play:
-		sstr = (shrt ? "P" : _("P"));
-		break;
-	case Touch:
-		sstr = (shrt ? "T" : _("T"));
-		break;
-	case Write:
-		sstr = (shrt ? "W" : _("W"));
-		break;
+		case ARDOUR::Off:
+			return shrt ? S_("Manual|M") : S_("Automation|Manual");
+		case Play:
+			return shrt ? S_("Play|P") : _("Play");
+		case Touch:
+			return shrt ? S_("Touch|T") : _("Touch");
+		case Latch:
+			return shrt ? S_("Latch|L") : _("Latch");
+		case Write:
+			return shrt ? S_("Write|W"): _("Write");
 	}
+	assert (0);
+	return "???";
+}
 
-	return sstr;
+string
+GainMeterBase::meterpt_string (MeterPoint mp)
+{
+	switch (mp) {
+		case MeterInput:
+			return _("Input");
+		case MeterPreFader:
+			return _("Pre Fader");
+		case MeterPostFader:
+			return _("Post Fader");
+		case MeterOutput:
+			return _("Output");
+		case MeterCustom:
+			return _("Custom");
+	}
+	assert (0);
+	return "???"; // make gcc and _FrnchFrgg_ happy
 }
 
 void
 GainMeterBase::gain_automation_state_changed ()
 {
 	ENSURE_GUI_THREAD (*this, &GainMeterBase::gain_automation_state_changed);
-
-	switch (_width) {
-	case Wide:
-		gain_automation_state_button.set_text (astate_string(_control->alist()->automation_state()));
-		break;
-	case Narrow:
-		gain_automation_state_button.set_text (short_astate_string(_control->alist()->automation_state()));
-		break;
-	}
+	gain_automation_state_button.set_text (short_astate_string(_control->alist()->automation_state()));
 
 	const bool automation_watch_required = (_control->alist()->automation_state() != ARDOUR::Off);
 
@@ -834,7 +865,15 @@ GainMeterBase::meter_channels() const
 void
 GainMeterBase::update_meters()
 {
-	char buf[32];
+	if (_clear_meters) {
+		max_peak = minus_infinity ();
+		level_meter->clear_meters ();
+		peak_display.set_text (_("-inf"));
+		peak_display.set_name ("MixerStripPeakDisplay");
+		_meter_peaked = false;
+		_clear_meters = false;
+	}
+
 	float mpeak = level_meter->update_meters();
 
 	if (mpeak > max_peak) {
@@ -842,12 +881,17 @@ GainMeterBase::update_meters()
 		if (mpeak <= -200.0f) {
 			peak_display.set_text (_("-inf"));
 		} else {
+			char buf[32];
 			snprintf (buf, sizeof(buf), "%.1f", mpeak);
 			peak_display.set_text (buf);
 		}
 	}
-	if (mpeak >= UIConfiguration::instance().get_meter_peak()) {
-		peak_display.set_name ("MixerStripPeakDisplayPeak");
+
+	bool peaking = mpeak >= UIConfiguration::instance().get_meter_peak();
+
+	if (!_meter_peaked && peaking) {
+			peak_display.set_name ("MixerStripPeakDisplayPeak");
+			_meter_peaked = true;
 	}
 }
 
@@ -905,7 +949,7 @@ GainMeter::GainMeter (Session* s, int fader_length)
 
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
 
-	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
+	gain_automation_state_button.set_can_focus (false);
 
 	gain_automation_state_button.set_size_request (PX_SCALE(12, 15), PX_SCALE(12, 15));
 
@@ -940,16 +984,18 @@ GainMeter::GainMeter (Session* s, int fader_length)
 	meter_hbox.pack_start (meter_alignment, false, false);
 	meter_hbox.pack_start (meter_ticks2_area, false, false);
 	meter_hbox.pack_start (meter_metric_area, false, false);
+
+	meter_metric_area.set_no_show_all ();
 }
 #undef PX_SCALE
 
 GainMeter::~GainMeter () { }
 
 void
-GainMeter::set_controls (boost::shared_ptr<Route> r,
-			 boost::shared_ptr<PeakMeter> meter,
-                         boost::shared_ptr<Amp> amp,
-			 boost::shared_ptr<GainControl> control)
+GainMeter::set_controls (std::shared_ptr<Route> r,
+			 std::shared_ptr<PeakMeter> meter,
+                         std::shared_ptr<Amp> amp,
+			 std::shared_ptr<GainControl> control)
 {
 	if (meter_hbox.get_parent()) {
 		hbox.remove (meter_hbox);
@@ -965,8 +1011,8 @@ GainMeter::set_controls (boost::shared_ptr<Route> r,
 		_meter->ConfigurationChanged.connect (
 			model_connections, invalidator (*this), boost::bind (&GainMeter::meter_configuration_changed, this, _1), gui_context()
 			);
-		_meter->TypeChanged.connect (
-			model_connections, invalidator (*this), boost::bind (&GainMeter::meter_type_changed, this, _1), gui_context()
+		_meter->MeterTypeChanged.connect (
+			model_connections, invalidator (*this), boost::bind (&GainMeter::redraw_metrics, this), gui_context()
 			);
 
 		meter_configuration_changed (_meter->input_streams ());
@@ -975,10 +1021,11 @@ GainMeter::set_controls (boost::shared_ptr<Route> r,
 
 	if (_route) {
 		_route->active_changed.connect (model_connections, invalidator (*this), boost::bind (&GainMeter::route_active_changed, this), gui_context ());
-		hbox.pack_start (meter_hbox, true, true);
-		meter_hbox.show ();
 	}
 
+	hbox.pack_start (meter_hbox, true, true);
+	meter_hbox.show ();
+
 //	if (r && !r->is_auditioner()) {
 //		fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
 //	}
@@ -998,12 +1045,12 @@ GainMeter::get_gm_width ()
 	Gtk::Requisition sz;
 	int min_w = 0;
 	sz.width = 0;
-	meter_metric_area.size_request (sz);
+	sz = meter_metric_area.size_request ();
 	min_w += sz.width;
-	level_meter->size_request (sz);
+	sz = level_meter->size_request ();
 	min_w += sz.width;
 
-	fader_alignment.size_request (sz);
+	sz = fader_alignment.size_request ();
 	if (_width == Wide)
 		return max(sz.width * 2, min_w * 2) + 6;
 	else
@@ -1048,13 +1095,13 @@ GainMeter::on_style_changed (const Glib::RefPtr<Gtk::Style>&)
 	peak_display.queue_draw();
 }
 
-boost::shared_ptr<PBD::Controllable>
+std::shared_ptr<PBD::Controllable>
 GainMeterBase::get_controllable()
 {
 	if (_amp) {
 		return _control;
 	} else {
-		return boost::shared_ptr<PBD::Controllable>();
+		return std::shared_ptr<PBD::Controllable>();
 	}
 }
 
@@ -1077,38 +1124,30 @@ GainMeter::meter_configuration_changed (ChanCount c)
 		}
 	}
 
-	if (_route
-			&& boost::dynamic_pointer_cast<AudioTrack>(_route) == 0
-			&& boost::dynamic_pointer_cast<MidiTrack>(_route) == 0
-			) {
-		if (_route->active()) {
-			set_meter_strip_name ("AudioBusMetrics");
-		} else {
-			set_meter_strip_name ("AudioBusMetricsInactive");
-		}
-	}
-	else if (
-			   (type == (1 << DataType::MIDI))
-			|| (_route && boost::dynamic_pointer_cast<MidiTrack>(_route))
-			) {
+	bool is_audio_track = _route && std::dynamic_pointer_cast<AudioTrack>(_route) != 0;
+	bool is_midi_track = _route && std::dynamic_pointer_cast<MidiTrack>(_route) != 0;
+
+	if (!is_audio_track && (is_midi_track || /* MIDI Bus */ (type == (1 << DataType::MIDI)))) {
 		if (!_route || _route->active()) {
 			set_meter_strip_name ("MidiTrackMetrics");
 		} else {
 			set_meter_strip_name ("MidiTrackMetricsInactive");
 		}
 	}
-	else if (type == (1 << DataType::AUDIO)) {
+	else if (_route && (!is_audio_track && !is_midi_track)) {
+			/* Bus */
+		if (_route->active()) {
+			set_meter_strip_name ("AudioBusMetrics");
+		} else {
+			set_meter_strip_name ("AudioBusMetricsInactive");
+		}
+	}
+	else {
 		if (!_route || _route->active()) {
 			set_meter_strip_name ("AudioTrackMetrics");
 		} else {
 			set_meter_strip_name ("AudioTrackMetricsInactive");
 		}
-	} else {
-		if (!_route || _route->active()) {
-			set_meter_strip_name ("AudioMidiTrackMetrics");
-		} else {
-			set_meter_strip_name ("AudioMidiTrackMetricsInactive");
-		}
 	}
 
 	setup_meters();
@@ -1125,8 +1164,7 @@ GainMeter::route_active_changed ()
 }
 
 void
-GainMeter::meter_type_changed (MeterType t)
+GainMeter::redraw_metrics ()
 {
-	_route->set_meter_type(t);
-	RedrawMetrics();
+	GainMeterBase::redraw_metrics ();
 }
