@@ -1,20 +1,22 @@
 /*
-    Copyright (C) 2009 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2009-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2009-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2014-2016 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cmath>
 #include <map>
@@ -22,9 +24,8 @@
 #include <gtkmm/cellrenderercombo.h>
 
 #include "evoral/midi_util.h"
-#include "evoral/Note.hpp"
+#include "evoral/Note.h"
 
-#include "ardour/beats_frames_converter.h"
 #include "ardour/midi_model.h"
 #include "ardour/midi_region.h"
 #include "ardour/midi_source.h"
@@ -46,24 +47,24 @@ using namespace Gtk;
 using namespace Gtkmm2ext;
 using namespace Glib;
 using namespace ARDOUR;
-using Timecode::BBT_Time;
+using Temporal::BBT_Time;
 
-static map<int,std::string> note_length_map;
+static map<int32_t,std::string> note_length_map;
 
 static void
 fill_note_length_map ()
 {
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat, _("Whole")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/2, _("Half")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/3, _("Triplet")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/4, _("Quarter")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/8, _("Eighth")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/16, _("Sixteenth")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/32, _("Thirty-second")));
-	note_length_map.insert (make_pair<int,string> (BBT_Time::ticks_per_beat/64, _("Sixty-fourth")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/1, _("Whole")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/2, _("Half")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/3, _("Triplet")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/4, _("Quarter")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/8, _("Eighth")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/16, _("Sixteenth")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/32, _("Thirty-second")));
+	note_length_map.insert (make_pair<int32_t,string> (Temporal::ticks_per_beat/64, _("Sixty-fourth")));
 }
 
-MidiListEditor::MidiListEditor (Session* s, boost::shared_ptr<MidiRegion> r, boost::shared_ptr<MidiTrack> tr)
+MidiListEditor::MidiListEditor (Session* s, std::shared_ptr<MidiRegion> r, std::shared_ptr<MidiTrack> tr)
 	: ArdourWindow (r->name())
 	, buttons (1, 1)
 	, region (r)
@@ -146,8 +147,10 @@ MidiListEditor::MidiListEditor (Session* s, boost::shared_ptr<MidiRegion> r, boo
 
 	redisplay_model ();
 
-	region->midi_source(0)->model()->ContentsChanged.connect (content_connection, invalidator (*this),
-								  boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
+	region->midi_source(0)->model()->ContentsChanged.connect (content_connections, invalidator (*this),
+	                                                          boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
+	region->PropertyChanged.connect (content_connections, invalidator (*this),
+	                                 boost::bind (&MidiListEditor::redisplay_model, this), gui_context());
 
 	buttons.attach (sound_notes_button, 0, 1, 0, 1);
 	Glib::RefPtr<Gtk::Action> act = ActionManager::get_action ("Editor", "sound-midi-notes");
@@ -182,7 +185,7 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 	int cellx;
 	int celly;
 	int idelta = 0;
-	double fdelta = 0;
+	Temporal::Beats beat_delta;
 	MidiModel::NoteDiffCommand::Property prop (MidiModel::NoteDiffCommand::NoteNumber);
 	bool apply = false;
 	bool was_selected = false;
@@ -205,12 +208,12 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 	switch (colnum) {
 	case 0:
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::SecondaryModifier)) {
-			fdelta = 1/64.0;
+			beat_delta = Temporal::Beats::ticks (Temporal::ticks_per_beat / 64);
 		} else {
-			fdelta = 1/4.0;
+			beat_delta = Temporal::Beats (1, 0);
 		}
 		if (ev->direction == GDK_SCROLL_DOWN || ev->direction == GDK_SCROLL_LEFT) {
-			fdelta = -fdelta;
+			beat_delta = beat_delta;
 		}
 		prop = MidiModel::NoteDiffCommand::StartTime;
 		opname = _("edit note start");
@@ -248,12 +251,12 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 
 	case 5:
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::SecondaryModifier)) {
-			fdelta = 1/64.0;
+			beat_delta = Temporal::Beats::ticks (Temporal::ticks_per_beat / 64);
 		} else {
-			fdelta = 1/4.0;
+			beat_delta = Temporal::Beats (1, 0);
 		}
 		if (ev->direction == GDK_SCROLL_DOWN || ev->direction == GDK_SCROLL_LEFT) {
-			fdelta = -fdelta;
+			beat_delta = -beat_delta;
 		}
 		prop = MidiModel::NoteDiffCommand::Length;
 		opname = _("edit note length");
@@ -267,7 +270,7 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 
 	if (apply) {
 
-		boost::shared_ptr<MidiModel> m (region->midi_source(0)->model());
+		std::shared_ptr<MidiModel> m (region->midi_source(0)->model());
 		MidiModel::NoteDiffCommand* cmd = m->new_note_diff_command (opname);
 		vector<TreeModel::Path> previous_selection;
 
@@ -277,7 +280,7 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 
 			TreeView::Selection::ListHandle_Path rows = view.get_selection()->get_selected_rows ();
 			TreeModel::iterator iter;
-			boost::shared_ptr<NoteType> note;
+			std::shared_ptr<NoteType> note;
 
 			for (TreeView::Selection::ListHandle_Path::iterator i = rows.begin(); i != rows.end(); ++i) {
 
@@ -289,21 +292,20 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 
 					switch (prop) {
 					case MidiModel::NoteDiffCommand::StartTime:
-						if (note->time() + fdelta >= 0) {
-							cmd->change (note, prop, note->time() + fdelta);
+						if (note->time() + beat_delta >= Temporal::Beats()) {
+							cmd->change (note, prop, note->time() + beat_delta);
 						} else {
-							cmd->change (note, prop, Evoral::Beats());
+							cmd->change (note, prop, Temporal::Beats());
 						}
 						break;
 					case MidiModel::NoteDiffCommand::Velocity:
 						cmd->change (note, prop, (uint8_t) (note->velocity() + idelta));
 						break;
 					case MidiModel::NoteDiffCommand::Length:
-						if (note->length().to_double() + fdelta >=
-						    Evoral::Beats::tick().to_double()) {
-							cmd->change (note, prop, note->length() + fdelta);
+						if (note->length() + beat_delta >= Temporal::Beats::one_tick()) {
+							cmd->change (note, prop, note->length() + beat_delta);
 						} else {
-							cmd->change (note, prop, Evoral::Beats::tick());
+							cmd->change (note, prop, Temporal::Beats::one_tick());
 						}
 						break;
 					case MidiModel::NoteDiffCommand::Channel:
@@ -328,25 +330,24 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 			previous_selection.push_back (path);
 
 			if (iter) {
-				boost::shared_ptr<NoteType> note = (*iter)[columns._note];
+				std::shared_ptr<NoteType> note = (*iter)[columns._note];
 
 				switch (prop) {
 				case MidiModel::NoteDiffCommand::StartTime:
-					if (note->time() + fdelta >= 0) {
-						cmd->change (note, prop, note->time() + fdelta);
+					if (note->time() + beat_delta >= Temporal::Beats()) {
+						cmd->change (note, prop, note->time() + beat_delta);
 					} else {
-						cmd->change (note, prop, Evoral::Beats());
+						cmd->change (note, prop, Temporal::Beats());
 					}
 					break;
 				case MidiModel::NoteDiffCommand::Velocity:
 					cmd->change (note, prop, (uint8_t) (note->velocity() + idelta));
 					break;
 				case MidiModel::NoteDiffCommand::Length:
-					if (note->length() + fdelta >=
-					    Evoral::Beats::tick().to_double()) {
-						cmd->change (note, prop, note->length() + fdelta);
+					if (note->length() + beat_delta >= Temporal::Beats::one_tick()) {
+						cmd->change (note, prop, note->length() + beat_delta);
 					} else {
-						cmd->change (note, prop, Evoral::Beats::tick());
+						cmd->change (note, prop, Temporal::Beats::one_tick());
 					}
 					break;
 				case MidiModel::NoteDiffCommand::Channel:
@@ -361,7 +362,7 @@ MidiListEditor::scroll_event (GdkEventScroll* ev)
 			}
 		}
 
-		m->apply_command (*_session, cmd);
+		m->apply_diff_command_as_commit (*_session, cmd);
 
 		/* reset selection to be as it was before we rebuilt */
 
@@ -449,9 +450,9 @@ MidiListEditor::key_release (GdkEventKey* ev)
 	TreeViewColumn* col;
 	TreeModel::iterator iter;
 	MidiModel::NoteDiffCommand* cmd;
-	boost::shared_ptr<MidiModel> m (region->midi_source(0)->model());
-	boost::shared_ptr<NoteType> note;
-	boost::shared_ptr<NoteType> copy;
+	std::shared_ptr<MidiModel> m (region->midi_source(0)->model());
+	std::shared_ptr<NoteType> note;
+	std::shared_ptr<NoteType> copy;
 
 	switch (ev->keyval) {
 	case GDK_Insert:
@@ -464,7 +465,7 @@ MidiListEditor::key_release (GdkEventKey* ev)
 		note = (*iter)[columns._note];
 		copy.reset (new NoteType (*note.get()));
 		cmd->add (copy);
-		m->apply_command (*_session, cmd);
+		m->apply_diff_command_as_commit (*_session, cmd);
 		/* model has been redisplayed by now */
 		path.next ();
 		/* select, start editing column 2 (note) */
@@ -511,26 +512,26 @@ MidiListEditor::delete_selected_note ()
 		return;
 	}
 
-	typedef vector<boost::shared_ptr<NoteType> > Notes;
+	typedef vector<std::shared_ptr<NoteType> > Notes;
 	Notes to_delete;
 
 	for (TreeView::Selection::ListHandle_Path::iterator i = rows.begin(); i != rows.end(); ++i) {
 		TreeIter iter;
 
 		if ((iter = model->get_iter (*i))) {
-			boost::shared_ptr<NoteType> note = (*iter)[columns._note];
+			std::shared_ptr<NoteType> note = (*iter)[columns._note];
 			to_delete.push_back (note);
 		}
 	}
 
-	boost::shared_ptr<MidiModel> m (region->midi_source(0)->model());
+	std::shared_ptr<MidiModel> m (region->midi_source(0)->model());
 	MidiModel::NoteDiffCommand* cmd = m->new_note_diff_command (_("delete notes (from list)"));
 
 	for (Notes::iterator i = to_delete.begin(); i != to_delete.end(); ++i) {
 		cmd->remove (*i);
 	}
 
-	m->apply_command (*_session, cmd);
+	m->apply_diff_command_as_commit (*_session, cmd);
 }
 
 void
@@ -582,14 +583,14 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 		return;
 	}
 
-	boost::shared_ptr<NoteType> note = (*iter)[columns._note];
+	std::shared_ptr<NoteType> note = (*iter)[columns._note];
 	MidiModel::NoteDiffCommand::Property prop (MidiModel::NoteDiffCommand::NoteNumber);
 
-	double fval;
+	Temporal::Beats  bval;
 	int    ival;
 	bool   apply = false;
 	int    idelta = 0;
-	double fdelta = 0;
+	Temporal::Beats delta;
 	char const * opname;
 	switch (edit_column) {
 	case 0: // start
@@ -630,18 +631,12 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 		break;
 	case 5: // length
 
-		if (sscanf (text.c_str(), "%lf", &fval) == 1) {
 
-			/* numeric value entered */
+		try {
+			stringstream ss (text);
+			ss >> bval;
 
-			if (text.find ('.') == string::npos && text.find (',') == string::npos) {
-				/* integral => units are ticks */
-				fval = fval / BBT_Time::ticks_per_beat;
-			} else {
-				/* non-integral => beats, so use as-is */
-			}
-
-		} else {
+		} catch (...) {
 
 			/* assume its text from the combo. look for the map
 			 * entry for the actual note ticks
@@ -664,12 +659,12 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 				}
 
 				if (x != note_length_map.end()) {
-					fval = x->first / BBT_Time::ticks_per_beat;
+					bval = x->first / Temporal::ticks_per_beat;
 				}
 
 			} else {
 
-				fval = -1.0;
+				bval = Temporal::Beats();
 
 				if (text != x->second) {
 
@@ -685,14 +680,14 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 
 					if (x != note_length_map.end()) {
 						/* convert to beats */
-						fval = (double) x->first / BBT_Time::ticks_per_beat;
+						bval = x->first / Temporal::ticks_per_beat;
 					}
 				}
 			}
 		}
 
-		if (fval > 0.0) {
-			fdelta = fval - note->length().to_double();
+		if (bval) {
+			delta = bval - note->length();
 			prop = MidiModel::NoteDiffCommand::Length;
 			opname = _("change note length");
 			apply = true;
@@ -705,7 +700,7 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 
 	if (apply) {
 
-		boost::shared_ptr<MidiModel> m (region->midi_source(0)->model());
+		std::shared_ptr<MidiModel> m (region->midi_source(0)->model());
 		MidiModel::NoteDiffCommand* cmd = m->new_note_diff_command (opname);
 
 		TreeView::Selection::ListHandle_Path rows = view.get_selection()->get_selected_rows ();
@@ -720,7 +715,7 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 					cmd->change (note, prop, (uint8_t) (note->velocity() + idelta));
 					break;
 				case MidiModel::NoteDiffCommand::Length:
-					cmd->change (note, prop, note->length() + fdelta);
+					cmd->change (note, prop, note->length() + delta);
 					break;
 				case MidiModel::NoteDiffCommand::Channel:
 					cmd->change (note, prop, (uint8_t) (note->channel() + idelta));
@@ -734,7 +729,7 @@ MidiListEditor::edited (const std::string& path, const std::string& text)
 			}
 		}
 
-		m->apply_command (*_session, cmd);
+		m->apply_diff_command_as_commit (*_session, cmd);
 
 		/* model has been redisplayed by now */
 		/* keep selected row(s), move cursor there, don't continue editing */
@@ -759,26 +754,28 @@ MidiListEditor::redisplay_model ()
 
 	if (_session) {
 
-		BeatsFramesConverter conv (_session->tempo_map(), region->position());
-		MidiModel::Notes notes = region->midi_source(0)->model()->notes();
+		std::shared_ptr<MidiModel> m (region->midi_source(0)->model());
 		TreeModel::Row row;
 		stringstream ss;
 
-		for (MidiModel::Notes::iterator i = notes.begin(); i != notes.end(); ++i) {
+		MidiModel::Notes::const_iterator i = m->note_lower_bound (region->start().beats());
+		Temporal::Beats end_time = (region->start() + region->length()).beats();
+
+		for (; i != m->notes().end() && (*i)->time() < end_time; ++i) {
 			row = *(model->append());
 			row[columns.channel] = (*i)->channel() + 1;
 			row[columns.note_name] = ParameterDescriptor::midi_note_name ((*i)->note());
 			row[columns.note] = (*i)->note();
 			row[columns.velocity] = (*i)->velocity();
 
-			Timecode::BBT_Time bbt (_session->tempo_map().bbt_at_frame (region->position() + conv.to ((*i)->time())));
+			Temporal::BBT_Time bbt = Temporal::TempoMap::use()->bbt_at ((region->position() + timepos_t ((*i)->time())).earlier (region->start()));
 
 			ss.str ("");
 			ss << bbt;
 			row[columns.start] = ss.str();
 
 			bbt.bars = 0;
-			const Evoral::Beats dur = (*i)->end_time() - (*i)->time();
+			const Temporal::Beats dur = (*i)->end_time() - (*i)->time();
 			bbt.beats = dur.get_beats ();
 			bbt.ticks = dur.get_ticks ();
 
@@ -809,7 +806,7 @@ MidiListEditor::selection_changed ()
 
 	TreeModel::Path path;
 	TreeModel::iterator iter;
-	boost::shared_ptr<NoteType> note;
+	std::shared_ptr<NoteType> note;
 	TreeView::Selection::ListHandle_Path rows = view.get_selection()->get_selected_rows ();
 
 	NotePlayer* player = new NotePlayer (track);
