@@ -1,20 +1,21 @@
 /*
-    Copyright (C) 2009 Paul Davis
-
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2009-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2009-2017 Paul Davis <paul@linuxaudiosystems.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_gtk2_midi_list_editor_h_
 #define __ardour_gtk2_midi_list_editor_h_
@@ -31,7 +32,6 @@
 
 namespace Evoral {
 	template<typename Time> class Note;
-	class                         Beats;
 };
 
 namespace ARDOUR {
@@ -44,10 +44,10 @@ namespace ARDOUR {
 class MidiListEditor : public ArdourWindow
 {
 public:
-	typedef Evoral::Note<Evoral::Beats> NoteType;
+	typedef Evoral::Note<Temporal::Beats> NoteType;
 
-	MidiListEditor(ARDOUR::Session*, boost::shared_ptr<ARDOUR::MidiRegion>,
-	               boost::shared_ptr<ARDOUR::MidiTrack>);
+	MidiListEditor(ARDOUR::Session*, std::shared_ptr<ARDOUR::MidiRegion>,
+	               std::shared_ptr<ARDOUR::MidiTrack>);
 	~MidiListEditor();
 
 private:
@@ -68,7 +68,7 @@ private:
 		Gtk::TreeModelColumn<uint8_t>     velocity;
 		Gtk::TreeModelColumn<std::string> start;
 		Gtk::TreeModelColumn<std::string> length;
-		Gtk::TreeModelColumn<boost::shared_ptr<NoteType> > _note;
+		Gtk::TreeModelColumn<std::shared_ptr<NoteType> > _note;
 	};
 
 	struct NoteLengthColumns : public Gtk::TreeModel::ColumnRecord
@@ -95,11 +95,11 @@ private:
 	Gtk::VBox                    vbox;
 	Gtk::ToggleButton            sound_notes_button;
 
-	boost::shared_ptr<ARDOUR::MidiRegion> region;
-	boost::shared_ptr<ARDOUR::MidiTrack>  track;
+	std::shared_ptr<ARDOUR::MidiRegion> region;
+	std::shared_ptr<ARDOUR::MidiTrack>  track;
 
 	/** connection used to connect to model's ContentChanged signal */
-	PBD::ScopedConnection content_connection;
+	PBD::ScopedConnectionList content_connections;
 
 	void edited (const std::string&, const std::string&);
 	void editing_started (Gtk::CellEditable*, const std::string& path, int);
