@@ -1,20 +1,26 @@
 /*
-    Copyright (C) 2001-2011 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2008-2012 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2008-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2015-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2015-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2015-2017 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __gtk_ardour_midi_region_view_h__
 #define __gtk_ardour_midi_region_view_h__
@@ -65,18 +71,18 @@ class CursorContext;
 class MidiRegionView : public RegionView
 {
 public:
-	typedef Evoral::Note<Evoral::Beats> NoteType;
-	typedef Evoral::Sequence<Evoral::Beats>::Notes Notes;
+	typedef Evoral::Note<Temporal::Beats> NoteType;
+	typedef Evoral::Sequence<Temporal::Beats>::Notes Notes;
 
 	MidiRegionView (ArdourCanvas::Container*              parent,
 	                RouteTimeAxisView&                    tv,
-	                boost::shared_ptr<ARDOUR::MidiRegion> r,
+	                std::shared_ptr<ARDOUR::MidiRegion> r,
 	                double                                samples_per_pixel,
 	                uint32_t                              basic_color);
 
 	MidiRegionView (ArdourCanvas::Container*              parent,
 	                RouteTimeAxisView&                    tv,
-	                boost::shared_ptr<ARDOUR::MidiRegion> r,
+	                std::shared_ptr<ARDOUR::MidiRegion> r,
 	                double                                samples_per_pixel,
 	                uint32_t                              basic_color,
 	                bool                                  recording,
@@ -84,13 +90,15 @@ public:
 
 
 	MidiRegionView (const MidiRegionView& other);
-	MidiRegionView (const MidiRegionView& other, boost::shared_ptr<ARDOUR::MidiRegion>);
+	MidiRegionView (const MidiRegionView& other, std::shared_ptr<ARDOUR::MidiRegion>);
 
 	~MidiRegionView ();
 
 	void init (bool wfd);
 
-	const boost::shared_ptr<ARDOUR::MidiRegion> midi_region() const;
+	void set_selected (bool yn);
+
+	const std::shared_ptr<ARDOUR::MidiRegion> midi_region() const;
 
 	inline MidiTimeAxisView* midi_view() const
 	{ return dynamic_cast<MidiTimeAxisView*>(&trackview); }
@@ -99,8 +107,8 @@ public:
 	{ return midi_view()->midi_view(); }
 
 	void step_add_note (uint8_t channel, uint8_t number, uint8_t velocity,
-	                    Evoral::Beats pos, Evoral::Beats len);
-	void step_sustain (Evoral::Beats beats);
+	                    Temporal::Beats pos, Temporal::Beats len);
+	void step_sustain (Temporal::Beats beats);
 	void set_height (double);
 	void apply_note_range(uint8_t lowest, uint8_t highest, bool force=false);
 
@@ -109,32 +117,29 @@ public:
 	uint32_t get_fill_color() const;
 	void color_handler ();
 
-	void show_step_edit_cursor (Evoral::Beats pos);
-	void move_step_edit_cursor (Evoral::Beats pos);
+	void show_step_edit_cursor (Temporal::Beats pos);
+	void move_step_edit_cursor (Temporal::Beats pos);
 	void hide_step_edit_cursor ();
-	void set_step_edit_cursor_width (Evoral::Beats beats);
-
-	void redisplay_model();
+	void set_step_edit_cursor_width (Temporal::Beats beats);
 
 	GhostRegion* add_ghost (TimeAxisView&);
 
-	NoteBase* add_note(const boost::shared_ptr<NoteType> note, bool visible);
-	void resolve_note(uint8_t note_num, Evoral::Beats end_time);
+	NoteBase* add_note(const std::shared_ptr<NoteType> note, bool visible);
 
 	void cut_copy_clear (Editing::CutCopyOp);
-	bool paste (framepos_t pos, const ::Selection& selection, PasteContext& ctx, const int32_t sub_num);
-	void paste_internal (framepos_t pos, unsigned paste_count, float times, const MidiCutBuffer&);
+	bool paste (Temporal::timepos_t const & pos, const ::Selection& selection, PasteContext& ctx);
+	void paste_internal (Temporal::timepos_t const & pos, unsigned paste_count, float times, const MidiCutBuffer&);
 
-	void add_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr patch, const std::string& displaytext, bool);
+	void add_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr patch);
 	void remove_canvas_patch_change (PatchChange* pc);
 
 	/** Look up the given time and channel in the 'automation' and set keys accordingly.
 	 * @param time the time of the patch change event
 	 * @param channel the MIDI channel of the event
-	 * @key a reference to an instance of MIDI::Name::PatchPrimaryKey whose fields will
+	 * @param key a reference to an instance of MIDI::Name::PatchPrimaryKey whose fields will
 	 *        will be set according to the result of the lookup
 	 */
-	void get_patch_key_at (Evoral::Beats time, uint8_t channel, MIDI::Name::PatchPrimaryKey& key) const;
+	void get_patch_key_at (Temporal::Beats time, uint8_t channel, MIDI::Name::PatchPrimaryKey& key) const;
 
 	/** Convert a given PatchChange into a PatchPrimaryKey
 	 */
@@ -145,16 +150,18 @@ public:
 	 * @param new_patch new patch
 	 */
 	void change_patch_change (PatchChange& old_patch, const MIDI::Name::PatchPrimaryKey& new_patch);
-	void change_patch_change (ARDOUR::MidiModel::PatchChangePtr, Evoral::PatchChange<Evoral::Beats> const &);
+	void change_patch_change (ARDOUR::MidiModel::PatchChangePtr, Evoral::PatchChange<Temporal::Beats> const &);
 
-	void add_patch_change (framecnt_t, Evoral::PatchChange<Evoral::Beats> const &);
-	void move_patch_change (PatchChange &, Evoral::Beats);
+	void add_patch_change (Temporal::timecnt_t const &, Evoral::PatchChange<Temporal::Beats> const &);
+	void move_patch_change (PatchChange &, Temporal::Beats);
 	void delete_patch_change (PatchChange *);
 	void edit_patch_change (PatchChange *);
 
 	void delete_sysex (SysEx*);
 
 	/** Change a patch to the next or previous bank/program.
+	 *
+	 * @param patch The patch-change instance (canvas item)
 	 * @param bank If true, step bank, otherwise, step program.
 	 * @param delta Amount to adjust number.
 	 */
@@ -172,16 +179,19 @@ public:
 	void end_write();
 	void extend_active_notes();
 
-	void display_model(boost::shared_ptr<ARDOUR::MidiModel> model);
+	void display_model(std::shared_ptr<ARDOUR::MidiModel> model);
 
+	/* note_diff commands should start here; this initiates an undo record */
 	void start_note_diff_command (std::string name = "midi edit");
+
 	void note_diff_add_change (NoteBase* ev, ARDOUR::MidiModel::NoteDiffCommand::Property, uint8_t val);
-	void note_diff_add_change (NoteBase* ev, ARDOUR::MidiModel::NoteDiffCommand::Property, Evoral::Beats val);
-	void note_diff_add_note (const boost::shared_ptr<NoteType> note, bool selected, bool show_velocity = false);
+	void note_diff_add_change (NoteBase* ev, ARDOUR::MidiModel::NoteDiffCommand::Property, Temporal::Beats val);
+	void note_diff_add_note (const std::shared_ptr<NoteType> note, bool selected, bool show_velocity = false);
 	void note_diff_remove_note (NoteBase* ev);
 
-	void apply_diff (bool as_subcommand = false, bool was_copy = false);
-	void abort_command();
+	/* note_diff commands should be completed with one of these calls; they may (or may not) commit the undo record */
+	void apply_note_diff (bool as_subcommand = false, bool was_copy = false);
+	void abort_note_diff();
 
 	void   note_entered(NoteBase* ev);
 	void   note_left(NoteBase* ev);
@@ -194,26 +204,31 @@ public:
 	void   note_selected(NoteBase* ev, bool add, bool extend=false);
 	void   note_deselected(NoteBase* ev);
 	void   delete_selection();
-	void   delete_note (boost::shared_ptr<NoteType>);
+	void   delete_note (std::shared_ptr<NoteType>);
 	size_t selection_size() { return _selection.size(); }
 	void   select_all_notes ();
-	void   select_range(framepos_t start, framepos_t end);
+	void   select_range(Temporal::timepos_t const & start, Temporal::timepos_t const & end);
 	void   invert_selection ();
+	void   extend_selection ();
+	void   duplicate_selection ();
 
-	Evoral::Beats earliest_in_selection ();
-	void move_selection(double dx, double dy, double cumulative_dy);
-	void note_dropped (NoteBase* ev, double d_qn, int8_t d_note, bool copy);
+	Temporal::Beats earliest_in_selection ();
+	void move_selection(Temporal::timecnt_t const & dx, double dy, double cumulative_dy);
+	void note_dropped (NoteBase* ev, Temporal::timecnt_t const & d_qn, int8_t d_note, bool copy);
 	NoteBase* copy_selection (NoteBase* primary);
-	void move_copies(double dx_qn, double dy, double cumulative_dy);
+	void move_copies(Temporal::timecnt_t const & dx_qn, double dy, double cumulative_dy);
 
-	void select_notes (std::list<Evoral::event_id_t>);
+	void select_notes (std::list<Evoral::event_id_t>, bool allow_audition);
 	void select_matching_notes (uint8_t notenum, uint16_t channel_mask, bool add, bool extend);
 	void toggle_matching_notes (uint8_t notenum, uint16_t channel_mask);
 
-	/** Return true iff the note is within the extent of the region.
+	/** Test if a note is within this region's range
+	 *
+	 * @param note the note to test
 	 * @param visible will be set to true if the note is within the visible note range, false otherwise.
+	 * @return true iff the note is within the (time) extent of the region.
 	 */
-	bool note_in_region_range(const boost::shared_ptr<NoteType> note, bool& visible) const;
+	bool note_in_region_range(const std::shared_ptr<NoteType> note, bool& visible) const;
 
 	/** Get the region position in pixels relative to session. */
 	double get_position_pixels();
@@ -228,7 +243,7 @@ public:
 	void begin_resizing(bool at_front);
 
 	void update_resizing (NoteBase* primary, bool at_front, double delta_x, bool relative, double snap_delta, bool with_snap);
-	void commit_resizing (NoteBase* primary, bool at_front, double delat_x, bool relative, double snap_delta, bool with_snap);
+	void finish_resizing (NoteBase* primary, bool at_front, double delat_x, bool relative, double snap_delta, bool with_snap);
 	void abort_resizing ();
 
 	/** Change the channel of the selection.
@@ -248,8 +263,8 @@ public:
 	MouseState mouse_state() const { return _mouse_state; }
 
 	struct NoteResizeData {
-		Note                     *note;
-		ArdourCanvas::Rectangle  *resize_rect;
+		::Note                  *note;
+		ArdourCanvas::Rectangle *resize_rect;
 	};
 
 	/** Snap a region relative pixel coordinate to pixel units.
@@ -259,50 +274,17 @@ public:
 	 */
 	double snap_to_pixel(double x, bool ensure_snap = false);
 
-	/** Snap a region relative pixel coordinate to frame units.
+	/** Snap a region relative pixel coordinate to time units.
 	 * @param x a pixel coordinate relative to region start
 	 * @param ensure_snap ignore SnapOff and magnetic snap.
 	 * Required for inverting snap logic with modifier keys and snap delta calculation.
-	 * @return the snapped framepos_t coordinate relative to region start
+	 * @return the snapped timecnt_t coordinate relative to region start
 	 */
-	framepos_t snap_pixel_to_sample(double x, bool ensure_snap = false);
-
-	/** Convert a timestamp in beats into frames (both relative to region position) */
-	framepos_t region_beats_to_region_frames(Evoral::Beats beats) const;
-	/** Convert a timestamp in beats into absolute frames */
-	framepos_t region_beats_to_absolute_frames(Evoral::Beats beats) const {
-		return _region->position() + region_beats_to_region_frames (beats);
-	}
-	/** Convert a timestamp in frames to beats (both relative to region position) */
-	Evoral::Beats region_frames_to_region_beats(framepos_t) const;
-	double region_frames_to_region_beats_double(framepos_t) const;
-
-	/** Convert a timestamp in beats measured from source start into absolute frames */
-	framepos_t source_beats_to_absolute_frames(Evoral::Beats beats) const;
-	/** Convert a timestamp in beats measured from source start into region-relative frames */
-	framepos_t source_beats_to_region_frames(Evoral::Beats beats) const {
-		return source_beats_to_absolute_frames (beats) - _region->position();
-	}
-	/** Convert a timestamp in absolute frames to beats measured from source start*/
-	Evoral::Beats absolute_frames_to_source_beats(framepos_t) const;
-
-	ARDOUR::BeatsFramesConverter const & region_relative_time_converter () const {
-		return _region_relative_time_converter;
-	}
-
-	ARDOUR::BeatsFramesConverter const & source_relative_time_converter () const {
-		return _source_relative_time_converter;
-	}
-
-	ARDOUR::DoubleBeatsFramesConverter const & region_relative_time_converter_double () const {
-		return _region_relative_time_converter_double;
-	}
-
-	double session_relative_qn (double qn) const;
+	Temporal::timecnt_t snap_pixel_to_time (double x, bool ensure_snap = false);
 
 	void goto_previous_note (bool add_to_selection);
 	void goto_next_note (bool add_to_selection);
-	void change_note_lengths (bool, bool, Evoral::Beats beats, bool start, bool end);
+	void change_note_lengths (bool, bool, Temporal::Beats beats, bool start, bool end);
 	void change_velocities (bool up, bool fine, bool allow_smush, bool all_together);
 	void transpose (bool up, bool fine, bool allow_smush);
 	void nudge_notes (bool forward, bool fine);
@@ -318,8 +300,6 @@ public:
 
 	void selection_as_notelist (Notes& selected, bool allow_all_if_none_selected = false);
 
-	void enable_display (bool);
-
 	void set_channel_selector_scoped_note(NoteBase* note){ _channel_selection_scoped_note = note; }
 	NoteBase* channel_selector_scoped_note(){  return _channel_selection_scoped_note; }
 
@@ -327,13 +307,13 @@ public:
 	void trim_front_ending ();
 
 	/** Add a note to the model, and the view, at a canvas (click) coordinate.
-	 * \param t time in frames relative to the position of the region
+	 * \param t time in samples relative to the position of the region
 	 * \param y vertical position in pixels
 	 * \param length duration of the note in beats
 	 * \param state the keyboard modifier mask for the canvas event (click).
 	 * \param shift_snap true alters snap behavior to round down always (false if the gui has already done that).
 	 */
-	void create_note_at (framepos_t t, double y, Evoral::Beats length, uint32_t state, bool shift_snap);
+	void create_note_at (Temporal::timepos_t const & t, double y, Temporal::Beats length, uint32_t state, bool shift_snap);
 
 	/** An external request to clear the note selection, remove MRV from editor
 	 * selection.
@@ -344,10 +324,9 @@ public:
 
 	void note_deleted (NoteBase*);
 
-	void show_verbose_cursor_for_new_note_value(boost::shared_ptr<NoteType> current_note,
-	                                            uint8_t new_note) const;
+	void show_verbose_cursor_for_new_note_value(std::shared_ptr<NoteType> current_note, uint8_t new_note) const;
 
-protected:
+  protected:
 	void region_resized (const PBD::PropertyChange&);
 
 	void set_flags (XMLNode *);
@@ -356,8 +335,65 @@ protected:
 	void reset_width_dependent_items (double pixel_width);
 
 	void parameter_changed (std::string const & p);
+	void _redisplay (bool view_only);
 
-private:
+  protected:
+	friend class Editor;
+
+	void clear_note_selection ();
+	void invert_note_selection ();
+	void extend_note_selection ();
+
+	void move_note_starts_earlier_fine () { change_note_lengths (true, false, Temporal::Beats(), true, false); }
+	void move_note_starts_earlier () { change_note_lengths (false, false, Temporal::Beats(), true, false); }
+	void move_note_ends_later_fine () { change_note_lengths (true, false, Temporal::Beats(), false, true); }
+	void move_note_ends_later () { change_note_lengths (false, false, Temporal::Beats(), false, true); }
+	void move_note_starts_later_fine () { change_note_lengths (true, true, Temporal::Beats(), true, false); }
+	void move_note_starts_later () { change_note_lengths (false, true, Temporal::Beats(), true, false); }
+	void move_note_ends_earlier_fine () { change_note_lengths (true, true, Temporal::Beats(), false, true); }
+	void move_note_ends_earlier () { change_note_lengths (false, true, Temporal::Beats(), false, true); }
+
+	void select_next_note () { goto_next_note (false); }
+	void select_previous_note () { goto_previous_note (false); }
+	void add_select_next_note () { goto_next_note (true); }
+	void add_select_previous_note () { goto_previous_note (true); }
+
+	void increase_note_velocity ()                     { change_velocities (true, false, false, false); }
+	void increase_note_velocity_fine ()                { change_velocities (true, true, false, false); }
+	void increase_note_velocity_smush ()               { change_velocities (true, false, true, false); }
+	void increase_note_velocity_together ()            { change_velocities (true, false, false, true); }
+	void increase_note_velocity_fine_smush ()          { change_velocities (true, true, true, false); }
+	void increase_note_velocity_fine_together ()       { change_velocities (true, true, false, true); }
+	void increase_note_velocity_smush_together ()      { change_velocities (true, false, true, true); }
+	void increase_note_velocity_fine_smush_together () { change_velocities (true, true, true, true); }
+
+	void decrease_note_velocity ()                     { change_velocities (false, false, false, false); }
+	void decrease_note_velocity_fine ()                { change_velocities (false, true, false, false); }
+	void decrease_note_velocity_smush ()               { change_velocities (false, false, true, false); }
+	void decrease_note_velocity_together ()            { change_velocities (false, false, false, true); }
+	void decrease_note_velocity_fine_smush ()          { change_velocities (false, true, true, false); }
+	void decrease_note_velocity_fine_together ()       { change_velocities (false, true, false, true); }
+	void decrease_note_velocity_smush_together ()      { change_velocities (false, false, true, true); }
+	void decrease_note_velocity_fine_smush_together () { change_velocities (false, true, true, true); }
+
+	void transpose_up_octave () { transpose (true, false, false); }
+	void transpose_up_octave_smush () { transpose (true, false, true); }
+	void transpose_up_tone () { transpose (true, true, false); }
+	void transpose_up_tone_smush () { transpose (true, true, true); }
+
+	void transpose_down_octave () { transpose (false, false, false); }
+	void transpose_down_octave_smush () { transpose (false, false, true); }
+	void transpose_down_tone () { transpose (false, true, false); }
+	void transpose_down_tone_smush () { transpose (false, true, true); }
+
+	void nudge_notes_later () { nudge_notes (true, false); }
+	void nudge_notes_later_fine () { nudge_notes (true, true); }
+	void nudge_notes_earlier () { nudge_notes (false, false); }
+	void nudge_notes_earlier_fine () { nudge_notes (false, true); }
+
+	void quantize_selected_notes ();
+
+  private:
 
 	friend class MidiRubberbandSelectDrag;
 	friend class MidiVerticalSelectDrag;
@@ -371,16 +407,14 @@ private:
 	/** Play the NoteOn event of the given note immediately
 	 * and schedule the playback of the corresponding NoteOff event.
 	 */
-	void play_midi_note (boost::shared_ptr<NoteType> note);
-	void start_playing_midi_note (boost::shared_ptr<NoteType> note);
-	void start_playing_midi_chord (std::vector<boost::shared_ptr<NoteType> > notes);
+	void play_midi_note (std::shared_ptr<NoteType> note);
+	void start_playing_midi_note (std::shared_ptr<NoteType> note);
+	void start_playing_midi_chord (std::vector<std::shared_ptr<NoteType> > notes);
 
 	/** Clear the note selection of just this midi region
 	 */
 	void clear_selection_internal ();
 
-	void clear_editor_note_selection ();
-
 	void clear_events ();
 
 	bool canvas_group_event(GdkEvent* ev);
@@ -393,38 +427,35 @@ private:
 
 	void change_note_channel (NoteBase *, int8_t, bool relative=false);
 	void change_note_velocity(NoteBase* ev, int8_t vel, bool relative=false);
-	void change_note_note(NoteBase* ev, int8_t note, bool relative=false);
+	uint8_t change_note_note(NoteBase* ev, int8_t note, bool relative=false);
 	void change_note_time(NoteBase* ev, ARDOUR::MidiModel::TimeType, bool relative=false);
 	void change_note_length (NoteBase *, ARDOUR::MidiModel::TimeType);
 	void trim_note(NoteBase* ev, ARDOUR::MidiModel::TimeType start_delta,
 	               ARDOUR::MidiModel::TimeType end_delta);
 
-	void update_drag_selection (framepos_t start, framepos_t end, double y0, double y1, bool extend);
+	void update_drag_selection (Temporal::timepos_t const & start, Temporal::timepos_t const & end, double y0, double y1, bool extend);
 	void update_vertical_drag_selection (double last_y, double y, bool extend);
 
 	void add_to_selection (NoteBase*);
 	void remove_from_selection (NoteBase*);
 
-	std::string get_note_name (boost::shared_ptr<NoteType> note, uint8_t note_value) const;
+	std::string get_note_name (std::shared_ptr<NoteType> note, uint8_t note_value) const;
 
 	void show_verbose_cursor (std::string const &, double, double) const;
-	void show_verbose_cursor (boost::shared_ptr<NoteType>) const;
+	void show_verbose_cursor (std::shared_ptr<NoteType>) const;
 
 	uint8_t get_velocity_for_add (ARDOUR::MidiModel::TimeType time) const;
+	uint8_t get_channel_for_add (ARDOUR::MidiModel::TimeType time) const;
 
 	uint8_t  _current_range_min;
 	uint8_t  _current_range_max;
 
-	typedef boost::unordered_map<boost::shared_ptr<NoteType>, NoteBase*>                             Events;
-	typedef boost::unordered_map<ARDOUR::MidiModel::PatchChangePtr, boost::shared_ptr<PatchChange> > PatchChanges;
-	typedef boost::unordered_map<ARDOUR::MidiModel::constSysExPtr, boost::shared_ptr<SysEx> >        SysExes;
+	typedef boost::unordered_map<std::shared_ptr<NoteType>, NoteBase*>                             Events;
+	typedef boost::unordered_map<ARDOUR::MidiModel::PatchChangePtr, std::shared_ptr<PatchChange> > PatchChanges;
+	typedef boost::unordered_map<ARDOUR::MidiModel::constSysExPtr, std::shared_ptr<SysEx> >        SysExes;
 	typedef std::vector<NoteBase*> CopyDragEvents;
 
-	ARDOUR::BeatsFramesConverter _region_relative_time_converter;
-	ARDOUR::BeatsFramesConverter _source_relative_time_converter;
-	ARDOUR::DoubleBeatsFramesConverter _region_relative_time_converter_double;
-
-	boost::shared_ptr<ARDOUR::MidiModel> _model;
+	std::shared_ptr<ARDOUR::MidiModel> _model;
 	Events                               _events;
 	CopyDragEvents                       _copy_drag_events;
 	PatchChanges                         _patch_changes;
@@ -436,8 +467,8 @@ private:
 	double                               _last_ghost_x;
 	double                               _last_ghost_y;
 	ArdourCanvas::Rectangle*             _step_edit_cursor;
-	Evoral::Beats                        _step_edit_cursor_width;
-	Evoral::Beats                        _step_edit_cursor_position;
+	Temporal::Beats                      _step_edit_cursor_width;
+	Temporal::Beats                      _step_edit_cursor_position;
 	NoteBase*                            _channel_selection_scoped_note;
 
 	MouseState _mouse_state;
@@ -450,26 +481,26 @@ private:
 
 	/** New notes (created in the current command) which should be selected
 	 * when they appear after the command is applied. */
-	std::set< boost::shared_ptr<NoteType> > _marked_for_selection;
+	std::set< std::shared_ptr<NoteType> > _marked_for_selection;
 
 	/** Notes that should be selected when the model is redisplayed. */
 	std::set<Evoral::event_id_t> _pending_note_selection;
 
 	/** New notes (created in the current command) which should have visible velocity
 	 * when they appear after the command is applied. */
-	std::set< boost::shared_ptr<NoteType> > _marked_for_velocity;
+	std::set< std::shared_ptr<NoteType> > _marked_for_velocity;
 
 	std::vector<NoteResizeData *> _resize_data;
 
 	/** connection used to connect to model's ContentChanged signal */
 	PBD::ScopedConnection content_connection;
 
-	NoteBase* find_canvas_note (boost::shared_ptr<NoteType>);
+	NoteBase* find_canvas_note (std::shared_ptr<NoteType>);
 	NoteBase* find_canvas_note (Evoral::event_id_t id);
 	Events::iterator _optimization_iterator;
 
-	boost::shared_ptr<PatchChange> find_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr p);
-	boost::shared_ptr<SysEx> find_canvas_sys_ex (ARDOUR::MidiModel::SysExPtr s);
+	std::shared_ptr<PatchChange> find_canvas_patch_change (ARDOUR::MidiModel::PatchChangePtr p);
+	std::shared_ptr<SysEx> find_canvas_sys_ex (ARDOUR::MidiModel::SysExPtr s);
 
 	void update_note (NoteBase*, bool update_ghost_regions = true);
 	void update_sustained (Note *, bool update_ghost_regions = true);
@@ -495,15 +526,17 @@ private:
 
 	void drop_down_keys ();
 	void maybe_select_by_position (GdkEventButton* ev, double x, double y);
-	void get_events (Events& e, Evoral::Sequence<Evoral::Beats>::NoteOperator op, uint8_t val, int chan_mask = 0);
+	void get_events (Events& e, Evoral::Sequence<Temporal::Beats>::NoteOperator op, uint8_t val, int chan_mask = 0);
 
 	void display_patch_changes_on_channel (uint8_t, bool);
 
 	void connect_to_diskstream ();
-	void data_recorded (boost::weak_ptr<ARDOUR::MidiSource>);
+	void data_recorded (std::weak_ptr<ARDOUR::MidiSource>);
 
 	/** Get grid type as beats, or default to 1 if not snapped to beats. */
-	Evoral::Beats get_grid_beats(framepos_t pos) const;
+	Temporal::Beats get_grid_beats(Temporal::timepos_t const & pos) const;
+
+	Temporal::Beats get_draw_length_beats(Temporal::timepos_t const & pos) const;
 
 	void remove_ghost_note ();
 	void mouse_mode_changed ();
@@ -511,24 +544,22 @@ private:
 	void leave_internal ();
 	void hide_verbose_cursor ();
 
-	framecnt_t _last_display_zoom;
+	samplecnt_t _last_display_zoom;
 
 	double    _last_event_x;
 	double    _last_event_y;
-	bool      _grabbed_keyboard;
 	bool      _entered;
 	NoteBase* _entered_note;
+	bool      _pasting;
 
 	bool _mouse_changed_selection;
 
 	Gtkmm2ext::Color _patch_change_outline;
 	Gtkmm2ext::Color _patch_change_fill;
 
-	Evoral::Beats snap_frame_to_grid_underneath (framepos_t p, int32_t divisions, bool shift_snap) const;
-
 	PBD::ScopedConnection _mouse_mode_connection;
 
-	boost::shared_ptr<CursorContext> _press_cursor_ctx;
+	std::shared_ptr<CursorContext> _press_cursor_ctx;
 
 	ARDOUR::ChannelMode get_channel_mode() const;
 	uint16_t get_selected_channels () const;
@@ -539,6 +570,11 @@ private:
 
 	double note_to_y (uint8_t note) const;
 	uint8_t y_to_note (double y) const;
+
+	void update_patch_changes ();
+	void update_sysexes ();
+	void view_changed ();
+	void model_changed ();
 };
 
 
