@@ -1,20 +1,26 @@
 /*
-    Copyright (C) 2001-2007 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2008-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2014-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2016 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cmath>
 #include <utility>
@@ -61,16 +67,22 @@ MidiStreamView::MidiStreamView (MidiTimeAxisView& tv)
 	, note_range_adjustment(0.0f, 0.0f, 0.0f)
 	, _range_dirty(false)
 	, _range_sum_cache(-1.0)
-	, _lowest_note(60)
-	, _highest_note(71)
+	, _lowest_note(UIConfiguration::instance().get_default_lower_midi_note())
+	, _highest_note(UIConfiguration::instance().get_default_upper_midi_note())
 	, _data_note_min(60)
 	, _data_note_max(71)
 	, _note_lines (0)
 	, _updates_suspended (false)
 {
+
 	/* use a group dedicated to MIDI underlays. Audio underlays are not in this group. */
-	midi_underlay_group = new ArdourCanvas::Container (_canvas_group);
-	midi_underlay_group->lower_to_bottom();
+	_midi_underlay = new ArdourCanvas::Container (_canvas_group);
+	_midi_underlay->lower_to_bottom();
+
+	/* use a dedicated group for MIDI regions (on top of the grid and lines) */
+	_region_group = new ArdourCanvas::Container (_canvas_group);
+	_region_group->raise_to_top ();
+	_region_group->set_render_with_alpha (UIConfiguration::instance().modifier ("region alpha").a());
 
 	/* put the note lines in the timeaxisview's group, so it
 	   can be put below ghost regions from MIDI underlays
@@ -87,6 +99,7 @@ MidiStreamView::MidiStreamView (MidiTimeAxisView& tv)
 	color_handler ();
 
 	UIConfiguration::instance().ColorsChanged.connect(sigc::mem_fun(*this, &MidiStreamView::color_handler));
+	UIConfiguration::instance().ParameterChanged.connect(sigc::mem_fun(*this, &MidiStreamView::parameter_changed));
 
 	note_range_adjustment.set_page_size(_highest_note - _lowest_note);
 	note_range_adjustment.set_value(_lowest_note);
@@ -99,10 +112,20 @@ MidiStreamView::~MidiStreamView ()
 {
 }
 
-RegionView*
-MidiStreamView::create_region_view (boost::shared_ptr<Region> r, bool /*wfd*/, bool recording)
+void
+MidiStreamView::parameter_changed (string const & param)
 {
-	boost::shared_ptr<MidiRegion> region = boost::dynamic_pointer_cast<MidiRegion> (r);
+	if (param == X_("max-note-height")) {
+		apply_note_range (_lowest_note, _highest_note, true);
+	} else {
+		StreamView::parameter_changed (param);
+	}
+}
+
+RegionView*
+MidiStreamView::create_region_view (std::shared_ptr<Region> r, bool /*wfd*/, bool recording)
+{
+	std::shared_ptr<MidiRegion> region = std::dynamic_pointer_cast<MidiRegion> (r);
 
 	if (region == 0) {
 		return 0;
@@ -111,11 +134,11 @@ MidiStreamView::create_region_view (boost::shared_ptr<Region> r, bool /*wfd*/, b
 	RegionView* region_view = NULL;
 	if (recording) {
 		region_view = new MidiRegionView (
-			_canvas_group, _trackview, region,
+			_region_group, _trackview, region,
 			_samples_per_pixel, region_color, recording,
 			TimeAxisViewItem::Visibility(TimeAxisViewItem::ShowFrame));
 	} else {
-		region_view = new MidiRegionView (_canvas_group, _trackview, region,
+		region_view = new MidiRegionView (_region_group, _trackview, region,
 		                                  _samples_per_pixel, region_color);
 	}
 
@@ -125,9 +148,9 @@ MidiStreamView::create_region_view (boost::shared_ptr<Region> r, bool /*wfd*/, b
 }
 
 RegionView*
-MidiStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_data, bool recording)
+MidiStreamView::add_region_view_internal (std::shared_ptr<Region> r, bool wait_for_data, bool recording)
 {
-	boost::shared_ptr<MidiRegion> region = boost::dynamic_pointer_cast<MidiRegion> (r);
+	std::shared_ptr<MidiRegion> region = std::dynamic_pointer_cast<MidiRegion> (r);
 
 	if (!region) {
 		return 0;
@@ -140,7 +163,7 @@ MidiStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait
 
 			(*i)->set_valid (true);
 
-			display_region(dynamic_cast<MidiRegionView*>(*i), wait_for_data);
+			display_region (dynamic_cast<MidiRegionView*>(*i), wait_for_data);
 
 			return 0;
 		}
@@ -153,16 +176,20 @@ MidiStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait
 
 	region_views.push_front (region_view);
 
-	/* display events and find note range */
-	display_region (region_view, wait_for_data);
+	{
+		RegionView::DisplaySuspender ds (*region_view, false);
 
-	/* fit note range if we are importing */
-	if (_trackview.session()->operation_in_progress (Operations::insert_file)) {
-		set_note_range (ContentsRange);
+		display_region (region_view, wait_for_data);
+
+		/* fit note range if we are importing */
+		if (_trackview.session()->operation_in_progress (Operations::insert_file)) {
+			/* this will call display_region() */
+			set_note_range (ContentsRange);
+		}
 	}
 
 	/* catch regionview going away */
-	boost::weak_ptr<Region> wr (region); // make this explicit
+	std::weak_ptr<Region> wr (region); // make this explicit
 	region->DropReferences.connect (*this, invalidator (*this), boost::bind (&MidiStreamView::remove_region_view, this, wr), gui_context());
 
 	RegionViewAdded (region_view);
@@ -171,60 +198,53 @@ MidiStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait
 }
 
 void
-MidiStreamView::display_region(MidiRegionView* region_view, bool load_model)
+MidiStreamView::display_region (MidiRegionView* region_view, bool)
 {
 	if (!region_view) {
 		return;
 	}
 
-	region_view->enable_display (true);
+	RegionView::DisplaySuspender ds (*region_view, false);
+
 	region_view->set_height (child_height());
 
-	boost::shared_ptr<MidiSource> source(region_view->midi_region()->midi_source(0));
+	std::shared_ptr<MidiSource> source (region_view->midi_region()->midi_source(0));
+
 	if (!source) {
 		error << _("attempt to display MIDI region with no source") << endmsg;
 		return;
 	}
 
-	if (load_model) {
-		Glib::Threads::Mutex::Lock lm(source->mutex());
-		source->load_model(lm);
-	}
-
 	if (!source->model()) {
 		error << _("attempt to display MIDI region with no model") << endmsg;
 		return;
 	}
 
-	_range_dirty = update_data_note_range(
-		source->model()->lowest_note(),
-		source->model()->highest_note());
+	_range_dirty = update_data_note_range (source->model()->lowest_note(), source->model()->highest_note());
 
 	// Display region contents
-	region_view->display_model(source->model());
+	region_view->display_model (source->model());
 }
 
 
 void
-MidiStreamView::display_track (boost::shared_ptr<Track> tr)
+MidiStreamView::display_track (std::shared_ptr<Track> tr)
 {
 	StreamView::display_track (tr);
 
 	draw_note_lines();
 
-	NoteRangeChanged();
+	NoteRangeChanged(); /* EMIT SIGNAL*/
 }
 
 void
-MidiStreamView::update_contents_metrics(boost::shared_ptr<Region> r)
+MidiStreamView::update_contents_metrics(std::shared_ptr<Region> r)
 {
-	boost::shared_ptr<MidiRegion> mr = boost::dynamic_pointer_cast<MidiRegion>(r);
+	std::shared_ptr<MidiRegion> mr = std::dynamic_pointer_cast<MidiRegion>(r);
+
 	if (mr) {
-		Glib::Threads::Mutex::Lock lm(mr->midi_source(0)->mutex());
-		mr->midi_source(0)->load_model(lm);
-		_range_dirty = update_data_note_range(
-			mr->model()->lowest_note(),
-			mr->model()->highest_note());
+		Source::ReaderLock lm (mr->midi_source(0)->mutex());
+		_range_dirty = update_data_note_range (mr->model()->lowest_note(), mr->model()->highest_note());
 	}
 }
 
@@ -253,6 +273,9 @@ MidiStreamView::set_layer_display (LayerDisplay d)
 //	}
 
 	StreamView::set_layer_display (d);
+	for (auto& rv : region_views) {
+		rv->set_frame_color ();
+	}
 }
 
 void
@@ -277,24 +300,24 @@ MidiStreamView::redisplay_track ()
 		_data_note_max = 71;
 	}
 
+	vector<RegionView::DisplaySuspender> vds;
+
 	// Flag region views as invalid and disable drawing
 	for (i = region_views.begin(); i != region_views.end(); ++i) {
-		(*i)->set_valid(false);
-		(*i)->enable_display(false);
+		(*i)->set_valid (false);
+		vds.push_back (RegionView::DisplaySuspender (**i, false));
 	}
 
 	// Add and display region views, and flag them as valid
-	_trackview.track()->playlist()->foreach_region(
-		sigc::hide_return (sigc::mem_fun (*this, &StreamView::add_region_view)));
+	_trackview.track()->playlist()->foreach_region (sigc::hide_return (sigc::mem_fun (*this, &StreamView::add_region_view)));
 
 	// Stack regions by layer, and remove invalid regions
 	layer_regions();
 
 	// Update note range (not regions which are correct) and draw note lines
-	apply_note_range(_lowest_note, _highest_note, false);
+	apply_note_range (_lowest_note, _highest_note, false);
 }
 
-
 void
 MidiStreamView::update_contents_height ()
 {
@@ -316,7 +339,7 @@ MidiStreamView::draw_note_lines()
 	double prev_y = .5;
 	uint32_t color;
 
-	_note_lines->clear();
+	ArdourCanvas::LineSet::ResetRAII lr (*_note_lines);
 
 	if (child_height() < 140 || note_height() < 3) {
 		/* track is too small for note lines, or there are too many */
@@ -336,7 +359,7 @@ MidiStreamView::draw_note_lines()
 		 */
 
 		if (i <= highest_note()) {
-			_note_lines->add (y, 1.0, UIConfiguration::instance().color ("piano roll black outline"));
+			_note_lines->add_coord (y, 1.0, UIConfiguration::instance().color ("piano roll black outline"));
 		}
 
 		/* now add a thicker line/bar which covers the entire vertical
@@ -360,7 +383,7 @@ MidiStreamView::draw_note_lines()
 		double mid = y + (h/2.0);
 
 		if (mid >= 0 && h > 1.0) {
-			_note_lines->add (mid, h, color);
+			_note_lines->add_coord (mid, h, color);
 		}
 
 		prev_y = y;
@@ -387,36 +410,35 @@ MidiStreamView::apply_note_range(uint8_t lowest, uint8_t highest, bool to_region
 	_highest_note = highest;
 	_lowest_note = lowest;
 
-	int const max_note_height = 20;  // This should probably be based on text size...
+	float uiscale = UIConfiguration::instance().get_ui_scale();
+	uiscale = expf (uiscale) / expf (1.f);
+
+	const int mnh = UIConfiguration::instance().get_max_note_height();
+	int const max_note_height = std::max<int> (mnh, mnh * uiscale);
 	int const range = _highest_note - _lowest_note;
-	int const pixels_per_note = floor (child_height () / range);
 
-	/* do not grow note height beyond 10 pixels */
-	if (pixels_per_note > max_note_height) {
+	int const available_note_range = floor (child_height() / max_note_height);
+	int additional_notes = available_note_range - range;
 
-		int const available_note_range = floor (child_height() / max_note_height);
-		int additional_notes = available_note_range - range;
+	/* distribute additional notes to higher and lower ranges, clamp at 0 and 127 */
+	for (int i = 0; i < additional_notes; i++){
 
-		/* distribute additional notes to higher and lower ranges, clamp at 0 and 127 */
-		for (int i = 0; i < additional_notes; i++){
-
-			if (i % 2 && _highest_note < 127){
-				_highest_note++;
-			}
-			else if (i % 2) {
-				_lowest_note--;
-			}
-			else if (_lowest_note > 0){
-				_lowest_note--;
-			}
-			else {
-				_highest_note++;
-			}
+		if (i % 2 && _highest_note < 127){
+			_highest_note++;
+		}
+		else if (i % 2) {
+			_lowest_note--;
+		}
+		else if (_lowest_note > 0){
+			_lowest_note--;
+		}
+		else {
+			_highest_note++;
 		}
 	}
 
-	note_range_adjustment.set_page_size(_highest_note - _lowest_note);
-	note_range_adjustment.set_value(_lowest_note);
+	note_range_adjustment.set_page_size (_highest_note - _lowest_note);
+	note_range_adjustment.set_value (_lowest_note);
 
 	draw_note_lines();
 
@@ -424,7 +446,7 @@ MidiStreamView::apply_note_range(uint8_t lowest, uint8_t highest, bool to_region
 		apply_note_range_to_regions ();
 	}
 
-	NoteRangeChanged();
+	NoteRangeChanged(); /* EMIT SIGNAL*/
 }
 
 void
@@ -449,7 +471,8 @@ MidiStreamView::setup_rec_box ()
 {
 	// cerr << _trackview.name() << " streamview SRB\n";
 
-	if (!_trackview.session()->transport_stopped()) {
+	if (!_trackview.session()->transport_stopped_or_stopping() &&
+	    (_trackview.session()->transport_rolling() || _trackview.session()->get_record_enabled())) {
 
 		if (!rec_active &&
 		    _trackview.session()->record_status() == Session::Recording &&
@@ -467,10 +490,10 @@ MidiStreamView::setup_rec_box ()
 
 				// handle multi
 
-				framepos_t start = 0;
+				timepos_t start;
 				if (rec_regions.size() > 0) {
 					start = rec_regions.back().first->start()
-						+ _trackview.track()->get_captured_frames(rec_regions.size()-1);
+						+ timepos_t (_trackview.track()->get_captured_samples (rec_regions.size() - 1));
 				}
 
 				if (!rec_regions.empty()) {
@@ -481,29 +504,26 @@ MidiStreamView::setup_rec_box ()
 				PropertyList plist;
 
 				plist.add (ARDOUR::Properties::start, start);
-				plist.add (ARDOUR::Properties::length, 1);
-				/* Just above we're setting this nascent region's length to 1.  I think this
-				   is so that the RegionView gets created with a non-zero width, as apparently
-				   creating a RegionView with a zero width causes it never to be displayed
-				   (there is a warning in TimeAxisViewItem::init about this).  However, we
-				   must also set length_beats to something non-zero, otherwise the frame length
-				   of 1 causes length_beats to be set to some small quantity << 1.  Then
-				   when the position is set up below, this length_beats is used to recompute
-				   length using BeatsFramesConverter::to, which is slightly innacurate for small
-				   beats values because it converts floating point beats to bars, beats and
-				   integer ticks.  The upshot of which being that length gets set back to 0,
-				   meaning no region view is ever seen, meaning no MIDI notes during record (#3820).
+				plist.add (ARDOUR::Properties::length, timepos_t (Temporal::Beats::ticks (1)));
+				/* Just above we're setting this nascent region's length to 1 tick.  I think this is so
+				   that the RegionView gets created with a non-zero width, as apparently creating a
+				   RegionView with a zero width causes it never to be displayed (there is a warning in
+				   TimeAxisViewItem::init about this). We don't want to use 1 sample since that results
+				   in zero length musical time duration.
 				*/
-				plist.add (ARDOUR::Properties::length_beats, 1);
 				plist.add (ARDOUR::Properties::name, string());
 				plist.add (ARDOUR::Properties::layer, 0);
 
-				boost::shared_ptr<MidiRegion> region (boost::dynamic_pointer_cast<MidiRegion>
+				std::shared_ptr<MidiRegion> region (std::dynamic_pointer_cast<MidiRegion>
 				                                      (RegionFactory::create (sources, plist, false)));
 				if (region) {
-					region->set_start (_trackview.track()->current_capture_start()
-					                   - _trackview.track()->get_capture_start_frame (0));
-					region->set_position (_trackview.session()->transport_frame());
+
+					/* MIDI regions should likely not be positioned using audio time, but this is
+					 * just a rec-region, so we don't really care
+					 */
+
+					region->set_start (timepos_t (_trackview.track()->current_capture_start() - _trackview.track()->get_capture_start_sample (0)));
+					region->set_position (timepos_t (_trackview.track()->current_capture_start ()));
 
 					RegionView* rv = add_region_view_internal (region, false, true);
 					MidiRegionView* mrv = dynamic_cast<MidiRegionView*> (rv);
@@ -521,7 +541,7 @@ MidiStreamView::setup_rec_box ()
 
 			/* start a new rec box */
 
-			create_rec_box(_trackview.midi_track()->current_capture_start(), 0);
+			create_rec_box (_trackview.midi_track()->current_capture_start(), 0);
 
 		} else if (rec_active &&
 		           (_trackview.session()->record_status() != Session::Recording ||
@@ -545,8 +565,8 @@ MidiStreamView::setup_rec_box ()
 
 			/* remove temp regions */
 
-			for (list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end();) {
-				list<pair<boost::shared_ptr<Region>,RegionView*> >::iterator tmp;
+			for (list<pair<std::shared_ptr<Region>,RegionView*> >::iterator iter = rec_regions.begin(); iter != rec_regions.end();) {
+				list<pair<std::shared_ptr<Region>,RegionView*> >::iterator tmp;
 
 				tmp = iter;
 				++tmp;
@@ -575,6 +595,7 @@ MidiStreamView::setup_rec_box ()
 void
 MidiStreamView::color_handler ()
 {
+	_region_group->set_render_with_alpha (UIConfiguration::instance().modifier ("region alpha").a());
 	draw_note_lines ();
 
 	if (_trackview.is_midi_track()) {
@@ -622,8 +643,8 @@ MidiStreamView::update_rec_box ()
 	}
 
 	/* Update the region being recorded to reflect where we currently are */
-	boost::shared_ptr<ARDOUR::Region> region = rec_regions.back().first;
-	region->set_length (_trackview.track()->current_capture_end () - _trackview.track()->current_capture_start(), 0);
+	std::shared_ptr<ARDOUR::Region> region = rec_regions.back().first;
+	region->set_length (timecnt_t (_trackview.track()->current_capture_end () - _trackview.track()->current_capture_start()));
 
 	MidiRegionView* mrv = dynamic_cast<MidiRegionView*> (rec_regions.back().second);
 	mrv->extend_active_notes ();
@@ -675,7 +696,7 @@ struct RegionPositionSorter {
 };
 
 bool
-MidiStreamView::paste (ARDOUR::framepos_t pos, const Selection& selection, PasteContext& ctx, const int32_t sub_num)
+MidiStreamView::paste (timepos_t const & pos, const Selection& selection, PasteContext& ctx)
 {
 	/* Paste into the first region which starts on or before pos.  Only called when
 	   using an internal editing tool. */
@@ -695,7 +716,7 @@ MidiStreamView::paste (ARDOUR::framepos_t pos, const Selection& selection, Paste
 		prev = i;
 	}
 
-	boost::shared_ptr<Region> r = (*prev)->region ();
+	std::shared_ptr<Region> r = (*prev)->region ();
 
 	/* If *prev doesn't cover pos, it's no good */
 	if (r->position() > pos || ((r->position() + r->length()) < pos)) {
@@ -703,5 +724,21 @@ MidiStreamView::paste (ARDOUR::framepos_t pos, const Selection& selection, Paste
 	}
 
 	MidiRegionView* mrv = dynamic_cast<MidiRegionView*> (*prev);
-	return mrv ? mrv->paste(pos, selection, ctx, sub_num) : false;
+	return mrv ? mrv->paste(pos, selection, ctx) : false;
+}
+
+void
+MidiStreamView::get_regions_with_selected_data (RegionSelection& rs)
+{
+	for (list<RegionView*>::const_iterator i = region_views.begin(); i != region_views.end(); ++i) {
+		MidiRegionView* mrv = dynamic_cast<MidiRegionView*> (*i);
+
+		if (!mrv) {
+			continue;
+		}
+
+		if (!mrv->selection().empty()) {
+			rs.add (*i);
+		}
+	}
 }
