@@ -1,20 +1,24 @@
 /*
-    Copyright (C) 2001, 2006 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2008-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2016-2017 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_midi_streamview_h__
 #define __ardour_midi_streamview_h__
@@ -59,8 +63,8 @@ public:
 	MidiStreamView (MidiTimeAxisView&);
 	~MidiStreamView ();
 
-	void set_selected_regionviews (RegionSelection&);
 	void get_inverted_selectables (Selection&, std::list<Selectable* >& results);
+	void get_regions_with_selected_data (RegionSelection&);
 
 	enum VisibleNoteRange {
 		FullRange,
@@ -68,7 +72,6 @@ public:
 	};
 
 	Gtk::Adjustment note_range_adjustment;
-	ArdourCanvas::Container* midi_underlay_group;
 
 	void set_note_range(VisibleNoteRange r);
 
@@ -101,34 +104,42 @@ public:
 
 	sigc::signal<void> NoteRangeChanged;
 
-	RegionView* create_region_view (boost::shared_ptr<ARDOUR::Region>, bool, bool);
+	RegionView* create_region_view (std::shared_ptr<ARDOUR::Region>, bool, bool);
 
-	bool paste (ARDOUR::framepos_t pos, const Selection& selection, PasteContext& ctx, const int32_t sub_num);
+	bool paste (Temporal::timepos_t const & pos, const Selection& selection, PasteContext& ctx);
 
 	void apply_note_range(uint8_t lowest, uint8_t highest, bool to_region_views);
 
 	void suspend_updates ();
 	void resume_updates ();
 
+	ArdourCanvas::Container* midi_underlay () const { return _midi_underlay; }
+	ArdourCanvas::Container* region_canvas () const { return _region_group; }
+
+	void parameter_changed (std::string const &);
+
 protected:
 	void setup_rec_box ();
 	void update_rec_box ();
 
+	ArdourCanvas::Container* _midi_underlay;
+	ArdourCanvas::Container* _region_group;
+
 private:
 
 	RegionView* add_region_view_internal (
-			boost::shared_ptr<ARDOUR::Region>,
+			std::shared_ptr<ARDOUR::Region>,
 			bool wait_for_waves,
 			bool recording = false);
 
 	void display_region(MidiRegionView* region_view, bool load_model);
-	void display_track (boost::shared_ptr<ARDOUR::Track> tr);
+	void display_track (std::shared_ptr<ARDOUR::Track> tr);
 
 	void update_contents_height ();
 
 	void draw_note_lines();
 	bool update_data_note_range(uint8_t min, uint8_t max);
-	void update_contents_metrics(boost::shared_ptr<ARDOUR::Region> r);
+	void update_contents_metrics(std::shared_ptr<ARDOUR::Region> r);
 
 	void color_handler ();
 
