@@ -1,26 +1,38 @@
 /*
-  Copyright (C) 2001-2009 Paul Davis
+ * Copyright (C) 2005-2006 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2005-2006 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2005-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Karsten Wiese <fzuuzf@googlemail.com>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2008-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2012-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015 John Emmas <john@creativepost.co.uk>
+ * Copyright (C) 2016 Julien "_FrnchFrgg_" RIVAUD <frnchfrgg@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+#include <ctype.h>
 #include <algorithm>
 
+#include <boost/tokenizer.hpp>
+
 #include <gtkmm/box.h>
 #include <gtkmm/alignment.h>
 #include "gtkmm2ext/utils.h"
+#include "gtkmm2ext/colors.h"
 
 #include "ardour/dB.h"
 #include "ardour/rc_configuration.h"
@@ -32,9 +44,12 @@
 #include "pbd/replace_all.h"
 #include "pbd/strsplit.h"
 
+#include "widgets/frame.h"
+
 #include "gui_thread.h"
 #include "option_editor.h"
 #include "public_editor.h"
+#include "ui_config.h"
 #include "utils.h"
 #include "pbd/i18n.h"
 
@@ -51,15 +66,19 @@ OptionEditorComponent::add_widget_to_page (OptionEditorPage* p, Gtk::Widget* w)
 	if (!_note.empty ()) {
 		++m;
 	}
+	_frame = manage (new ArdourWidgets::Frame);
+	_frame->add (*w);
+	_frame->set_draw (false);
+	_frame->set_edge_color (UIConfiguration::instance().color (X_("preference highlight")));
 
 	p->table.resize (m, 3);
-	p->table.attach (*w, 1, 3, n, n + 1, FILL | EXPAND);
+	p->table.attach (*_frame, 1, 3, n, n + 1, FILL | EXPAND);
 
 	maybe_add_note (p, n + 1);
 }
 
 void
-OptionEditorComponent::add_widgets_to_page (OptionEditorPage* p, Gtk::Widget* wa, Gtk::Widget* wb, bool expand)
+OptionEditorComponent::add_widgets_to_page (OptionEditorPage* p, Gtk::Widget* wa, Gtk::Widget* wb, bool /*notused*/)
 {
 	int const n = p->table.property_n_rows();
 	int m = n + 1;
@@ -67,15 +86,18 @@ OptionEditorComponent::add_widgets_to_page (OptionEditorPage* p, Gtk::Widget* wa
 		++m;
 	}
 
+	_frame = manage (new ArdourWidgets::Frame);
+	_frame->add (*wa);
+	_frame->set_draw (false);
+	_frame->set_edge_color (UIConfiguration::instance().color (X_("preference highlight")));
+
 	p->table.resize (m, 3);
-	p->table.attach (*wa, 1, 2, n, n + 1, FILL);
-	if (expand) {
-		p->table.attach (*wb, 2, 3, n, n + 1, FILL | EXPAND);
-	} else {
-		Alignment* a = manage (new Alignment (0, 0.5, 0, 1.0));
-		a->add (*wb);
-		p->table.attach (*a, 2, 3, n, n + 1, FILL | EXPAND);
-	}
+	p->table.attach (*_frame, 1, 2, n, n + 1, FILL);
+
+	Alignment* a = manage (new Alignment (0, 0.5, 0, 1.0));
+	a->add (*wb);
+	p->table.attach (*a, 2, 3, n, n + 1, FILL | EXPAND);
+
 	maybe_add_note (p, n + 1);
 }
 
@@ -96,6 +118,34 @@ OptionEditorComponent::set_note (string const & n)
 	_note = n;
 }
 
+void
+OptionEditorComponent::highlight ()
+{
+	if (_frame) {
+		_frame->set_draw (true);
+	}
+}
+
+void
+OptionEditorComponent::end_highlight ()
+{
+	if (_frame) {
+		_frame->set_draw (false);
+	}
+}
+
+PBD::Configuration::Metadata const *
+OptionEditorComponent::get_metadata () const
+{
+	return _metadata;
+}
+
+void
+OptionEditorComponent::set_metadata (PBD::Configuration::Metadata const & m)
+{
+	_metadata = &m;
+}
+
 /*--------------------------*/
 
 OptionEditorHeading::OptionEditorHeading (string const & h)
@@ -193,15 +243,70 @@ RcActionButton::add_to_page (OptionEditorPage *p)
 {
 	int const n = p->table.property_n_rows();
 	int m = n + 1;
+	if (!_note.empty ()) {
+		++m;
+	}
 	p->table.resize (m, 3);
+	Alignment* a = manage (new Alignment (0, 0.5, 0, 1.0));
+	a->add (*_button);
+
 	if (_label) {
-		p->table.attach (*_label,  1, 2, n, n + 1, FILL | EXPAND);
-		p->table.attach (*_button, 2, 3, n, n + 1, FILL | EXPAND);
+		p->table.attach (*_label,  1, 2, n, n + 1);
+		p->table.attach (*a, 2, 3, n, n + 1, FILL|EXPAND);
 	} else {
-		p->table.attach (*_button, 1, 3, n, n + 1, FILL | EXPAND);
+		p->table.attach (*a, 1, 3, n, n + 1, FILL|EXPAND);
+	}
+	maybe_add_note (p, n + 1);
+}
+
+/*--------------------------*/
+
+CheckOption::CheckOption (string const & i, string const & n, Glib::RefPtr<Gtk::Action> act)
+{
+	_button = manage (new CheckButton);
+	_label = manage (new Label);
+	_label->set_markup (n);
+	_button->add (*_label);
+	_button->signal_toggled().connect (sigc::mem_fun (*this, &CheckOption::toggled));
+
+	Gtkmm2ext::Activatable::set_related_action (act);
+	assert (_action);
+
+	action_sensitivity_changed ();
+
+	Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic (_action);
+	if (tact) {
+		action_toggled ();
+		tact->signal_toggled().connect (sigc::mem_fun (*this, &CheckOption::action_toggled));
+	}
+
+	_action->connect_property_changed ("sensitive", sigc::mem_fun (*this, &CheckOption::action_sensitivity_changed));
+}
+
+void
+CheckOption::action_toggled ()
+{
+	Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic (_action);
+	if (tact) {
+		_button->set_active (tact->get_active());
 	}
 }
 
+void
+CheckOption::add_to_page (OptionEditorPage* p)
+{
+	add_widget_to_page (p, _button);
+}
+
+void
+CheckOption::toggled ()
+{
+	Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic (_action);
+
+	tact->set_active (_button->get_active ());
+}
+
+
 /*--------------------------*/
 
 BoolOption::BoolOption (string const & i, string const & n, sigc::slot<bool> g, sigc::slot<bool, bool> s)
@@ -287,9 +392,21 @@ void
 EntryOption::filter_text (const Glib::ustring&, int*)
 {
 	std::string text = _entry->get_text ();
+
+	if (!_valid.empty()) {
+		for (std::string::const_iterator t = text.begin(); t != text.end(); ) {
+			if (_valid.find_first_of (*t) == std::string::npos) {
+				t = text.erase (t);
+			} else {
+				++t;
+			}
+		}
+	}
+
 	for (size_t i = 0; i < _invalid.length(); ++i) {
 		text.erase (std::remove(text.begin(), text.end(), _invalid.at(i)), text.end());
 	}
+
 	if (text != _entry->get_text ()) {
 		_entry->set_text (text);
 	}
@@ -334,6 +451,13 @@ HSliderOption::HSliderOption (
 	_adj.set_value (_get());
 	_adj.signal_value_changed().connect (sigc::mem_fun (*this, &HSliderOption::changed));
 	_hscale.set_update_policy (Gtk::UPDATE_DISCONTINUOUS);
+
+	/* make the slider be a fixed, font-relative width */
+
+	_hscale.ensure_style ();
+	int width, height;
+	_hscale.create_pango_layout (X_("a piece of text that is as wide sliders should be"))->get_pixel_size (width, height);
+	_hscale.set_size_request (width, -1);
 }
 
 void
@@ -402,15 +526,15 @@ ComboStringOption::add_to_page (OptionEditorPage* p)
  */
 void
 ComboStringOption::set_popdown_strings (const std::vector<std::string>& strings) {
-	_combo->clear_items ();
+	_combo->remove_all ();
 	for (std::vector<std::string>::const_iterator i = strings.begin(); i != strings.end(); ++i) {
-		_combo->append_text (*i);
+		_combo->append (*i);
 	}
 }
 
 void
 ComboStringOption::clear () {
-	_combo->clear_items();
+	_combo->remove_all();
 }
 
 void
@@ -446,9 +570,9 @@ BoolComboOption::BoolComboOption (
 	_combo = manage (new ComboBoxText);
 
 	/* option 0 is the false option */
-	_combo->append_text (f);
+	_combo->append (f);
 	/* and option 1 is the true */
-	_combo->append_text (t);
+	_combo->append (t);
 
 	_combo->signal_changed().connect (sigc::mem_fun (*this, &BoolComboOption::changed));
 }
@@ -485,7 +609,7 @@ FaderOption::FaderOption (string const & i, string const & n, sigc::slot<gain_t>
 	, _get (g)
 	, _set (s)
 {
-	_db_slider = manage (new ArdourWidgets::HSliderController (&_db_adjustment, boost::shared_ptr<PBD::Controllable>(), 220, 18));
+	_db_slider = manage (new ArdourWidgets::HSliderController (&_db_adjustment, std::shared_ptr<PBD::Controllable>(), 220, 18));
 
 	_label.set_text (n + ":");
 	_label.set_alignment (0, 0.5);
@@ -574,22 +698,22 @@ void
 ClockOption::set_state_from_config ()
 {
 	Timecode::Time TC;
-	framepos_t when;
+	samplepos_t when;
 	if (!Timecode::parse_timecode_format(_get(), TC)) {
-		_clock.set (0, true);
+		_clock.set (timepos_t (0), true);
 	}
 	TC.rate = _session->samples_per_timecode_frame();
 	TC.drop = _session->timecode_drop_frames();
 	_session->timecode_to_sample(TC, when, false, false);
 	if (TC.negative) { when=-when; }
-	_clock.set (when, true);
+	_clock.set (timepos_t (when), true);
 }
 
 void
 ClockOption::save_clock_time ()
 {
 	Timecode::Time TC;
-	_session->sample_to_timecode(_clock.current_time(), TC, false, false);
+	_session->sample_to_timecode (_clock.last_when().samples(), TC, false, false);
 	_set (Timecode::timecode_format_time(TC));
 }
 
@@ -608,6 +732,20 @@ ClockOption::set_session (Session* s)
 
 /*--------------------------*/
 
+WidgetOption::WidgetOption (string const & i, string const & n, Gtk::Widget& w)
+	: Option (i, n)
+	, _widget (&w)
+{
+}
+
+void
+WidgetOption::add_to_page (OptionEditorPage* p)
+{
+	add_widget_to_page (p, _widget);
+}
+
+/*--------------------------*/
+
 OptionEditorPage::OptionEditorPage ()
 	: table (1, 3)
 {
@@ -654,6 +792,9 @@ OptionEditorMiniPage::add_to_page (OptionEditorPage* p)
 OptionEditor::OptionEditor (PBD::Configuration* c)
 	: _config (c)
 	, option_tree (TreeStore::create (option_columns))
+	, search_results (0)
+	, search_current_highlight (0)
+	, search_not_found_count (0)
 	, option_treeview (option_tree)
 {
 	using namespace Notebook_Helpers;
@@ -673,6 +814,18 @@ OptionEditor::OptionEditor (PBD::Configuration* c)
 
 	/* Watch out for changes to parameters */
 	_config->ParameterChanged.connect (config_connection, invalidator (*this), boost::bind (&OptionEditor::parameter_changed, this, _1), gui_context());
+
+	search_entry.show ();
+	search_entry.set_text (_("Search here..."));
+	search_entry.set_name (X_("ShadedEntry"));
+	set_size_request_to_display_given_text (search_entry, X_("a long enough search string"), 2, 2);
+	search_packer.pack_start (search_entry, true, true);
+	search_packer.show ();
+
+	search_entry.signal_activate().connect (sigc::mem_fun (*this, &OptionEditor::search));
+	search_entry.signal_key_press_event().connect (sigc::mem_fun (*this, &OptionEditor::search_key_press), false);
+	search_entry.signal_focus_in_event().connect (sigc::mem_fun (*this, &OptionEditor::search_key_focus), false);
+	search_entry.signal_focus_out_event().connect (sigc::mem_fun (*this, &OptionEditor::search_key_focus), false);
 }
 
 OptionEditor::~OptionEditor ()
@@ -685,6 +838,171 @@ OptionEditor::~OptionEditor ()
 	}
 }
 
+bool
+OptionEditor::search_key_focus (GdkEventFocus* ev)
+{
+	if (ev->in) {
+		if (search_entry.get_name () == X_("ShadedEntry")) {
+			search_entry.set_text ("");
+			search_entry.set_name (X_("GtkEntry"));
+		}
+	} else {
+		if (search_entry.get_text().empty() && search_entry.get_name () != X_("ShadedEntry")) {
+			search_entry.set_text (_("Search here..."));
+			search_entry.set_name (X_("ShadedEntry"));
+		}
+		if (search_current_highlight) {
+			search_current_highlight->end_highlight ();
+			search_current_highlight = 0;
+		}
+
+	}
+	return false;
+}
+
+bool
+OptionEditor::search_key_press (GdkEventKey* ev)
+{
+	if (search_entry.get_name () == X_("ShadedEntry")) {
+		search_entry.set_text ("");
+		search_entry.set_name (X_("GtkEntry"));
+	}
+
+	/* any key press should remove the current highlight, since something
+	 * is changing.
+	 */
+
+	if (search_current_highlight) {
+		search_current_highlight->end_highlight ();
+		search_current_highlight = 0;
+	}
+
+	return false;
+}
+
+void
+OptionEditor::search ()
+{
+	string search_for = search_entry.get_text();
+
+	not_found_callback ();
+
+	if (search_for.empty()) {
+		return;
+	}
+
+	if (!search_results || search_for != last_search_string) {
+
+		boost::char_separator<char> sep (" ");
+		typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+		tokenizer t (search_for, sep);
+
+		search_targets.clear ();
+		for (tokenizer::iterator ti = t.begin (); ti != t.end (); ++ti) {
+			string word = *ti;
+			transform (word.begin (), word.end (), word.begin (), ::toupper);
+			search_targets.push_back (word);
+		}
+
+		/* (re)build search results */
+
+		delete search_results;
+		search_results = new std::vector<SearchResult>;
+
+		for (auto p : pages()) {
+			for (auto oc : p.second->components) {
+				PBD::Configuration::Metadata const * metadata (oc->get_metadata());
+
+				if (!metadata) {
+					continue;
+				}
+
+				std::vector<SearchResult>::size_type found_cnt = 0;
+
+				for (auto const & s : search_targets) {
+					for (auto const & m : *metadata) {
+						if (m.find (s) != string::npos) {
+							found_cnt++;
+							break;
+						}
+					}
+				}
+
+				if (found_cnt == search_targets.size()) {
+					search_results->push_back (SearchResult (p.first, *oc));
+				}
+			}
+		}
+
+		if (search_results->empty()) {
+			not_found ();
+			delete search_results;
+			search_results = 0;
+			return;
+		}
+
+		last_search_string = search_for;
+		search_iterator = search_results->begin ();
+
+	} else {
+
+		/* have results and still searching for the same string. End
+		 * highlight of previous find (if not at end) and move on to
+		 * the next if we can.
+		 */
+
+		if (search_current_highlight) {
+			search_current_highlight->end_highlight ();
+			search_current_highlight = 0;
+		}
+
+		if (search_iterator != search_results->end()) {
+			++search_iterator;
+		}
+
+		if (search_iterator == search_results->end()) {
+
+			search_iterator = search_results->begin();
+			search_highlight ((*search_iterator).page_title, (*search_iterator).component);
+			search_iterator++;
+
+			return;
+		}
+	}
+
+	/* Move to next result, and highlight it */
+
+	search_highlight ((*search_iterator).page_title, (*search_iterator).component);
+}
+
+void
+OptionEditor::not_found ()
+{
+	search_entry.set_sensitive (false);
+	not_found_timeout = Glib::signal_timeout().connect (mem_fun (*this, &OptionEditor::not_found_callback), 250);
+	search_not_found_count++;
+}
+
+bool
+OptionEditor::not_found_callback ()
+{
+	search_entry.set_sensitive (true);
+	search_entry.grab_focus ();
+	search_not_found_count = 0;
+	return false;
+}
+
+void
+OptionEditor::search_highlight (std::string const & page_title, OptionEditorComponent& component)
+{
+	if (current_page() != page_title) {
+		set_current_page (page_title);
+	}
+	search_current_highlight = &component;
+	search_current_highlight->highlight ();
+}
+
+
 /** Called when a configuration parameter has been changed.
  *  @param p Parameter name.
  */
@@ -700,6 +1018,20 @@ OptionEditor::parameter_changed (std::string const & p)
 	}
 }
 
+std::string
+OptionEditor::current_page ()
+{
+	Glib::RefPtr<Gtk::TreeSelection> selection = option_treeview.get_selection();
+	TreeModel::const_iterator iter = selection->get_selected();
+
+	if (iter) {
+		TreeModel::Row row = *iter;
+		return row[option_columns.name];
+	}
+
+	return string();
+}
+
 void
 OptionEditor::treeview_row_selected ()
 {
@@ -833,14 +1165,17 @@ DirectoryOption::DirectoryOption (string const & i, string const & n, sigc::slot
 	, _get (g)
 	, _set (s)
 {
+	Gtkmm2ext::add_volume_shortcuts (_file_chooser);
 	_file_chooser.set_action (Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
-	_file_chooser.signal_selection_changed().connect (sigc::mem_fun (*this, &DirectoryOption::selection_changed));
+	_changed_connection = _file_chooser.signal_selection_changed().connect (sigc::mem_fun (*this, &DirectoryOption::selection_changed));
 }
 
 void
 DirectoryOption::set_state_from_config ()
 {
-	_file_chooser.set_current_folder (poor_mans_glob(_get ()));
+	_changed_connection.block ();
+	_file_chooser.set_filename (poor_mans_glob(_get ()));
+	_changed_connection.unblock ();
 }
 
 void
@@ -860,7 +1195,7 @@ DirectoryOption::selection_changed ()
 
 /*--------------------------*/
 
-OptionEditorContainer::OptionEditorContainer (PBD::Configuration* c, string const& str)
+OptionEditorContainer::OptionEditorContainer (PBD::Configuration* c)
 	: OptionEditor (c)
 {
 	set_border_width (4);
@@ -868,7 +1203,9 @@ OptionEditorContainer::OptionEditorContainer (PBD::Configuration* c, string cons
 	f->add (treeview());
 	f->set_shadow_type (Gtk::SHADOW_OUT);
 	f->set_border_width (0);
-	hpacker.pack_start (*f, false, false, 4);
+	treeview_packer.pack_start (*f, true, true);
+
+	hpacker.pack_start (treeview_packer, false, false, 4);
 	hpacker.pack_start (notebook(), false, false);
 	pack_start (hpacker, true, true);
 
@@ -879,18 +1216,19 @@ OptionEditorWindow::OptionEditorWindow (PBD::Configuration* c, string const& str
 	: OptionEditor (c)
 	, ArdourWindow (str)
 {
-	container.set_border_width (4);
+	hpacker.set_border_width (4);
 	Frame* f = manage (new Frame ());
+
 	f->add (treeview());
 	f->set_shadow_type (Gtk::SHADOW_OUT);
 	f->set_border_width (0);
-	hpacker.pack_start (*f, false, false);
+	vpacker.pack_start (*f, true, true);
+
+	hpacker.pack_start (vpacker, false, false);
 	hpacker.pack_start (notebook(), true, true, 4);
 
-	container.pack_start (hpacker, true, true);
-
 	hpacker.show_all ();
-	container.show ();
+	vpacker.show ();
 
-	add (container);
+	add (hpacker);
 }
