@@ -1,21 +1,25 @@
 /*
-    Copyright (C) 2001 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2018 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2010-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2014-2015 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cmath>
 
@@ -45,7 +49,7 @@ using namespace PBD;
 using namespace std;
 using namespace Gtkmm2ext;
 
-RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
+RegionEditor::RegionEditor (Session* s, std::shared_ptr<Region> r)
 	: ArdourDialog (_("Region"))
 	, _table (9, 2)
 	, _table_row (0)
@@ -64,7 +68,9 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 {
 	set_session (s);
 
-	_clock_group->set_clock_mode (ARDOUR_UI::instance()->secondary_clock->mode());
+	_clock_group->set_clock_mode (ARDOUR_UI::instance()->primary_clock->mode());
+	ARDOUR_UI::instance()->primary_clock->mode_changed.connect (sigc::mem_fun (*this, &RegionEditor::set_clock_mode_from_primary));
+
 	_clock_group->add (position_clock);
 	_clock_group->add (end_clock);
 	_clock_group->add (length_clock);
@@ -81,7 +87,7 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 
 	ArdourWidgets::set_tooltip (audition_button, _("audition this region"));
 
-	audition_button.unset_flags (Gtk::CAN_FOCUS);
+	audition_button.set_can_focus (false);
 
 	audition_button.set_events (audition_button.get_events() & ~(Gdk::ENTER_NOTIFY_MASK|Gdk::LEAVE_NOTIFY_MASK));
 
@@ -101,7 +107,7 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 	start_label.set_text (_("File start:"));
 	_sources_label.set_name ("RegionEditorLabel");
 
-	if (_region->n_channels() > 1) {
+	if (_region->sources().size() > 1) {
 		_sources_label.set_text (_("Sources:"));
 	} else {
 		_sources_label.set_text (_("Source:"));
@@ -133,24 +139,24 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 	_table.attach (position_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
- 	_table.attach (end_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
+	_table.attach (end_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
 	_table.attach (end_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
- 	_table.attach (length_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
+	_table.attach (length_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
 	_table.attach (length_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
- 	_table.attach (sync_relative_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
- 	_table.attach (sync_offset_relative_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
+	_table.attach (sync_relative_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
+	_table.attach (sync_offset_relative_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
- 	_table.attach (sync_absolute_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
- 	_table.attach (sync_offset_absolute_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
+	_table.attach (sync_absolute_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
+	_table.attach (sync_offset_absolute_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
- 	_table.attach (start_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
- 	_table.attach (start_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
+	_table.attach (start_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
+	_table.attach (start_clock, 1, 2, _table_row, _table_row + 1, Gtk::FILL | Gtk::EXPAND, Gtk::FILL);
 	++_table_row;
 
 	_table.attach (_sources_label, 0, 1, _table_row, _table_row + 1, Gtk::FILL, Gtk::FILL);
@@ -168,8 +174,8 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 
 	set_title (string_compose (_("Region '%1'"), _region->name()));
 
-	for (uint32_t i = 0; i < _region->n_channels(); ++i) {
-		_sources.append_text (_region->source(i)->name());
+	for (uint32_t i = 0; i < _region->sources().size(); ++i) {
+		_sources.append (_region->source(i)->name());
 	}
 
 	_sources.set_headers_visible (false);
@@ -185,7 +191,6 @@ RegionEditor::RegionEditor (Session* s, boost::shared_ptr<Region> r)
 
 	change.add (ARDOUR::Properties::start);
 	change.add (ARDOUR::Properties::length);
-	change.add (ARDOUR::Properties::position);
 	change.add (ARDOUR::Properties::sync_position);
 
 	bounds_changed (change);
@@ -202,6 +207,12 @@ RegionEditor::~RegionEditor ()
 	delete _clock_group;
 }
 
+void
+RegionEditor::set_clock_mode_from_primary ()
+{
+	_clock_group->set_clock_mode (ARDOUR_UI::instance()->primary_clock->mode());
+}
+
 void
 RegionEditor::region_changed (const PBD::PropertyChange& what_changed)
 {
@@ -211,7 +222,6 @@ RegionEditor::region_changed (const PBD::PropertyChange& what_changed)
 
 	PropertyChange interesting_stuff;
 
-	interesting_stuff.add (ARDOUR::Properties::position);
 	interesting_stuff.add (ARDOUR::Properties::length);
 	interesting_stuff.add (ARDOUR::Properties::start);
 	interesting_stuff.add (ARDOUR::Properties::sync_position);
@@ -274,14 +284,14 @@ void
 RegionEditor::position_clock_changed ()
 {
 	bool in_command = false;
-	boost::shared_ptr<Playlist> pl = _region->playlist();
+	std::shared_ptr<Playlist> pl = _region->playlist();
 
 	if (pl) {
 		PublicEditor::instance().begin_reversible_command (_("change region start position"));
 		in_command = true;
 
 		_region->clear_changes ();
-		_region->set_position (position_clock.current_time());
+		_region->set_position (position_clock.last_when());
 		_session->add_command(new StatefulDiffCommand (_region));
 	}
 
@@ -294,14 +304,14 @@ void
 RegionEditor::end_clock_changed ()
 {
 	bool in_command = false;
-	boost::shared_ptr<Playlist> pl = _region->playlist();
+	std::shared_ptr<Playlist> pl = _region->playlist();
 
 	if (pl) {
 		PublicEditor::instance().begin_reversible_command (_("change region end position"));
 		in_command = true;
 
 		_region->clear_changes ();
-		_region->trim_end (end_clock.current_time());
+		_region->trim_end (end_clock.last_when());
 		_session->add_command(new StatefulDiffCommand (_region));
 	}
 
@@ -309,22 +319,26 @@ RegionEditor::end_clock_changed ()
 		PublicEditor::instance().commit_reversible_command ();
 	}
 
-	end_clock.set (_region->position() + _region->length() - 1, true);
+	end_clock.set (_region->nt_last(), true);
 }
 
 void
 RegionEditor::length_clock_changed ()
 {
-	framecnt_t frames = length_clock.current_time();
+	timecnt_t len = length_clock.current_duration();
 	bool in_command = false;
-	boost::shared_ptr<Playlist> pl = _region->playlist();
+	std::shared_ptr<Playlist> pl = _region->playlist();
 
 	if (pl) {
 		PublicEditor::instance().begin_reversible_command (_("change region length"));
 		in_command = true;
 
 		_region->clear_changes ();
-		_region->trim_end (_region->position() + frames - 1);
+		/* new end is actually 1 domain unit before the clock duration
+		 * would otherwise indicate
+		 */
+		const timepos_t new_end = (_region->position() + len).decrement ();
+		_region->trim_end (new_end);
 		_session->add_command(new StatefulDiffCommand (_region));
 	}
 
@@ -332,7 +346,7 @@ RegionEditor::length_clock_changed ()
 		PublicEditor::instance().commit_reversible_command ();
 	}
 
-	length_clock.set (_region->length());
+	length_clock.set_duration (_region->length());
 }
 
 void
@@ -356,34 +370,28 @@ RegionEditor::name_changed ()
 void
 RegionEditor::bounds_changed (const PropertyChange& what_changed)
 {
-	if (what_changed.contains (ARDOUR::Properties::position) && what_changed.contains (ARDOUR::Properties::length)) {
+	if (what_changed.contains (ARDOUR::Properties::length)) {
 		position_clock.set (_region->position(), true);
-		end_clock.set (_region->position() + _region->length() - 1, true);
-		length_clock.set (_region->length(), true);
-	} else if (what_changed.contains (ARDOUR::Properties::position)) {
-		position_clock.set (_region->position(), true);
-		end_clock.set (_region->position() + _region->length() - 1, true);
-	} else if (what_changed.contains (ARDOUR::Properties::length)) {
-		end_clock.set (_region->position() + _region->length() - 1, true);
-		length_clock.set (_region->length(), true);
+		end_clock.set (_region->nt_last(), true);
+		length_clock.set_duration (_region->length(), true);
 	}
 
-	if (what_changed.contains (ARDOUR::Properties::sync_position) || what_changed.contains (ARDOUR::Properties::position)) {
+	if (what_changed.contains (ARDOUR::Properties::sync_position) || what_changed.contains (ARDOUR::Properties::length)) {
 		int dir;
-		frameoffset_t off = _region->sync_offset (dir);
+		timecnt_t off = _region->sync_offset (dir);
 		if (dir == -1) {
 			off = -off;
 		}
 
 		if (what_changed.contains (ARDOUR::Properties::sync_position)) {
-			sync_offset_relative_clock.set (off, true);
+			sync_offset_relative_clock.set_duration (off, true);
 		}
 
-		sync_offset_absolute_clock.set (off + _region->position (), true);
+		sync_offset_absolute_clock.set (_region->position () + off, true);
 	}
 
 	if (what_changed.contains (ARDOUR::Properties::start)) {
-		start_clock.set (_region->start(), true);
+		start_clock.set (timepos_t (_region->start()), true);
 	}
 }
 
@@ -417,7 +425,7 @@ RegionEditor::sync_offset_absolute_clock_changed ()
 	PublicEditor::instance().begin_reversible_command (_("change region sync point"));
 
 	_region->clear_changes ();
-	_region->set_sync_position (sync_offset_absolute_clock.current_time());
+	_region->set_sync_position (sync_offset_absolute_clock.last_when());
 	_session->add_command (new StatefulDiffCommand (_region));
 
 	PublicEditor::instance().commit_reversible_command ();
@@ -429,7 +437,7 @@ RegionEditor::sync_offset_relative_clock_changed ()
 	PublicEditor::instance().begin_reversible_command (_("change region sync point"));
 
 	_region->clear_changes ();
-	_region->set_sync_position (sync_offset_relative_clock.current_time() + _region->position ());
+	_region->set_sync_position (sync_offset_relative_clock.last_when() + _region->position ());
 	_session->add_command (new StatefulDiffCommand (_region));
 
 	PublicEditor::instance().commit_reversible_command ();
@@ -442,7 +450,6 @@ RegionEditor::on_delete_event (GdkEventAny*)
 
 	change.add (ARDOUR::Properties::start);
 	change.add (ARDOUR::Properties::length);
-	change.add (ARDOUR::Properties::position);
 	change.add (ARDOUR::Properties::sync_position);
 
 	bounds_changed (change);
