@@ -1,27 +1,32 @@
 /*
-    Copyright (C) 2002 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2006-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2014-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2014-2018 Ben Loftis <ben@harrisonconsoles.com>
+ * Copyright (C) 2014-2018 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2017 Tim Mayberry <mojofunk@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <algorithm>
 #include <sigc++/bind.h>
 
 #include "pbd/error.h"
-#include "pbd/stacktrace.h"
 #include "pbd/types_convert.h"
 
 #include "ardour/evoral_types_convert.h"
@@ -43,6 +48,7 @@
 #include "automation_time_axis.h"
 #include "public_editor.h"
 #include "control_point.h"
+#include "triggerbox_ui.h"
 #include "vca_time_axis.h"
 
 #include "pbd/i18n.h"
@@ -51,15 +57,14 @@ using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
 
-struct AudioRangeComparator {
-	bool operator()(AudioRange a, AudioRange b) {
-		return a.start < b.start;
+struct TimelineRangeComparator {
+	bool operator()(TimelineRange a, TimelineRange b) {
+		return a.start() < b.start();
 	}
 };
 
 Selection::Selection (const PublicEditor* e, bool mls)
-	: tracks (e)
-	, editor (e)
+	: editor (e)
 	, next_time_id (0)
 	, manage_libardour_selection (mls)
 {
@@ -98,8 +103,7 @@ operator== (const Selection& a, const Selection& b)
 		a.time == b.time &&
 		a.lines == b.lines &&
 		a.playlists == b.playlists &&
-		a.midi_notes == b.midi_notes &&
-		a.midi_regions == b.midi_regions;
+		a.midi_notes == b.midi_notes;
 }
 
 /** Clear everything from the Selection */
@@ -113,8 +117,8 @@ Selection::clear ()
 	clear_time ();
 	clear_playlists ();
 	clear_midi_notes ();
-	clear_midi_regions ();
 	clear_markers ();
+	clear_triggers ();
 	pending_midi_note_selection.clear();
 }
 
@@ -126,7 +130,7 @@ Selection::clear_objects (bool with_signal)
 	clear_lines(with_signal);
 	clear_playlists (with_signal);
 	clear_midi_notes (with_signal);
-	clear_midi_regions (with_signal);
+	clear_triggers (with_signal);
 }
 
 void
@@ -162,6 +166,10 @@ Selection::clear_regions (bool with_signal)
 void
 Selection::clear_midi_notes (bool with_signal)
 {
+	/* Remember: MIDI notes are only stored here if we're using a Selection
+	   object as a cut buffer.
+	*/
+
 	if (!midi_notes.empty()) {
 		for (MidiNoteSelection::iterator x = midi_notes.begin(); x != midi_notes.end(); ++x) {
 			delete *x;
@@ -171,30 +179,6 @@ Selection::clear_midi_notes (bool with_signal)
 			MidiNotesChanged ();
 		}
 	}
-
-	// clear note selections for MRV's that have note selections
-	// this will cause the MRV to be removed from the list
-	for (MidiRegionSelection::iterator i = midi_regions.begin();
-	     i != midi_regions.end();) {
-		MidiRegionSelection::iterator tmp = i;
-		++tmp;
-		MidiRegionView* mrv = dynamic_cast<MidiRegionView*>(*i);
-		if (mrv) {
-			mrv->clear_selection();
-		}
-		i = tmp;
-	}
-}
-
-void
-Selection::clear_midi_regions (bool with_signal)
-{
-	if (!midi_regions.empty()) {
-		midi_regions.clear ();
-		if (with_signal) {
-			MidiRegionsChanged ();
-		}
-	}
 }
 
 void
@@ -239,10 +223,29 @@ Selection::clear_markers (bool with_signal)
 }
 
 void
-Selection::toggle (boost::shared_ptr<Playlist> pl)
+Selection::clear_triggers (bool with_signal)
 {
-	clear_time();  //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	if (!triggers.empty()) {
+		triggers.clear ();
+		if (with_signal) {
+			TriggersChanged ();
+		}
+	}
+}
+
+RegionSelection
+Selection::trigger_regionview_proxy () const
+{
+	RegionSelection rs;
+	return rs;
+}
+
+void
+Selection::toggle (std::shared_ptr<Playlist> pl)
+{
+	clear_time(); // enforce object/range exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	PlaylistSelection::iterator i;
 
@@ -259,8 +262,9 @@ Selection::toggle (boost::shared_ptr<Playlist> pl)
 void
 Selection::toggle (const MidiNoteSelection& midi_note_list)
 {
-	clear_time();  //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce object/range exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	for (MidiNoteSelection::const_iterator i = midi_note_list.begin(); i != midi_note_list.end(); ++i) {
 		toggle ((*i));
@@ -287,8 +291,9 @@ Selection::toggle (MidiCutBuffer* midi)
 void
 Selection::toggle (RegionView* r)
 {
-	clear_time();  //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce object/range exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	RegionSelection::iterator i;
 
@@ -301,28 +306,12 @@ Selection::toggle (RegionView* r)
 	RegionsChanged ();
 }
 
-void
-Selection::toggle (MidiRegionView* mrv)
-{
-	clear_time();   //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
-
-	MidiRegionSelection::iterator i;
-
-	if ((i = find (midi_regions.begin(), midi_regions.end(), mrv)) == midi_regions.end()) {
-		add (mrv);
-	} else {
-		midi_regions.erase (i);
-	}
-
-	MidiRegionsChanged ();
-}
-
 void
 Selection::toggle (vector<RegionView*>& r)
 {
-	clear_time();  //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce object/range exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	RegionSelection::iterator i;
 
@@ -338,15 +327,15 @@ Selection::toggle (vector<RegionView*>& r)
 }
 
 long
-Selection::toggle (framepos_t start, framepos_t end)
+Selection::toggle (timepos_t const & start, timepos_t const & end)
 {
-	clear_objects();  //enforce object/range exclusivity
+	clear_objects(); // enforce object/range exclusivity
 
-	AudioRangeComparator cmp;
+	TimelineRangeComparator cmp;
 
 	/* XXX this implementation is incorrect */
 
-	time.push_back (AudioRange (start, end, ++next_time_id));
+	time.push_back (TimelineRange (start, end, ++next_time_id));
 	time.consolidate ();
 	time.sort (cmp);
 
@@ -356,12 +345,13 @@ Selection::toggle (framepos_t start, framepos_t end)
 }
 
 void
-Selection::add (boost::shared_ptr<Playlist> pl)
+Selection::add (std::shared_ptr<Playlist> pl)
 {
 
 	if (find (playlists.begin(), playlists.end(), pl) == playlists.end()) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 		pl->use ();
 		playlists.push_back(pl);
 		PlaylistsChanged ();
@@ -369,11 +359,11 @@ Selection::add (boost::shared_ptr<Playlist> pl)
 }
 
 void
-Selection::add (const list<boost::shared_ptr<Playlist> >& pllist)
+Selection::add (const list<std::shared_ptr<Playlist> >& pllist)
 {
 	bool changed = false;
 
-	for (list<boost::shared_ptr<Playlist> >::const_iterator i = pllist.begin(); i != pllist.end(); ++i) {
+	for (list<std::shared_ptr<Playlist> >::const_iterator i = pllist.begin(); i != pllist.end(); ++i) {
 		if (find (playlists.begin(), playlists.end(), (*i)) == playlists.end()) {
 			(*i)->use ();
 			playlists.push_back (*i);
@@ -382,8 +372,9 @@ Selection::add (const list<boost::shared_ptr<Playlist> >& pllist)
 	}
 
 	if (changed) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 		PlaylistsChanged ();
 	}
 }
@@ -395,8 +386,9 @@ Selection::add (const MidiNoteSelection& midi_list)
 	const MidiNoteSelection::const_iterator e = midi_list.end();
 
 	if (!midi_list.empty()) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 		midi_notes.insert (midi_notes.end(), b, e);
 		MidiNotesChanged ();
 	}
@@ -428,8 +420,9 @@ Selection::add (vector<RegionView*>& v)
 	}
 
 	if (changed) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers ();
 		RegionsChanged ();
 	}
 }
@@ -449,8 +442,9 @@ Selection::add (const RegionSelection& rs)
 	}
 
 	if (changed) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers ();
 		RegionsChanged ();
 	}
 }
@@ -461,38 +455,24 @@ Selection::add (RegionView* r)
 	if (find (regions.begin(), regions.end(), r) == regions.end()) {
 		bool changed = regions.add (r);
 		if (changed) {
-			clear_time();  //enforce object/range exclusivity
-			clear_tracks();  //enforce object/track exclusivity
+			clear_time(); // enforce object/range exclusivity
+			clear_tracks(); // enforce object/track exclusivity
+			clear_triggers ();
 			RegionsChanged ();
 		}
 	}
 }
 
-void
-Selection::add (MidiRegionView* mrv)
-{
-	DEBUG_TRACE(DEBUG::Selection, string_compose("Selection::add MRV %1\n", mrv));
-
-	clear_time();  //enforce object/range exclusivity
-	clear_tracks();  //enforce object/track exclusivity
-
-	if (find (midi_regions.begin(), midi_regions.end(), mrv) == midi_regions.end()) {
-		midi_regions.push_back (mrv);
-		/* XXX should we do this? */
-		MidiRegionsChanged ();
-	}
-}
-
 long
-Selection::add (framepos_t start, framepos_t end)
+Selection::add (timepos_t const & start, timepos_t const & end)
 {
-	clear_objects();  //enforce object/range exclusivity
+	clear_objects(); // enforce object/range exclusivity
 
-	AudioRangeComparator cmp;
+	TimelineRangeComparator cmp;
 
 	/* XXX this implementation is incorrect */
 
-	time.push_back (AudioRange (start, end, ++next_time_id));
+	time.push_back (TimelineRange (start, end, ++next_time_id));
 	time.consolidate ();
 	time.sort (cmp);
 
@@ -502,34 +482,34 @@ Selection::add (framepos_t start, framepos_t end)
 }
 
 void
-Selection::move_time (framecnt_t distance)
+Selection::move_time (timecnt_t const & distance)
 {
-	if (distance == 0) {
+	if (distance.is_zero ()) {
 		return;
 	}
 
-	for (list<AudioRange>::iterator i = time.begin(); i != time.end(); ++i) {
-		(*i).start += distance;
-		(*i).end += distance;
+	for (list<TimelineRange>::iterator i = time.begin(); i != time.end(); ++i) {
+		(*i).start() += distance;
+		(*i).end() += distance;
 	}
 
 	TimeChanged ();
 }
 
 void
-Selection::replace (uint32_t sid, framepos_t start, framepos_t end)
+Selection::replace (uint32_t sid, timepos_t const & start, timepos_t const & end)
 {
-	clear_objects();  //enforce object/range exclusivity
+	clear_objects(); // enforce object/range exclusivity
 
-	for (list<AudioRange>::iterator i = time.begin(); i != time.end(); ++i) {
+	for (list<TimelineRange>::iterator i = time.begin(); i != time.end(); ++i) {
 		if ((*i).id == sid) {
 			time.erase (i);
-			time.push_back (AudioRange(start,end, sid));
+			time.push_back (TimelineRange (start,end, sid));
 
 			/* don't consolidate here */
 
 
-			AudioRangeComparator cmp;
+			TimelineRangeComparator cmp;
 			time.sort (cmp);
 
 			TimeChanged ();
@@ -539,9 +519,9 @@ Selection::replace (uint32_t sid, framepos_t start, framepos_t end)
 }
 
 void
-Selection::add (boost::shared_ptr<Evoral::ControlList> cl)
+Selection::add (std::shared_ptr<Evoral::ControlList> cl)
 {
-	boost::shared_ptr<ARDOUR::AutomationList> al = boost::dynamic_pointer_cast<ARDOUR::AutomationList>(cl);
+	std::shared_ptr<ARDOUR::AutomationList> al = std::dynamic_pointer_cast<ARDOUR::AutomationList>(cl);
 
 	if (!al) {
 		warning << "Programming error: Selected list is not an ARDOUR::AutomationList" << endmsg;
@@ -549,16 +529,17 @@ Selection::add (boost::shared_ptr<Evoral::ControlList> cl)
 	}
 
 	if (!cl->empty()) {
-		clear_time();  //enforce object/range exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce object/range exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 	}
 
 	/* The original may change so we must store a copy (not a pointer) here.
 	 * e.g AutomationLine rewrites the list with gain mapping.
-	 * the downside is that we can't perfom duplicate checks.
+	 * the downside is that we can't perform duplicate checks.
 	 * This code was changed in response to #6842
 	 */
-	lines.push_back (boost::shared_ptr<ARDOUR::AutomationList> (new ARDOUR::AutomationList(*al)));
+	lines.push_back (std::shared_ptr<ARDOUR::AutomationList> (new ARDOUR::AutomationList(*al)));
 	LinesChanged();
 }
 
@@ -605,9 +586,9 @@ Selection::remove (MidiCutBuffer* midi)
 }
 
 void
-Selection::remove (boost::shared_ptr<Playlist> track)
+Selection::remove (std::shared_ptr<Playlist> track)
 {
-	list<boost::shared_ptr<Playlist> >::iterator i;
+	list<std::shared_ptr<Playlist> >::iterator i;
 	if ((i = find (playlists.begin(), playlists.end(), track)) != playlists.end()) {
 		playlists.erase (i);
 		PlaylistsChanged();
@@ -615,13 +596,13 @@ Selection::remove (boost::shared_ptr<Playlist> track)
 }
 
 void
-Selection::remove (const list<boost::shared_ptr<Playlist> >& pllist)
+Selection::remove (const list<std::shared_ptr<Playlist> >& pllist)
 {
 	bool changed = false;
 
-	for (list<boost::shared_ptr<Playlist> >::const_iterator i = pllist.begin(); i != pllist.end(); ++i) {
+	for (list<std::shared_ptr<Playlist> >::const_iterator i = pllist.begin(); i != pllist.end(); ++i) {
 
-		list<boost::shared_ptr<Playlist> >::iterator x;
+		list<std::shared_ptr<Playlist> >::iterator x;
 
 		if ((x = find (playlists.begin(), playlists.end(), (*i))) != playlists.end()) {
 			playlists.erase (x);
@@ -643,15 +624,10 @@ Selection::remove (RegionView* r)
 }
 
 void
-Selection::remove (MidiRegionView* mrv)
+Selection::remove (vector<RegionView*> rv)
 {
-	DEBUG_TRACE(DEBUG::Selection, string_compose("Selection::remove MRV %1\n", mrv));
-
-	MidiRegionSelection::iterator x;
-
-	if ((x = find (midi_regions.begin(), midi_regions.end(), mrv)) != midi_regions.end()) {
-		midi_regions.erase (x);
-		MidiRegionsChanged ();
+	if (regions.remove (rv)) {
+		RegionsChanged ();
 	}
 }
 
@@ -662,7 +638,7 @@ Selection::remove (uint32_t selection_id)
 		return;
 	}
 
-	for (list<AudioRange>::iterator i = time.begin(); i != time.end(); ++i) {
+	for (list<TimelineRange>::iterator i = time.begin(); i != time.end(); ++i) {
 		if ((*i).id == selection_id) {
 			time.erase (i);
 
@@ -673,12 +649,12 @@ Selection::remove (uint32_t selection_id)
 }
 
 void
-Selection::remove (framepos_t /*start*/, framepos_t /*end*/)
+Selection::remove (samplepos_t /*start*/, samplepos_t /*end*/)
 {
 }
 
 void
-Selection::remove (boost::shared_ptr<ARDOUR::AutomationList> ac)
+Selection::remove (std::shared_ptr<ARDOUR::AutomationList> ac)
 {
 	AutomationSelection::iterator i;
 	if ((i = find (lines.begin(), lines.end(), ac)) != lines.end()) {
@@ -691,29 +667,31 @@ void
 Selection::set (const MidiNoteSelection& midi_list)
 {
 	if (!midi_list.empty()) {
-		clear_time ();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time (); // enforce region/object exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 	}
 	clear_objects ();
 	add (midi_list);
 }
 
 void
-Selection::set (boost::shared_ptr<Playlist> playlist)
+Selection::set (std::shared_ptr<Playlist> playlist)
 {
 	if (playlist) {
-		clear_time ();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time (); // enforce region/object exclusivity
+		clear_tracks(); // enforce object/track exclusivity
+		clear_triggers(); // enforce trigger exclusivity
 	}
 	clear_objects ();
 	add (playlist);
 }
 
 void
-Selection::set (const list<boost::shared_ptr<Playlist> >& pllist)
+Selection::set (const list<std::shared_ptr<Playlist> >& pllist)
 {
 	if (!pllist.empty()) {
-		clear_time();  //enforce region/object exclusivity
+		clear_time(); // enforce region/object exclusivity
 	}
 	clear_objects ();
 	add (pllist);
@@ -723,31 +701,20 @@ void
 Selection::set (const RegionSelection& rs)
 {
 	if (!rs.empty()) {
-		clear_time();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce region/object exclusivity
+		clear_tracks(); // enforce object/track exclusivity
 	}
 	clear_objects();
 	regions = rs;
 	RegionsChanged(); /* EMIT SIGNAL */
 }
 
-void
-Selection::set (MidiRegionView* mrv)
-{
-	if (mrv) {
-		clear_time();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
-	}
-	clear_objects ();
-	add (mrv);
-}
-
 void
 Selection::set (RegionView* r, bool /*also_clear_tracks*/)
 {
 	if (r) {
-		clear_time();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce region/object exclusivity
+		clear_tracks(); // enforce object/track exclusivity
 	}
 	clear_objects ();
 	add (r);
@@ -757,8 +724,8 @@ void
 Selection::set (vector<RegionView*>& v)
 {
 	if (!v.empty()) {
-		clear_time();  //enforce region/object exclusivity
-		clear_tracks();  //enforce object/track exclusivity
+		clear_time(); // enforce region/object exclusivity
+		clear_tracks(); // enforce object/track exclusivity
 	}
 
 	clear_objects();
@@ -770,25 +737,25 @@ Selection::set (vector<RegionView*>& v)
  *  the list of tracks it applies to.
  */
 long
-Selection::set (framepos_t start, framepos_t end)
+Selection::set (timepos_t const & start, timepos_t const & end)
 {
-	clear_objects();  //enforce region/object exclusivity
+	clear_objects(); // enforce region/object exclusivity
 	clear_time();
 
-	if ((start == 0 && end == 0) || end < start) {
+	if ((start.is_zero () && end.is_zero ()) || end < start) {
 		return 0;
 	}
 
 	if (time.empty()) {
-		time.push_back (AudioRange (start, end, ++next_time_id));
+		time.push_back (TimelineRange (start, end, ++next_time_id));
 	} else {
 		/* reuse the first entry, and remove all the rest */
 
 		while (time.size() > 1) {
 			time.pop_front();
 		}
-		time.front().start = start;
-		time.front().end = end;
+		time.front().start() = start;
+		time.front().end() = end;
 	}
 
 	time.consolidate ();
@@ -807,20 +774,20 @@ Selection::set (framepos_t start, framepos_t end)
  *  @param end New end time.
  */
 void
-Selection::set_preserving_all_ranges (framepos_t start, framepos_t end)
+Selection::set_preserving_all_ranges (timepos_t const & start, timepos_t const & end)
 {
-	clear_objects();  //enforce region/object exclusivity
+	clear_objects(); // enforce region/object exclusivity
 
-	if ((start == 0 && end == 0) || (end < start)) {
+	if ((start.is_zero () && end.is_zero ()) || (end < start)) {
 		return;
 	}
 
 	if (time.empty ()) {
-		time.push_back (AudioRange (start, end, ++next_time_id));
+		time.push_back (TimelineRange (start, end, ++next_time_id));
 	} else {
-		time.sort (AudioRangeComparator ());
-		time.front().start = start;
-		time.back().end = end;
+		time.sort (TimelineRangeComparator ());
+		time.front().set_start (start);
+		time.back().set_end (end);
 	}
 
 	time.consolidate ();
@@ -829,10 +796,10 @@ Selection::set_preserving_all_ranges (framepos_t start, framepos_t end)
 }
 
 void
-Selection::set (boost::shared_ptr<Evoral::ControlList> ac)
+Selection::set (std::shared_ptr<Evoral::ControlList> ac)
 {
-	clear_time();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
 	clear_objects();
 
 	add (ac);
@@ -859,7 +826,7 @@ Selection::selected (ControlPoint* cp) const
 bool
 Selection::empty (bool internal_selection)
 {
-	bool object_level_empty =  regions.empty () &&
+	bool object_level_empty = regions.empty () &&
 		tracks.empty () &&
 		points.empty () &&
 		playlists.empty () &&
@@ -867,7 +834,7 @@ Selection::empty (bool internal_selection)
 		time.empty () &&
 		playlists.empty () &&
 		markers.empty() &&
-		midi_regions.empty()
+		triggers.empty()
 		;
 
 	if (!internal_selection) {
@@ -884,8 +851,9 @@ Selection::empty (bool internal_selection)
 void
 Selection::toggle (ControlPoint* cp)
 {
-	clear_time();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	cp->set_selected (!cp->selected ());
 	PointSelection::iterator i = find (points.begin(), points.end(), cp);
@@ -901,8 +869,9 @@ Selection::toggle (ControlPoint* cp)
 void
 Selection::toggle (vector<ControlPoint*> const & cps)
 {
-	clear_time();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	for (vector<ControlPoint*>::const_iterator i = cps.begin(); i != cps.end(); ++i) {
 		toggle (*i);
@@ -912,8 +881,9 @@ Selection::toggle (vector<ControlPoint*> const & cps)
 void
 Selection::toggle (list<Selectable*> const & selectables)
 {
-	clear_time();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time(); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	RegionView* rv;
 	ControlPoint* cp;
@@ -945,8 +915,8 @@ Selection::toggle (list<Selectable*> const & selectables)
 void
 Selection::set (list<Selectable*> const & selectables)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
 	clear_objects ();
 
 	add (selectables);
@@ -955,8 +925,9 @@ Selection::set (list<Selectable*> const & selectables)
 void
 Selection::add (PointSelection const & s)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	for (PointSelection::const_iterator i = s.begin(); i != s.end(); ++i) {
 		points.push_back (*i);
@@ -966,8 +937,9 @@ Selection::add (PointSelection const & s)
 void
 Selection::add (list<Selectable*> const & selectables)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	RegionView* rv;
 	ControlPoint* cp;
@@ -1010,8 +982,9 @@ Selection::clear_points (bool with_signal)
 void
 Selection::add (ControlPoint* cp)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	cp->set_selected (true);
 	points.push_back (cp);
@@ -1021,8 +994,9 @@ Selection::add (ControlPoint* cp)
 void
 Selection::add (vector<ControlPoint*> const & cps)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	for (vector<ControlPoint*>::const_iterator i = cps.begin(); i != cps.end(); ++i) {
 		(*i)->set_selected (true);
@@ -1034,8 +1008,9 @@ Selection::add (vector<ControlPoint*> const & cps)
 void
 Selection::set (ControlPoint* cp)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	if (cp->selected () && points.size () == 1) {
 		return;
@@ -1052,8 +1027,9 @@ Selection::set (ControlPoint* cp)
 void
 Selection::set (ArdourMarker* m)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time ();  // enforce region/object exclusivity
+	clear_tracks();  // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 	markers.clear ();
 
 	add (m);
@@ -1085,8 +1061,9 @@ Selection::remove (ArdourMarker* m)
 void
 Selection::add (ArdourMarker* m)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); //enforce region/object exclusivity
+	clear_tracks(); //enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	if (find (markers.begin(), markers.end(), m) == markers.end()) {
 		markers.push_back (m);
@@ -1097,8 +1074,9 @@ Selection::add (ArdourMarker* m)
 void
 Selection::add (const list<ArdourMarker*>& m)
 {
-	clear_time ();  //enforce region/object exclusivity
-	clear_tracks();  //enforce object/track exclusivity
+	clear_time (); // enforce region/object exclusivity
+	clear_tracks(); // enforce object/track exclusivity
+	clear_triggers(); // enforce trigger exclusivity
 
 	markers.insert (markers.end(), m.begin(), m.end());
 	markers.sort ();
@@ -1108,10 +1086,16 @@ Selection::add (const list<ArdourMarker*>& m)
 }
 
 void
-MarkerSelection::range (framepos_t& s, framepos_t& e)
+MarkerSelection::range (timepos_t& s, timepos_t& e)
 {
-	s = max_framepos;
-	e = 0;
+	if (empty()) {
+		s = timepos_t::zero (Temporal::AudioTime);
+		e = timepos_t::zero (Temporal::AudioTime);
+		return;
+	}
+
+	s = timepos_t::max (front()->position().time_domain());
+	e = timepos_t::zero (front()->position().time_domain());
 
 	for (MarkerSelection::iterator i = begin(); i != end(); ++i) {
 
@@ -1147,6 +1131,7 @@ Selection::get_state () const
 			XMLNode* t = node->add_child (X_("AutomationView"));
 			t->set_property (X_("id"), atv->parent_stripable()->id ());
 			t->set_property (X_("parameter"), EventTypeMap::instance().to_symbol (atv->parameter ()));
+			t->set_property (X_("ctrl_id"), atv->control()->id());
 		}
 	}
 
@@ -1159,15 +1144,15 @@ Selection::get_state () const
 	}
 
 	/* midi region views have thir own internal selection. */
-	list<pair<PBD::ID, std::set<boost::shared_ptr<Evoral::Note<Evoral::Beats> > > > > rid_notes;
+	list<pair<PBD::ID, std::set<std::shared_ptr<Evoral::Note<Temporal::Beats> > > > > rid_notes;
 	editor->get_per_region_note_selection (rid_notes);
 
-	list<pair<PBD::ID, std::set<boost::shared_ptr<Evoral::Note<Evoral::Beats> > > > >::iterator rn_it;
+	list<pair<PBD::ID, std::set<std::shared_ptr<Evoral::Note<Temporal::Beats> > > > >::iterator rn_it;
 	for (rn_it = rid_notes.begin(); rn_it != rid_notes.end(); ++rn_it) {
 		XMLNode* n = node->add_child (X_("MIDINotes"));
 		n->set_property (X_("region-id"), (*rn_it).first);
 
-		for (std::set<boost::shared_ptr<Evoral::Note<Evoral::Beats> > >::iterator i = (*rn_it).second.begin(); i != (*rn_it).second.end(); ++i) {
+		for (std::set<std::shared_ptr<Evoral::Note<Temporal::Beats> > >::iterator i = (*rn_it).second.begin(); i != (*rn_it).second.end(); ++i) {
 			XMLNode* nc = n->add_child(X_("note"));
 			nc->set_property(X_("note-id"), (*i)->id());
 		}
@@ -1197,9 +1182,9 @@ Selection::get_state () const
 	}
 
 	for (TimeSelection::const_iterator i = time.begin(); i != time.end(); ++i) {
-		XMLNode* t = node->add_child (X_("AudioRange"));
-		t->set_property (X_("start"), (*i).start);
-		t->set_property (X_("end"), (*i).end);
+		XMLNode* t = node->add_child (X_("TimelineRange"));
+		t->set_property (X_("start"), (*i).start());
+		t->set_property (X_("end"), (*i).end());
 	}
 
 	for (MarkerSelection::const_iterator i = markers.begin(); i != markers.end(); ++i) {
@@ -1227,6 +1212,7 @@ Selection::set_state (XMLNode const & node, int)
 	clear_points ();
 	clear_time ();
 	clear_markers ();
+	clear_triggers();
 
 	/* NOTE: stripable/time-axis-view selection is saved/restored by
 	 * ARDOUR::CoreSelection, not this Selection object
@@ -1289,7 +1275,7 @@ Selection::set_state (XMLNode const & node, int)
 			for (RegionSelection::iterator rsi = rs.begin(); rsi != rs.end(); ++rsi) {
 				MidiRegionView* mrv = dynamic_cast<MidiRegionView*> (*rsi);
 				if (mrv) {
-					mrv->select_notes(notes);
+					mrv->select_notes(notes, false);
 				}
 			}
 
@@ -1298,7 +1284,7 @@ Selection::set_state (XMLNode const & node, int)
 				pending_midi_note_selection.push_back (make_pair (id, notes));
 			}
 
-		} else if  ((*i)->name() == X_("ControlPoint")) {
+		} else if ((*i)->name() == X_("ControlPoint")) {
 			XMLProperty const * prop_type = (*i)->property (X_("type"));
 
 			assert(prop_type);
@@ -1321,17 +1307,12 @@ Selection::set_state (XMLNode const & node, int)
 				vector <ControlPoint *> cps;
 
 				if (stv) {
-					boost::shared_ptr<AutomationTimeAxisView> atv = stv->automation_child (EventTypeMap::instance().from_symbol (param));
-					if (atv) {
-						list<boost::shared_ptr<AutomationLine> > lines = atv->lines();
-						for (list<boost::shared_ptr<AutomationLine> > ::iterator li = lines.begin(); li != lines.end(); ++li) {
-							if ((*li)->the_list()->id() == alist_id) {
-								ControlPoint* cp = (*li)->nth(view_index);
-								if (cp) {
-									cps.push_back (cp);
-									cp->show();
-								}
-							}
+					std::shared_ptr<AutomationLine> li = stv->automation_child_by_alist_id (alist_id);
+					if (li) {
+						ControlPoint* cp = li->nth(view_index);
+						if (cp) {
+							cps.push_back (cp);
+							cp->show();
 						}
 					}
 				}
@@ -1355,7 +1336,7 @@ Selection::set_state (XMLNode const & node, int)
 					for (RegionSelection::iterator rsi = rs.begin(); rsi != rs.end(); ++rsi) {
 						AudioRegionView* arv = dynamic_cast<AudioRegionView*> (*rsi);
 						if (arv) {
-							boost::shared_ptr<AudioRegionGainLine> gl = arv->get_gain_line ();
+							std::shared_ptr<AudioRegionGainLine> gl = arv->get_gain_line ();
 							ControlPoint* cp = gl->nth(view_index);
 							if (cp) {
 								cps.push_back (cp);
@@ -1369,9 +1350,9 @@ Selection::set_state (XMLNode const & node, int)
 				}
 			}
 
-		} else if  ((*i)->name() == X_("AudioRange")) {
-			framepos_t start;
-			framepos_t end;
+		} else if ((*i)->name() == X_("TimelineRange")) {
+			timepos_t start;
+			timepos_t end;
 
 			if (!(*i)->get_property (X_("start"), start) || !(*i)->get_property (X_("end"), end)) {
 				assert(false);
@@ -1381,7 +1362,9 @@ Selection::set_state (XMLNode const & node, int)
 
 		} else if ((*i)->name() == X_("AutomationView")) {
 
+			// XXX is this even used? -> StripableAutomationControl
 			std::string param;
+			PBD::ID ctrl_id (0);
 
 			if (!(*i)->get_property (X_("id"), id) || !(*i)->get_property (X_("parameter"), param)) {
 				assert (false);
@@ -1389,13 +1372,13 @@ Selection::set_state (XMLNode const & node, int)
 
 			StripableTimeAxisView* stv = editor->get_stripable_time_axis_by_id (id);
 
-			if (stv) {
-				boost::shared_ptr<AutomationTimeAxisView> atv = stv->automation_child (EventTypeMap::instance().from_symbol (param));
+			if (stv && (*i)->get_property (X_("control_id"), ctrl_id)) {
+				std::shared_ptr<AutomationTimeAxisView> atv = stv->automation_child (EventTypeMap::instance().from_symbol (param), ctrl_id);
 
 				/* the automation could be for an entity that was never saved
-				   in the session file. Don't freak out if we can't find
-				   it.
-				*/
+				 * in the session file. Don't freak out if we can't find
+				 * it.
+				 */
 
 				if (atv) {
 					add (atv.get());
@@ -1453,8 +1436,8 @@ Selection::toggle (const TrackViewList& track_list)
 	PresentationInfo::ChangeSuspender cs;
 
 	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-		boost::shared_ptr<Stripable> s = (*i)->stripable ();
-		boost::shared_ptr<AutomationControl> c = (*i)->control ();
+		std::shared_ptr<Stripable> s = (*i)->stripable ();
+		std::shared_ptr<AutomationControl> c = (*i)->control ();
 		selection.toggle (s, c);
 	}
 }
@@ -1476,8 +1459,8 @@ Selection::add (TrackViewList const & track_list)
 	PresentationInfo::ChangeSuspender cs;
 
 	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-		boost::shared_ptr<Stripable> s = (*i)->stripable ();
-		boost::shared_ptr<AutomationControl> c = (*i)->control ();
+		std::shared_ptr<Stripable> s = (*i)->stripable ();
+		std::shared_ptr<AutomationControl> c = (*i)->control ();
 		selection.add (s, c);
 	}
 }
@@ -1505,8 +1488,8 @@ Selection::remove (const TrackViewList& t)
 	PresentationInfo::ChangeSuspender cs;
 
 	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-		boost::shared_ptr<Stripable> s = (*i)->stripable ();
-		boost::shared_ptr<AutomationControl> c = (*i)->control ();
+		std::shared_ptr<Stripable> s = (*i)->stripable ();
+		std::shared_ptr<AutomationControl> c = (*i)->control ();
 		selection.remove (s, c);
 	}
 }
@@ -1526,7 +1509,7 @@ Selection::set (const TrackViewList& track_list)
 
 	CoreSelection& selection (editor->session()->selection());
 
-#if 1 // crazy optmization hack
+#if 1 // crazy optimization hack
 	/* check is the selection actually changed, ignore NO-OPs
 	 *
 	 * There are excessive calls from EditorRoutes::selection_changed():
@@ -1542,8 +1525,8 @@ Selection::set (const TrackViewList& track_list)
 	CoreSelection::StripableAutomationControls sac;
 	selection.get_stripables (sac);
 	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-		boost::shared_ptr<Stripable> s = (*i)->stripable ();
-		boost::shared_ptr<AutomationControl> c = (*i)->control ();
+		std::shared_ptr<Stripable> s = (*i)->stripable ();
+		std::shared_ptr<AutomationControl> c = (*i)->control ();
 		bool found = false;
 		for (CoreSelection::StripableAutomationControls::iterator j = sac.begin (); j != sac.end (); ++j) {
 			if (j->stripable == s && j->controllable == c) {
@@ -1567,8 +1550,8 @@ Selection::set (const TrackViewList& track_list)
 	selection.clear_stripables ();
 
 	for (TrackSelection::const_iterator i = t.begin(); i != t.end(); ++i) {
-		boost::shared_ptr<Stripable> s = (*i)->stripable ();
-		boost::shared_ptr<AutomationControl> c = (*i)->control ();
+		std::shared_ptr<Stripable> s = (*i)->stripable ();
+		std::shared_ptr<AutomationControl> c = (*i)->control ();
 		selection.add (s, c);
 	}
 }
@@ -1597,8 +1580,8 @@ Selection::selected (TimeAxisView* tv) const
 	}
 
 	CoreSelection& selection (session->selection());
-	boost::shared_ptr<Stripable> s = tv->stripable ();
-	boost::shared_ptr<AutomationControl> c = tv->control ();
+	std::shared_ptr<Stripable> s = tv->stripable ();
+	std::shared_ptr<AutomationControl> c = tv->control ();
 
 	if (c) {
 		return selection.selected (c);
@@ -1671,7 +1654,7 @@ Selection::core_selection_changed (PropertyChange const & what_changed)
 	CoreSelection& selection (editor->session()->selection());
 
 	if (selection.selected()) {
-		clear_objects();  // enforce object/range exclusivity
+		clear_objects(); // enforce object/range exclusivity
 	}
 
 	tracks.clear (); // clear stage for whatever tracks are now selected (maybe none)
@@ -1693,4 +1676,79 @@ Selection::core_selection_changed (PropertyChange const & what_changed)
 			tracks.push_back (tav);
 		}
 	}
+
+	TracksChanged();
 }
+
+MidiRegionSelection
+Selection::midi_regions ()
+{
+	MidiRegionSelection ms;
+
+	for (RegionSelection::iterator i = regions.begin(); i != regions.end(); ++i) {
+		MidiRegionView* mrv = dynamic_cast<MidiRegionView*>(*i);
+		if (mrv) {
+			ms.add (mrv);
+		}
+	}
+
+	return ms;
+}
+
+bool
+Selection::selected (TriggerEntry* te) const
+{
+	return find (triggers.begin(), triggers.end(), te) != triggers.end();
+}
+
+void
+Selection::set (TriggerEntry* te)
+{
+#if 0
+	clear();
+#else
+	clear_tracks ();
+	clear_regions ();
+	clear_points ();
+	clear_lines ();
+	clear_time ();
+	clear_playlists ();
+	clear_midi_notes ();
+	clear_markers ();
+	pending_midi_note_selection.clear();
+#endif
+	clear_triggers (te ? false: true); /* Do not emit signal here, add() emits signal */
+	add (te);
+}
+
+void
+Selection::add (TriggerEntry* te)
+{
+	triggers.push_back (te);
+	TriggersChanged ();
+}
+
+void
+Selection::remove (TriggerEntry* te)
+{
+	TriggerSelection::iterator e = find (triggers.begin(), triggers.end(), te);
+
+	if (e != triggers.end()) {
+		triggers.erase (e);
+		TriggersChanged ();
+	}
+}
+
+void
+Selection::toggle (TriggerEntry* te)
+{
+	TriggerSelection::iterator e;
+
+	if ((e = find (triggers.begin(), triggers.end(), te)) != triggers.end()) {
+		add (te);
+	} else {
+		triggers.erase (e);
+	}
+	TriggersChanged ();
+}
+
