@@ -1,32 +1,39 @@
 /*
-    Copyright (C) 2001, 2006 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2005-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2007 Jesse Chappell <jesse@essej.net>
+ * Copyright (C) 2006-2015 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2014-2015 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2014-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2016 Nick Mainsbridge <mainsbridge@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cmath>
 
 #include <gtkmm.h>
 
+#include <gtkmm2ext/colors.h>
 #include <gtkmm2ext/gtk_ui.h>
 #include <gtkmm2ext/utils.h>
 
-#include "ardour/playlist.h"
 #include "ardour/region.h"
 #include "ardour/track.h"
-#include "ardour/session.h"
 
 #include "pbd/compose.h"
 
@@ -40,16 +47,13 @@
 #include "selection.h"
 #include "public_editor.h"
 #include "timers.h"
-#include "rgb_macros.h"
 #include "gui_thread.h"
 #include "ui_config.h"
-#include "utils.h"
 
 #include "pbd/i18n.h"
 
 using namespace std;
 using namespace ARDOUR;
-using namespace ARDOUR_UI_UTILS;
 using namespace PBD;
 using namespace Editing;
 
@@ -63,7 +67,7 @@ StreamView::StreamView (RouteTimeAxisView& tv, ArdourCanvas::Container* canvas_g
 	, _layers (1)
 	, _layer_display (Overlaid)
 	, height (tv.height)
-	, last_rec_data_frame(0)
+	, last_rec_data_sample(0)
 {
 	CANVAS_DEBUG_NAME (_canvas_group, string_compose ("SV canvas group %1", _trackview.name()));
 
@@ -77,7 +81,6 @@ StreamView::StreamView (RouteTimeAxisView& tv, ArdourCanvas::Container* canvas_g
 	canvas_rect->Event.connect (sigc::bind (sigc::mem_fun (_trackview.editor(), &PublicEditor::canvas_stream_view_event), canvas_rect, &_trackview));
 
 	if (_trackview.is_track()) {
-		_trackview.track()->DiskstreamChanged.connect (*this, invalidator (*this), boost::bind (&StreamView::diskstream_changed, this), gui_context());
 		_trackview.track()->rec_enable_control()->Changed.connect (*this, invalidator (*this), boost::bind (&StreamView::rec_enable_changed, this), gui_context());
 
 		_trackview.session()->TransportStateChange.connect (*this, invalidator (*this), boost::bind (&StreamView::transport_changed, this), gui_context());
@@ -115,7 +118,7 @@ StreamView::set_height (double h)
 {
 	/* limit the values to something sane-ish */
 
-	if (h < 10.0 || h > 1000.0) {
+	if (h < 10.0 || h > 2500.0) {
 		return -1;
 	}
 
@@ -159,15 +162,15 @@ StreamView::set_samples_per_pixel (double fpp)
 		recbox.rectangle->set_x1 (xend);
 	}
 
-	update_coverage_frames ();
+	update_coverage_frame ();
 
 	return 0;
 }
 
 void
-StreamView::add_region_view (boost::weak_ptr<Region> wr)
+StreamView::add_region_view (std::weak_ptr<Region> wr)
 {
-	boost::shared_ptr<Region> r (wr.lock());
+	std::shared_ptr<Region> r (wr.lock());
 	if (!r) {
 		return;
 	}
@@ -180,11 +183,11 @@ StreamView::add_region_view (boost::weak_ptr<Region> wr)
 }
 
 void
-StreamView::remove_region_view (boost::weak_ptr<Region> weak_r)
+StreamView::remove_region_view (std::weak_ptr<Region> weak_r)
 {
 	ENSURE_GUI_THREAD (*this, &StreamView::remove_region_view, weak_r)
 
-	boost::shared_ptr<Region> r (weak_r.lock());
+	std::shared_ptr<Region> r (weak_r.lock());
 
 	if (!r) {
 		return;
@@ -216,11 +219,11 @@ StreamView::undisplay_track ()
 }
 
 void
-StreamView::display_track (boost::shared_ptr<Track> tr)
+StreamView::display_track (std::shared_ptr<Track> tr)
 {
 	playlist_switched_connection.disconnect();
 	playlist_switched (tr);
-	tr->PlaylistChanged.connect (playlist_switched_connection, invalidator (*this), boost::bind (&StreamView::playlist_switched, this, boost::weak_ptr<Track> (tr)), gui_context());
+	tr->PlaylistChanged.connect (playlist_switched_connection, invalidator (*this), boost::bind (&StreamView::playlist_switched, this, std::weak_ptr<Track> (tr)), gui_context());
 }
 
 void
@@ -240,8 +243,6 @@ StreamView::layer_regions()
 			region_views.erase (i);
 			i = tmp;
 			continue;
-		} else {
-			(*i)->enable_display(true);
 		}
 
 		if (copy.size() == 0) {
@@ -281,9 +282,9 @@ StreamView::layer_regions()
 }
 
 void
-StreamView::playlist_layered (boost::weak_ptr<Track> wtr)
+StreamView::playlist_layered (std::weak_ptr<Track> wtr)
 {
-	boost::shared_ptr<Track> tr (wtr.lock());
+	std::shared_ptr<Track> tr (wtr.lock());
 
 	if (!tr) {
 		return;
@@ -297,7 +298,7 @@ StreamView::playlist_layered (boost::weak_ptr<Track> wtr)
 	if (_layer_display == Stacked) {
 		update_contents_height ();
 		/* tricky. playlist_changed() does this as well, and its really inefficient. */
-		update_coverage_frames ();
+		update_coverage_frame ();
 	} else {
 		/* layering has probably been modified. reflect this in the canvas. */
 		layer_regions();
@@ -305,9 +306,9 @@ StreamView::playlist_layered (boost::weak_ptr<Track> wtr)
 }
 
 void
-StreamView::playlist_switched (boost::weak_ptr<Track> wtr)
+StreamView::playlist_switched (std::weak_ptr<Track> wtr)
 {
-	boost::shared_ptr<Track> tr (wtr.lock());
+	std::shared_ptr<Track> tr (wtr.lock());
 
 	if (!tr) {
 		return;
@@ -316,7 +317,7 @@ StreamView::playlist_switched (boost::weak_ptr<Track> wtr)
 	/* disconnect from old playlist */
 
 	playlist_connections.drop_connections ();
-	//undisplay_track ();
+	undisplay_track ();
 
 	/* draw it */
 	tr->playlist()->freeze();
@@ -325,33 +326,20 @@ StreamView::playlist_switched (boost::weak_ptr<Track> wtr)
 	/* update layers count and the y positions and heights of our regions */
 	_layers = tr->playlist()->top_layer() + 1;
 	update_contents_height ();
-	update_coverage_frames ();
+	update_coverage_frame ();
 
 	/* catch changes */
 
-	tr->playlist()->LayeringChanged.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::playlist_layered, this, boost::weak_ptr<Track> (tr)), gui_context());
+	tr->playlist()->LayeringChanged.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::playlist_layered, this, std::weak_ptr<Track> (tr)), gui_context());
 	tr->playlist()->RegionAdded.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::add_region_view, this, _1), gui_context());
 	tr->playlist()->RegionRemoved.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::remove_region_view, this, _1), gui_context());
-	tr->playlist()->ContentsChanged.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::update_coverage_frames, this), gui_context());
-}
-
-
-void
-StreamView::diskstream_changed ()
-{
-	boost::shared_ptr<Track> t;
-
-	if ((t = _trackview.track()) != 0) {
-		Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&StreamView::display_track, this, t));
-	} else {
-		Gtkmm2ext::UI::instance()->call_slot (invalidator (*this), boost::bind (&StreamView::undisplay_track, this));
-	}
+	tr->playlist()->ContentsChanged.connect (playlist_connections, invalidator (*this), boost::bind (&StreamView::update_coverage_frame, this), gui_context());
 }
 
 void
 StreamView::apply_color (Gdk::Color const& c, ColorTarget target)
 {
-	return apply_color (gdk_color_to_rgba (c), target);
+	return apply_color (Gtkmm2ext::gdk_color_to_rgba (c), target);
 }
 
 void
@@ -410,9 +398,9 @@ StreamView::transport_looped()
 }
 
 void
-StreamView::create_rec_box(framepos_t frame_pos, double width)
+StreamView::create_rec_box(samplepos_t sample_pos, double width)
 {
-	const double   xstart     = _trackview.editor().sample_to_pixel(frame_pos);
+	const double   xstart     = _trackview.editor().sample_to_pixel(sample_pos);
 	const double   xend       = xstart + width;
 	const uint32_t fill_color = UIConfiguration::instance().color_mod("recording rect", "recording_rect");
 
@@ -433,7 +421,7 @@ StreamView::create_rec_box(framepos_t frame_pos, double width)
 	if (rec_rects.empty()) {
 		recbox.start = _trackview.session()->record_location ();
 	} else {
-		recbox.start = _trackview.session()->transport_frame ();
+		recbox.start = _trackview.session()->transport_sample ();
 	}
 
 	rec_rects.push_back (recbox);
@@ -451,7 +439,7 @@ StreamView::update_rec_box ()
 	if (rec_active && rec_rects.size() > 0) {
 		/* only update the last box */
 		RecBoxInfo & rect = rec_rects.back();
-		framepos_t const at = _trackview.track()->current_capture_end ();
+		samplepos_t const at = _trackview.track()->current_capture_end ();
 		double xstart;
 		double xend;
 
@@ -464,12 +452,6 @@ StreamView::update_rec_box ()
 			xend = _trackview.editor().sample_to_pixel (at);
 			break;
 
-		case Destructive:
-			rect.length = 2;
-			xstart = _trackview.editor().sample_to_pixel (_trackview.track()->current_capture_start());
-			xend = _trackview.editor().sample_to_pixel (at);
-			break;
-
 		default:
 			fatal << string_compose (_("programming error: %1"), "illegal track mode") << endmsg;
 			abort(); /*NOTREACHED*/
@@ -482,7 +464,7 @@ StreamView::update_rec_box ()
 }
 
 RegionView*
-StreamView::find_view (boost::shared_ptr<const Region> region)
+StreamView::find_view (std::shared_ptr<const Region> region)
 {
 	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
 
@@ -545,14 +527,14 @@ StreamView::set_selected_regionviews (RegionSelection& regions)
 }
 
 /** Get selectable things within a given range.
- *  @param start Start time in session frames.
- *  @param end End time in session frames.
+ *  @param start Start time in session samples.
+ *  @param end End time in session samples.
  *  @param top Top y range, in trackview coordinates (ie 0 is the top of the track view)
  *  @param bot Bottom y range, in trackview coordinates (ie 0 is the top of the track view)
  *  @param result Filled in with selectable things.
  */
 void
-StreamView::get_selectables (framepos_t start, framepos_t end, double top, double bottom, list<Selectable*>& results, bool within)
+StreamView::get_selectables (timepos_t const & start, timepos_t const & end, double top, double bottom, list<Selectable*>& results, bool within)
 {
 	if (_trackview.editor().internal_editing()) {
 		return;  // Don't select regions with an internal tool
@@ -590,11 +572,11 @@ StreamView::get_selectables (framepos_t start, framepos_t end, double top, doubl
 		}
 
 		if (within) {
-			if ((*i)->region()->coverage (start, end) == Evoral::OverlapExternal && layer_ok) {
+			if ((*i)->region()->coverage (start, end) == Temporal::OverlapExternal && layer_ok) {
 				results.push_back (*i);
 			}
 		} else {
-			if ((*i)->region()->coverage (start, end) != Evoral::OverlapNone && layer_ok) {
+			if ((*i)->region()->coverage (start, end) != Temporal::OverlapNone && layer_ok) {
 				results.push_back (*i);
 			}
 		}
@@ -612,6 +594,16 @@ StreamView::get_inverted_selectables (Selection& sel, list<Selectable*>& results
 	}
 }
 
+void
+StreamView::get_regionviews_at_or_after (timepos_t const & pos, RegionSelection& regions)
+{
+	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
+		if ((*i)->region()->position() >= pos) {
+			regions.push_back (*i);
+		}
+	}
+}
+
 /** @return height of a child region view, depending on stacked / overlaid mode */
 double
 StreamView::child_height () const
@@ -677,19 +669,19 @@ StreamView::set_layer_display (LayerDisplay d)
 	}
 
 	update_contents_height ();
-	update_coverage_frames ();
+	update_coverage_frame ();
 }
 
 void
-StreamView::update_coverage_frames ()
+StreamView::update_coverage_frame ()
 {
 	for (RegionViewList::iterator i = region_views.begin (); i != region_views.end (); ++i) {
-		(*i)->update_coverage_frames (_layer_display);
+		(*i)->update_coverage_frame (_layer_display);
 	}
 }
 
 void
-StreamView::check_record_layers (boost::shared_ptr<Region> region, framepos_t to)
+StreamView::check_record_layers (std::shared_ptr<Region> region, samplepos_t to)
 {
 	if (_new_rec_layer_time < to) {
 		/* The region being recorded has overlapped the start of a top-layered region, so
@@ -699,7 +691,7 @@ StreamView::check_record_layers (boost::shared_ptr<Region> region, framepos_t to
 		*/
 
 		/* Stop this happening again */
-		_new_rec_layer_time = max_framepos;
+		_new_rec_layer_time = max_samplepos;
 
 		/* Make space in the view for the new layer */
 		++_layers;
@@ -713,15 +705,25 @@ StreamView::check_record_layers (boost::shared_ptr<Region> region, framepos_t to
 }
 
 void
-StreamView::setup_new_rec_layer_time (boost::shared_ptr<Region> region)
+StreamView::setup_new_rec_layer_time (std::shared_ptr<Region> region)
 {
 	/* If we are in Stacked mode, we may need to (visually) create a new layer to put the
 	   recorded region in.  To work out where this needs to happen, find the start of the next
 	   top-layered region after the start of the region we are recording and make a note of it.
 	*/
 	if (_layer_display == Stacked) {
-		_new_rec_layer_time = _trackview.track()->playlist()->find_next_top_layer_position (region->start());
+		_new_rec_layer_time = _trackview.track()->playlist()->find_next_top_layer_position (region->position()).samples();
 	} else {
-		_new_rec_layer_time = max_framepos;
+		_new_rec_layer_time = max_samplepos;
+	}
+}
+
+void
+StreamView::parameter_changed (string const & what)
+{
+	if (what == "show-region-name") {
+		for (RegionViewList::iterator i = region_views.begin (); i != region_views.end (); ++i) {
+			(*i)->update_visibility ();
+		}
 	}
 }
