@@ -1,21 +1,23 @@
 /*
-    Copyright (C) 2009 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2009-2011 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2009-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2009-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2015-2019 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <iostream>
 
@@ -24,6 +26,8 @@
 #include <gtkmm/progressbar.h>
 #include <gtkmm/stock.h>
 
+#include "pbd/pthread_utils.h"
+
 #include "ardour/audioregion.h"
 #include "ardour/dB.h"
 #include "ardour/logmeter.h"
@@ -49,6 +53,9 @@ StripSilenceDialog::StripSilenceDialog (Session* s, list<RegionView*> const & v)
 	, _destroying (false)
 	, analysis_progress_cur (0)
 	, analysis_progress_max (0)
+	, _threshold_value (-60)
+	, _minimum_length_value (1000)
+	, _fade_length_value (64)
 {
 	set_session (s);
 
@@ -61,6 +68,14 @@ StripSilenceDialog::StripSilenceDialog (Session* s, list<RegionView*> const & v)
 	Gtk::Table* table = Gtk::manage (new Gtk::Table (3, 3));
 	table->set_spacings (6);
 
+	//get the last used settings for this
+	XMLNode* node = _session->extra_xml(X_("StripSilence"));
+	if (node) {
+		node->get_property(X_("threshold"), _threshold_value);
+		node->get_property(X_("min-length"), _minimum_length_value);
+		node->get_property(X_("fade-length"), _fade_length_value);
+	}
+
 	int n = 0;
 
 	table->attach (*Gtk::manage (new Gtk::Label (_("Threshold"), 1, 0.5)), 0, 1, n, n + 1, Gtk::FILL);
@@ -71,7 +86,7 @@ StripSilenceDialog::StripSilenceDialog (Session* s, list<RegionView*> const & v)
 	_threshold.set_digits (1);
 	_threshold.set_increments (1, 10);
 	_threshold.set_range (-120, 0);
-	_threshold.set_value (-60);
+	_threshold.set_value (_threshold_value);
 	_threshold.set_activates_default ();
 
 	table->attach (*Gtk::manage (new Gtk::Label (_("Minimum length"), 1, 0.5)), 0, 1, n, n + 1, Gtk::FILL);
@@ -79,16 +94,16 @@ StripSilenceDialog::StripSilenceDialog (Session* s, list<RegionView*> const & v)
 	++n;
 
 	_minimum_length->set_session (s);
-	_minimum_length->set_mode (AudioClock::Frames);
-	_minimum_length->set (1000, true);
+	_minimum_length->set_mode (AudioClock::Samples);
+	_minimum_length->set_duration (timecnt_t (_minimum_length_value), true);
 
 	table->attach (*Gtk::manage (new Gtk::Label (_("Fade length"), 1, 0.5)), 0, 1, n, n + 1, Gtk::FILL);
 	table->attach (*_fade_length, 1, 2, n, n + 1, Gtk::FILL);
 	++n;
 
 	_fade_length->set_session (s);
-	_fade_length->set_mode (AudioClock::Frames);
-	_fade_length->set (64, true);
+	_fade_length->set_mode (AudioClock::Samples);
+	_fade_length->set_duration (timecnt_t (_fade_length_value), true);
 
 	hbox->pack_start (*table);
 
@@ -118,6 +133,8 @@ StripSilenceDialog::StripSilenceDialog (Session* s, list<RegionView*> const & v)
 	Completed.connect (_completed_connection, invalidator(*this), boost::bind (&StripSilenceDialog::update, this), gui_context ());
 	_thread_should_finish = false;
 	pthread_create (&_thread, 0, StripSilenceDialog::_detection_thread_work, this);
+
+	signal_response().connect(sigc::mem_fun (*this, &StripSilenceDialog::finished));
 }
 
 
@@ -158,7 +175,7 @@ void
 StripSilenceDialog::silences (AudioIntervalMap& m)
 {
 	for (list<ViewInterval>::iterator v = views.begin(); v != views.end(); ++v) {
-		pair<boost::shared_ptr<Region>,AudioIntervalResult> newpair (v->view->region(), v->intervals);
+		pair<std::shared_ptr<Region>,AudioIntervalResult> newpair (v->view->region(), v->intervals);
 		m.insert (newpair);
 	}
 }
@@ -166,8 +183,8 @@ StripSilenceDialog::silences (AudioIntervalMap& m)
 void
 StripSilenceDialog::drop_rects ()
 {
-	// called by parent when starting to progess (dialog::run returned),
-	// but before the dialog is destoyed.
+	// called by parent when starting to progress (dialog::run returned),
+	// but before the dialog is destroyed.
 
 	_interthread_info.cancel = true;
 
@@ -231,6 +248,7 @@ void *
 StripSilenceDialog::_detection_thread_work (void* arg)
 {
 	StripSilenceDialog* d = reinterpret_cast<StripSilenceDialog*> (arg);
+	pthread_set_name ("SilenceDetect");
 	return d->detection_thread_work ();
 }
 
@@ -250,7 +268,7 @@ StripSilenceDialog::detection_thread_work ()
 		analysis_progress_cur = 0;
 		analysis_progress_max = views.size();
 		for (list<ViewInterval>::iterator i = views.begin(); i != views.end(); ++i) {
-			boost::shared_ptr<AudioRegion> ar = boost::dynamic_pointer_cast<AudioRegion> ((*i).view->region());
+			std::shared_ptr<AudioRegion> ar = std::dynamic_pointer_cast<AudioRegion> ((*i).view->region());
 
 			if (ar) {
 				i->intervals = ar->find_silence (dB_to_coefficient (threshold ()), minimum_length (), fade_length(), _interthread_info);
@@ -320,16 +338,16 @@ StripSilenceDialog::threshold_changed ()
 	restart_thread ();
 }
 
-framecnt_t
+samplecnt_t
 StripSilenceDialog::minimum_length () const
 {
-	return std::max((framecnt_t)1, _minimum_length->current_duration (views.front().view->region()->position()));
+	return std::max((samplecnt_t)1, _minimum_length->current_duration (views.front().view->region()->position()).samples());
 }
 
-framecnt_t
+samplecnt_t
 StripSilenceDialog::fade_length () const
 {
-	return std::max((framecnt_t)0, _fade_length->current_duration (views.front().view->region()->position()));
+	return std::max((samplecnt_t)0, _fade_length->current_duration (views.front().view->region()->position()).samples());
 }
 
 void
@@ -337,3 +355,26 @@ StripSilenceDialog::update_progress_gui (float p)
 {
 	_progress_bar.set_fraction (p);
 }
+
+XMLNode&
+StripSilenceDialog::get_state () const
+{
+	XMLNode* node = new XMLNode(X_("StripSilence"));
+	node->set_property(X_("threshold"), threshold());
+	node->set_property(X_("min-length"), minimum_length());
+	node->set_property(X_("fade-length"), fade_length());
+	return *node;
+}
+
+void
+StripSilenceDialog::set_state (const XMLNode &)
+{
+}
+
+void
+StripSilenceDialog::finished(int response)
+{
+	if(response == Gtk::RESPONSE_OK) {
+		_session->add_extra_xml(get_state());
+	}
+}
