@@ -1,21 +1,28 @@
 /*
-    Copyright (C) 2003 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2008 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2005-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2005 Karsten Wiese <fzuuzf@googlemail.com>
+ * Copyright (C) 2005 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007 Doug McLain <doug@nostar.net>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2014-2015 Ben Loftis <ben@harrisonconsoles.com>
+ * Copyright (C) 2014-2019 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_gtk_time_axis_h__
 #define __ardour_gtk_time_axis_h__
@@ -36,7 +43,7 @@
 #include "pbd/stateful.h"
 #include "pbd/signals.h"
 
-#include "evoral/Parameter.hpp"
+#include "evoral/Parameter.h"
 
 #include "ardour/types.h"
 #include "ardour/presentation_info.h"
@@ -90,13 +97,13 @@ class PasteContext;
  */
 class TimeAxisView : public virtual AxisView
 {
-	private:
+private:
 	enum NamePackingBits {
 		NameLabelPacked = 0x1,
 		NameEntryPacked = 0x2
 	};
 
-	public:
+public:
 	TimeAxisView(ARDOUR::Session* sess, PublicEditor& ed, TimeAxisView* parent, ArdourCanvas::Canvas& canvas);
 	virtual ~TimeAxisView ();
 
@@ -152,7 +159,7 @@ class TimeAxisView : public virtual AxisView
 		HeightPerLane
 	};
 
-	virtual void set_height (uint32_t h, TrackHeightMode m = OnlySelf);
+	virtual void set_height (uint32_t h, TrackHeightMode m = OnlySelf, bool from_idle = false);
 	void set_height_enum (Height, bool apply_to_selection = false);
 	void reset_height();
 
@@ -164,13 +171,13 @@ class TimeAxisView : public virtual AxisView
 	virtual void step_height (bool);
 
 	virtual ARDOUR::RouteGroup* route_group() const { return 0; }
-	virtual boost::shared_ptr<ARDOUR::Playlist> playlist() const { return boost::shared_ptr<ARDOUR::Playlist> (); }
+	virtual std::shared_ptr<ARDOUR::Playlist> playlist() const { return std::shared_ptr<ARDOUR::Playlist> (); }
 
 	virtual void set_samples_per_pixel (double);
 	virtual void show_selection (TimeSelection&);
 	virtual void hide_selection ();
 	virtual void reshow_selection (TimeSelection&);
-	virtual void show_timestretch (framepos_t start, framepos_t end, int layers, int layer);
+	virtual void show_timestretch (Temporal::timepos_t const & start, Temporal::timepos_t const & end, int layers, int layer);
 	virtual void hide_timestretch ();
 
 	/* editing operations */
@@ -178,28 +185,33 @@ class TimeAxisView : public virtual AxisView
 	virtual void cut_copy_clear (Selection&, Editing::CutCopyOp) {}
 
 	/** Paste a selection.
-	 *  @param pos Position to paste to (session frames).
+	 *  @param pos Position to paste to (session samples).
 	 *  @param selection Selection to paste.
 	 *  @param ctx Paste context.
+	 *  @param sub_num music-time sub-division: \c -1: snap to bar, \c 1: exact beat, \c >1: \c (1 \c / \p sub_num \c ) beat-divisions
 	 */
-	virtual bool paste (ARDOUR::framepos_t pos,
-	                    const Selection&   selection,
-	                    PasteContext&      ctx,
-			    const int32_t sub_num) { return false; }
+	virtual bool paste (Temporal::timepos_t const & pos,
+	                    const Selection&    selection,
+	                    PasteContext&       ctx)
+	{
+		return false;
+	}
+
 
 	virtual void set_selected_regionviews (RegionSelection&) {}
 	virtual void set_selected_points (PointSelection&);
 
 	virtual void fade_range (TimeSelection&) {}
 
-	virtual boost::shared_ptr<ARDOUR::Region> find_next_region (framepos_t /*pos*/, ARDOUR::RegionPoint, int32_t /*dir*/) {
-		return boost::shared_ptr<ARDOUR::Region> ();
+	virtual std::shared_ptr<ARDOUR::Region> find_next_region (ARDOUR::timepos_t const & /*pos*/, ARDOUR::RegionPoint, int32_t /*dir*/) {
+		return std::shared_ptr<ARDOUR::Region> ();
 	}
 
 	void order_selection_trims (ArdourCanvas::Item *item, bool put_start_on_top);
 
-	virtual void get_selectables (ARDOUR::framepos_t, ARDOUR::framepos_t, double, double, std::list<Selectable*>&, bool within = false);
+	virtual void get_selectables (Temporal::timepos_t const &, Temporal::timepos_t  const &, double, double, std::list<Selectable*>&, bool within = false);
 	virtual void get_inverted_selectables (Selection&, std::list<Selectable *>& results);
+	virtual void get_regionviews_at_or_after (Temporal::timepos_t const &, RegionSelection&) {}
 
 	void add_ghost (RegionView*);
 	void remove_ghost (RegionView*);
@@ -215,12 +227,12 @@ class TimeAxisView : public virtual AxisView
 	virtual LayerDisplay layer_display () const { return Overlaid; }
 	virtual StreamView* view () const { return 0; }
 
-	typedef std::vector<boost::shared_ptr<TimeAxisView> > Children;
+	typedef std::vector<std::shared_ptr<TimeAxisView> > Children;
 	Children get_child_list () const;
 
 	static uint32_t preset_height (Height);
 
-	protected:
+protected:
 	static Glib::RefPtr<Gtk::SizeGroup> controls_meters_size_group;
 	static Glib::RefPtr<Gtk::SizeGroup> midi_scroomer_size_group;
 	static unsigned int name_width_px;
@@ -263,7 +275,7 @@ class TimeAxisView : public virtual AxisView
 
 	virtual bool name_entry_changed (std::string const&);
 
-	/** Handle mouse relaese on our LHS control name ebox.
+	/** Handle mouse release on our LHS control name ebox.
 	 *
 	 *@ param ev the event
 	 */
@@ -292,20 +304,21 @@ class TimeAxisView : public virtual AxisView
 	bool is_child (TimeAxisView*);
 	virtual bool propagate_time_selection () const { return false; }
 
-	virtual void remove_child (boost::shared_ptr<TimeAxisView>);
-	void add_child (boost::shared_ptr<TimeAxisView>);
+	virtual void remove_child (std::shared_ptr<TimeAxisView>);
+	void add_child (std::shared_ptr<TimeAxisView>);
 
 	/* selection display */
 
 	virtual void selection_click (GdkEventButton*);
 
 	void color_handler ();
+	void parameter_changed (std::string const &);
 
 	void conditionally_add_to_selection ();
 
 	void build_size_menu ();
 
-  private:
+private:
 	Gtk::VBox*            control_parent;
 	int                  _order;
 	uint32_t             _effective_height;
