@@ -1,20 +1,22 @@
 /*
-    Copyright (C) 2016 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2016-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2016-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2017-2018 Ben Loftis <ben@harrisonconsoles.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <gtkmm/stock.h>
 
@@ -25,16 +27,17 @@
 #include "ardour/vca.h"
 #include "ardour/vca_manager.h"
 
+#include "gtkmm2ext/colors.h"
 #include "gtkmm2ext/doi.h"
 #include "gtkmm2ext/keyboard.h"
 #include "widgets/tooltips.h"
 
 #include "ardour_dialog.h"
+#include "ardour_message.h"
 #include "floating_text_entry.h"
 #include "gui_thread.h"
 #include "mixer_ui.h"
 #include "ui_config.h"
-#include "utils.h"
 #include "vca_master_strip.h"
 
 #include "pbd/i18n.h"
@@ -48,7 +51,9 @@ using std::string;
 
 PBD::Signal1<void,VCAMasterStrip*> VCAMasterStrip::CatchDeletion;
 
-VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
+static bool no_propagate (GdkEventButton*) { return false; }
+
+VCAMasterStrip::VCAMasterStrip (Session* s, std::shared_ptr<VCA> v)
 	: SessionHandlePtr (s)
 	, _vca (v)
 	, gain_meter (s, 254) /* magic number, don't adjust blindly */
@@ -59,26 +64,30 @@ VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
 	/* set color for the VCA, if not already done. */
 
 	if (!_vca->presentation_info().color_set()) {
-		_vca->presentation_info().set_color (ARDOUR_UI_UTILS::gdk_color_to_rgba (unique_random_color()));
+		if (UIConfiguration::instance().get_use_palette_for_new_vca ()) {
+			_vca->presentation_info().set_color (Gtkmm2ext::gdk_color_to_rgba (AxisView::round_robin_palette_color ()));
+		} else {
+			_vca->presentation_info().set_color (UIConfiguration::instance ().color (X_("neutral:midground")));
+		}
 	}
 
-	control_slave_ui.set_stripable (boost::dynamic_pointer_cast<Stripable> (v));
+	control_slave_ui.set_stripable (std::dynamic_pointer_cast<Stripable> (v));
 
-	gain_meter.set_controls (boost::shared_ptr<Route>(),
-	                         boost::shared_ptr<PeakMeter>(),
-	                         boost::shared_ptr<Amp>(),
+	gain_meter.set_controls (std::shared_ptr<Route>(),
+	                         std::shared_ptr<PeakMeter>(),
+	                         std::shared_ptr<Amp>(),
 	                         _vca->gain_control());
 
 	solo_button.set_name ("solo button");
-	set_tooltip (solo_button, _("Solo slaves"));
+	set_tooltip (solo_button, _("Solo assigned channels"));
 	solo_button.signal_button_release_event().connect (sigc::mem_fun (*this, &VCAMasterStrip::solo_release), false);
 
 	mute_button.set_name ("mute button");
 	mute_button.set_text (_("M"));
-	set_tooltip (mute_button, _("Mute slaves"));
+	set_tooltip (mute_button, _("Mute assigned channels"));
 	mute_button.signal_button_release_event().connect (sigc::mem_fun (*this, &VCAMasterStrip::mute_release), false);
 
-	hide_button.set_icon (ArdourIcon::CloseCross);
+	hide_button.set_icon (ArdourIcon::HideEye);
 	set_tooltip (&hide_button, _("Hide this VCA strip"));
 
 	hide_button.signal_clicked.connect (sigc::mem_fun(*this, &VCAMasterStrip::hide_clicked));
@@ -106,10 +115,11 @@ VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
 	/* horizontally centered, with a little space (5%) at the top */
 	vertical_button.set_angle (90);
 	vertical_button.set_layout_font (UIConfiguration::instance().get_NormalBoldFont());
-	vertical_button.signal_button_release_event().connect (sigc::mem_fun (*this, &VCAMasterStrip::vertical_button_press));
+	vertical_button.signal_button_press_event().connect (sigc::ptr_fun (&no_propagate), false);
+	vertical_button.signal_button_release_event().connect (sigc::mem_fun (*this, &VCAMasterStrip::vertical_button_release), false);
 	vertical_button.set_fallthrough_to_parent (true);
 	vertical_button.set_active_color (_vca->presentation_info().color ());
-	set_tooltip (vertical_button, _("Click to show slaves only")); /* tooltip updated dynamically */
+	set_tooltip (vertical_button, _("Click to show assigned channels only")); /* tooltip updated dynamically */
 
 	global_vpacker.set_border_width (0);
 	global_vpacker.set_spacing (0);
@@ -132,7 +142,6 @@ VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
 
 	global_vpacker.show ();
 	global_frame.show ();
-	top_padding.show ();
 	vertical_button.show ();
 	hide_button.show ();
 	number_label.show ();
@@ -149,7 +158,7 @@ VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
 	update_vca_name ();
 	solo_changed ();
 	mute_changed ();
-	spill_change (boost::shared_ptr<VCA>());
+	spill_change (std::shared_ptr<VCA>());
 
 	Mixer_UI::instance()->show_spill_change.connect (sigc::mem_fun (*this, &VCAMasterStrip::spill_change));
 
@@ -160,6 +169,7 @@ VCAMasterStrip::VCAMasterStrip (Session* s, boost::shared_ptr<VCA> v)
 	_vca->solo_control()->Changed.connect (vca_connections, invalidator (*this), boost::bind (&VCAMasterStrip::solo_changed, this), gui_context());
 	_vca->mute_control()->Changed.connect (vca_connections, invalidator (*this), boost::bind (&VCAMasterStrip::mute_changed, this), gui_context());
 
+	_session->MonitorBusAddedOrRemoved.connect (*this, invalidator (*this), boost::bind (&VCAMasterStrip::set_button_names, this), gui_context());
 
 	s->config.ParameterChanged.connect (*this, invalidator (*this), boost::bind (&VCAMasterStrip::parameter_changed, this, _1), gui_context());
 	Config->ParameterChanged.connect (*this, invalidator (*this), boost::bind (&VCAMasterStrip::parameter_changed, this, _1), gui_context());
@@ -170,7 +180,7 @@ VCAMasterStrip::~VCAMasterStrip ()
 {
 	if ((_session && !_session->deletion_in_progress()) && Mixer_UI::instance()->showing_spill_for (_vca)) {
 		/* cancel spill for this VCA */
-		Mixer_UI::instance()->show_spill (boost::shared_ptr<Stripable>());
+		Mixer_UI::instance()->show_spill (std::shared_ptr<Stripable>());
 	}
 
 	delete delete_dialog;
@@ -184,21 +194,21 @@ VCAMasterStrip::self_delete ()
 {
 	if ((_session && !_session->deletion_in_progress()) && Mixer_UI::instance()->showing_spill_for (_vca)) {
 		/* cancel spill for this VCA */
-		Mixer_UI::instance()->show_spill (boost::shared_ptr<Stripable>());
+		Mixer_UI::instance()->show_spill (std::shared_ptr<Stripable>());
 	}
 	/* Drop reference immediately, delete self when idle */
 	_vca.reset ();
-	gain_meter.set_controls (boost::shared_ptr<Route>(),
-	                         boost::shared_ptr<PeakMeter>(),
-	                         boost::shared_ptr<Amp>(),
-	                         boost::shared_ptr<GainControl>());
+	gain_meter.set_controls (std::shared_ptr<Route>(),
+	                         std::shared_ptr<PeakMeter>(),
+	                         std::shared_ptr<Amp>(),
+	                         std::shared_ptr<GainControl>());
 	delete_when_idle (this);
 }
 
 void
 VCAMasterStrip::parameter_changed (std::string const & p)
 {
-	if (p == "use-monitor-bus" || p == "solo-control-is-listen-control" || p == "listen-position") {
+	if (p == "solo-control-is-listen-control" || p == "listen-position") {
 		set_button_names ();
 	} else if (p == "mixer-element-visibility") {
 		update_bottom_padding ();
@@ -377,7 +387,7 @@ VCAMasterStrip::solo_changed ()
 }
 
 bool
-VCAMasterStrip::vertical_button_press (GdkEventButton* ev)
+VCAMasterStrip::vertical_button_release (GdkEventButton* ev)
 {
 	if (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS) {
 		start_name_edit ();
@@ -472,14 +482,14 @@ void
 VCAMasterStrip::spill ()
 {
 	if (Mixer_UI::instance()->showing_spill_for (_vca)) {
-		Mixer_UI::instance()->show_spill (boost::shared_ptr<Stripable>());
+		Mixer_UI::instance()->show_spill (std::shared_ptr<Stripable>());
 	} else {
 		Mixer_UI::instance()->show_spill (_vca);
 	}
 }
 
 void
-VCAMasterStrip::spill_change (boost::shared_ptr<Stripable> vca)
+VCAMasterStrip::spill_change (std::shared_ptr<Stripable> vca)
 {
 	if (vca != _vca) {
 		vertical_button.set_active_state (Gtkmm2ext::Off);
@@ -497,6 +507,32 @@ VCAMasterStrip::remove ()
 		return;
 	}
 
+	ArdourMessageDialog checker (_("Do you really want to remove this VCA?"),
+	                             true,
+	                             Gtk::MESSAGE_QUESTION,
+	                             Gtk::BUTTONS_NONE);
+
+	string title = string_compose (_("Remove %1"), "VCA");
+	checker.set_title (title);
+
+	checker.set_secondary_text(_("This action cannot be undone."));
+
+	checker.add_button (_("No, do nothing."), RESPONSE_CANCEL);
+	checker.add_button (_("Yes, remove it."), RESPONSE_ACCEPT);
+	checker.set_default_response (RESPONSE_CANCEL);
+
+	checker.set_name (X_("RemoveVcaDialog"));
+	checker.set_wmclass (X_("ardour_vca_remove"), PROGRAM_NAME);
+	checker.set_position (Gtk::WIN_POS_MOUSE);
+
+	switch (checker.run()) {
+	case RESPONSE_ACCEPT:
+		break;
+	default:
+		return;
+	}
+	checker.hide();
+
 	_session->vca_manager().remove_vca (_vca);
 }
 
@@ -518,14 +554,14 @@ VCAMasterStrip::drop_all_slaves ()
 	_vca->Drop (); /* EMIT SIGNAL */
 
 	if (Mixer_UI::instance()->showing_spill_for (_vca)) {
-		Mixer_UI::instance()->show_spill (boost::shared_ptr<Stripable>());
+		Mixer_UI::instance()->show_spill (std::shared_ptr<Stripable>());
 	}
 }
 
 Gdk::Color
 VCAMasterStrip::color () const
 {
-	return ARDOUR_UI_UTILS::gdk_color_from_rgba (_vca->presentation_info().color ());
+	return Gtkmm2ext::gdk_color_from_rgba (_vca->presentation_info().color ());
 }
 
 string
@@ -562,7 +598,7 @@ VCAMasterStrip::presentation_info () const
 	return _vca->presentation_info();
 }
 
-boost::shared_ptr<Stripable>
+std::shared_ptr<Stripable>
 VCAMasterStrip::stripable () const
 {
 	return _vca;
