@@ -1,21 +1,24 @@
 /*
-    Copyright (C) 2000 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2006-2011 David Robillard <d@drobilla.net>
+ * Copyright (C) 2006-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2007-2011 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2016-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2017 Julien "_FrnchFrgg_" RIVAUD <frnchfrgg@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_io_h__
 #define __ardour_io_h__
@@ -65,9 +68,9 @@ class UserBundle;
  * An IO can contain ports of varying types, making routes/inserts/etc with
  * varied combinations of types (eg MIDI and audio) possible.
  */
-class LIBARDOUR_API IO : public SessionObject, public Latent
+class LIBARDOUR_API IO : public SessionObject
 {
-  public:
+public:
 	static const std::string state_node_name;
 
 	enum Direction {
@@ -92,49 +95,54 @@ class LIBARDOUR_API IO : public SessionObject, public Latent
 	void set_pretty_name (const std::string& str);
 	std::string pretty_name () const { return _pretty_name_prefix; }
 
-	virtual void silence (framecnt_t);
-	void increment_port_buffer_offset (pframes_t offset);
+	virtual void silence (samplecnt_t);
 
 	int ensure_io (ChanCount cnt, bool clear, void *src);
 
-	int connect_ports_to_bundle (boost::shared_ptr<Bundle>, bool exclusive, void *);
-	int connect_ports_to_bundle (boost::shared_ptr<Bundle>, bool, bool, void *);
-	int disconnect_ports_from_bundle (boost::shared_ptr<Bundle>, void *);
+	int connect_ports_to_bundle (std::shared_ptr<Bundle>, bool exclusive, void *);
+	int connect_ports_to_bundle (std::shared_ptr<Bundle>, bool, bool, void *);
+	int disconnect_ports_from_bundle (std::shared_ptr<Bundle>, void *);
 
 	BundleList bundles_connected ();
 
-	boost::shared_ptr<Bundle> bundle () { return _bundle; }
+	std::shared_ptr<Bundle> bundle () { return _bundle; }
+
+	bool can_add_port (DataType) const;
 
 	int add_port (std::string connection, void *src, DataType type = DataType::NIL);
-	int remove_port (boost::shared_ptr<Port>, void *src);
-	int connect (boost::shared_ptr<Port> our_port, std::string other_port, void *src);
-	int disconnect (boost::shared_ptr<Port> our_port, std::string other_port, void *src);
+	int remove_port (std::shared_ptr<Port>, void *src);
+	int connect (std::shared_ptr<Port> our_port, std::string other_port, void *src);
+	int disconnect (std::shared_ptr<Port> our_port, std::string other_port, void *src);
 	int disconnect (void *src);
-	bool connected_to (boost::shared_ptr<const IO>) const;
+	bool connected_to (std::shared_ptr<const IO>) const;
 	bool connected_to (const std::string&) const;
 	bool connected () const;
 	bool physically_connected () const;
 
-	framecnt_t signal_latency () const { return 0; }
-	framecnt_t latency () const;
+	samplecnt_t latency () const;
+	samplecnt_t connected_latency (bool for_playback) const;
+
+	void set_private_port_latencies (samplecnt_t value, bool playback);
+	void set_public_port_latencies (samplecnt_t value, bool playback) const;
+	void set_public_port_latency_from_connections () const;
 
 	PortSet& ports() { return _ports; }
 	const PortSet& ports() const { return _ports; }
 
-	bool has_port (boost::shared_ptr<Port>) const;
+	bool has_port (std::shared_ptr<Port>) const;
 
-	boost::shared_ptr<Port> nth (uint32_t n) const {
+	std::shared_ptr<Port> nth (uint32_t n) const {
 		if (n < _ports.num_ports()) {
 			return _ports.port(n);
 		} else {
-			return boost::shared_ptr<Port> ();
+			return std::shared_ptr<Port> ();
 		}
 	}
 
-	boost::shared_ptr<Port> port_by_name (const std::string& str) const;
+	std::shared_ptr<Port> port_by_name (const std::string& str) const;
 
-	boost::shared_ptr<AudioPort> audio(uint32_t n) const;
-	boost::shared_ptr<MidiPort>  midi(uint32_t n) const;
+	std::shared_ptr<AudioPort> audio(uint32_t n) const;
+	std::shared_ptr<MidiPort>  midi(uint32_t n) const;
 
 	const ChanCount& n_ports ()  const { return _ports.count(); }
 
@@ -144,8 +152,8 @@ class LIBARDOUR_API IO : public SessionObject, public Latent
 	 */
 	PBD::Signal2<void, IOChange, void *> changed;
 
-	virtual XMLNode& state (bool full);
-	XMLNode& get_state (void);
+	XMLNode& get_state () const;
+
 	int set_state (const XMLNode&, int version);
 	int set_state_2X (const XMLNode&, int, bool);
 	static void prepare_for_reset (XMLNode&, const std::string&);
@@ -175,53 +183,40 @@ class LIBARDOUR_API IO : public SessionObject, public Latent
 	 */
 	PBD::Signal1<bool, ChanCount, BoolCombiner> PortCountChanging;
 
-	static int disable_connecting ();
-	static int enable_connecting ();
-
 	static PBD::Signal1<void, ChanCount> PortCountChanged; // emitted when the number of ports changes
 
 	static std::string name_from_state (const XMLNode&);
 	static void set_name_in_state (XMLNode&, const std::string&);
 
-	/* we have to defer/order port connection. this is how we do it.
-	*/
-
-	static PBD::Signal0<int> ConnectingLegal;
-	static bool              connecting_legal;
-
-	XMLNode *pending_state_node;
-	int pending_state_node_version;
-	bool pending_state_node_in;
-
 	/* three utility functions - this just seems to be simplest place to put them */
 
 	void collect_input (BufferSet& bufs, pframes_t nframes, ChanCount offset);
-	void process_input (boost::shared_ptr<Processor>, framepos_t start_frame, framepos_t end_frame, double speed, pframes_t nframes);
-	void copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, framecnt_t offset);
+	void copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, samplecnt_t offset);
 
 	/* AudioTrack::deprecated_use_diskstream_connections() needs these */
 
 	int set_ports (const std::string& str);
 
-  private:
-	mutable Glib::Threads::Mutex io_lock;
+protected:
+	virtual XMLNode& state () const;
 
-  protected:
-	PortSet   _ports;
 	Direction _direction;
 	DataType _default_type;
 	bool     _active;
 	bool     _sendish;
 
-  private:
-	int connecting_became_legal ();
-	PBD::ScopedConnection connection_legal_c;
+private:
+	mutable Glib::Threads::RWLock _io_lock;
+	PortSet   _ports;
 
-	boost::shared_ptr<Bundle> _bundle; ///< a bundle representing our ports
+	void reestablish_port_subscriptions ();
+	PBD::ScopedConnectionList _port_connections;
+
+	std::shared_ptr<Bundle> _bundle; ///< a bundle representing our ports
 
 	struct UserBundleInfo {
-		UserBundleInfo (IO*, boost::shared_ptr<UserBundle> b);
-		boost::shared_ptr<UserBundle> bundle;
+		UserBundleInfo (IO*, std::shared_ptr<UserBundle> b);
+		std::shared_ptr<UserBundle> bundle;
 		PBD::ScopedConnection changed;
 	};
 
@@ -231,14 +226,13 @@ class LIBARDOUR_API IO : public SessionObject, public Latent
 	int ensure_ports (ChanCount, bool clear, void *src);
 
 	void bundle_changed (Bundle::Change);
+	int set_port_state_2X (const XMLNode& node, int version, bool in);
 
-	int get_port_counts (const XMLNode& node, int version, ChanCount& n, boost::shared_ptr<Bundle>& c);
-	int get_port_counts_2X (const XMLNode& node, int version, ChanCount& n, boost::shared_ptr<Bundle>& c);
+	int get_port_counts (const XMLNode& node, int version, ChanCount& n, std::shared_ptr<Bundle>& c);
+	int get_port_counts_2X (const XMLNode& node, int version, ChanCount& n, std::shared_ptr<Bundle>& c);
 	int create_ports (const XMLNode&, int version);
-	int make_connections (const XMLNode&, int, bool);
-	int make_connections_2X (const XMLNode &, int, bool);
 
-	boost::shared_ptr<Bundle> find_possible_bundle (const std::string &desired_name);
+	std::shared_ptr<Bundle> find_possible_bundle (const std::string &desired_name);
 
 	int ensure_ports_locked (ChanCount, bool clear, bool& changed);
 
@@ -251,7 +245,7 @@ class LIBARDOUR_API IO : public SessionObject, public Latent
 	void apply_pretty_name ();
 	std::string _pretty_name_prefix;
 	BufferSet _buffers;
-	void disconnect_check (boost::shared_ptr<ARDOUR::Port>, boost::shared_ptr<ARDOUR::Port>);
+	void connection_change (std::shared_ptr<ARDOUR::Port>, std::shared_ptr<ARDOUR::Port>);
 };
 
 } // namespace ARDOUR
