@@ -1,28 +1,28 @@
 /*
-    Copyright (C) 2016 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify it
-    under the terms of the GNU General Public License as published by the Free
-    Software Foundation; either version 2 of the License, or (at your option)
-    any later version.
-
-    This program is distributed in the hope that it will be useful, but WITHOUT
-    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-    for more details.
-
-    You should have received a copy of the GNU General Public License along
-    with this program; if not, write to the Free Software Foundation, Inc.,
-    675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2016-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2016-2017 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_mute_control_h__
 #define __ardour_mute_control_h__
 
+#include <memory>
 #include <string>
 
-#include <boost/shared_ptr.hpp>
-
 #include "ardour/slavable_automation_control.h"
 
 #include "ardour/mute_master.h"
@@ -36,7 +36,7 @@ class Muteable;
 class LIBARDOUR_API MuteControl : public SlavableAutomationControl
 {
 public:
-	MuteControl (Session& session, std::string const& name, Muteable&);
+	MuteControl (Session& session, std::string const& name, Muteable&, Temporal::TimeDomain td);
 
 	double get_value () const;
 	double get_save_value() const { return muted_by_self(); }
@@ -68,14 +68,14 @@ public:
 	void set_mute_points (MuteMaster::MutePoint);
 	MuteMaster::MutePoint mute_points () const;
 
-	void automation_run (framepos_t start, pframes_t nframes);
+	void automation_run (samplepos_t start, pframes_t nframes);
 
 protected:
-	bool handle_master_change (boost::shared_ptr<AutomationControl>);
+	bool handle_master_change (std::shared_ptr<AutomationControl>);
 	void actually_set_value (double, PBD::Controllable::GroupControlDisposition group_override);
 
-	void pre_remove_master (boost::shared_ptr<AutomationControl>);
-	void post_add_master (boost::shared_ptr<AutomationControl>);
+	void pre_remove_master (std::shared_ptr<AutomationControl>);
+	void post_add_master (std::shared_ptr<AutomationControl>);
 
 private:
 	Muteable& _muteable;
