@@ -1,28 +1,31 @@
 /*
-    Copyright (C) 2012 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2009-2018 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2010-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2010-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2015-2018 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef __ardour_session_event_h__
 #define __ardour_session_event_h__
 
 #include <list>
+#include <memory>
+
 #include <boost/function.hpp>
-#include <boost/shared_ptr.hpp>
 
 #include "pbd/pool.h"
 #include "pbd/ringbuffer.h"
@@ -33,27 +36,26 @@
 
 namespace ARDOUR {
 
-class Slave;
+class TransportMaster;
 class Region;
+class Track;
 
 class LIBARDOUR_API SessionEvent {
 public:
 	enum Type {
 		SetTransportSpeed,
-		SetTrackSpeed,
+		SetDefaultPlaySpeed,
 		Locate,
 		LocateRoll,
 		LocateRollLocate,
 		SetLoop,
 		PunchIn,
 		PunchOut,
-		RecordStart,
 		RangeStop,
 		RangeLocate,
 		Overwrite,
-		SetSyncSource,
+		OverwriteAll,
 		Audition,
-		InputConfigurationChange,
 		SetPlayAudioRange,
 		CancelPlayAudioRange,
 		RealTimeOperation,
@@ -61,12 +63,15 @@ public:
 		AdjustCaptureBuffering,
 		SetTimecodeTransmission,
 		Skip,
+		SetTransportMaster,
+		StartRoll,
+		EndRoll,
+		TransportStateChange,
+		SyncCues,
 
 		/* only one of each of these events can be queued at any one time */
 
-		StopOnce,
 		AutoLoop,
-		AutoLoopDeclick,
 	};
 
 	enum Action {
@@ -78,21 +83,23 @@ public:
 
 	Type       type;
 	Action     action;
-	framepos_t action_frame;
-	framepos_t target_frame;
+	samplepos_t action_sample;
+	samplepos_t target_sample;
 	double     speed;
 
 	union {
-		void*        ptr;
-		bool         yes_or_no;
-		framepos_t   target2_frame;
-		Slave*       slave;
-		Route*       route;
+		bool             yes_or_no;
+		samplepos_t      target2_sample;
+		OverwriteReason  overwrite;
+		int32_t          scene;
 	};
 
+	std::weak_ptr<Track> track;
+
 	union {
 		bool second_yes_or_no;
 		double control_value;
+		LocateTransportDisposition locate_transport_disposition;
 	};
 
 	union {
@@ -103,29 +110,30 @@ public:
 
 	typedef boost::function<void (SessionEvent*)> RTeventCallback;
 
-	boost::shared_ptr<ControlList> controls; /* apply to */
-	boost::shared_ptr<RouteList> routes;     /* apply to */
+	std::shared_ptr<ControlList> controls; /* apply to */
+	std::shared_ptr<RouteList> routes;     /* apply to */
 	boost::function<void (void)> rt_slot;    /* what to call in RT context */
 	RTeventCallback              rt_return;  /* called after rt_slot, with this event as an argument */
 	PBD::EventLoop*              event_loop;
 
-	std::list<AudioRange> audio_range;
-	std::list<MusicRange> music_range;
+	std::list<TimelineRange> audio_range;
+	std::list<TimelineRange> music_range;
 
-	boost::shared_ptr<Region> region;
+	std::shared_ptr<Region> region;
+	std::shared_ptr<TransportMaster> transport_master;
 
-	SessionEvent (Type t, Action a, framepos_t when, framepos_t where, double spd, bool yn = false, bool yn2 = false, bool yn3 = false);
+	SessionEvent (Type t, Action a, samplepos_t when, samplepos_t where, double spd, bool yn = false, bool yn2 = false, bool yn3 = false);
 
-	void set_ptr (void* p) {
-		ptr = p;
+	void set_track (std::shared_ptr<Track> t) {
+		track = t;
 	}
 
 	bool before (const SessionEvent& other) const {
-		return action_frame < other.action_frame;
+		return action_sample < other.action_sample;
 	}
 
 	bool after (const SessionEvent& other) const {
-		return action_frame > other.action_frame;
+		return action_sample > other.action_sample;
 	}
 
 	static bool compare (const SessionEvent *e1, const SessionEvent *e2) {
@@ -135,17 +143,18 @@ public:
 	void* operator new (size_t);
 	void  operator delete (void *ptr, size_t /*size*/);
 
-	static const framepos_t Immediate = -1;
+	static const samplepos_t Immediate = -1;
 
 	static bool has_per_thread_pool ();
 	static void create_per_thread_pool (const std::string& n, uint32_t nitems);
 	static void init_event_pool ();
+	static guint pool_available ();
 
-	CrossThreadPool* event_pool() const { return own_pool; }
+	PBD::CrossThreadPool* event_pool() const { return own_pool; }
 
 private:
-	static PerThreadPool* pool;
-	CrossThreadPool* own_pool;
+	static PBD::PerThreadPool* pool;
+	PBD::CrossThreadPool*      own_pool;
 
 	friend class Butler;
 };
@@ -161,7 +170,7 @@ public:
 	void clear_events (SessionEvent::Type type, boost::function<void (void)> after);
 
 protected:
-	RingBuffer<SessionEvent*> pending_events;
+	PBD::RingBuffer<SessionEvent*> pending_events;
 	typedef std::list<SessionEvent *> Events;
 	Events           events;
 	Events           immediate_events;
@@ -177,13 +186,13 @@ protected:
 
 	void dump_events () const;
 	void merge_event (SessionEvent*);
-	void replace_event (SessionEvent::Type, framepos_t action_frame, framepos_t target = 0);
+	void replace_event (SessionEvent::Type, samplepos_t action_sample, samplepos_t target = 0);
 	bool _replace_event (SessionEvent*);
 	bool _remove_event (SessionEvent *);
 	void _clear_event_type (SessionEvent::Type);
 
-	void add_event (framepos_t action_frame, SessionEvent::Type type, framepos_t target_frame = 0);
-	void remove_event (framepos_t frame, SessionEvent::Type type);
+	void add_event (samplepos_t action_sample, SessionEvent::Type type, samplepos_t target_sample = 0);
+	void remove_event (samplepos_t sample, SessionEvent::Type type);
 
 	virtual void process_event(SessionEvent*) = 0;
 	virtual void set_next_event () = 0;
@@ -191,4 +200,6 @@ protected:
 
 } /* namespace */
 
+LIBARDOUR_API std::ostream& operator<<(std::ostream&, const ARDOUR::SessionEvent&);
+
 #endif /* __ardour_session_event_h__ */
