@@ -1,21 +1,28 @@
 /*
-    Copyright (C) 2002 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2005-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2006-2016 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2008-2010 Sakari Bergen <sakari.bergen@beatwaves.net>
+ * Copyright (C) 2008 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2012-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2013-2014 John Emmas <john@creativepost.co.uk>
+ * Copyright (C) 2013-2015 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2015 GZharun <grygoriiz@wavesglobal.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <unistd.h>
 #include <cerrno>
@@ -32,13 +39,16 @@
 #include "pbd/epa.h"
 #include "pbd/file_utils.h"
 #include "pbd/pthread_utils.h"
-#include "pbd/stacktrace.h"
 #include "pbd/unknown_type.h"
 
+#include "temporal/superclock.h"
+#include "temporal/tempo.h"
+
 #include "midi++/port.h"
 #include "midi++/mmc.h"
 
 #include "ardour/async_midi_port.h"
+#include "ardour/ardour.h"
 #include "ardour/audio_port.h"
 #include "ardour/audio_backend.h"
 #include "ardour/audioengine.h"
@@ -55,6 +65,7 @@
 #include "ardour/process_thread.h"
 #include "ardour/rc_configuration.h"
 #include "ardour/session.h"
+#include "ardour/transport_master_manager.h"
 
 #include "pbd/i18n.h"
 
@@ -64,7 +75,7 @@ using namespace PBD;
 
 AudioEngine* AudioEngine::_instance = 0;
 
-static gint audioengine_thread_cnt = 1;
+static std::atomic<int> audioengine_thread_cnt (1);
 
 #ifdef SILENCE_AFTER
 #define SILENCE_AFTER_SECONDS 600
@@ -73,30 +84,26 @@ static gint audioengine_thread_cnt = 1;
 AudioEngine::AudioEngine ()
 	: session_remove_pending (false)
 	, session_removal_countdown (-1)
+	, session_deleted (false)
 	, _running (false)
 	, _freewheeling (false)
 	, monitor_check_interval (INT32_MAX)
 	, last_monitor_check (0)
-	, _processed_frames (0)
-	, m_meter_thread (0)
+	, _processed_samples (-1)
 	, _main_thread (0)
 	, _mtdm (0)
 	, _mididm (0)
 	, _measuring_latency (MeasureNone)
-	, _latency_input_port (0)
-	, _latency_output_port (0)
-	, _latency_flush_frames (0)
+	, _latency_flush_samples (0)
 	, _latency_signal_latency (0)
 	, _stopped_for_latency (false)
 	, _started_for_latency (false)
 	, _in_destructor (false)
-	, _last_backend_error_string(AudioBackend::get_error_string((AudioBackend::ErrorCode)-1))
+	, _last_backend_error_string(AudioBackend::get_error_string(AudioBackend::NoError))
 	, _hw_reset_event_thread(0)
-	, _hw_reset_request_count(0)
-	, _stop_hw_reset_processing(0)
 	, _hw_devicelist_update_thread(0)
-	, _hw_devicelist_update_count(0)
-	, _stop_hw_devicelist_processing(0)
+	, _start_cnt (0)
+	, _init_countdown (0)
 #ifdef SILENCE_AFTER_SECONDS
 	, _silence_countdown (0)
 	, _silence_hit_cnt (0)
@@ -105,6 +112,13 @@ AudioEngine::AudioEngine ()
 	reset_silence_countdown ();
 	start_hw_event_processing();
 	discover_backends ();
+
+	_hw_reset_request_count.store (0);
+	_pending_playback_latency_callback.store (0);
+	_pending_capture_latency_callback.store (0);
+	_hw_devicelist_update_count.store (0);
+	_stop_hw_reset_processing.store (0);
+	_stop_hw_devicelist_processing.store (0);
 }
 
 AudioEngine::~AudioEngine ()
@@ -131,18 +145,44 @@ AudioEngine::create ()
 }
 
 void
-AudioEngine::split_cycle (pframes_t offset)
+AudioEngine::split_cycle (pframes_t nframes)
 {
 	/* caller must hold process lock */
 
-	Port::increment_global_port_buffer_offset (offset);
+	std::shared_ptr<Ports const> p = _ports.reader();
+
+	/* This is mainly for the benefit of rt-control ports (MTC, MClk)
+	 *
+	 * Normally ports are flushed by the route:
+	 *   ARDOUR::MidiPort::flush_buffers(unsigned int)
+	 *   ARDOUR::Delivery::flush_buffers(long)
+	 *   ARDOUR::Route::flush_processor_buffers_locked(long)
+	 *   ARDOUR::Route::run_route(long, long, unsigned int, bool, bool)
+	 *   ...
+	 *
+	 * This is required so that route -> route connections work during
+	 * normal processing.
+	 *
+	 * However some non-route ports may contain MIDI events
+	 * from current Port::port_offset() .. Port::port_offset() + nframes.
+	 * If those events are not pushed to ports before the cycle split,
+	 * MidiPort::flush_buffers will drop them (event time is out of bounds).
+	 *
+	 * TODO: for optimized builds MidiPort::flush_buffers() could
+	 * be relaxed, ignore ev->time() checks, and simply send
+	 * all events as-is.
+	 */
+	for (auto const& i : *p) {
+		i.second->flush_buffers (nframes);
+	}
+
+	Port::increment_global_port_buffer_offset (nframes);
 
 	/* tell all Ports that we're going to start a new (split) cycle */
 
-	boost::shared_ptr<Ports> p = ports.reader();
 
-	for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
-		i->second->cycle_split ();
+	for (auto const& i : *p) {
+		i.second->cycle_split ();
 	}
 }
 
@@ -155,7 +195,9 @@ AudioEngine::sample_rate_change (pframes_t nframes)
 	last_monitor_check = 0;
 
 	if (_session) {
-		_session->set_frame_rate (nframes);
+		_session->set_sample_rate (nframes);
+	} else {
+		Temporal::set_sample_rate (nframes);
 	}
 
 	SampleRateChanged (nframes); /* EMIT SIGNAL */
@@ -170,6 +212,9 @@ AudioEngine::sample_rate_change (pframes_t nframes)
 int
 AudioEngine::buffer_size_change (pframes_t bufsiz)
 {
+	Glib::Threads::Mutex::Lock pl (_process_lock);
+	set_port_buffer_sizes (bufsiz);
+
 	if (_session) {
 		_session->set_block_size (bufsiz);
 		last_monitor_check = 0;
@@ -181,7 +226,7 @@ AudioEngine::buffer_size_change (pframes_t bufsiz)
 }
 
 /** Method called by our ::process_thread when there is work to be done.
- *  @param nframes Number of frames to process.
+ *  @param nframes Number of samples to process.
  */
 #ifdef __clang__
 __attribute__((annotate("realtime")))
@@ -189,20 +234,27 @@ __attribute__((annotate("realtime")))
 int
 AudioEngine::process_callback (pframes_t nframes)
 {
+	TimerRAII tr (dsp_stats[ProcessCallback]);
 	Glib::Threads::Mutex::Lock tm (_process_lock, Glib::Threads::TRY_LOCK);
+	Port::set_varispeed_ratio (1.0);
 
 	PT_TIMING_REF;
 	PT_TIMING_CHECK (1);
 
-	/// The number of frames that will have been processed when we've finished
-	pframes_t next_processed_frames;
+	/// The number of samples that will have been processed when we've finished
+	pframes_t next_processed_samples;
 
-	/* handle wrap around of total frames counter */
+	if (_processed_samples < 0) {
+		_processed_samples = sample_time();
+		cerr << "IIIIINIT PS to " << _processed_samples << endl;
+	}
 
-	if (max_framepos - _processed_frames < nframes) {
-		next_processed_frames = nframes - (max_framepos - _processed_frames);
+	/* handle wrap around of total samples counter */
+
+	if (max_samplepos - _processed_samples < nframes) {
+		next_processed_samples = nframes - (max_samplepos - _processed_samples);
 	} else {
-		next_processed_frames = _processed_frames + nframes;
+		next_processed_samples = _processed_samples + nframes;
 	}
 
 	if (!tm.locked()) {
@@ -210,12 +262,12 @@ AudioEngine::process_callback (pframes_t nframes)
 		if (_session) {
 			Xrun();
 		}
-		/* really only JACK requires this
-		 * (other backends clear the output buffers
-		 * before the process_callback. it may even be
-		 * jack/alsa only). but better safe than sorry.
+		/* only JACK requires this (other backends clear the
+		 * output buffers before the process_callback.
 		 */
-		PortManager::silence_outputs (nframes);
+		if (!session_deleted) {
+			PortManager::silence_outputs (nframes);
+		}
 		return 0;
 	}
 
@@ -239,6 +291,74 @@ AudioEngine::process_callback (pframes_t nframes)
 		thread_init_callback (NULL);
 	}
 
+	Temporal::TempoMap::SharedPtr current_map = Temporal::TempoMap::read ();
+	if (current_map != Temporal::TempoMap::use()) {
+		Temporal::TempoMap::set (current_map);
+	}
+
+	/* This is for JACK, where the latency callback arrives in sync with
+	 * port registration (usually while ardour holds the process-lock
+	 * or with _adding_routes_in_progress or _route_deletion_in_progress set,
+	 * potentially while processing in parallel.
+	 *
+	 * Note: this must be done without holding the _process_lock
+	 */
+	if (_session && !_session->processing_blocked ()) {
+		bool lp = false;
+		bool lc = false;
+		int canderef (1);
+		if (_pending_playback_latency_callback.compare_exchange_strong (canderef, 0)) {
+			lp = true;
+		}
+		canderef = 1;
+		if (_pending_capture_latency_callback.compare_exchange_strong (canderef, 0)) {
+			lc = true;
+		}
+		if (lp || lc) {
+			/* synchronize with Session::processing_blocked
+			 * This lock is not contended by this thread, but acts
+			 * as barrier for Session::block_processing (Session::write_one_track).
+			 */
+			Glib::Threads::Mutex::Lock ll (_latency_lock, Glib::Threads::TRY_LOCK);
+			/* re-check after talking latency-lock */
+			if (!ll.locked () || _session->processing_blocked ()) {
+				if (lc) {
+					queue_latency_update (false);
+				}
+				if (lp) {
+					queue_latency_update (true);
+				}
+			} else {
+				/* release process-lock to call Session::update_latency */
+				tm.release ();
+				if (lc) {
+					_session->update_latency (false);
+				}
+				if (lp) {
+					_session->update_latency (true);
+				}
+				/* release latency lock, **before** reacquiring process-lock */
+				ll.release ();
+				tm.acquire ();
+			}
+		}
+	}
+
+	if (_session && _init_countdown > 0) {
+		--_init_countdown;
+		/* Warm up caches */
+		PortManager::cycle_start (nframes, _session);
+		_session->process (nframes);
+		PortManager::silence (nframes);
+		PortManager::cycle_end (nframes);
+		if (_init_countdown == 0) {
+			_session->reset_xrun_count();
+			ARDOUR::reset_performance_meters (_session);
+		}
+
+		return 0;
+	}
+
 	bool return_after_remove_check = false;
 
 	if (_measuring_latency == MeasureAudio && _mtdm) {
@@ -286,7 +406,7 @@ AudioEngine::process_callback (pframes_t nframes)
 		PortManager::cycle_end (nframes);
 		return_after_remove_check = true;
 
-	} else if (_latency_flush_frames) {
+	} else if (_latency_flush_samples) {
 
 		/* wait for the appropriate duration for the MTDM signal to
 		 * drain from the ports before we revert to normal behaviour.
@@ -296,10 +416,10 @@ AudioEngine::process_callback (pframes_t nframes)
 		PortManager::silence (nframes);
 		PortManager::cycle_end (nframes);
 
-		if (_latency_flush_frames > nframes) {
-			_latency_flush_frames -= nframes;
+		if (_latency_flush_samples > nframes) {
+			_latency_flush_samples -= nframes;
 		} else {
-			_latency_flush_frames = 0;
+			_latency_flush_samples = 0;
 		}
 
 		return_after_remove_check = true;
@@ -329,12 +449,15 @@ AudioEngine::process_callback (pframes_t nframes)
 			*/
 
 			if (session_removal_countdown <= nframes) {
+				assert (_session);
 				_session->midi_panic ();
 			}
 
 		} else {
 			/* fade out done */
-			_session = 0;
+			PortManager::silence_outputs (nframes);
+			session_deleted = true;
+			SessionHandlePtr::set_session (0);
 			session_removal_countdown = -1; // reset to "not in progress"
 			session_remove_pending = false;
 			session_removed.signal(); // wakes up thread that initiated session removal
@@ -345,25 +468,64 @@ AudioEngine::process_callback (pframes_t nframes)
 		return 0;
 	}
 
+	TransportMasterManager& tmm (TransportMasterManager::instance());
+
+	/* make sure the TMM is up to date about the current session */
+
+	if (_session != tmm.session()) {
+		tmm.set_session (_session);
+	}
+
 	if (_session == 0) {
 
-		if (!_freewheeling) {
-			PortManager::cycle_start (nframes);
-			PortManager::cycle_end (nframes);
+		if (!_freewheeling && !session_deleted) {
+			PortManager::silence_outputs (nframes);
 		}
 
-		_processed_frames = next_processed_frames;
+		_processed_samples = next_processed_samples;
 
 		return 0;
 	}
 
+	if (!_freewheeling || Freewheel.empty()) {
+		/* catch_speed is the speed that we estimate we need to run at
+		   to catch (or remain locked to) a transport master.
+		*/
+		double catch_speed = tmm.pre_process_transport_masters (nframes, sample_time_at_cycle_start());
+		catch_speed = _session->plan_master_strategy (nframes, tmm.get_current_speed_in_process_context(), tmm.get_current_position_in_process_context(), catch_speed);
+		Port::set_varispeed_ratio (catch_speed);
+		DEBUG_TRACE (DEBUG::Slave, string_compose ("transport master (current=%1) gives speed %2 (ports using %3)\n", tmm.current() ? tmm.current()->name() : string("[]"), catch_speed, Port::speed_ratio()));
+
+#if 0 // USE FOR DEBUG ONLY
+		/* use with Dummy backend, engine pulse and
+		 * scripts/_find_nonzero_sample.lua
+		 * to correlate with recorded region alignment.
+		 */
+		static bool was_rolling = false;
+		bool is_rolling = _session->transport_rolling();
+		if (!was_rolling && is_rolling) {
+			samplepos_t stacs = sample_time_at_cycle_start ();
+			samplecnt_t sr = sample_rate ();
+			samplepos_t tp = _session->transport_sample ();
+			/* Note: this does not take Port latency into account:
+			 * - always add 12 samples (Port::_resampler_quality)
+			 * - ExistingMaterial: subtract playback latency from engine-pulse
+			 *   We assume the player listens and plays along. Recorded region is moved
+			 *   back by playback_latency
+			 */
+			printf (" ******** Starting play at %ld, next pulse: %ld\n", stacs, ((sr - (stacs % sr)) %sr) + tp);
+		}
+		was_rolling = is_rolling;
+#endif
+	}
+
 	/* tell all relevant objects that we're starting a new cycle */
 
 	InternalSend::CycleStart (nframes);
 
 	/* tell all Ports that we're starting a new cycle */
 
-	PortManager::cycle_start (nframes);
+	PortManager::cycle_start (nframes, _session);
 
 	/* test if we are freewheeling and there are freewheel signals connected.
 	 * ardour should act normally even when freewheeling unless /it/ is
@@ -373,23 +535,52 @@ AudioEngine::process_callback (pframes_t nframes)
 	if (_freewheeling && !Freewheel.empty()) {
 		Freewheel (nframes);
 	} else {
-		_session->process (nframes);
+		samplepos_t start_sample = _session->transport_sample ();
+		samplecnt_t pre_roll = _session->remaining_latency_preroll ();
+
+		if (Port::cycle_nframes () <= nframes) {
+			_session->process (Port::cycle_nframes ());
+		} else {
+			pframes_t remain = Port::cycle_nframes ();
+			while (remain > 0) {
+				/* keep track of split_cycle() calls by Session::process */
+				samplecnt_t poff = Port::port_offset ();
+				pframes_t nf = std::min (remain, nframes);
+				_session->process (nf);
+				remain -= nf;
+				if (remain > 0) {
+					/* calculate split-cycle offset */
+					samplecnt_t delta = Port::port_offset () - poff;
+					assert (delta >= 0 && delta <= nf);
+					if (nf > delta) {
+						split_cycle (nf - delta);
+					}
+				}
+			}
+		}
+
+		/* send timecode for current cycle */
+		samplepos_t end_sample = _session->transport_sample ();
+		_session->send_ltc_for_cycle (start_sample, end_sample, nframes);
+		/* and MIDI Clock */
+		_session->send_mclk_for_cycle (start_sample, end_sample, nframes, pre_roll);
 	}
 
 	if (_freewheeling) {
-		PortManager::cycle_end (nframes);
+		PortManager::cycle_end (nframes, _session);
 		return 0;
 	}
 
 	if (!_running) {
-		_processed_frames = next_processed_frames;
+		_processed_samples = next_processed_samples;
+		PortManager::cycle_end (nframes, _session);
 		return 0;
 	}
 
-	if (last_monitor_check + monitor_check_interval < next_processed_frames) {
+	if (last_monitor_check + monitor_check_interval < next_processed_samples) {
 
 		PortManager::check_monitoring ();
-		last_monitor_check = next_processed_frames;
+		last_monitor_check = next_processed_samples;
 	}
 
 #ifdef SILENCE_AFTER_SECONDS
@@ -419,7 +610,7 @@ AudioEngine::process_callback (pframes_t nframes)
 
 	if (session_remove_pending && session_removal_countdown) {
 
-		PortManager::fade_out (session_removal_gain, session_removal_gain_step, nframes);
+		PortManager::cycle_end_fade_out (session_removal_gain, session_removal_gain_step, nframes, _session);
 
 		if (session_removal_countdown > nframes) {
 			session_removal_countdown -= nframes;
@@ -428,11 +619,11 @@ AudioEngine::process_callback (pframes_t nframes)
 		}
 
 		session_removal_gain -= (nframes * session_removal_gain_step);
+	} else {
+		PortManager::cycle_end (nframes, _session);
 	}
 
-	PortManager::cycle_end (nframes);
-
-	_processed_frames = next_processed_frames;
+	_processed_samples = next_processed_samples;
 
 	PT_TIMING_CHECK (2);
 
@@ -467,31 +658,32 @@ void
 AudioEngine::request_backend_reset()
 {
 	Glib::Threads::Mutex::Lock guard (_reset_request_lock);
-	g_atomic_int_inc (&_hw_reset_request_count);
+	_hw_reset_request_count.fetch_add (1);
 	_hw_reset_condition.signal ();
 }
 
 int
 AudioEngine::backend_reset_requested()
 {
-	return g_atomic_int_get (&_hw_reset_request_count);
+	return _hw_reset_request_count.load ();
 }
 
 void
 AudioEngine::do_reset_backend()
 {
 	SessionEvent::create_per_thread_pool (X_("Backend reset processing thread"), 1024);
+	pthread_set_name ("EngineWatchdog");
 
 	Glib::Threads::Mutex::Lock guard (_reset_request_lock);
 
-	while (!_stop_hw_reset_processing) {
+	while (!_stop_hw_reset_processing.load ()) {
 
-		if (g_atomic_int_get (&_hw_reset_request_count) != 0 && _backend) {
+		if (_hw_reset_request_count.load () != 0 && _backend) {
 
 			_reset_request_lock.unlock();
 
 			Glib::Threads::RecMutex::Lock pl (_state_lock);
-			g_atomic_int_dec_and_test (&_hw_reset_request_count);
+			PBD::atomic_dec_and_test (_hw_reset_request_count);
 
 			std::cout << "AudioEngine::RESET::Reset request processing. Requests left: " << _hw_reset_request_count << std::endl;
 			DeviceResetStarted(); // notify about device reset to be started
@@ -499,7 +691,7 @@ AudioEngine::do_reset_backend()
 			// backup the device name
 			std::string name = _backend->device_name ();
 
-			std::cout << "AudioEngine::RESET::Reseting device..." << std::endl;
+			std::cout << "AudioEngine::RESET::Resetting device..." << std::endl;
 			if ( ( 0 == stop () ) &&
 					( 0 == _backend->reset_device () ) &&
 					( 0 == start () ) ) {
@@ -533,7 +725,7 @@ void
 AudioEngine::request_device_list_update()
 {
 	Glib::Threads::Mutex::Lock guard (_devicelist_update_lock);
-	g_atomic_int_inc (&_hw_devicelist_update_count);
+	_hw_devicelist_update_count.fetch_add (1);
 	_hw_devicelist_update_condition.signal ();
 }
 
@@ -541,18 +733,19 @@ void
 AudioEngine::do_devicelist_update()
 {
 	SessionEvent::create_per_thread_pool (X_("Device list update processing thread"), 512);
+	pthread_set_name ("DeviceList");
 
 	Glib::Threads::Mutex::Lock guard (_devicelist_update_lock);
 
 	while (!_stop_hw_devicelist_processing) {
 
-		if (_hw_devicelist_update_count) {
+		if (_hw_devicelist_update_count.load ()) {
 
 			_devicelist_update_lock.unlock();
 
 			Glib::Threads::RecMutex::Lock pl (_state_lock);
 
-			g_atomic_int_dec_and_test (&_hw_devicelist_update_count);
+			PBD::atomic_dec_and_test (_hw_devicelist_update_count);
 			DeviceListChanged (); /* EMIT SIGNAL */
 
 			_devicelist_update_lock.lock();
@@ -568,15 +761,15 @@ void
 AudioEngine::start_hw_event_processing()
 {
 	if (_hw_reset_event_thread == 0) {
-		g_atomic_int_set(&_hw_reset_request_count, 0);
-		g_atomic_int_set(&_stop_hw_reset_processing, 0);
-		_hw_reset_event_thread = Glib::Threads::Thread::create (boost::bind (&AudioEngine::do_reset_backend, this));
+		_hw_reset_request_count.store (0);
+		_stop_hw_reset_processing.store (0);
+		_hw_reset_event_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_reset_backend, this));
 	}
 
 	if (_hw_devicelist_update_thread == 0) {
-		g_atomic_int_set(&_hw_devicelist_update_count, 0);
-		g_atomic_int_set(&_stop_hw_devicelist_processing, 0);
-		_hw_devicelist_update_thread = Glib::Threads::Thread::create (boost::bind (&AudioEngine::do_devicelist_update, this));
+		_hw_devicelist_update_count.store (0);
+		_stop_hw_devicelist_processing.store (0);
+		_hw_devicelist_update_thread = PBD::Thread::create (boost::bind (&AudioEngine::do_devicelist_update, this));
 	}
 }
 
@@ -585,16 +778,16 @@ void
 AudioEngine::stop_hw_event_processing()
 {
 	if (_hw_reset_event_thread) {
-		g_atomic_int_set(&_stop_hw_reset_processing, 1);
-		g_atomic_int_set(&_hw_reset_request_count, 0);
+		_stop_hw_reset_processing.store (1);
+		_hw_reset_request_count.store (0);
 		_hw_reset_condition.signal ();
 		_hw_reset_event_thread->join ();
 		_hw_reset_event_thread = 0;
 	}
 
 	if (_hw_devicelist_update_thread) {
-		g_atomic_int_set(&_stop_hw_devicelist_processing, 1);
-		g_atomic_int_set(&_hw_devicelist_update_count, 0);
+		_stop_hw_devicelist_processing.store (1);
+		_hw_devicelist_update_count.store (0);
 		_hw_devicelist_update_condition.signal ();
 		_hw_devicelist_update_thread->join ();
 		_hw_devicelist_update_thread = 0;
@@ -609,21 +802,10 @@ AudioEngine::set_session (Session *s)
 	SessionHandlePtr::set_session (s);
 
 	if (_session) {
-
-		pframes_t blocksize = samples_per_cycle ();
-
-		PortManager::cycle_start (blocksize);
-
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-		_session->process (blocksize);
-
-		PortManager::cycle_end (blocksize);
+		session_deleted = false;
+		_init_countdown = std::max (4, (int)(_backend->sample_rate () / _backend->buffer_size ()) / 8);
+		_pending_playback_latency_callback.store (0);
+		_pending_capture_latency_callback.store (0);
 	}
 }
 
@@ -642,24 +824,13 @@ AudioEngine::remove_session ()
 		}
 
 	} else {
+		session_deleted = true;
 		SessionHandlePtr::set_session (0);
 	}
 
 	remove_all_ports ();
 }
 
-
-void
-AudioEngine::reconnect_session_routes (bool reconnect_inputs, bool reconnect_outputs)
-{
-#ifdef USE_TRACKS_CODE_FEATURES
-	if (_session) {
-		_session->reconnect_existing_routes(true, true, reconnect_inputs, reconnect_outputs);
-	}
-#endif
-}
-
-
 void
 AudioEngine::died ()
 {
@@ -813,32 +984,33 @@ void
 AudioEngine::drop_backend ()
 {
 	if (_backend) {
+		/* see also ::stop() */
 		_backend->stop ();
-		// Stopped is needed for Graph to explicitly terminate threads
+		_running = false;
+		if (_session && !_session->loading() && !_session->deletion_in_progress()) {
+			// it's not a halt, but should be handled the same way:
+			// disable record, stop transport and I/O processign but save the data.
+			_session->engine_halted ();
+		}
+		Port::PortDrop (); /* EMIT SIGNAL */
+		TransportMasterManager& tmm (TransportMasterManager::instance());
+		tmm.engine_stopped ();
+		tmm.set_session (0); // unregister TMM ports
+
+		/* Stopped is needed for Graph to explicitly terminate threads */
 		Stopped (); /* EMIT SIGNAL */
 		_backend->drop_device ();
 		_backend.reset ();
-		_running = false;
 	}
 }
 
-boost::shared_ptr<AudioBackend>
-AudioEngine::set_default_backend ()
-{
-	if (_backends.empty()) {
-		return boost::shared_ptr<AudioBackend>();
-	}
-
-	return set_backend (_backends.begin()->first, "", "");
-}
-
-boost::shared_ptr<AudioBackend>
+std::shared_ptr<AudioBackend>
 AudioEngine::set_backend (const std::string& name, const std::string& arg1, const std::string& arg2)
 {
 	BackendMap::iterator b = _backends.find (name);
 
 	if (b == _backends.end()) {
-		return boost::shared_ptr<AudioBackend>();
+		return std::shared_ptr<AudioBackend>();
 	}
 
 	drop_backend ();
@@ -852,7 +1024,7 @@ AudioEngine::set_backend (const std::string& name, const std::string& arg1, cons
 
 	} catch (exception& e) {
 		error << string_compose (_("Could not create backend for %1: %2"), name, e.what()) << endmsg;
-		return boost::shared_ptr<AudioBackend>();
+		return std::shared_ptr<AudioBackend>();
 	}
 
 	return _backend;
@@ -867,11 +1039,15 @@ AudioEngine::start (bool for_latency)
 		return -1;
 	}
 
+	if (_running && _backend->can_change_systemic_latency_when_running()) {
+		_started_for_latency = for_latency;
+	}
+
 	if (_running) {
 		return 0;
 	}
 
-	_processed_frames = 0;
+	_processed_samples = 0;
 	last_monitor_check = 0;
 
 	int error_code = _backend->start (for_latency);
@@ -884,7 +1060,7 @@ AudioEngine::start (bool for_latency)
 	_running = true;
 
 	if (_session) {
-		_session->set_frame_rate (_backend->sample_rate());
+		_session->set_sample_rate (_backend->sample_rate());
 
 		if (_session->config.get_jack_time_master()) {
 			_backend->set_time_master (true);
@@ -892,14 +1068,24 @@ AudioEngine::start (bool for_latency)
 
 	}
 
-	/* XXX MIDI ports may not actually be available here yet .. */
-
-	PortManager::fill_midi_port_info ();
-
 	if (!for_latency) {
-		Running(); /* EMIT SIGNAL */
+		/* Call the library-wide ::init_post_engine() before emitting
+		 * running to ensure that its tasks are complete before any
+		 * signal handlers execute. PBD::Signal does not ensure
+		 * ordering of signal handlers so even if ::init_post_engine()
+		 * is connected first, it may not run first.
+		 */
+
+		ARDOUR::init_post_engine (_start_cnt);
+
+		Running (_start_cnt); /* EMIT SIGNAL */
+
+		/* latency start/stop cycles do not count as "starts" */
+
+		_start_cnt++;
 	}
 
+
 	return 0;
 }
 
@@ -920,7 +1106,7 @@ AudioEngine::stop (bool for_latency)
 
 	if (for_latency && _backend->can_change_systemic_latency_when_running()) {
 		stop_engine = false;
-		if (_running) {
+		if (_running && _started_for_latency) {
 			_backend->start (false); // keep running, reload latencies
 		}
 	} else {
@@ -936,32 +1122,37 @@ AudioEngine::stop (bool for_latency)
 		pl.release ();
 	}
 
-	if (_session && _running && stop_engine &&
-	    (_session->state_of_the_state() & Session::Loading) == 0 &&
-	    (_session->state_of_the_state() & Session::Deletion) == 0) {
+	const bool was_running_will_stop = (_running && stop_engine);
+
+	if (was_running_will_stop) {
+		_running = false;
+	}
+
+	if (_session && was_running_will_stop && !_session->loading() && !_session->deletion_in_progress()) {
 		// it's not a halt, but should be handled the same way:
 		// disable record, stop transport and I/O processign but save the data.
 		_session->engine_halted ();
 	}
 
-	if (stop_engine && _running) {
-		_running = false;
+	if (was_running_will_stop) {
 		if (!for_latency) {
 			_started_for_latency = false;
 		} else if (!_started_for_latency) {
 			_stopped_for_latency = true;
 		}
 	}
-	_processed_frames = 0;
+	_processed_samples = 0;
 	_measuring_latency = MeasureNone;
-	_latency_output_port = 0;
-	_latency_input_port = 0;
+	_latency_output_port.reset ();
+	_latency_input_port.reset ();
 
 	if (stop_engine) {
 		Port::PortDrop ();
 	}
 
 	if (stop_engine) {
+		TransportMasterManager& tmm (TransportMasterManager::instance());
+		tmm.engine_stopped ();
 		Stopped (); /* EMIT SIGNAL */
 	}
 
@@ -999,14 +1190,26 @@ AudioEngine::is_realtime() const
 	return _backend->is_realtime();
 }
 
-bool
-AudioEngine::connected() const
+int
+AudioEngine::client_real_time_priority ()
 {
 	if (!_backend) {
-		return false;
+		assert (0);
+		return PBD_RT_PRI_PROC;
+	}
+	if (!_backend->is_realtime ()) {
+		/* this is only an issue with the Dummy backend.
+		 * - with JACK, we require rt permissions.
+		 * - with ALSA/PulseAudio this can only happen if rt permissions
+		 *   are n/a. Other attempts to get rt will fail likewise.
+		 *
+		 * perhaps:
+		 * TODO: use is_realtime () ? PBD_SCHED_FIFO : PBD_SCHED_OTHER
+		 */
+		return PBD_RT_PRI_PROC; // XXX
 	}
 
-	return _backend->available();
+	return _backend->client_real_time_priority();
 }
 
 void
@@ -1037,7 +1240,7 @@ AudioEngine::transport_state ()
 }
 
 void
-AudioEngine::transport_locate (framepos_t pos)
+AudioEngine::transport_locate (samplepos_t pos)
 {
 	if (!_backend) {
 		return;
@@ -1045,19 +1248,22 @@ AudioEngine::transport_locate (framepos_t pos)
 	return _backend->transport_locate (pos);
 }
 
-framepos_t
-AudioEngine::transport_frame()
+samplepos_t
+AudioEngine::transport_sample()
 {
 	if (!_backend) {
 		return 0;
 	}
-	return _backend->transport_frame ();
+	return _backend->transport_sample ();
 }
 
-framecnt_t
+samplecnt_t
 AudioEngine::sample_rate () const
 {
 	if (!_backend) {
+		if (_session) {
+			return _session->nominal_sample_rate ();
+		}
 		return 0;
 	}
 	return _backend->sample_rate ();
@@ -1090,7 +1296,7 @@ AudioEngine::raw_buffer_size (DataType t)
 	return _backend->raw_buffer_size (t);
 }
 
-framepos_t
+samplepos_t
 AudioEngine::sample_time ()
 {
 	if (!_backend) {
@@ -1099,7 +1305,7 @@ AudioEngine::sample_time ()
 	return _backend->sample_time ();
 }
 
-framepos_t
+samplepos_t
 AudioEngine::sample_time_at_cycle_start ()
 {
 	if (!_backend) {
@@ -1250,15 +1456,17 @@ AudioEngine::thread_init_callback (void* arg)
 	   knows about it.
 	*/
 
-	pthread_set_name (X_("audioengine"));
-
-	const int thread_num = g_atomic_int_add (&audioengine_thread_cnt, 1);
+	const int thread_num = audioengine_thread_cnt.fetch_add (1);
 	const string thread_name = string_compose (X_("AudioEngine %1"), thread_num);
 
+	pthread_set_name (thread_name.c_str());
+
 	SessionEvent::create_per_thread_pool (thread_name, 512);
 	PBD::notify_event_loops_about_thread_creation (pthread_self(), thread_name, 4096);
 	AsyncMIDIPort::set_process_thread (pthread_self());
 
+	Temporal::TempoMap::fetch ();
+
 	if (arg) {
 		delete AudioEngine::instance()->_main_thread;
 		/* the special thread created/managed by the backend */
@@ -1267,8 +1475,9 @@ AudioEngine::thread_init_callback (void* arg)
 }
 
 int
-AudioEngine::sync_callback (TransportState state, framepos_t position)
+AudioEngine::sync_callback (TransportState state, samplepos_t position)
 {
+	DEBUG_TRACE (DEBUG::BackendCallbacks, string_compose (X_("sync callback %1, %2\n"), state, position));
 	if (_session) {
 		return _session->backend_sync_callback (state, position);
 	}
@@ -1278,14 +1487,54 @@ AudioEngine::sync_callback (TransportState state, framepos_t position)
 void
 AudioEngine::freewheel_callback (bool onoff)
 {
+	DEBUG_TRACE (DEBUG::BackendCallbacks, string_compose (X_("freewheel callback onoff %1\n"), onoff));
 	_freewheeling = onoff;
+	if (!_freewheeling) {
+		PortManager::reinit ();
+	}
 }
 
 void
 AudioEngine::latency_callback (bool for_playback)
 {
-	if (_session) {
-		_session->update_latency (for_playback);
+	DEBUG_TRACE (DEBUG::BackendCallbacks, string_compose (X_("latency callback playback ? %1\n"), for_playback));
+	if (!_session) {
+		return;
+	}
+
+	if (in_process_thread ()) {
+		/* internal backends emit the latency callback in the rt-callback,
+		 * async to connect/disconnect or port creation/deletion.
+		 * All is fine.
+		 */
+		Glib::Threads::Mutex::Lock ll (_latency_lock, Glib::Threads::TRY_LOCK);
+		if (!ll.locked () || _session->processing_blocked ()) {
+		 /* Except Session::write_one_track() might just have called block_processing() */
+			queue_latency_update (for_playback);
+		} else {
+			_session->update_latency (for_playback);
+		}
+	} else {
+		/* However jack 1/2 emit the callback in sync with creating the port
+		 * (or while handling the connection change).
+		 * e.g. JACK2 jack_port_register() blocks and the jack_latency_callback
+		 * from a different thread: https://pastebin.com/mitGBwpq
+		 * but at this point in time Ardour still holds the process callback
+		 * because JACK2 can process in parallel to latency callbacks.
+		 *
+		 * see also Session::update_latency() and git-ref 1983f56592dfea5f7498
+		 */
+		queue_latency_update (for_playback);
+	}
+}
+
+void
+AudioEngine::queue_latency_update (bool for_playback)
+{
+	if (for_playback) {
+		_pending_playback_latency_callback.store (1);
+	} else {
+		_pending_capture_latency_callback.store (1);
 	}
 }
 
@@ -1300,6 +1549,7 @@ AudioEngine::update_latencies ()
 void
 AudioEngine::halted_callback (const char* why)
 {
+	DEBUG_TRACE (DEBUG::BackendCallbacks, string_compose (X_("halted callback why: [%1]\n"), why));
 	if (_in_destructor) {
 		/* everything is under control */
 		return;
@@ -1452,7 +1702,7 @@ AudioEngine::start_latency_detection (bool for_midi)
 	_latency_signal_latency += lr.max;
 
 	/* all created and connected, lets go */
-	_latency_flush_frames = samples_per_cycle();
+	_latency_flush_samples = samples_per_cycle();
 	_measuring_latency = for_midi ? MeasureMIDI : MeasureAudio;
 
 	return 0;
@@ -1465,11 +1715,11 @@ AudioEngine::stop_latency_detection ()
 
 	if (_latency_output_port) {
 		port_engine().unregister_port (_latency_output_port);
-		_latency_output_port = 0;
+		_latency_output_port.reset();
 	}
 	if (_latency_input_port) {
 		port_engine().unregister_port (_latency_input_port);
-		_latency_input_port = 0;
+		_latency_input_port.reset();
 	}
 
 	if (_running && _backend->can_change_systemic_latency_when_running()) {
