@@ -1,20 +1,28 @@
 /*
-    Copyright (C) 2000-2006 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2000-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2006-2014 David Robillard <d@drobilla.net>
+ * Copyright (C) 2006 Jesse Chappell <jesse@essej.net>
+ * Copyright (C) 2007-2011 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2013-2015 John Emmas <john@creativepost.co.uk>
+ * Copyright (C) 2013-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015 GZharun <grygoriiz@wavesglobal.com>
+ * Copyright (C) 2016-2017 Julien "_FrnchFrgg_" RIVAUD <frnchfrgg@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <algorithm>
 #include <cmath>
@@ -55,22 +63,25 @@ using namespace ARDOUR;
 using namespace PBD;
 
 const string                 IO::state_node_name = "IO";
-bool                         IO::connecting_legal = false;
-PBD::Signal0<int>            IO::ConnectingLegal;
 PBD::Signal1<void,ChanCount> IO::PortCountChanged;
 
+static std::string
+legalize_io_name (std::string n)
+{
+	replace_all (n, ":", "-");
+	return n;
+}
+
 /** @param default_type The type of port that will be created by ensure_io
  * and friends if no type is explicitly requested (to avoid breakage).
  */
 IO::IO (Session& s, const string& name, Direction dir, DataType default_type, bool sendish)
-	: SessionObject (s, name)
+	: SessionObject (s, legalize_io_name (name))
 	, _direction (dir)
 	, _default_type (default_type)
 	, _sendish (sendish)
 {
 	_active = true;
-	Port::PostDisconnect.connect_same_thread (*this, boost::bind (&IO::disconnect_check, this, _1, _2));
-	pending_state_node = 0;
 	setup_bundle ();
 }
 
@@ -81,8 +92,6 @@ IO::IO (Session& s, const XMLNode& node, DataType dt, bool sendish)
 	, _sendish (sendish)
 {
 	_active = true;
-	pending_state_node = 0;
-	Port::PostDisconnect.connect_same_thread (*this, boost::bind (&IO::disconnect_check, this, _1, _2));
 
 	set_state (node, Stateful::loading_state_version);
 	setup_bundle ();
@@ -90,22 +99,20 @@ IO::IO (Session& s, const XMLNode& node, DataType dt, bool sendish)
 
 IO::~IO ()
 {
-	Glib::Threads::Mutex::Lock lm (io_lock);
-
 	DEBUG_TRACE (DEBUG::Ports, string_compose ("IO %1 unregisters %2 ports\n", name(), _ports.num_ports()));
 
 	BLOCK_PROCESS_CALLBACK ();
+	Glib::Threads::RWLock::WriterLock wl (_io_lock);
 
 	for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
 		_session.engine().unregister_port (*i);
 	}
-	delete pending_state_node; pending_state_node = 0;
 }
 
 void
-IO::disconnect_check (boost::shared_ptr<Port> a, boost::shared_ptr<Port> b)
+IO::connection_change (std::shared_ptr<Port> a, std::shared_ptr<Port> b)
 {
-	if (_session.state_of_the_state () & Session::Deletion) {
+	if (_session.deletion_in_progress ()) {
 		return;
 	}
 	/* this could be called from within our own ::disconnect() method(s)
@@ -114,12 +121,13 @@ IO::disconnect_check (boost::shared_ptr<Port> a, boost::shared_ptr<Port> b)
 	   we assume that its safely locked by our own ::disconnect().
 	*/
 
-	Glib::Threads::Mutex::Lock tm (io_lock, Glib::Threads::TRY_LOCK);
+	Glib::Threads::RWLock::WriterLock wl (_io_lock, Glib::Threads::TRY_LOCK);
 
-	if (tm.locked()) {
+	if (wl.locked()) {
 		/* we took the lock, so we cannot be here from inside
 		 * ::disconnect()
 		 */
+		wl.release (); // release lock before emitting signal
 		if (_ports.contains (a) || _ports.contains (b)) {
 			changed (IOChange (IOChange::ConnectionsChanged), this); /* EMIT SIGNAL */
 		}
@@ -131,19 +139,7 @@ IO::disconnect_check (boost::shared_ptr<Port> a, boost::shared_ptr<Port> b)
 }
 
 void
-IO::increment_port_buffer_offset (pframes_t offset)
-{
-	/* io_lock, not taken: function must be called from Session::process() calltree */
-
-        if (_direction == Output) {
-                for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
-                        i->increment_port_buffer_offset (offset);
-                }
-        }
-}
-
-void
-IO::silence (framecnt_t nframes)
+IO::silence (samplecnt_t nframes)
 {
 	/* io_lock, not taken: function must be called from Session::process() calltree */
 
@@ -155,30 +151,33 @@ IO::silence (framecnt_t nframes)
 }
 
 int
-IO::disconnect (boost::shared_ptr<Port> our_port, string other_port, void* src)
+IO::disconnect (std::shared_ptr<Port> our_port, string other_port, void* src)
 {
 	if (other_port.length() == 0 || our_port == 0) {
 		return 0;
 	}
 
-        {
-                Glib::Threads::Mutex::Lock lm (io_lock);
+	{
+		Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 
-                /* check that our_port is really one of ours */
+		/* check that our_port is really one of ours */
 
-                if ( ! _ports.contains(our_port)) {
-                        return -1;
-                }
+		if ( ! _ports.contains(our_port)) {
+			return -1;
+		}
 
-                /* disconnect it from the source */
+		/* disconnect it from the source */
 
-                if (our_port->disconnect (other_port)) {
-                        error << string_compose(_("IO: cannot disconnect port %1 from %2"), our_port->name(), other_port) << endmsg;
-                        return -1;
-                }
-        }
+		DEBUG_TRACE (DEBUG::PortConnectIO,
+		             string_compose("IO::disconnect %1 from %2\n", our_port->name(), other_port));
 
-        changed (IOChange (IOChange::ConnectionsChanged), src); /* EMIT SIGNAL */
+		if (our_port->disconnect (other_port)) {
+			error << string_compose(_("IO: cannot disconnect port %1 from %2"), our_port->name(), other_port) << endmsg;
+			return -1;
+		}
+	}
+
+	changed (IOChange (IOChange::ConnectionsChanged), src); /* EMIT SIGNAL */
 
 	_session.set_dirty ();
 
@@ -186,14 +185,14 @@ IO::disconnect (boost::shared_ptr<Port> our_port, string other_port, void* src)
 }
 
 int
-IO::connect (boost::shared_ptr<Port> our_port, string other_port, void* src)
+IO::connect (std::shared_ptr<Port> our_port, string other_port, void* src)
 {
 	if (other_port.length() == 0 || our_port == 0) {
 		return 0;
 	}
 
 	{
-		Glib::Threads::Mutex::Lock lm (io_lock);
+		Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 
 		/* check that our_port is really one of ours */
 
@@ -203,6 +202,9 @@ IO::connect (boost::shared_ptr<Port> our_port, string other_port, void* src)
 
 		/* connect it to the source */
 
+		DEBUG_TRACE (DEBUG::PortConnectIO,
+		             string_compose("IO::connect %1 to %2\n", our_port->name(), other_port));
+
 		if (our_port->connect (other_port)) {
 			return -1;
 		}
@@ -212,15 +214,30 @@ IO::connect (boost::shared_ptr<Port> our_port, string other_port, void* src)
 	return 0;
 }
 
+bool
+IO::can_add_port (DataType type) const
+{
+	switch (type) {
+		case DataType::NIL:
+			return false;
+		case DataType::AUDIO:
+			return true;
+		case DataType::MIDI:
+			return _ports.count ().n_midi() < 1;
+	}
+	abort(); /*NOTREACHED*/
+	return false;
+}
+
 int
-IO::remove_port (boost::shared_ptr<Port> port, void* src)
+IO::remove_port (std::shared_ptr<Port> port, void* src)
 {
 	ChanCount before = _ports.count ();
 	ChanCount after = before;
 	after.set (port->type(), after.get (port->type()) - 1);
 
 	boost::optional<bool> const r = PortCountChanging (after); /* EMIT SIGNAL */
-	if (r.get_value_or (false)) {
+	if (r.value_or (false)) {
 		return -1;
 	}
 
@@ -230,7 +247,7 @@ IO::remove_port (boost::shared_ptr<Port> port, void* src)
 		BLOCK_PROCESS_CALLBACK ();
 
 		{
-			Glib::Threads::Mutex::Lock lm (io_lock);
+			Glib::Threads::RWLock::WriterLock wl (_io_lock);
 
 			if (_ports.remove(port)) {
 				change.type = IOChange::Type (change.type | IOChange::ConfigurationChanged);
@@ -275,12 +292,16 @@ IO::remove_port (boost::shared_ptr<Port> port, void* src)
 int
 IO::add_port (string destination, void* src, DataType type)
 {
-	boost::shared_ptr<Port> our_port;
+	std::shared_ptr<Port> our_port;
 
 	if (type == DataType::NIL) {
 		type = _default_type;
 	}
 
+	if (!can_add_port (type)) {
+		return -1;
+	}
+
 	ChanCount before = _ports.count ();
 	ChanCount after = before;
 	after.set (type, after.get (type) + 1);
@@ -297,7 +318,7 @@ IO::add_port (string destination, void* src, DataType type)
 
 
 		{
-			Glib::Threads::Mutex::Lock lm (io_lock);
+			Glib::Threads::RWLock::WriterLock wl (_io_lock);
 
 			/* Create a new port */
 
@@ -343,7 +364,7 @@ int
 IO::disconnect (void* src)
 {
 	{
-		Glib::Threads::Mutex::Lock lm (io_lock);
+		Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 
 		for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
 			i->disconnect_all ();
@@ -363,8 +384,8 @@ IO::ensure_ports_locked (ChanCount count, bool clear, bool& changed)
 	assert (!AudioEngine::instance()->process_lock().trylock());
 #endif
 
-	boost::shared_ptr<Port> port;
-	vector<boost::shared_ptr<Port> > deleted_ports;
+	std::shared_ptr<Port> port;
+	vector<std::shared_ptr<Port> > deleted_ports;
 
 	changed    = false;
 
@@ -466,7 +487,7 @@ IO::ensure_ports (ChanCount count, bool clear, void* src)
 	change.before = _ports.count ();
 
 	{
-		Glib::Threads::Mutex::Lock im (io_lock);
+		Glib::Threads::RWLock::WriterLock wl (_io_lock);
 		if (ensure_ports_locked (count, clear, changed)) {
 			return -1;
 		}
@@ -484,6 +505,15 @@ IO::ensure_ports (ChanCount count, bool clear, void* src)
 	return 0;
 }
 
+void
+IO::reestablish_port_subscriptions ()
+{
+	_port_connections.drop_connections ();
+	for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		i->ConnectedOrDisconnected.connect_same_thread (*this, boost::bind (&IO::connection_change, this, _1, _2));
+	}
+}
+
 /** Caller must hold process lock */
 int
 IO::ensure_io (ChanCount count, bool clear, void* src)
@@ -496,17 +526,17 @@ IO::ensure_io (ChanCount count, bool clear, void* src)
 }
 
 XMLNode&
-IO::get_state ()
+IO::get_state () const
 {
-	return state (true);
+	return state ();
 }
 
 XMLNode&
-IO::state (bool /*full_state*/)
+IO::state () const
 {
 	XMLNode* node = new XMLNode (state_node_name);
 	int n;
-	Glib::Threads::Mutex::Lock lm (io_lock);
+	Glib::Threads::RWLock::WriterLock wl (_io_lock);
 
 	node->set_property ("name", name());
 	node->set_property ("id", id ());
@@ -517,7 +547,7 @@ IO::state (bool /*full_state*/)
 		node->set_property("pretty-name", _pretty_name_prefix);
 	}
 
-	for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
+	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
 
 		vector<string> connections;
 
@@ -548,8 +578,6 @@ IO::state (bool /*full_state*/)
 		node->add_child_nocopy (*pnode);
 	}
 
-	node->set_property (X_("user-latency"), _user_latency);
-
 	return *node;
 }
 
@@ -589,27 +617,55 @@ IO::set_state (const XMLNode& node, int version)
 		return -1;
 	}
 
+	if (_sendish && _direction == Output) {
+		/* ignore <Port name="...">  from XML for sends, but use the names
+		 * ::ensure_ports_locked() creates port using ::build_legal_port_name()
+		 * This is needed to properly restore connections when creating
+		 * external sends from templates because the IO name changes.
+		 */
+		PortSet::iterator i = _ports.begin();
+		XMLNodeConstIterator x = node.children().begin();
+		for (; i != _ports.end() && x != node.children().end(); ++i, ++x) {
+			if ((*x)->name() == "Port") {
+				(*x)->remove_property (X_("name"));
+				(*x)->set_property (X_("name"), i->name());
+			}
+		}
+	}
+
 	// after create_ports, updates names
+
 	if (node.get_property ("pretty-name", name)) {
 		set_pretty_name (name);
 	}
 
-	if (connecting_legal) {
+	/* now set port state (this will *not* connect them, but will store
+	   the names of connected ports).
+	 */
 
-		if (make_connections (node, version, false)) {
-			return -1;
-		}
-
-	} else {
-
-		delete pending_state_node;
-		pending_state_node = new XMLNode (node);
-		pending_state_node_version = version;
-		pending_state_node_in = false;
-		ConnectingLegal.connect_same_thread (connection_legal_c, boost::bind (&IO::connecting_became_legal, this));
+	if (version < 3000) {
+		return set_port_state_2X (node, version, false);
 	}
 
-	node.get_property ("user-latency", _user_latency);
+	XMLProperty const * prop;
+
+	for (XMLNodeConstIterator i = node.children().begin(); i != node.children().end(); ++i) {
+
+		if ((*i)->name() == "Port") {
+
+			prop = (*i)->property (X_("name"));
+
+			if (!prop) {
+				continue;
+			}
+
+			std::shared_ptr<Port> p = port_by_name (prop->value());
+
+			if (p) {
+				p->set_state (**i, version);
+			}
+		}
+	}
 
 	return 0;
 }
@@ -618,7 +674,6 @@ int
 IO::set_state_2X (const XMLNode& node, int version, bool in)
 {
 	XMLProperty const * prop;
-	XMLNodeConstIterator iter;
 	LocaleGuard lg;
 
 	/* force use of non-localized representation of decimal point,
@@ -647,52 +702,21 @@ IO::set_state_2X (const XMLNode& node, int version, bool in)
 		return -1;
 	}
 
-	if (connecting_legal) {
-
-		if (make_connections_2X (node, version, in)) {
-			return -1;
-		}
-
-	} else {
-
-		delete pending_state_node;
-		pending_state_node = new XMLNode (node);
-		pending_state_node_version = version;
-		pending_state_node_in = in;
-		ConnectingLegal.connect_same_thread (connection_legal_c, boost::bind (&IO::connecting_became_legal, this));
+	if (set_port_state_2X (node, version, in)) {
+		return -1;
 	}
 
 	return 0;
 }
 
-int
-IO::connecting_became_legal ()
-{
-	int ret = 0;
-
-	assert (pending_state_node);
-
-	connection_legal_c.disconnect ();
-
-    // it's not required for TracksLive, as long as TracksLive's session does all the connections when it's being loaded
-    if (!Profile->get_trx() ) {
-        ret = make_connections (*pending_state_node, pending_state_node_version, pending_state_node_in);
-    }
-
-	delete pending_state_node;
-	pending_state_node = 0;
-
-	return ret;
-}
-
-boost::shared_ptr<Bundle>
+std::shared_ptr<Bundle>
 IO::find_possible_bundle (const string &desired_name)
 {
 	static const string digits = "0123456789";
 	const string &default_name = (_direction == Input ? _("in") : _("out"));
 	const string &bundle_type_name = (_direction == Input ? _("input") : _("output"));
 
-	boost::shared_ptr<Bundle> c = _session.bundle_by_name (desired_name);
+	std::shared_ptr<Bundle> c = _session.bundle_by_name (desired_name);
 
 	if (!c) {
 		int bundle_number, mask;
@@ -776,7 +800,7 @@ IO::find_possible_bundle (const string &desired_name)
 }
 
 int
-IO::get_port_counts_2X (XMLNode const & node, int /*version*/, ChanCount& n, boost::shared_ptr<Bundle>& /*c*/)
+IO::get_port_counts_2X (XMLNode const & node, int /*version*/, ChanCount& n, std::shared_ptr<Bundle>& /*c*/)
 {
 	XMLProperty const * prop;
 	XMLNodeList children = node.children ();
@@ -804,7 +828,7 @@ IO::get_port_counts_2X (XMLNode const & node, int /*version*/, ChanCount& n, boo
 }
 
 int
-IO::get_port_counts (const XMLNode& node, int version, ChanCount& n, boost::shared_ptr<Bundle>& c)
+IO::get_port_counts (const XMLNode& node, int version, ChanCount& n, std::shared_ptr<Bundle>& c)
 {
 	if (version < 3000) {
 		return get_port_counts_2X (node, version, n, c);
@@ -861,7 +885,7 @@ int
 IO::create_ports (const XMLNode& node, int version)
 {
 	ChanCount n;
-	boost::shared_ptr<Bundle> c;
+	std::shared_ptr<Bundle> c;
 
 	get_port_counts (node, version, n, c);
 
@@ -880,103 +904,7 @@ IO::create_ports (const XMLNode& node, int version)
 }
 
 int
-IO::make_connections (const XMLNode& node, int version, bool in)
-{
-	if (version < 3000) {
-		return make_connections_2X (node, version, in);
-	}
-
-	XMLProperty const * prop;
-
-	for (XMLNodeConstIterator i = node.children().begin(); i != node.children().end(); ++i) {
-
-		if ((*i)->name() == "Bundle") {
-			XMLProperty const * prop = (*i)->property ("name");
-			if (prop) {
-				boost::shared_ptr<Bundle> b = find_possible_bundle (prop->value());
-				if (b) {
-					connect_ports_to_bundle (b, true, this);
-				}
-			}
-
-			return 0;
-		}
-
-		if ((*i)->name() == "Port") {
-
-			prop = (*i)->property (X_("name"));
-
-			if (!prop) {
-				continue;
-			}
-
-			boost::shared_ptr<Port> p = port_by_name (prop->value());
-
-			if (p) {
-				for (XMLNodeConstIterator c = (*i)->children().begin(); c != (*i)->children().end(); ++c) {
-
-					XMLNode* cnode = (*c);
-
-					if (cnode->name() != X_("Connection")) {
-						continue;
-					}
-
-					if ((prop = cnode->property (X_("other"))) == 0) {
-						continue;
-					}
-
-					if (prop) {
-                                                connect (p, prop->value(), this);
-					}
-				}
-			}
-		}
-	}
-
-	return 0;
-}
-
-void
-IO::prepare_for_reset (XMLNode& node, const std::string& name)
-{
-	/* reset name */
-	node.set_property ("name", name);
-
-	/* now find connections and reset the name of the port
-	   in one so that when we re-use it it will match
-	   the name of the thing we're applying it to.
-	*/
-
-	XMLProperty * prop;
-	XMLNodeList children = node.children();
-
-	for (XMLNodeIterator i = children.begin(); i != children.end(); ++i) {
-
-		if ((*i)->name() == "Port") {
-
-			prop = (*i)->property (X_("name"));
-
-			if (prop) {
-				string new_name;
-				string old = prop->value();
-				string::size_type slash = old.find ('/');
-
-				if (slash != string::npos) {
-					/* port name is of form: <IO-name>/<port-name> */
-
-					new_name = name;
-					new_name += old.substr (old.find ('/'));
-
-					prop->set_value (new_name);
-				}
-			}
-		}
-	}
-}
-
-
-int
-IO::make_connections_2X (const XMLNode& node, int /*version*/, bool in)
+IO::set_port_state_2X (const XMLNode& node, int /*version*/, bool in)
 {
 	XMLProperty const * prop;
 
@@ -1071,6 +999,46 @@ IO::make_connections_2X (const XMLNode& node, int /*version*/, bool in)
 	return 0;
 }
 
+void
+IO::prepare_for_reset (XMLNode& node, const std::string& name)
+{
+	/* reset name */
+	node.set_property ("name", name);
+
+	/* now find connections and reset the name of the port
+	   in one so that when we re-use it it will match
+	   the name of the thing we're applying it to.
+	*/
+
+	XMLProperty * prop;
+	XMLNodeList children = node.children();
+
+	for (XMLNodeIterator i = children.begin(); i != children.end(); ++i) {
+
+		if ((*i)->name() == "Port") {
+
+			prop = (*i)->property (X_("name"));
+
+			if (prop) {
+				string new_name;
+				string old = prop->value();
+				string::size_type slash = old.find ('/');
+
+				if (slash != string::npos) {
+					/* port name is of form: <IO-name>/<port-name> */
+
+					new_name = name;
+					new_name += old.substr (old.find ('/'));
+
+					prop->set_value (new_name);
+				}
+			}
+		}
+	}
+}
+
+
+
 int
 IO::set_ports (const string& str)
 {
@@ -1176,10 +1144,11 @@ IO::set_name (const string& requested_name)
 
 	/* replace all colons in the name. i wish we didn't have to do this */
 
-	replace_all (name, ":", "-");
+	name = legalize_io_name (name);
 
 	for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
 		string current_name = i->name();
+		assert (current_name.find (_name) != std::string::npos);
 		current_name.replace (current_name.find (_name), _name.val().length(), name);
 		i->set_name (current_name);
 	}
@@ -1216,45 +1185,156 @@ IO::apply_pretty_name ()
 	}
 }
 
-framecnt_t
-IO::latency () const
+void
+IO::set_private_port_latencies (samplecnt_t value, bool playback)
 {
-	framecnt_t max_latency;
-	framecnt_t latency;
+	LatencyRange lat;
+	lat.min = lat.max = value;
+	for (PortSet::iterator i = _ports.begin (); i != _ports.end(); ++i) {
+		 i->set_private_latency_range (lat, playback);
+	}
+}
 
-	max_latency = 0;
-
-	/* io lock not taken - must be protected by other means */
+void
+IO::set_public_port_latency_from_connections () const
+{
+	/* get min/max of connected up/downstream ports */
+	bool connected = false;
+	bool playback = _direction == Output;
+	LatencyRange lr;
+	lr.min = ~((pframes_t) 0);
+	lr.max = 0;
 
 	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		if (i->connected()) {
+			connected = true;
+		}
+		i->collect_latency_from_backend (lr, playback);
+	}
+
+	if (!connected) {
+		/* if output is not connected to anything, use private latency */
+		lr.min = lr.max = latency ();
+	}
+
+	for (PortSet::const_iterator i = _ports.begin (); i != _ports.end(); ++i) {
+		 i->set_public_latency_range (lr, playback);
+	}
+}
+
+void
+IO::set_public_port_latencies (samplecnt_t value, bool playback) const
+{
+	LatencyRange lat;
+	lat.min = lat.max = value;
+	for (PortSet::const_iterator i = _ports.begin (); i != _ports.end(); ++i) {
+		 i->set_public_latency_range (lat, playback);
+	}
+}
+
+samplecnt_t
+IO::latency () const
+{
+	samplecnt_t max_latency = 0;
+
+	Glib::Threads::RWLock::ReaderLock rl (_io_lock);
+
+	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		samplecnt_t latency;
 		if ((latency = i->private_latency_range (_direction == Output).max) > max_latency) {
-                        DEBUG_TRACE (DEBUG::Latency, string_compose ("port %1 has %2 latency of %3 - use\n",
-                                                                     name(),
-                                                                     ((_direction == Output) ? "PLAYBACK" : "CAPTURE"),
-                                                                     latency));
+			DEBUG_TRACE (DEBUG::LatencyIO, string_compose ("port %1 has %2 latency of %3 - use\n",
+			                                               name(),
+			                                               ((_direction == Output) ? "PLAYBACK" : "CAPTURE"),
+			                                               latency));
 			max_latency = latency;
 		}
 	}
 
-        DEBUG_TRACE (DEBUG::Latency, string_compose ("%1: max %4 latency from %2 ports = %3\n",
-                                                     name(), _ports.num_ports(), max_latency,
-                                                     ((_direction == Output) ? "PLAYBACK" : "CAPTURE")));
+	DEBUG_TRACE (DEBUG::LatencyIO, string_compose ("%1: max %4 latency from %2 ports = %3\n",
+	                                               name(), _ports.num_ports(), max_latency,
+	                                               ((_direction == Output) ? "PLAYBACK" : "CAPTURE")));
+	return max_latency;
+}
+
+#if 0 // not used, but may some day be handy for debugging
+samplecnt_t
+IO::public_latency () const
+{
+	samplecnt_t max_latency = 0;
+
+	/* io lock not taken - must be protected by other means */
+
+	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		samplecnt_t latency;
+		if ((latency = i->public_latency_range (_direction == Output).max) > max_latency) {
+			DEBUG_TRACE (DEBUG::LatencyIO, string_compose ("port %1 has %2 latency of %3 - use\n",
+			                                               name(),
+			                                               ((_direction == Output) ? "PLAYBACK" : "CAPTURE"),
+			                                               latency));
+			max_latency = latency;
+		}
+	}
+
+	DEBUG_TRACE (DEBUG::LatencyIO, string_compose ("%1: max %4 public latency from %2 ports = %3\n",
+	                                               name(), _ports.num_ports(), max_latency,
+	                                               ((_direction == Output) ? "PLAYBACK" : "CAPTURE")));
+	return max_latency;
+}
+#endif
+
+samplecnt_t
+IO::connected_latency (bool for_playback) const
+{
+	/* may be called concurrently with processing via
+	 *
+	 * Session::auto_connect_thread_run ()
+	 * -> Session::update_latency_compensation ()
+	 * -> Session::update_route_latency ()
+	 * -> Route::update_signal_latency ()
+	 * -> IO::connected_latency ()
+	 */
+	Glib::Threads::RWLock::ReaderLock rl (_io_lock);
+
+	samplecnt_t max_latency = 0;
+	bool connected = false;
+
+	/* if output is not connected to anything, use private latency */
+	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
+		if (i->connected()) {
+			connected = true;
+			max_latency = 0;
+			break;
+		}
+		samplecnt_t latency;
+		if ((latency = i->private_latency_range (for_playback).max) > max_latency) {
+			max_latency = latency;
+		}
+	}
+	if (connected) {
+		for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
+			LatencyRange lr;
+			i->get_connected_latency_range (lr, for_playback);
+			if (lr.max > max_latency) {
+				max_latency = lr.max;
+			}
+		}
+	}
 	return max_latency;
 }
 
 int
-IO::connect_ports_to_bundle (boost::shared_ptr<Bundle> c, bool exclusive, void* src) {
+IO::connect_ports_to_bundle (std::shared_ptr<Bundle> c, bool exclusive, void* src) {
 	return connect_ports_to_bundle(c, exclusive, false, src);
 }
 
 int
-IO::connect_ports_to_bundle (boost::shared_ptr<Bundle> c, bool exclusive,
+IO::connect_ports_to_bundle (std::shared_ptr<Bundle> c, bool exclusive,
                              bool allow_partial, void* src)
 {
 	BLOCK_PROCESS_CALLBACK ();
 
 	{
-		Glib::Threads::Mutex::Lock lm2 (io_lock);
+		Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 
 		if (exclusive) {
 			for (PortSet::iterator i = _ports.begin(); i != _ports.end(); ++i) {
@@ -1271,12 +1351,12 @@ IO::connect_ports_to_bundle (boost::shared_ptr<Bundle> c, bool exclusive,
 }
 
 int
-IO::disconnect_ports_from_bundle (boost::shared_ptr<Bundle> c, void* src)
+IO::disconnect_ports_from_bundle (std::shared_ptr<Bundle> c, void* src)
 {
 	BLOCK_PROCESS_CALLBACK ();
 
 	{
-		Glib::Threads::Mutex::Lock lm2 (io_lock);
+		Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 
 		c->disconnect (_bundle, _session.engine());
 
@@ -1288,28 +1368,9 @@ IO::disconnect_ports_from_bundle (boost::shared_ptr<Bundle> c, void* src)
 	return 0;
 }
 
-
-int
-IO::disable_connecting ()
-{
-	connecting_legal = false;
-	return 0;
-}
-
-int
-IO::enable_connecting ()
-{
-	Glib::Threads::Mutex::Lock lm (AudioEngine::instance()->process_lock());
-	connecting_legal = true;
-	boost::optional<int> r = ConnectingLegal ();
-	return r.get_value_or (0);
-}
-
 void
 IO::bundle_changed (Bundle::Change /*c*/)
 {
-	/* XXX */
-//	connect_input_ports_to_bundle (_input_bundle, this);
 }
 
 
@@ -1357,8 +1418,7 @@ IO::build_legal_port_name (DataType type)
 
 	/* colons are illegal in port names, so fix that */
 
-	string nom = _name.val();
-	replace_all (nom, ":", ";");
+	string nom = legalize_io_name (_name.val());
 
 	snprintf (&buf1[0], name_size+1, ("%.*s/%s"), limit, nom.c_str(), suffix.c_str());
 
@@ -1402,14 +1462,14 @@ IO::find_port_hole (const char* base)
 }
 
 
-boost::shared_ptr<AudioPort>
+std::shared_ptr<AudioPort>
 IO::audio(uint32_t n) const
 {
 	return _ports.nth_audio_port (n);
 
 }
 
-boost::shared_ptr<MidiPort>
+std::shared_ptr<MidiPort>
 IO::midi(uint32_t n) const
 {
 	return _ports.nth_midi_port (n);
@@ -1418,12 +1478,9 @@ IO::midi(uint32_t n) const
 /**
  *  Setup a bundle that describe our inputs or outputs. Also creates the bundle if necessary.
  */
-
 void
 IO::setup_bundle ()
 {
-        char buf[32];
-
 	if (!_bundle) {
 		_bundle.reset (new Bundle (_direction == Input));
 	}
@@ -1432,12 +1489,7 @@ IO::setup_bundle ()
 
 	_bundle->remove_channels ();
 
-	if (_direction == Input) {
-		snprintf(buf, sizeof (buf), _("%s in"), _name.val().c_str());
-	} else {
-		snprintf(buf, sizeof (buf), _("%s out"), _name.val().c_str());
-	}
-        _bundle->set_name (buf);
+	_bundle->set_name (string_compose ("%1 %2", _name, _direction == Input ? _("in") : _("out")));
 
 	int c = 0;
 	for (DataType::iterator i = DataType::begin(); i != DataType::end(); ++i) {
@@ -1451,6 +1503,8 @@ IO::setup_bundle ()
 
 	}
 
+	reestablish_port_subscriptions ();
+
 	_bundle->resume_signals ();
 }
 
@@ -1461,27 +1515,27 @@ IO::bundles_connected ()
 	BundleList bundles;
 
 	/* Session bundles */
-	boost::shared_ptr<ARDOUR::BundleList> b = _session.bundles ();
-	for (ARDOUR::BundleList::iterator i = b->begin(); i != b->end(); ++i) {
-		if ((*i)->connected_to (_bundle, _session.engine())) {
-			bundles.push_back (*i);
+	std::shared_ptr<ARDOUR::BundleList const> b = _session.bundles ();
+	for (auto const& i : *b) {
+		if (i->connected_to (_bundle, _session.engine())) {
+			bundles.push_back (i);
 		}
 	}
 
 	/* Route bundles */
 
-	boost::shared_ptr<ARDOUR::RouteList> r = _session.get_routes ();
+	std::shared_ptr<ARDOUR::RouteList const> r = _session.get_routes ();
 
 	if (_direction == Input) {
-		for (ARDOUR::RouteList::iterator i = r->begin(); i != r->end(); ++i) {
-			if ((*i)->output()->bundle()->connected_to (_bundle, _session.engine())) {
-				bundles.push_back ((*i)->output()->bundle());
+		for (auto const& i : *r) {
+			if (i->output()->bundle()->connected_to (_bundle, _session.engine())) {
+				bundles.push_back (i->output()->bundle());
 			}
 		}
 	} else {
-		for (ARDOUR::RouteList::iterator i = r->begin(); i != r->end(); ++i) {
-			if ((*i)->input()->bundle()->connected_to (_bundle, _session.engine())) {
-				bundles.push_back ((*i)->input()->bundle());
+		for (auto const& i : *r) {
+			if (i->input()->bundle()->connected_to (_bundle, _session.engine())) {
+				bundles.push_back (i->input()->bundle());
 			}
 		}
 	}
@@ -1490,7 +1544,7 @@ IO::bundles_connected ()
 }
 
 
-IO::UserBundleInfo::UserBundleInfo (IO* io, boost::shared_ptr<UserBundle> b)
+IO::UserBundleInfo::UserBundleInfo (IO* io, std::shared_ptr<UserBundle> b)
 {
 	bundle = b;
 	b->Changed.connect_same_thread (changed, boost::bind (&IO::bundle_changed, io, _1));
@@ -1552,22 +1606,22 @@ IO::set_name_in_state (XMLNode& node, const string& new_name)
 bool
 IO::connected () const
 {
-        /* do we have any connections at all? */
+	/* do we have any connections at all? */
 
-        for (PortSet::const_iterator p = _ports.begin(); p != _ports.end(); ++p) {
-                if (p->connected()) {
-                        return true;
-                }
-        }
+	for (PortSet::const_iterator p = _ports.begin(); p != _ports.end(); ++p) {
+		if (p->connected()) {
+			return true;
+		}
+	}
 
-        return false;
+	return false;
 }
 
 bool
-IO::connected_to (boost::shared_ptr<const IO> other) const
+IO::connected_to (std::shared_ptr<const IO> other) const
 {
 	if (!other) {
-                return connected ();
+		return connected ();
 	}
 
 	assert (_direction != other->direction());
@@ -1578,10 +1632,10 @@ IO::connected_to (boost::shared_ptr<const IO> other) const
 
 	for (i = 0; i < no; ++i) {
 		for (j = 0; j < ni; ++j) {
-			if ((NULL != nth(i).get()) && (NULL != other->nth(j).get())) {
-				if (nth(i)->connected_to (other->nth(j)->name())) {
-					return true;
-				}
+			std::shared_ptr<Port> pa (nth(i));
+			std::shared_ptr<Port> pb (other->nth(j));
+			if (pa && pb && pa->connected_to (pb->name())) {
+				return true;
 			}
 		}
 	}
@@ -1601,25 +1655,6 @@ IO::connected_to (const string& str) const
 	return false;
 }
 
-/** Call a processor's ::run() method, giving it our buffers
- *  Caller must hold process lock.
- */
-void
-IO::process_input (boost::shared_ptr<Processor> proc, framepos_t start_frame, framepos_t end_frame, double speed, pframes_t nframes)
-{
-	/* don't read the data into new buffers - just use the port buffers directly */
-
-	if (n_ports().n_total() == 0) {
-		/* We have no ports, so nothing to process */
-		return;
-	}
-
-	_buffers.get_backend_port_addresses (_ports, nframes);
-	if (proc) {
-		proc->run (_buffers, start_frame, end_frame, speed, nframes, true);
-	}
-}
-
 void
 IO::collect_input (BufferSet& bufs, pframes_t nframes, ChanCount offset)
 {
@@ -1649,7 +1684,7 @@ IO::collect_input (BufferSet& bufs, pframes_t nframes, ChanCount offset)
 }
 
 void
-IO::copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, framecnt_t offset)
+IO::copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, samplecnt_t offset)
 {
 	PortSet::iterator o = _ports.begin(type);
 	BufferSet::iterator i = bufs.begin(type);
@@ -1674,9 +1709,15 @@ IO::copy_to_outputs (BufferSet& bufs, DataType type, pframes_t nframes, framecnt
 		port_buffer.read_from (*prev, nframes, offset);
 		++o;
 	}
+
+	/* when port is both externally and internally connected,
+	 * make data directly available to downstream internal ports */
+	for (auto const& p : _ports) {
+		p->flush_buffers (nframes);
+	}
 }
 
-boost::shared_ptr<Port>
+std::shared_ptr<Port>
 IO::port_by_name (const std::string& str) const
 {
 	/* to be called only from ::set_state() - no locking */
@@ -1684,28 +1725,28 @@ IO::port_by_name (const std::string& str) const
 	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
 
 		if (i->name() == str) {
-			return boost::const_pointer_cast<Port> (*i);
+			return std::const_pointer_cast<Port> (*i);
 		}
 	}
 
-	return boost::shared_ptr<Port> ();
+	return std::shared_ptr<Port> ();
 }
 
 bool
 IO::physically_connected () const
 {
 	for (PortSet::const_iterator i = _ports.begin(); i != _ports.end(); ++i) {
-                if (i->physically_connected()) {
-                        return true;
-                }
-        }
+		if (i->physically_connected()) {
+			return true;
+		}
+	}
 
-        return false;
+	return false;
 }
 
 bool
-IO::has_port (boost::shared_ptr<Port> p) const
+IO::has_port (std::shared_ptr<Port> p) const
 {
-	Glib::Threads::Mutex::Lock lm (io_lock);
+	Glib::Threads::RWLock::ReaderLock rl (_io_lock);
 	return _ports.contains (p);
 }
