@@ -1,22 +1,27 @@
 /*
-  Copyright (C) 2007 Paul Davis
-  Author: David Robillard
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2007-2016 David Robillard <d@drobilla.net>
+ * Copyright (C) 2008-2012 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2008-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2011 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2014-2016 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2017 Nick Mainsbridge <mainsbridge@gmail.com>
+ * Copyright (C) 2015 André Nusser <andre.nusser@googlemail.com>
+ * Copyright (C) 2016 Julien "_FrnchFrgg_" RIVAUD <frnchfrgg@free.fr>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <algorithm>
 #include <iostream>
@@ -28,7 +33,7 @@
 #include "pbd/enumwriter.h"
 #include "pbd/error.h"
 
-#include "evoral/Control.hpp"
+#include "evoral/Control.h"
 
 #include "midi++/events.h"
 
@@ -53,55 +58,41 @@ using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
 
-MidiModel::MidiModel (boost::shared_ptr<MidiSource> s)
-	: AutomatableSequence<TimeType>(s->session())
+MidiModel::MidiModel (MidiSource& s)
+	: AutomatableSequence<TimeType> (s.session(), Temporal::BeatTime)
+	, _midi_source (s)
 {
-	set_midi_source (s);
+	_midi_source.InterpolationChanged.connect_same_thread (_midi_source_connections, boost::bind (&MidiModel::source_interpolation_changed, this, _1, _2));
+	_midi_source.AutomationStateChanged.connect_same_thread (_midi_source_connections, boost::bind (&MidiModel::source_automation_state_changed, this, _1, _2));
 }
 
-/** Start a new NoteDiff command.
- *
- * This has no side-effects on the model or Session, the returned command
- * can be held on to for as long as the caller wishes, or discarded without
- * formality, until apply_command is called and ownership is taken.
- */
 MidiModel::NoteDiffCommand*
 MidiModel::new_note_diff_command (const string& name)
 {
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-
-	return new NoteDiffCommand (ms->model(), name);
+	/* return via the MidiSource to get a shared_ptr to
+	 *  ourselves. Probably faster than shared_from_this()
+	 */
+	return new NoteDiffCommand (_midi_source.model(), name);
 }
 
-/** Start a new SysExDiff command */
 MidiModel::SysExDiffCommand*
 MidiModel::new_sysex_diff_command (const string& name)
 {
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-
-	return new SysExDiffCommand (ms->model(), name);
+	/* return via the MidiSource to get a shared_ptr to
+	 *  ourselves. Probably faster than shared_from_this()
+	 */
+	return new SysExDiffCommand (_midi_source.model(), name);
 }
 
-/** Start a new PatchChangeDiff command */
 MidiModel::PatchChangeDiffCommand*
 MidiModel::new_patch_change_diff_command (const string& name)
 {
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-
-	return new PatchChangeDiffCommand (ms->model(), name);
+	return new PatchChangeDiffCommand (_midi_source.model(), name);
 }
 
 
-/** Apply a command.
- *
- * Ownership of cmd is taken, it must not be deleted by the caller.
- * The command will constitute one item on the undo stack.
- */
 void
-MidiModel::apply_command(Session& session, Command* cmd)
+MidiModel::apply_diff_command_as_commit(Session& session, Command* cmd)
 {
 	session.begin_reversible_command (cmd->name());
 	(*cmd)();
@@ -109,20 +100,22 @@ MidiModel::apply_command(Session& session, Command* cmd)
 	set_edited (true);
 }
 
-/** Apply a command as part of a larger reversible transaction
- *
- * Ownership of cmd is taken, it must not be deleted by the caller.
- * The command will constitute one item on the undo stack.
- */
 void
-MidiModel::apply_command_as_subcommand(Session& session, Command* cmd)
+MidiModel::apply_diff_command_as_subcommand(Session& session, Command* cmd)
 {
 	(*cmd)();
 	session.add_command (cmd);
 	set_edited (true);
 }
 
-/************** DIFF COMMAND ********************/
+void
+MidiModel::apply_diff_command_only(Session& session, Command* cmd)
+{
+	(*cmd)();
+	set_edited (true);
+}
+
+/* ************* DIFF COMMAND ********************/
 
 #define NOTE_DIFF_COMMAND_ELEMENT "NoteDiffCommand"
 #define DIFF_NOTES_ELEMENT "ChangedNotes"
@@ -136,7 +129,7 @@ MidiModel::apply_command_as_subcommand(Session& session, Command* cmd)
 #define REMOVED_PATCH_CHANGES_ELEMENT "RemovedPatchChanges"
 #define DIFF_PATCH_CHANGES_ELEMENT "ChangedPatchChanges"
 
-MidiModel::DiffCommand::DiffCommand(boost::shared_ptr<MidiModel> m, const std::string& name)
+MidiModel::DiffCommand::DiffCommand(std::shared_ptr<MidiModel> m, const std::string& name)
 	: Command (name)
 	, _model (m)
 	, _name (name)
@@ -144,7 +137,7 @@ MidiModel::DiffCommand::DiffCommand(boost::shared_ptr<MidiModel> m, const std::s
 	assert(_model);
 }
 
-MidiModel::NoteDiffCommand::NoteDiffCommand (boost::shared_ptr<MidiModel> m, const XMLNode& node)
+MidiModel::NoteDiffCommand::NoteDiffCommand (std::shared_ptr<MidiModel> m, const XMLNode& node)
 	: DiffCommand (m, "")
 {
 	assert (_model);
@@ -451,7 +444,7 @@ MidiModel::NoteDiffCommand::undo ()
 }
 
 XMLNode&
-MidiModel::NoteDiffCommand::marshal_note(const NotePtr note)
+MidiModel::NoteDiffCommand::marshal_note(const NotePtr note) const
 {
 	XMLNode* xml_note = new XMLNode("note");
 
@@ -468,40 +461,34 @@ MidiModel::NoteDiffCommand::marshal_note(const NotePtr note)
 Evoral::Sequence<MidiModel::TimeType>::NotePtr
 MidiModel::NoteDiffCommand::unmarshal_note (XMLNode *xml_note)
 {
-	Evoral::event_id_t id;
+	Evoral::event_id_t id = -1;
 	if (!xml_note->get_property ("id", id)) {
 		error << "note information missing ID value" << endmsg;
-		id = -1;
 	}
 
-	uint8_t note;
+	uint8_t note = 127;
 	if (!xml_note->get_property("note", note)) {
 		warning << "note information missing note value" << endmsg;
-		note = 127;
 	}
 
-	uint8_t channel;
+	uint8_t channel = 0;
 	if (!xml_note->get_property("channel", channel)) {
 		warning << "note information missing channel" << endmsg;
-		channel = 0;
 	}
 
-	MidiModel::TimeType time;
+	MidiModel::TimeType time = MidiModel::TimeType();
 	if (!xml_note->get_property("time", time)) {
 		warning << "note information missing time" << endmsg;
-		time = MidiModel::TimeType();
 	}
 
-	MidiModel::TimeType length;
+	MidiModel::TimeType length = MidiModel::TimeType();
 	if (!xml_note->get_property("length", length)) {
 		warning << "note information missing length" << endmsg;
-		length = MidiModel::TimeType(1);
 	}
 
-	uint8_t velocity;
+	uint8_t velocity = 127;
 	if (!xml_note->get_property("velocity", velocity)) {
 		warning << "note information missing velocity" << endmsg;
-		velocity = 127;
 	}
 
 	NotePtr note_ptr(new Evoral::Note<TimeType>(channel, time, length, note, velocity));
@@ -511,7 +498,7 @@ MidiModel::NoteDiffCommand::unmarshal_note (XMLNode *xml_note)
 }
 
 XMLNode&
-MidiModel::NoteDiffCommand::marshal_change (const NoteChange& change)
+MidiModel::NoteDiffCommand::marshal_change (const NoteChange& change) const
 {
 	XMLNode* xml_change = new XMLNode("Change");
 
@@ -561,7 +548,7 @@ MidiModel::NoteDiffCommand::unmarshal_change (XMLNode *xml_change)
 	}
 
 	int old_val;
-	Evoral::Beats old_time;
+	Temporal::Beats old_time;
 	if ((change.property == StartTime || change.property == Length) &&
 	    xml_change->get_property ("old", old_time)) {
 		change.old_value = old_time;
@@ -573,7 +560,7 @@ MidiModel::NoteDiffCommand::unmarshal_change (XMLNode *xml_change)
 	}
 
 	int new_val;
-	Evoral::Beats new_time;
+	Temporal::Beats new_time;
 	if ((change.property == StartTime || change.property == Length) &&
 	    xml_change->get_property ("new", new_time)) {
 		change.new_value = new_time;
@@ -655,25 +642,25 @@ MidiModel::NoteDiffCommand::set_state (const XMLNode& diff_command, int /*versio
 }
 
 XMLNode&
-MidiModel::NoteDiffCommand::get_state ()
+MidiModel::NoteDiffCommand::get_state () const
 {
 	XMLNode* diff_command = new XMLNode (NOTE_DIFF_COMMAND_ELEMENT);
-	diff_command->set_property("midi-source", _model->midi_source()->id().to_s());
+	diff_command->set_property("midi-source", _model->midi_source().id().to_s());
 
 	XMLNode* changes = diff_command->add_child(DIFF_NOTES_ELEMENT);
-	for_each(_changes.begin(), _changes.end(),
+	for_each(_changes.cbegin(), _changes.cend(),
 	         boost::bind (
 		         boost::bind (&XMLNode::add_child_nocopy, changes, _1),
 		         boost::bind (&NoteDiffCommand::marshal_change, this, _1)));
 
 	XMLNode* added_notes = diff_command->add_child(ADDED_NOTES_ELEMENT);
-	for_each(_added_notes.begin(), _added_notes.end(),
+	for_each(_added_notes.cbegin(), _added_notes.cend(),
 	         boost::bind(
 		         boost::bind (&XMLNode::add_child_nocopy, added_notes, _1),
 		         boost::bind (&NoteDiffCommand::marshal_note, this, _1)));
 
 	XMLNode* removed_notes = diff_command->add_child(REMOVED_NOTES_ELEMENT);
-	for_each(_removed_notes.begin(), _removed_notes.end(),
+	for_each(_removed_notes.cbegin(), _removed_notes.cend(),
 	         boost::bind (
 		         boost::bind (&XMLNode::add_child_nocopy, removed_notes, _1),
 		         boost::bind (&NoteDiffCommand::marshal_note, this, _1)));
@@ -692,7 +679,7 @@ MidiModel::NoteDiffCommand::get_state ()
 	return *diff_command;
 }
 
-MidiModel::SysExDiffCommand::SysExDiffCommand (boost::shared_ptr<MidiModel> m, const XMLNode& node)
+MidiModel::SysExDiffCommand::SysExDiffCommand (std::shared_ptr<MidiModel> m, const XMLNode& node)
 	: DiffCommand (m, "")
 {
 	assert (_model);
@@ -700,7 +687,7 @@ MidiModel::SysExDiffCommand::SysExDiffCommand (boost::shared_ptr<MidiModel> m, c
 }
 
 void
-MidiModel::SysExDiffCommand::change (boost::shared_ptr<Evoral::Event<TimeType> > s, TimeType new_time)
+MidiModel::SysExDiffCommand::change (std::shared_ptr<Evoral::Event<TimeType> > s, TimeType new_time)
 {
 	Change change;
 
@@ -781,7 +768,7 @@ MidiModel::SysExDiffCommand::remove (SysExPtr sysex)
 }
 
 XMLNode&
-MidiModel::SysExDiffCommand::marshal_change (const Change& change)
+MidiModel::SysExDiffCommand::marshal_change (const Change& change) const
 {
 	XMLNode* xml_change = new XMLNode ("Change");
 
@@ -855,10 +842,10 @@ MidiModel::SysExDiffCommand::set_state (const XMLNode& diff_command, int /*versi
 }
 
 XMLNode&
-MidiModel::SysExDiffCommand::get_state ()
+MidiModel::SysExDiffCommand::get_state () const
 {
 	XMLNode* diff_command = new XMLNode (SYSEX_DIFF_COMMAND_ELEMENT);
-	diff_command->set_property ("midi-source", _model->midi_source()->id().to_s());
+	diff_command->set_property ("midi-source", _model->midi_source().id().to_s());
 
 	XMLNode* changes = diff_command->add_child(DIFF_SYSEXES_ELEMENT);
 	for_each (_changes.begin(), _changes.end(),
@@ -869,13 +856,13 @@ MidiModel::SysExDiffCommand::get_state ()
 	return *diff_command;
 }
 
-MidiModel::PatchChangeDiffCommand::PatchChangeDiffCommand (boost::shared_ptr<MidiModel> m, const string& name)
+MidiModel::PatchChangeDiffCommand::PatchChangeDiffCommand (std::shared_ptr<MidiModel> m, const string& name)
 	: DiffCommand (m, name)
 {
 	assert (_model);
 }
 
-MidiModel::PatchChangeDiffCommand::PatchChangeDiffCommand (boost::shared_ptr<MidiModel> m, const XMLNode & node)
+MidiModel::PatchChangeDiffCommand::PatchChangeDiffCommand (std::shared_ptr<MidiModel> m, const XMLNode & node)
 	: DiffCommand (m, "")
 {
 	assert (_model);
@@ -1060,7 +1047,7 @@ MidiModel::PatchChangeDiffCommand::undo ()
 }
 
 XMLNode &
-MidiModel::PatchChangeDiffCommand::marshal_patch_change (constPatchChangePtr p)
+MidiModel::PatchChangeDiffCommand::marshal_patch_change (constPatchChangePtr p) const
 {
 	XMLNode* n = new XMLNode ("patch-change");
 
@@ -1074,7 +1061,7 @@ MidiModel::PatchChangeDiffCommand::marshal_patch_change (constPatchChangePtr p)
 }
 
 XMLNode&
-MidiModel::PatchChangeDiffCommand::marshal_change (const Change& c)
+MidiModel::PatchChangeDiffCommand::marshal_change (const Change& c) const
 {
 	XMLNode* n = new XMLNode (X_("Change"));
 
@@ -1113,7 +1100,7 @@ MidiModel::PatchChangeDiffCommand::unmarshal_patch_change (XMLNode* n)
 		assert(false);
 	}
 
-	Evoral::Beats time = Evoral::Beats();
+	Temporal::Beats time = Temporal::Beats();
 	if (!n->get_property ("time", time)) {
 		// warning??
 	}
@@ -1200,13 +1187,13 @@ MidiModel::PatchChangeDiffCommand::set_state (const XMLNode& diff_command, int /
 }
 
 XMLNode &
-MidiModel::PatchChangeDiffCommand::get_state ()
+MidiModel::PatchChangeDiffCommand::get_state () const
 {
 	XMLNode* diff_command = new XMLNode (PATCH_CHANGE_DIFF_COMMAND_ELEMENT);
-	diff_command->set_property("midi-source", _model->midi_source()->id().to_s());
+	diff_command->set_property("midi-source", _model->midi_source().id().to_s());
 
 	XMLNode* added = diff_command->add_child (ADDED_PATCH_CHANGES_ELEMENT);
-	for_each (_added.begin(), _added.end(),
+	for_each (_added.cbegin(), _added.cend(),
 		  boost::bind (
 			  boost::bind (&XMLNode::add_child_nocopy, added, _1),
 			  boost::bind (&PatchChangeDiffCommand::marshal_patch_change, this, _1)
@@ -1214,7 +1201,7 @@ MidiModel::PatchChangeDiffCommand::get_state ()
 		);
 
 	XMLNode* removed = diff_command->add_child (REMOVED_PATCH_CHANGES_ELEMENT);
-	for_each (_removed.begin(), _removed.end(),
+	for_each (_removed.cbegin(), _removed.cend(),
 		  boost::bind (
 			  boost::bind (&XMLNode::add_child_nocopy, removed, _1),
 			  boost::bind (&PatchChangeDiffCommand::marshal_patch_change, this, _1)
@@ -1222,7 +1209,7 @@ MidiModel::PatchChangeDiffCommand::get_state ()
 		);
 
 	XMLNode* changes = diff_command->add_child (DIFF_PATCH_CHANGES_ELEMENT);
-	for_each (_changes.begin(), _changes.end(),
+	for_each (_changes.cbegin(), _changes.cend(),
 		  boost::bind (
 			  boost::bind (&XMLNode::add_child_nocopy, changes, _1),
 			  boost::bind (&PatchChangeDiffCommand::marshal_change, this, _1)
@@ -1243,25 +1230,24 @@ MidiModel::PatchChangeDiffCommand::get_state ()
  * `Discrete' mode).
  */
 bool
-MidiModel::write_to (boost::shared_ptr<MidiSource>     source,
-                     const Glib::Threads::Mutex::Lock& source_lock)
+MidiModel::write_to (std::shared_ptr<MidiSource>     source,
+                     const Source::WriterLock& source_lock)
 {
-	ReadLock lock(read_lock());
-
-	const bool old_percussive = percussive();
-	set_percussive(false);
+	ReadLock lock (read_lock()); /* Sequence read-lock */
 
 	source->drop_model(source_lock);
-	source->mark_streaming_midi_write_started (source_lock, note_mode());
+	/* as of March 2022 or long before , the note mode argument does nothing */
+	source->mark_streaming_midi_write_started (source_lock, Sustained);
 
 	for (Evoral::Sequence<TimeType>::const_iterator i = begin(TimeType(), true); i != end(); ++i) {
 		source->append_event_beats(source_lock, *i);
 	}
 
-	set_percussive(old_percussive);
 	source->mark_streaming_write_completed(source_lock);
 
-	set_edited(false);
+	/* no call to set_edited() because writing to "newsrc" doesn't remove
+	 * the need to write to "our own" source in ::sync_to_source()
+	 */
 
 	return true;
 }
@@ -1272,31 +1258,22 @@ MidiModel::write_to (boost::shared_ptr<MidiSource>     source,
     of the model.
 */
 bool
-MidiModel::sync_to_source (const Glib::Threads::Mutex::Lock& source_lock)
+MidiModel::sync_to_source (const Source::WriterLock& source_lock)
 {
 	ReadLock lock(read_lock());
 
-	const bool old_percussive = percussive();
-	set_percussive(false);
-
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	if (!ms) {
-		error << "MIDI model has no source to sync to" << endmsg;
-		return false;
-	}
-
 	/* Invalidate and store active notes, which will be picked up by the iterator
 	   on the next roll if time progresses linearly. */
-	ms->invalidate(source_lock);
+	_midi_source.invalidate(source_lock);
 
-	ms->mark_streaming_midi_write_started (source_lock, note_mode());
+	/* as of March 2022 or long before , the note mode argument does nothing */
+	_midi_source.mark_streaming_midi_write_started (source_lock, Sustained);
 
 	for (Evoral::Sequence<TimeType>::const_iterator i = begin(TimeType(), true); i != end(); ++i) {
-		ms->append_event_beats(source_lock, *i);
+		_midi_source.append_event_beats(source_lock, *i);
 	}
 
-	set_percussive (old_percussive);
-	ms->mark_streaming_write_completed (source_lock);
+	_midi_source.mark_streaming_write_completed (source_lock);
 
 	set_edited (false);
 
@@ -1311,20 +1288,18 @@ MidiModel::sync_to_source (const Glib::Threads::Mutex::Lock& source_lock)
  * destroying the original note durations.
  */
 bool
-MidiModel::write_section_to (boost::shared_ptr<MidiSource>     source,
-                             const Glib::Threads::Mutex::Lock& source_lock,
+MidiModel::write_section_to (std::shared_ptr<MidiSource>     source,
+                             const Source::WriterLock&         source_lock,
                              TimeType                          begin_time,
                              TimeType                          end_time,
                              bool                              offset_events)
 {
 	ReadLock lock(read_lock());
-	MidiStateTracker mst;
-
-	const bool old_percussive = percussive();
-	set_percussive(false);
+	MidiNoteTracker mst;
 
 	source->drop_model(source_lock);
-	source->mark_streaming_midi_write_started (source_lock, note_mode());
+	/* as of March 2022 or long before , the note mode argument does nothing */
+	source->mark_streaming_midi_write_started (source_lock, Sustained);
 
 	for (Evoral::Sequence<TimeType>::const_iterator i = begin(TimeType(), true); i != end(); ++i) {
 		if (i->time() >= begin_time && i->time() < end_time) {
@@ -1362,7 +1337,6 @@ MidiModel::write_section_to (boost::shared_ptr<MidiSource>     source,
 	}
 	mst.resolve_notes (*source, source_lock, end_time);
 
-	set_percussive(old_percussive);
 	source->mark_streaming_write_completed(source_lock);
 
 	set_edited(false);
@@ -1371,7 +1345,7 @@ MidiModel::write_section_to (boost::shared_ptr<MidiSource>     source,
 }
 
 XMLNode&
-MidiModel::get_state()
+MidiModel::get_state() const
 {
 	XMLNode *node = new XMLNode("MidiModel");
 	return *node;
@@ -1399,7 +1373,7 @@ MidiModel::find_note (NotePtr other)
 }
 
 Evoral::Sequence<MidiModel::TimeType>::NotePtr
-MidiModel::find_note (gint note_id)
+MidiModel::find_note (Evoral::event_id_t note_id)
 {
 	/* used only for looking up notes when reloading history from disk,
 	   so we don't care about performance *too* much.
@@ -1426,8 +1400,8 @@ MidiModel::find_patch_change (Evoral::event_id_t id)
 	return PatchChangePtr ();
 }
 
-boost::shared_ptr<Evoral::Event<MidiModel::TimeType> >
-MidiModel::find_sysex (gint sysex_id)
+std::shared_ptr<Evoral::Event<MidiModel::TimeType> >
+MidiModel::find_sysex (Evoral::event_id_t sysex_id)
 {
 	/* used only for looking up notes when reloading history from disk,
 	   so we don't care about performance *too* much.
@@ -1439,7 +1413,7 @@ MidiModel::find_sysex (gint sysex_id)
 		}
 	}
 
-	return boost::shared_ptr<Evoral::Event<TimeType> > ();
+	return std::shared_ptr<Evoral::Event<TimeType> > ();
 }
 
 /** Lock and invalidate the source.
@@ -1448,18 +1422,15 @@ MidiModel::find_sysex (gint sysex_id)
 MidiModel::WriteLock
 MidiModel::edit_lock()
 {
-	boost::shared_ptr<MidiSource> ms          = _midi_source.lock();
-	Glib::Threads::Mutex::Lock*   source_lock = 0;
+	Source::WriterLock*   source_lock = 0;
 
-	if (ms) {
-		/* Take source lock and invalidate iterator to release its lock on model.
-		   Add currently active notes to _active_notes so we can restore them
-		   if playback resumes at the same point after the edit. */
-		source_lock = new Glib::Threads::Mutex::Lock(ms->mutex());
-		ms->invalidate(*source_lock);
-	}
-
-	return WriteLock(new WriteLockImpl(source_lock, _lock, _control_lock));
+	/* Take source lock and invalidate iterator to release its lock on model.
+	 * Add currently active notes to _active_notes so we can restore them
+	 * if playback resumes at the same point after the edit.
+	 */
+	source_lock = new Source::WriterLock (_midi_source.mutex());
+	_midi_source.invalidate(*source_lock);
+	return WriteLock (new WriteLockImpl (source_lock, _lock, _control_lock));
 }
 
 int
@@ -1491,16 +1462,16 @@ MidiModel::resolve_overlaps_unlocked (const NotePtr note, void* arg)
 
 		TimeType sb = (*i)->time();
 		TimeType eb = (*i)->end_time();
-		OverlapType overlap = OverlapNone;
+		Temporal::OverlapType overlap = Temporal::OverlapNone;
 
 		if ((sb > sa) && (eb <= ea)) {
-			overlap = OverlapInternal;
+			overlap = Temporal::OverlapInternal;
 		} else if ((eb > sa) && (eb <= ea)) {
-			overlap = OverlapStart;
+			overlap = Temporal::OverlapStart;
 		} else if ((sb > sa) && (sb < ea)) {
-			overlap = OverlapEnd;
+			overlap = Temporal::OverlapEnd;
 		} else if ((sa >= sb) && (sa <= eb) && (ea <= eb)) {
-			overlap = OverlapExternal;
+			overlap = Temporal::OverlapExternal;
 		} else {
 			/* no overlap */
 			continue;
@@ -1516,7 +1487,7 @@ MidiModel::resolve_overlaps_unlocked (const NotePtr note, void* arg)
 		}
 
 		switch (overlap) {
-		case OverlapStart:
+		case Temporal::OverlapStart:
 			cerr << "OverlapStart\n";
 			/* existing note covers start of new note */
 			switch (insert_merge_policy()) {
@@ -1549,7 +1520,7 @@ MidiModel::resolve_overlaps_unlocked (const NotePtr note, void* arg)
 			}
 			break;
 
-		case OverlapEnd:
+		case Temporal::OverlapEnd:
 			cerr << "OverlapEnd\n";
 			/* existing note covers end of new note */
 			switch (insert_merge_policy()) {
@@ -1585,7 +1556,7 @@ MidiModel::resolve_overlaps_unlocked (const NotePtr note, void* arg)
 			}
 			break;
 
-		case OverlapExternal:
+		case Temporal::OverlapExternal:
 			cerr << "OverlapExt\n";
 			/* existing note overlaps all the new note */
 			switch (insert_merge_policy()) {
@@ -1604,7 +1575,7 @@ MidiModel::resolve_overlaps_unlocked (const NotePtr note, void* arg)
 			}
 			break;
 
-		case OverlapInternal:
+		case Temporal::OverlapInternal:
 			cerr << "OverlapInt\n";
 			/* new note fully overlaps an existing note */
 			switch (insert_merge_policy()) {
@@ -1658,33 +1629,7 @@ InsertMergePolicy
 MidiModel::insert_merge_policy () const
 {
 	/* XXX ultimately this should be a per-track or even per-model policy */
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-
-	return ms->session().config.get_insert_merge_policy ();
-}
-
-void
-MidiModel::set_midi_source (boost::shared_ptr<MidiSource> s)
-{
-	boost::shared_ptr<MidiSource> old = _midi_source.lock ();
-
-	if (old) {
-		Source::Lock lm(old->mutex());
-		old->invalidate (lm);
-	}
-
-	_midi_source_connections.drop_connections ();
-
-	_midi_source = s;
-
-	s->InterpolationChanged.connect_same_thread (
-		_midi_source_connections, boost::bind (&MidiModel::source_interpolation_changed, this, _1, _2)
-		);
-
-	s->AutomationStateChanged.connect_same_thread (
-		_midi_source_connections, boost::bind (&MidiModel::source_automation_state_changed, this, _1, _2)
-		);
+	return _midi_source.session().config.get_insert_merge_policy ();
 }
 
 /** The source has signalled that the interpolation style for a parameter has changed.  In order to
@@ -1695,77 +1640,68 @@ MidiModel::set_midi_source (boost::shared_ptr<MidiSource> s)
  *  or the other is listened to by the GUI.
  */
 void
-MidiModel::source_interpolation_changed (Evoral::Parameter p, Evoral::ControlList::InterpolationStyle s)
+MidiModel::source_interpolation_changed (Evoral::Parameter const& p, AutomationList::InterpolationStyle s)
 {
-	Glib::Threads::Mutex::Lock lm (_control_lock);
-	control(p)->list()->set_interpolation (s);
+	{
+		Glib::Threads::Mutex::Lock lm (_control_lock);
+		control(p)->list()->set_interpolation (s);
+	}
+	/* re-read MIDI */
+	ContentsChanged (); /* EMIT SIGNAL */
 }
 
 /** A ControlList has signalled that its interpolation style has changed.  Again, in order to keep
  *  MidiSource and ControlList interpolation state in sync, we pass this change onto our MidiSource.
  */
 void
-MidiModel::control_list_interpolation_changed (Evoral::Parameter p, Evoral::ControlList::InterpolationStyle s)
+MidiModel::control_list_interpolation_changed (Evoral::Parameter const& p, AutomationList::InterpolationStyle s)
 {
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-
-	ms->set_interpolation_of (p, s);
+	_midi_source.set_interpolation_of (p, s);
 }
 
 void
-MidiModel::source_automation_state_changed (Evoral::Parameter p, AutoState s)
+MidiModel::source_automation_state_changed (Evoral::Parameter const& p, AutoState s)
 {
-	Glib::Threads::Mutex::Lock lm (_control_lock);
-	boost::shared_ptr<AutomationList> al = boost::dynamic_pointer_cast<AutomationList> (control(p)->list ());
-	al->set_automation_state (s);
+	{
+		Glib::Threads::Mutex::Lock lm (_control_lock);
+		std::shared_ptr<AutomationList> al = std::dynamic_pointer_cast<AutomationList> (control(p)->list ());
+		al->set_automation_state (s);
+	}
+	/* re-read MIDI */
+	ContentsChanged (); /* EMIT SIGNAL */
 }
 
 void
-MidiModel::automation_list_automation_state_changed (Evoral::Parameter p, AutoState s)
+MidiModel::automation_list_automation_state_changed (Evoral::Parameter const& p, AutoState s)
 {
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
-	ms->set_automation_state_of (p, s);
+	_midi_source.set_automation_state_of (p, s);
 }
 
-boost::shared_ptr<Evoral::Control>
+std::shared_ptr<Evoral::Control>
 MidiModel::control_factory (Evoral::Parameter const & p)
 {
-	boost::shared_ptr<Evoral::Control> c = Automatable::control_factory (p);
+	std::shared_ptr<Evoral::Control> c = Automatable::control_factory (p);
 
 	/* Set up newly created control's lists to the appropriate interpolation and
 	   automation state from our source.
 	*/
 
-	boost::shared_ptr<MidiSource> ms = _midi_source.lock ();
-	assert (ms);
+	c->list()->set_interpolation (_midi_source.interpolation_of (p));
 
-	c->list()->set_interpolation (ms->interpolation_of (p));
-
-	boost::shared_ptr<AutomationList> al = boost::dynamic_pointer_cast<AutomationList> (c->list ());
+	std::shared_ptr<AutomationList> al = std::dynamic_pointer_cast<AutomationList> (c->list ());
 	assert (al);
 
-	al->set_automation_state (ms->automation_state_of (p));
+	al->set_automation_state (_midi_source.automation_state_of (p));
 
 	return c;
 }
 
-boost::shared_ptr<const MidiSource>
-MidiModel::midi_source ()
-{
-	return _midi_source.lock ();
-}
-
 /** Moves notes, patch changes, controllers and sys-ex to insert silence at the start of the model.
  *  Adds commands to the session's current undo stack to reflect the movements.
  */
 void
 MidiModel::insert_silence_at_start (TimeType t)
 {
-	boost::shared_ptr<MidiSource> s = _midi_source.lock ();
-	assert (s);
-
 	/* Notes */
 
 	if (!notes().empty ()) {
@@ -1775,7 +1711,7 @@ MidiModel::insert_silence_at_start (TimeType t)
 			c->change (*i, NoteDiffCommand::StartTime, (*i)->time() + t);
 		}
 
-		apply_command_as_subcommand (s->session(), c);
+		apply_diff_command_as_subcommand (_midi_source.session(), c);
 	}
 
 	/* Patch changes */
@@ -1787,17 +1723,17 @@ MidiModel::insert_silence_at_start (TimeType t)
 			c->change_time (*i, (*i)->time() + t);
 		}
 
-		apply_command_as_subcommand (s->session(), c);
+		apply_diff_command_as_subcommand (_midi_source.session(), c);
 	}
 
 	/* Controllers */
 
 	for (Controls::iterator i = controls().begin(); i != controls().end(); ++i) {
-		boost::shared_ptr<AutomationControl> ac = boost::dynamic_pointer_cast<AutomationControl> (i->second);
+		std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl> (i->second);
 		XMLNode& before = ac->alist()->get_state ();
-		i->second->list()->shift (0, t.to_double());
+		i->second->list()->shift (timepos_t::zero (i->second->list()->time_domain()), timecnt_t (t));
 		XMLNode& after = ac->alist()->get_state ();
-		s->session().add_command (new MementoCommand<AutomationList> (new MidiAutomationListBinder (s, i->first), &before, &after));
+		_midi_source.session().add_command (new MementoCommand<AutomationList> (new MidiAutomationListBinder (_midi_source, i->first), &before, &after));
 	}
 
 	/* Sys-ex */
@@ -1809,10 +1745,10 @@ MidiModel::insert_silence_at_start (TimeType t)
 			c->change (*i, (*i)->time() + t);
 		}
 
-		apply_command_as_subcommand (s->session(), c);
+		apply_diff_command_as_subcommand (_midi_source.session(), c);
 	}
 
-	ContentsShifted (t.to_double());
+	ContentsShifted (timecnt_t (t));
 }
 
 void
@@ -1832,7 +1768,7 @@ MidiModel::transpose (NoteDiffCommand* c, const NotePtr note_ptr, int semitones)
 void
 MidiModel::control_list_marked_dirty ()
 {
-	AutomatableSequence<Evoral::Beats>::control_list_marked_dirty ();
+	AutomatableSequence<Temporal::Beats>::control_list_marked_dirty ();
 
 	ContentsChanged (); /* EMIT SIGNAL */
 }
