@@ -1,21 +1,27 @@
 /*
-    Copyright (C) 2006 Paul Davis
-    Author: David Robillard
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2016 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2007-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2008-2012 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2013-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2013-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2013 John Emmas <john@creativepost.co.uk>
+ * Copyright (C) 2015-2018 Ben Loftis <ben@harrisonconsoles.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 #include <cmath>
 
 #ifdef COMPILER_MSVC
@@ -33,13 +39,17 @@
 #include "pbd/types_convert.h"
 #include "evoral/midi_util.h"
 
-#include "ardour/beats_frames_converter.h"
+#include "ardour/amp.h"
+#ifdef HAVE_BEATBOX
+#include "ardour/beatbox.h"
+#endif
 #include "ardour/buffer_set.h"
 #include "ardour/debug.h"
 #include "ardour/delivery.h"
+#include "ardour/disk_reader.h"
+#include "ardour/disk_writer.h"
 #include "ardour/event_type_map.h"
 #include "ardour/meter.h"
-#include "ardour/midi_diskstream.h"
 #include "ardour/midi_playlist.h"
 #include "ardour/midi_port.h"
 #include "ardour/midi_region.h"
@@ -71,12 +81,17 @@ using namespace PBD;
 MidiTrack::MidiTrack (Session& sess, string name, TrackMode mode)
 	: Track (sess, name, PresentationInfo::MidiTrack, mode, DataType::MIDI)
 	, _immediate_events(6096) // FIXME: size?
+	, _immediate_event_buffer(6096)
 	, _step_edit_ring_buffer(64) // FIXME: size?
-	, _note_mode(Sustained)
+	, _note_mode (Sustained)
 	, _step_editing (false)
 	, _input_active (true)
+	, _restore_pgm_on_load (true)
 {
 	_session.SessionLoaded.connect_same_thread (*this, boost::bind (&MidiTrack::restore_controls, this));
+
+	_playback_filter.ChannelModeChanged.connect_same_thread (*this, boost::bind (&Track::playlist_modified, this));
+	_playback_filter.ChannelMaskChanged.connect_same_thread (*this, boost::bind (&Track::playlist_modified, this));
 }
 
 MidiTrack::~MidiTrack ()
@@ -92,20 +107,25 @@ MidiTrack::init ()
 
 	_input->changed.connect_same_thread (*this, boost::bind (&MidiTrack::track_input_active, this, _1, _2));
 
+	_disk_writer->set_note_mode (_note_mode);
+	_disk_reader->reset_tracker ();
+
+	_disk_writer->DataRecorded.connect_same_thread (*this, boost::bind (&MidiTrack::data_recorded, this, _1));
+
+#ifdef HAVE_BEATBOX
+	_beatbox.reset (new BeatBox (_session));
+	add_processor (_beatbox, PostFader);
+#endif
+
 	return 0;
 }
 
-boost::shared_ptr<Diskstream>
-MidiTrack::create_diskstream ()
+void
+MidiTrack::data_recorded (std::weak_ptr<MidiSource> src)
 {
-	MidiDiskstream::Flag dflags = MidiDiskstream::Flag (MidiDiskstream::Recordable);
-
-	assert(_mode != Destructive);
-
-	return boost::shared_ptr<Diskstream> (new MidiDiskstream (_session, name(), dflags));
+	DataRecorded (src); /* EMIT SIGNAL */
 }
 
-
 bool
 MidiTrack::can_be_record_safe ()
 {
@@ -126,34 +146,17 @@ MidiTrack::can_be_record_enabled ()
 	return Track::can_be_record_enabled ();
 }
 
-void
-MidiTrack::set_diskstream (boost::shared_ptr<Diskstream> ds)
+MonitorState
+MidiTrack::get_input_monitoring_state (bool recording, bool talkback) const
 {
-	/* We have to do this here, as Track::set_diskstream will cause a buffer refill,
-	   and the diskstream must be set up to fill its buffers using the correct _note_mode.
-	*/
-	boost::shared_ptr<MidiDiskstream> mds = boost::dynamic_pointer_cast<MidiDiskstream> (ds);
-	mds->set_note_mode (_note_mode);
-
-	Track::set_diskstream (ds);
-
-	mds->reset_tracker ();
-
-	_diskstream->set_track (this);
-	_diskstream->set_record_enabled (false);
-
-	_diskstream_data_recorded_connection.disconnect ();
-	mds->DataRecorded.connect_same_thread (
-		_diskstream_data_recorded_connection,
-		boost::bind (&MidiTrack::diskstream_data_recorded, this, _1));
-
-	DiskstreamChanged (); /* EMIT SIGNAL */
-}
-
-boost::shared_ptr<MidiDiskstream>
-MidiTrack::midi_diskstream() const
-{
-	return boost::dynamic_pointer_cast<MidiDiskstream>(_diskstream);
+	RecordMode rmode = _session.config.get_record_mode ();
+	if (rmode == RecSoundOnSound && (recording || talkback)) {
+		return MonitoringCue;
+	} else if (rmode != RecNonLayered || recording || talkback) {
+		return MonitoringInput;
+	} else {
+		return MonitoringSilence;
+	}
 }
 
 int
@@ -179,6 +182,10 @@ MidiTrack::set_state (const XMLNode& node, int version)
 		set_input_active (yn);
 	}
 
+	if (node.get_property ("restore-pgm", yn)) {
+		set_restore_pgm_on_load (yn);
+	}
+
 	ChannelMode playback_channel_mode = AllChannels;
 	ChannelMode capture_channel_mode = AllChannels;
 
@@ -211,7 +218,7 @@ MidiTrack::set_state (const XMLNode& node, int version)
 
 	pending_state = const_cast<XMLNode*> (&node);
 
-	if (_session.state_of_the_state() & Session::Loading) {
+	if (_session.loading ()) {
 		_session.StateReady.connect_same_thread (
 			*this, boost::bind (&MidiTrack::set_state_part_two, this));
 	} else {
@@ -222,9 +229,9 @@ MidiTrack::set_state (const XMLNode& node, int version)
 }
 
 XMLNode&
-MidiTrack::state(bool full_state)
+MidiTrack::state(bool save_template) const
 {
-	XMLNode& root (Track::state(full_state));
+	XMLNode& root (Track::state (save_template));
 	XMLNode* freeze_node;
 	char buf[64];
 
@@ -233,9 +240,10 @@ MidiTrack::state(bool full_state)
 
 		freeze_node = new XMLNode (X_("freeze-info"));
 		freeze_node->set_property ("playlist", _freeze_record.playlist->name());
+		freeze_node->set_property ("playlist-id", _freeze_record.playlist->id().to_s());
 		freeze_node->set_property ("state", _freeze_record.state);
 
-		for (vector<FreezeRecordProcessorInfo*>::iterator i = _freeze_record.processor_info.begin(); i != _freeze_record.processor_info.end(); ++i) {
+		for (vector<FreezeRecordProcessorInfo*>::const_iterator i = _freeze_record.processor_info.begin(); i != _freeze_record.processor_info.end(); ++i) {
 			inode = new XMLNode (X_("processor"));
 			inode->set_property (X_("id"), id());
 			inode->add_child_copy ((*i)->state);
@@ -256,10 +264,11 @@ MidiTrack::state(bool full_state)
 	root.set_property ("note-mode", _note_mode);
 	root.set_property ("step-editing", _step_editing);
 	root.set_property ("input-active", _input_active);
+	root.set_property ("restore-pgm", _restore_pgm_on_load);
 
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
-		if (boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) {
-			boost::shared_ptr<AutomationControl> ac = boost::dynamic_pointer_cast<AutomationControl> (c->second);
+		if (std::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) {
+			std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl> (c->second);
 			assert (ac);
 			root.add_child_nocopy (ac->get_state ());
 		}
@@ -272,6 +281,7 @@ void
 MidiTrack::set_state_part_two ()
 {
 	XMLNode* fnode;
+	XMLProperty const * prop;
 
 	/* This is called after all session state has been restored but before
 	   have been made ports and connections are established.
@@ -290,23 +300,28 @@ MidiTrack::set_state_part_two ()
 		}
 		_freeze_record.processor_info.clear ();
 
-		std::string str;
-		if (fnode->get_property (X_("playlist"), str)) {
-			boost::shared_ptr<Playlist> pl = _session.playlists->by_name (str);
-			if (pl) {
-				_freeze_record.playlist = boost::dynamic_pointer_cast<MidiPlaylist> (pl);
-			} else {
-				_freeze_record.playlist.reset();
-				_freeze_record.state = NoFreeze;
-				return;
-			}
+		std::shared_ptr<Playlist> freeze_pl;
+		if ((prop = fnode->property (X_("playlist-id"))) != 0) {
+			freeze_pl = _session.playlists()->by_id (prop->value());
+		} else if ((prop = fnode->property (X_("playlist"))) != 0) {
+			freeze_pl = _session.playlists()->by_name (prop->value());
 		}
+		if (freeze_pl) {
+			_freeze_record.playlist = std::dynamic_pointer_cast<MidiPlaylist> (freeze_pl);
+			_freeze_record.playlist->use();
+		} else {
+			_freeze_record.playlist.reset ();
+			_freeze_record.state = NoFreeze;
+			return;
+		}
+
 
 		fnode->get_property (X_("state"), _freeze_record.state);
 
 		XMLNodeConstIterator citer;
 		XMLNodeList clist = fnode->children();
 
+		std::string str;
 		for (citer = clist.begin(); citer != clist.end(); ++citer) {
 			if ((*citer)->name() != X_("processor")) {
 				continue;
@@ -317,153 +332,61 @@ MidiTrack::set_state_part_two ()
 			}
 
 			FreezeRecordProcessorInfo* frii = new FreezeRecordProcessorInfo (*((*citer)->children().front()),
-										   boost::shared_ptr<Processor>());
+										   std::shared_ptr<Processor>());
 			frii->id = str;
 			_freeze_record.processor_info.push_back (frii);
 		}
 	}
 
-	if (midi_diskstream ()) {
-		midi_diskstream()->set_block_size (_session.get_block_size ());
-	}
-
 	return;
 }
 
 void
 MidiTrack::restore_controls ()
 {
-	// TODO order events (CC before PGM to set banks)
+	/* first CC (bank select) */
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
-		boost::shared_ptr<MidiTrack::MidiControl> mctrl = boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second);
-		if (mctrl) {
+		std::shared_ptr<MidiTrack::MidiControl> mctrl = std::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second);
+		if (mctrl && mctrl->parameter().type () != MidiPgmChangeAutomation) {
+			mctrl->restore_value();
+		}
+	}
+
+	if (!_restore_pgm_on_load) {
+		return;
+	}
+
+	/* then restore PGM */
+	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
+		std::shared_ptr<MidiTrack::MidiControl> mctrl = std::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second);
+		if (mctrl && mctrl->parameter().type () == MidiPgmChangeAutomation) {
 			mctrl->restore_value();
 		}
 	}
 }
 
 void
-MidiTrack::update_controls(const BufferSet& bufs)
+MidiTrack::update_controls (BufferSet const& bufs)
 {
 	const MidiBuffer& buf = bufs.get_midi(0);
 	for (MidiBuffer::const_iterator e = buf.begin(); e != buf.end(); ++e) {
-		const Evoral::Event<framepos_t>&         ev      = *e;
+		const Evoral::Event<samplepos_t>&         ev     = *e;
 		const Evoral::Parameter                  param   = midi_parameter(ev.buffer(), ev.size());
-		const boost::shared_ptr<AutomationControl> control = automation_control (param);
+		const std::shared_ptr<AutomationControl> control = automation_control (param);
 		if (control) {
-			control->set_double(ev.value(), _session.transport_frame(), false);
-			control->Changed (false, Controllable::NoGroup);
+			double old = control->get_double ();
+			control->set_double (ev.value(), timepos_t::zero (false), false);
+			if (old != ev.value()) {
+				control->Changed (false, Controllable::NoGroup);
+			}
 		}
 	}
 }
 
-/** @param need_butler to be set to true if this track now needs the butler, otherwise it can be left alone
- *  or set to false.
- */
 int
-MidiTrack::roll (pframes_t nframes, framepos_t start_frame, framepos_t end_frame, int declick, bool& need_butler)
+MidiTrack::no_roll_unlocked (pframes_t nframes, samplepos_t start_sample, samplepos_t end_sample, bool state_changing)
 {
-	Glib::Threads::RWLock::ReaderLock lm (_processor_lock, Glib::Threads::TRY_LOCK);
-	if (!lm.locked()) {
-		boost::shared_ptr<MidiDiskstream> diskstream = midi_diskstream();
-		framecnt_t playback_distance = diskstream->calculate_playback_distance(nframes);
-		if (can_internal_playback_seek(::llabs(playback_distance))) {
-			/* TODO should declick, and/or note-off */
-			internal_playback_seek(playback_distance);
-		}
-		return 0;
-	}
-
-	boost::shared_ptr<MidiDiskstream> diskstream = midi_diskstream();
-
-	if (n_outputs().n_total() == 0 && _processors.empty()) {
-		return 0;
-	}
-
-	if (!_active) {
-		silence (nframes);
-		if (_meter_point == MeterInput && ((_monitoring_control->monitoring_choice() & MonitorInput) || _diskstream->record_enabled())) {
-			_meter->reset();
-		}
-		return 0;
-	}
-
-	framepos_t transport_frame = _session.transport_frame();
-
-	int dret;
-	framecnt_t playback_distance;
-
-	if ((nframes = check_initial_delay (nframes, transport_frame)) == 0) {
-		/* need to do this so that the diskstream sets its
-		   playback distance to zero, thus causing diskstream::commit
-		   to do nothing.
-		   */
-		BufferSet bufs; /* empty set - is OK, since nothing will happen */
-
-		dret = diskstream->process (bufs, transport_frame, 0, playback_distance, false);
-		need_butler = diskstream->commit (playback_distance);
-		return dret;
-	}
-
-	BufferSet& bufs = _session.get_route_buffers (n_process_buffers());
-
-	fill_buffers_with_input (bufs, _input, nframes);
-
-	/* filter captured data before meter sees it */
-	_capture_filter.filter (bufs);
-
-	if (_meter_point == MeterInput && ((_monitoring_control->monitoring_choice() & MonitorInput) || _diskstream->record_enabled())) {
-		_meter->run (bufs, start_frame, end_frame, 1.0 /*speed()*/, nframes, true);
-	}
-
-
-	_silent = false;
-
-	if ((dret = diskstream->process (bufs, transport_frame, nframes, playback_distance, (monitoring_state() == MonitoringDisk))) != 0) {
-		need_butler = diskstream->commit (playback_distance);
-		silence (nframes);
-		return dret;
-	}
-
-	/* note diskstream uses our filter to filter/map playback channels appropriately. */
-
-	if (monitoring_state() == MonitoringInput) {
-
-		/* not actually recording, but we want to hear the input material anyway,
-		   at least potentially (depending on monitoring options)
-		*/
-
-		/* because the playback buffer is event based and not a
-		 * continuous stream, we need to make sure that we empty
-		 * it of events every cycle to avoid it filling up with events
-		 * read from disk, while we are actually monitoring input
-		 */
-
-		diskstream->flush_playback (start_frame, end_frame);
-
-	}
-
-
-	/* append immediate messages to the first MIDI buffer (thus sending it to the first output port) */
-
-	write_out_of_band_data (bufs, start_frame, end_frame, nframes);
-
-	/* final argument: don't waste time with automation if we're not recording or rolling */
-
-	process_output_buffers (bufs, start_frame, end_frame, nframes,
-				declick, (!diskstream->record_enabled() && !_session.transport_stopped()));
-
-	flush_processor_buffers_locked (nframes);
-
-	need_butler = diskstream->commit (playback_distance);
-
-	return 0;
-}
-
-int
-MidiTrack::no_roll (pframes_t nframes, framepos_t start_frame, framepos_t end_frame, bool state_changing)
-{
-	int ret = Track::no_roll (nframes, start_frame, end_frame, state_changing);
+	int ret = Track::no_roll_unlocked (nframes, start_sample, end_sample, state_changing);
 
 	if (ret == 0 && _step_editing) {
 		push_midi_input_to_step_edit_ringbuffer (nframes);
@@ -473,7 +396,7 @@ MidiTrack::no_roll (pframes_t nframes, framepos_t start_frame, framepos_t end_fr
 }
 
 void
-MidiTrack::realtime_locate ()
+MidiTrack::realtime_locate (bool for_loop_end)
 {
 	Glib::Threads::RWLock::ReaderLock lm (_processor_lock, Glib::Threads::TRY_LOCK);
 
@@ -482,39 +405,25 @@ MidiTrack::realtime_locate ()
 	}
 
 	for (ProcessorList::iterator i = _processors.begin(); i != _processors.end(); ++i) {
-		(*i)->realtime_locate ();
-	}
-
-	midi_diskstream()->reset_tracker ();
-}
-
-void
-MidiTrack::realtime_handle_transport_stopped ()
-{
-	Glib::Threads::RWLock::ReaderLock lm (_processor_lock, Glib::Threads::TRY_LOCK);
-
-	if (!lm.locked ()) {
-		return;
-	}
-
-	for (ProcessorList::iterator i = _processors.begin(); i != _processors.end(); ++i) {
-		(*i)->realtime_handle_transport_stopped ();
+		(*i)->realtime_locate (for_loop_end);
 	}
 }
 
 void
-MidiTrack::non_realtime_locate (framepos_t pos)
+MidiTrack::non_realtime_locate (samplepos_t spos)
 {
-	Track::non_realtime_locate(pos);
+	timepos_t pos (spos);
 
-	boost::shared_ptr<MidiPlaylist> playlist = midi_diskstream()->midi_playlist();
+	Track::non_realtime_locate (spos);
+
+	std::shared_ptr<MidiPlaylist> playlist = _disk_writer->midi_playlist();
 	if (!playlist) {
 		return;
 	}
 
 	/* Get the top unmuted region at this position. */
-	boost::shared_ptr<MidiRegion> region = boost::dynamic_pointer_cast<MidiRegion>(
-		playlist->top_unmuted_region_at(pos));
+	std::shared_ptr<MidiRegion> region = std::dynamic_pointer_cast<MidiRegion> (playlist->top_unmuted_region_at (pos));
+
 	if (!region) {
 		return;
 	}
@@ -530,23 +439,32 @@ MidiTrack::non_realtime_locate (framepos_t pos)
 	}
 
 	/* Update track controllers based on its "automation". */
-	const framepos_t     origin = region->position() - region->start();
-	BeatsFramesConverter bfc(_session.tempo_map(), origin);
+	const timepos_t pos_beats = timepos_t (region->source_position().distance (pos).beats ()); /* relative to source start */
+
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
-		boost::shared_ptr<MidiTrack::MidiControl> tcontrol;
-		boost::shared_ptr<Evoral::Control>        rcontrol;
-		if ((tcontrol = boost::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) &&
+
+		std::shared_ptr<AutomationControl> ac = std::dynamic_pointer_cast<AutomationControl> (c->second);
+
+		if (!ac->automation_playback()) {
+			continue;
+		}
+
+		std::shared_ptr<MidiTrack::MidiControl> tcontrol;
+		std::shared_ptr<Evoral::Control>        rcontrol;
+
+		if ((tcontrol = std::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) &&
+
 		    (rcontrol = region->control(tcontrol->parameter()))) {
-			const Evoral::Beats pos_beats = bfc.from(pos - origin);
+
 			if (rcontrol->list()->size() > 0) {
-				tcontrol->set_value(rcontrol->list()->eval(pos_beats.to_double()), Controllable::NoGroup);
+				tcontrol->set_value(rcontrol->list()->eval(pos_beats), Controllable::NoGroup);
 			}
 		}
 	}
 }
 
 void
-MidiTrack::push_midi_input_to_step_edit_ringbuffer (framecnt_t nframes)
+MidiTrack::push_midi_input_to_step_edit_ringbuffer (samplecnt_t nframes)
 {
 	PortSet& ports (_input->ports());
 
@@ -558,7 +476,7 @@ MidiTrack::push_midi_input_to_step_edit_ringbuffer (framecnt_t nframes)
 
 		for (MidiBuffer::const_iterator e = mb->begin(); e != mb->end(); ++e) {
 
-			const Evoral::Event<framepos_t> ev(*e, false);
+			const Evoral::Event<samplepos_t> ev(*e, false);
 
 			/* note on, since for step edit, note length is determined
 			   elsewhere
@@ -573,79 +491,103 @@ MidiTrack::push_midi_input_to_step_edit_ringbuffer (framecnt_t nframes)
 }
 
 void
-MidiTrack::write_out_of_band_data (BufferSet& bufs, framepos_t /*start*/, framepos_t /*end*/, framecnt_t nframes)
+MidiTrack::snapshot_out_of_band_data (samplecnt_t nframes)
+{
+	_immediate_event_buffer.clear ();
+	if (0 == _immediate_events.read_space()) {
+		return;
+	}
+
+	assert (nframes > 0);
+
+	DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 has %2 of immediate events to deliver\n", name(), _immediate_events.read_space()));
+
+	/* write as many of the immediate events as we can, but give "true" as
+	 * the last argument ("stop on overflow in destination") so that we'll
+	 * ship the rest out next time.
+	 *
+	 * the (nframes-1) argument puts all these events at the last
+	 * possible position of the output buffer, so that we do not
+	 * violate monotonicity when writing.
+	 */
+
+	_immediate_events.read (_immediate_event_buffer, 0, 1, nframes - 1, true);
+}
+
+void
+MidiTrack::write_out_of_band_data (BufferSet& bufs, samplecnt_t nframes) const
 {
 	MidiBuffer& buf (bufs.get_midi (0));
-
-	update_controls (bufs);
-
-	// Append immediate events
-
-	if (_immediate_events.read_space()) {
-
-		DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 has %2 of immediate events to deliver\n",
-		                                            name(), _immediate_events.read_space()));
-
-		/* write as many of the immediate events as we can, but give "true" as
-		 * the last argument ("stop on overflow in destination") so that we'll
-		 * ship the rest out next time.
-		 *
-		 * the Port::port_offset() + (nframes-1) argument puts all these events at the last
-		 * possible position of the output buffer, so that we do not
-		 * violate monotonicity when writing. Port::port_offset() will
-		 * be non-zero if we're in a split process cycle.
-		 */
-		_immediate_events.read (buf, 0, 1, Port::port_offset() + nframes - 1, true);
-	}
+	buf.merge_from (_immediate_event_buffer, nframes);
 }
 
 int
 MidiTrack::export_stuff (BufferSet&                   buffers,
-                         framepos_t                   start,
-                         framecnt_t                   nframes,
-                         boost::shared_ptr<Processor> endpoint,
+                         samplepos_t                  start,
+                         samplecnt_t                  nframes,
+                         std::shared_ptr<Processor> endpoint,
                          bool                         include_endpoint,
                          bool                         for_export,
-                         bool                         for_freeze)
+                         bool                         for_freeze,
+                         MidiNoteTracker&            tracker)
 {
 	if (buffers.count().n_midi() == 0) {
 		return -1;
 	}
 
-	boost::shared_ptr<MidiDiskstream> diskstream = midi_diskstream();
-
 	Glib::Threads::RWLock::ReaderLock rlock (_processor_lock);
 
-	boost::shared_ptr<MidiPlaylist> mpl = boost::dynamic_pointer_cast<MidiPlaylist>(diskstream->playlist());
+	std::shared_ptr<MidiPlaylist> mpl = _disk_writer->midi_playlist();
 	if (!mpl) {
 		return -2;
 	}
 
 	buffers.get_midi(0).clear();
-	if (mpl->read(buffers.get_midi(0), start, nframes, 0) != nframes) {
-		return -1;
+
+	/* Can't use a note tracker here, because the note off's might be in a
+	 * subsequent call
+	 */
+
+	MidiNoteTracker ignored;
+
+	/* XXX this doesn't fail, other than if the lock cannot be obtained */
+	mpl->rendered()->read (buffers.get_midi(0), start, start+nframes, ignored, start);
+
+	MidiBuffer& buf = buffers.get_midi(0);
+
+	if (endpoint && !for_export) {
+		for (MidiBuffer::iterator i = buf.begin(); i != buf.end(); ++i) {
+			MidiBuffer::TimeType *t = i.timeptr ();
+			*t -= start;
+		}
+		bounce_process (buffers, start, nframes, endpoint, include_endpoint, for_export, for_freeze);
 	}
 
-	//bounce_process (buffers, start, nframes, endpoint, include_endpoint, for_export, for_freeze);
+	/* Add to tracker so that we can resolve at the end of the export (in Session::write_one_track()) */
+
+	for (MidiBuffer::iterator i = buf.begin(); i != buf.end(); ++i) {
+		tracker.track (*i);
+	}
 
 	return 0;
 }
 
-boost::shared_ptr<Region>
-MidiTrack::bounce (InterThreadInfo& itt)
+std::shared_ptr<Region>
+MidiTrack::bounce (InterThreadInfo& itt, std::string const& name)
 {
-	return bounce_range (_session.current_start_frame(), _session.current_end_frame(), itt, main_outs(), false);
+	return bounce_range (_session.current_start_sample(), _session.current_end_sample(), itt, main_outs(), false, name);
 }
 
-boost::shared_ptr<Region>
-MidiTrack::bounce_range (framepos_t                   start,
-                         framepos_t                   end,
+std::shared_ptr<Region>
+MidiTrack::bounce_range (samplepos_t                  start,
+                         samplepos_t                  end,
                          InterThreadInfo&             itt,
-                         boost::shared_ptr<Processor> endpoint,
-                         bool                         include_endpoint)
+                         std::shared_ptr<Processor> endpoint,
+                         bool                         include_endpoint,
+                         std::string const&           name)
 {
-	vector<boost::shared_ptr<Source> > srcs;
-	return _session.write_one_track (*this, start, end, false, srcs, itt, endpoint, include_endpoint, false, false);
+	vector<std::shared_ptr<Source> > srcs;
+	return _session.write_one_track (*this, start, end, false, srcs, itt, endpoint, include_endpoint, false, false, name);
 }
 
 void
@@ -665,7 +607,7 @@ void
 MidiTrack::set_note_mode (NoteMode m)
 {
 	_note_mode = m;
-	midi_diskstream()->set_note_mode(m);
+	_disk_writer->set_note_mode(m);
 }
 
 std::string
@@ -681,24 +623,24 @@ MidiTrack::midi_panic()
 	DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 delivers panic data\n", name()));
 	for (uint8_t channel = 0; channel <= 0xF; channel++) {
 		uint8_t ev[3] = { ((uint8_t) (MIDI_CMD_CONTROL | channel)), ((uint8_t) MIDI_CTL_SUSTAIN), 0 };
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 		ev[1] = MIDI_CTL_ALL_NOTES_OFF;
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 		ev[1] = MIDI_CTL_RESET_CONTROLLERS;
-		write_immediate_event(3, ev);
+		write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 	}
 }
 
 /** \return true on success, false on failure (no buffer space left)
  */
 bool
-MidiTrack::write_immediate_event(size_t size, const uint8_t* buf)
+MidiTrack::write_immediate_event(Evoral::EventType event_type, size_t size, const uint8_t* buf)
 {
 	if (!Evoral::midi_event_is_valid(buf, size)) {
 		cerr << "WARNING: Ignoring illegal immediate MIDI event" << endl;
 		return false;
 	}
-	return (_immediate_events.write (0, Evoral::MIDI_EVENT, size, buf) == size);
+	return (_immediate_events.write (0, event_type, size, buf) == size);
 }
 
 void
@@ -749,45 +691,52 @@ MidiTrack::MidiControl::actually_set_value (double val, PBD::Controllable::Group
 		return;
 	}
 
+	if (_session.loading ()) {
+		/* send events later in MidiTrack::restore_controls */
+		AutomationControl::actually_set_value (val, group_override);
+		return;
+	}
+
 	assert(val <= desc.upper);
-	if ( ! _list || ! automation_playback()) {
+	if (!_list || !automation_playback ()) {
 		size_t size = 3;
 		uint8_t ev[3] = { parameter.channel(), uint8_t (val), 0 };
 		switch(parameter.type()) {
 		case MidiCCAutomation:
-			ev[0] += MIDI_CMD_CONTROL;
+			ev[0] |= MIDI_CMD_CONTROL;
 			ev[1] = parameter.id();
 			ev[2] = int(val);
 			break;
 
 		case MidiPgmChangeAutomation:
 			size = 2;
-			ev[0] += MIDI_CMD_PGM_CHANGE;
+			ev[0] |= MIDI_CMD_PGM_CHANGE;
 			ev[1] = int(val);
 			break;
 
 		case MidiChannelPressureAutomation:
 			size = 2;
-			ev[0] += MIDI_CMD_CHANNEL_PRESSURE;
+			ev[0] |= MIDI_CMD_CHANNEL_PRESSURE;
 			ev[1] = int(val);
 			break;
 
 		case MidiNotePressureAutomation:
-			ev[0] += MIDI_CMD_NOTE_PRESSURE;
+			ev[0] |= MIDI_CMD_NOTE_PRESSURE;
 			ev[1] = parameter.id();
 			ev[2] = int(val);
 			break;
 
 		case MidiPitchBenderAutomation:
-			ev[0] += MIDI_CMD_BENDER;
+			ev[0] |= MIDI_CMD_BENDER;
 			ev[1] = 0x7F & int(val);
 			ev[2] = 0x7F & (int(val) >> 7);
 			break;
 
 		default:
+			size = 0;
 			assert(false);
 		}
-		_route->write_immediate_event(size,  ev);
+		_route->write_immediate_event(Evoral::LIVE_MIDI_EVENT, size, ev);
 	}
 
 	AutomationControl::actually_set_value(val, group_override);
@@ -806,10 +755,10 @@ MidiTrack::set_step_editing (bool yn)
 	}
 }
 
-boost::shared_ptr<SMFSource>
+std::shared_ptr<SMFSource>
 MidiTrack::write_source (uint32_t)
 {
-	return midi_diskstream()->write_source ();
+	return _disk_writer->midi_write_source ();
 }
 
 void
@@ -844,16 +793,26 @@ MidiTrack::set_capture_channel_mask (uint16_t mask)
 	}
 }
 
-boost::shared_ptr<MidiPlaylist>
+std::shared_ptr<MidiPlaylist>
 MidiTrack::midi_playlist ()
 {
-	return midi_diskstream()->midi_playlist ();
+	return std::dynamic_pointer_cast<MidiPlaylist> (_playlists[DataType::MIDI]);
 }
 
 void
-MidiTrack::diskstream_data_recorded (boost::weak_ptr<MidiSource> src)
+MidiTrack::set_restore_pgm_on_load (bool yn)
 {
-	DataRecorded (src); /* EMIT SIGNAL */
+	if (_restore_pgm_on_load == yn) {
+		return;
+	}
+	_restore_pgm_on_load = yn;
+	_session.set_dirty();
+}
+
+bool
+MidiTrack::restore_pgm_on_load () const
+{
+	return _restore_pgm_on_load;
 }
 
 bool
@@ -882,7 +841,7 @@ MidiTrack::map_input_active (bool yn)
 	PortSet& ports (_input->ports());
 
 	for (PortSet::iterator p = ports.begin(DataType::MIDI); p != ports.end(DataType::MIDI); ++p) {
-		boost::shared_ptr<MidiPort> mp = boost::dynamic_pointer_cast<MidiPort> (*p);
+		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (*p);
 		if (yn != mp->input_active()) {
 			mp->set_input_active (yn);
 		}
@@ -897,16 +856,10 @@ MidiTrack::track_input_active (IOChange change, void* /* src */)
 	}
 }
 
-boost::shared_ptr<Diskstream>
-MidiTrack::diskstream_factory (XMLNode const & node)
-{
-	return boost::shared_ptr<Diskstream> (new MidiDiskstream (_session, node));
-}
-
-boost::shared_ptr<MidiBuffer>
+std::shared_ptr<MidiBuffer>
 MidiTrack::get_gui_feed_buffer () const
 {
-	return midi_diskstream()->get_gui_feed_buffer ();
+	return _disk_writer->get_gui_feed_buffer ();
 }
 
 void
@@ -922,7 +875,7 @@ MidiTrack::act_on_mute ()
 	/* If we haven't got a diskstream yet, there's nothing to worry about,
 	   and we can't call get_channel_mask() anyway.
 	*/
-	if (!midi_diskstream()) {
+	if (!_disk_writer) {
 		return;
 	}
 
@@ -937,7 +890,7 @@ MidiTrack::act_on_mute ()
 
 				DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 delivers mute message to channel %2\n", name(), channel+1));
 				uint8_t ev[3] = { ((uint8_t) (MIDI_CMD_CONTROL | channel)), MIDI_CTL_SUSTAIN, 0 };
-				write_immediate_event (3, ev);
+				write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
 
 				/* Note we do not send MIDI_CTL_ALL_NOTES_OFF here, since this may
 				   silence notes that came from another non-muted track. */
@@ -945,7 +898,7 @@ MidiTrack::act_on_mute ()
 		}
 
 		/* Resolve active notes. */
-		midi_diskstream()->resolve_tracker(_immediate_events, Port::port_offset());
+		_disk_reader->resolve_tracker (_immediate_events, 0);
 	}
 }
 
@@ -961,17 +914,13 @@ MidiTrack::monitoring_changed (bool self, Controllable::GroupControlDisposition
 	PortSet& ports (_output->ports());
 
 	for (PortSet::iterator p = ports.begin(); p != ports.end(); ++p) {
-		boost::shared_ptr<MidiPort> mp = boost::dynamic_pointer_cast<MidiPort> (*p);
+		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (*p);
 		if (mp) {
 			mp->require_resolve ();
 		}
 	}
 
-	boost::shared_ptr<MidiDiskstream> md (midi_diskstream());
-
-	if (md) {
-		md->reset_tracker ();
-	}
+	_disk_reader->reset_tracker ();
 }
 
 MonitorState
@@ -979,7 +928,27 @@ MidiTrack::monitoring_state () const
 {
 	MonitorState ms = Track::monitoring_state();
 	if (ms == MonitoringSilence) {
+		/* MIDI always monitor input as fallback */
 		return MonitoringInput;
 	}
 	return ms;
 }
+
+void
+MidiTrack::filter_input (BufferSet& bufs)
+{
+	_capture_filter.filter (bufs);
+}
+
+void
+MidiTrack::realtime_handle_transport_stopped ()
+{
+	Route::realtime_handle_transport_stopped ();
+	_disk_reader->resolve_tracker (_immediate_events, 0);
+}
+
+void
+MidiTrack::playlist_contents_changed ()
+
+{
+}
