@@ -1,29 +1,35 @@
 /*
-  Copyright (C) 1999-2002 Paul Davis
+ * Copyright (C) 2005-2009 Taybin Rutkin <taybin@taybin.com>
+ * Copyright (C) 2005-2019 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2006-2012 David Robillard <d@drobilla.net>
+ * Copyright (C) 2008 Hans Baier <hansfbaier@googlemail.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2012-2019 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2013 John Emmas <john@creativepost.co.uk>
+ * Copyright (C) 2013 Michael Fisher <mfisher31@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
-
-#include <string>
-#include <cmath>
-#include <cerrno>
 #include <cassert>
-#include <unistd.h>
+#include <cerrno>
+#include <cmath>
+#include <memory>
+#include <string>
 
-#include <boost/shared_ptr.hpp>
+#include <unistd.h>
 
 #include <glibmm/main.h>
 
@@ -33,9 +39,8 @@
 #include "pbd/error.h"
 #include "pbd/pthread_utils.h"
 #include "pbd/timersub.h"
-#include "pbd/stacktrace.h"
 
-#include "timecode/time.h"
+#include "temporal/time.h"
 
 #include "ardour/audio_track.h"
 #include "ardour/audioengine.h"
@@ -44,8 +49,10 @@
 #include "ardour/midi_track.h"
 #include "ardour/midi_ui.h"
 #include "ardour/profile.h"
+#include "ardour/route_group.h"
 #include "ardour/session.h"
-#include "ardour/slave.h"
+#include "ardour/transport_master.h"
+#include "ardour/transport_fsm.h"
 #include "ardour/ticker.h"
 
 #include "pbd/i18n.h"
@@ -60,11 +67,11 @@ void
 Session::midi_panic()
 {
 	{
-		boost::shared_ptr<RouteList> r = routes.reader ();
+		std::shared_ptr<RouteList const> r = routes.reader ();
 
-		for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
-			MidiTrack *track = dynamic_cast<MidiTrack*>((*i).get());
-			if (track != 0) {
+		for (auto const& i : *r) {
+			std::shared_ptr<MidiTrack> track = std::dynamic_pointer_cast<MidiTrack>(i);
+			if (track) {
 				track->midi_panic();
 			}
 		}
@@ -93,7 +100,7 @@ void
 Session::spp_start ()
 {
 	if (Config->get_mmc_control ()) {
-		request_transport_speed (1.0);
+		request_roll (TRS_MMC);
 	}
 }
 
@@ -115,7 +122,7 @@ void
 Session::mmc_deferred_play (MIDI::MachineControl &/*mmc*/)
 {
 	if (Config->get_mmc_control ()) {
-		request_transport_speed (1.0);
+		request_roll (TRS_MMC);
 	}
 }
 
@@ -130,26 +137,13 @@ Session::mmc_record_pause (MIDI::MachineControl &/*mmc*/)
 void
 Session::mmc_record_strobe (MIDI::MachineControl &/*mmc*/)
 {
-	if (Profile->get_trx()) {
-
-		/* In Tracks Live, there is no concept of punch, so we just
-		   treat RecordStrobe like RecordPause. This violates the MMC
-		   specification.
-		*/
-
-		if (Config->get_mmc_control()) {
-			maybe_enable_record();
-		}
-		return;
-	}
-
 	if (!Config->get_mmc_control() || (_step_editors > 0)) {
 		return;
 	}
 
 	/* record strobe does an implicit "Play" command */
 
-	if (_transport_speed != 1.0) {
+	if (_transport_fsm->transport_speed() != 1.0) {
 
 		/* start_transport() will move from Enabled->Recording, so we
 		   don't need to do anything here except enable recording.
@@ -158,10 +152,10 @@ Session::mmc_record_strobe (MIDI::MachineControl &/*mmc*/)
 		*/
 
 		save_state ("", true);
-		g_atomic_int_set (&_record_status, Enabled);
+		_record_status.store (Enabled);
 		RecordStateChanged (); /* EMIT SIGNAL */
 
-		request_transport_speed (1.0);
+		request_roll (TRS_MMC);
 
 	} else {
 
@@ -229,7 +223,7 @@ Session::mmc_step (MIDI::MachineControl &/*mmc*/, int steps)
 	double diff_secs = diff.tv_sec + (diff.tv_usec / 1000000.0);
 	double cur_speed = (((steps * 0.5) * timecode_frames_per_second()) / diff_secs) / timecode_frames_per_second();
 
-	if (_transport_speed == 0 || cur_speed * _transport_speed < 0) {
+	if (_transport_fsm->transport_speed() == 0 || cur_speed * _transport_fsm->transport_speed() < 0) {
 		/* change direction */
 		step_speed = cur_speed;
 	} else {
@@ -240,7 +234,7 @@ Session::mmc_step (MIDI::MachineControl &/*mmc*/, int steps)
 
 #if 0
 	cerr << "delta = " << diff_secs
-	     << " ct = " << _transport_speed
+	     << " ct = " << _transport_fsm->transport_speed()
 	     << " steps = " << steps
 	     << " new speed = " << cur_speed
 	     << " speed = " << step_speed
@@ -283,7 +277,7 @@ Session::mmc_locate (MIDI::MachineControl &/*mmc*/, const MIDI::byte* mmc_tc)
 		return;
 	}
 
-	framepos_t target_frame;
+	samplepos_t target_sample;
 	Timecode::Time timecode;
 
 	timecode.hours = mmc_tc[0] & 0xf;
@@ -294,10 +288,10 @@ Session::mmc_locate (MIDI::MachineControl &/*mmc*/, const MIDI::byte* mmc_tc)
 	timecode.drop = timecode_drop_frames();
 
 	// Also takes timecode offset into account:
-	timecode_to_sample( timecode, target_frame, true /* use_offset */, false /* use_subframes */ );
+	timecode_to_sample( timecode, target_sample, true /* use_offset */, false /* use_subframes */ );
 
-	if (target_frame > max_framepos) {
-		target_frame = max_framepos;
+	if (target_sample > max_samplepos) {
+		target_sample = max_samplepos;
 	}
 
 	/* Some (all?) MTC/MMC devices do not send a full MTC frame
@@ -306,14 +300,14 @@ Session::mmc_locate (MIDI::MachineControl &/*mmc*/, const MIDI::byte* mmc_tc)
 	   of an MTC slave to become out of date. Catch this.
 	*/
 
-	MTC_Slave* mtcs = dynamic_cast<MTC_Slave*> (_slave);
+	std::shared_ptr<MTC_TransportMaster> mtcs = std::dynamic_pointer_cast<MTC_TransportMaster> (transport_master());
 
-	if (mtcs != 0) {
+	if (mtcs) {
 		// cerr << "Locate *with* MTC slave\n";
 		mtcs->handle_locate (mmc_tc);
 	} else {
 		// cerr << "Locate without MTC slave\n";
-		request_locate (target_frame, false);
+		request_locate (target_sample, false, MustStop);
 	}
 }
 
@@ -335,7 +329,7 @@ Session::mmc_shuttle (MIDI::MachineControl &/*mmc*/, float speed, bool forw)
 	}
 }
 
-boost::shared_ptr<Route>
+std::shared_ptr<Route>
 Session::get_midi_nth_route_by_id (PresentationInfo::order_t n) const
 {
 	PresentationInfo::Flag f;
@@ -351,18 +345,18 @@ Session::get_midi_nth_route_by_id (PresentationInfo::order_t n) const
 		f = PresentationInfo::Route;
 	}
 
-	boost::shared_ptr<RouteList> r = routes.reader ();
+	std::shared_ptr<RouteList const> r = routes.reader ();
 	PresentationInfo::order_t match_cnt = 0;
 
-	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
-		if ((*i)->presentation_info().flag_match (f)) {
+	for (auto const& i : *r) {
+		if (i->presentation_info().flag_match (f)) {
 			if (match_cnt++ == n) {
-				return *i;
+				return i;
 			}
 		}
 	}
 
-	return boost::shared_ptr<Route>();
+	return std::shared_ptr<Route>();
 }
 
 void
@@ -372,26 +366,39 @@ Session::mmc_record_enable (MIDI::MachineControl &mmc, size_t trk, bool enabled)
 		return;
 	}
 
-	boost::shared_ptr<Route> r = get_midi_nth_route_by_id (trk);
+	std::shared_ptr<Route> r = get_midi_nth_route_by_id (trk);
 
 	if (r) {
-		boost::shared_ptr<AudioTrack> at;
+		std::shared_ptr<AudioTrack> at;
 
-		if ((at = boost::dynamic_pointer_cast<AudioTrack> (r))) {
+		if ((at = std::dynamic_pointer_cast<AudioTrack> (r))) {
 			at->rec_enable_control()->set_value (enabled, Controllable::UseGroup);
 		}
 	}
 }
 
+void
+Session::mtc_tx_resync_latency (bool playback)
+{
+	if (deletion_in_progress() || !playback) {
+		return;
+	}
+	std::shared_ptr<Port> mtxport = _midi_ports->mtc_output_port ();
+	if (mtxport) {
+		mtxport->get_connected_latency_range(mtc_out_latency, true);
+		DEBUG_TRACE (DEBUG::MTC, string_compose ("resync latency: %1\n", mtc_out_latency.max));
+	}
+}
+
 /** Send MTC Full Frame message (complete Timecode time) for the start of this cycle.
  * This resets the MTC code, the next quarter frame message that is sent will be
  * the first one with the beginning of this cycle as the new start point.
  * @param t time to send.
  */
 int
-Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
+Session::send_full_time_code (samplepos_t const t, MIDI::pframes_t nframes)
 {
-	/* This function could easily send at a given frame offset, but would
+	/* This function could easily send at a given sample offset, but would
 	 * that be useful?  Does ardour do sub-block accurate locating? [DR] */
 
 	MIDI::byte msg[10];
@@ -402,7 +409,8 @@ Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
 	if (_engine.freewheeling() || !Config->get_send_mtc()) {
 		return 0;
 	}
-	if (_slave && !_slave->locked()) {
+
+	if (transport_master_is_external() && !transport_master()->locked()) {
 		return 0;
 	}
 
@@ -410,26 +418,24 @@ Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
 	sample_to_timecode (t, timecode, true /* use_offset */, false /* no subframes */);
 
 	// sample-align outbound to rounded (no subframes) timecode
-	framepos_t mtc_tc;
+	samplepos_t mtc_tc;
 	timecode_to_sample(timecode, mtc_tc, true, false);
 	outbound_mtc_timecode_frame = mtc_tc;
 	transmitting_timecode_time = timecode;
 
-	LatencyRange mtc_out_latency = {0, 0}; // TODO cache this, update on engine().GraphReordered()
-	_midi_ports->mtc_output_port ()->get_connected_latency_range (ltc_out_latency, true);
-	frameoffset_t mtc_offset = worst_playback_latency() - mtc_out_latency.max;
+	sampleoffset_t mtc_offset = mtc_out_latency.max;
 
 	// only if rolling.. ?
 	outbound_mtc_timecode_frame += mtc_offset;
 
-	// outbound_mtc_timecode_frame needs to be >= _transport_frame
+	// outbound_mtc_timecode_frame needs to be >= _transport_sample
 	// or a new full timecode will be queued next cycle.
 	while (outbound_mtc_timecode_frame < t) {
 		Timecode::increment (transmitting_timecode_time, config.get_subframes_per_frame());
 		outbound_mtc_timecode_frame += _samples_per_timecode_frame;
 	}
 
-	double const quarter_frame_duration = ((framecnt_t) _samples_per_timecode_frame) / 4.0;
+	double const quarter_frame_duration = ((samplecnt_t) _samples_per_timecode_frame) / 4.0;
 	if (ceil((t - mtc_tc) / quarter_frame_duration) > 0) {
 		Timecode::increment (transmitting_timecode_time, config.get_subframes_per_frame());
 		outbound_mtc_timecode_frame += _samples_per_timecode_frame;
@@ -437,11 +443,9 @@ Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
 
 	DEBUG_TRACE (DEBUG::MTC, string_compose ("Full MTC TC %1 (off %2)\n", outbound_mtc_timecode_frame, mtc_offset));
 
-	// I don't understand this bit yet.. [DR]
-	// I do [rg]:
-	// according to MTC spec 24, 30 drop and 30 non-drop TC, the frame-number represented by 8 quarter frames must be even.
+	/* according to MTC spec 24, 30 drop and 30 non-drop TC, the frame-number represented by 8 quarter frames must be even. */
 	if (((mtc_timecode_bits >> 5) != MIDI::MTC_25_FPS) && (transmitting_timecode_time.frames % 2)) {
-		// start MTC quarter frame transmission on an even frame
+		/* start MTC quarter frame transmission on an even frame */
 		Timecode::increment (transmitting_timecode_time, config.get_subframes_per_frame());
 		outbound_mtc_timecode_frame += _samples_per_timecode_frame;
 	}
@@ -464,7 +468,7 @@ Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
 	// Send message at offset 0, sent time is for the start of this cycle
 
 	MidiBuffer& mb (_midi_ports->mtc_output_port()->get_midi_buffer (nframes));
-	mb.push_back (Port::port_offset(), sizeof (msg), msg);
+	mb.push_back (0, Evoral::MIDI_EVENT, sizeof (msg), msg);
 
 	_pframes_since_last_mtc = 0;
 	return 0;
@@ -478,19 +482,19 @@ Session::send_full_time_code (framepos_t const t, MIDI::pframes_t nframes)
  * earlier already this cycle by send_full_time_code)
  */
 int
-Session::send_midi_time_code_for_cycle (framepos_t start_frame, framepos_t end_frame, ARDOUR::pframes_t nframes)
+Session::send_midi_time_code_for_cycle (samplepos_t start_sample, samplepos_t end_sample, ARDOUR::pframes_t nframes)
 {
-	// start_frame == start_frame  for normal cycles
-	// start_frame > _transport_frame  for split cycles
+	// start_sample == start_sample  for normal cycles
+	// start_sample > _transport_sample  for split cycles
 	if (_engine.freewheeling() || !_send_qf_mtc || transmitting_timecode_time.negative || (next_quarter_frame_to_send < 0)) {
 		// cerr << "(MTC) Not sending MTC\n";
 		return 0;
 	}
-	if (_slave && !_slave->locked()) {
+	if (transport_master_is_external() && !transport_master()->locked()) {
 		return 0;
 	}
 
-	if (_transport_speed < 0) {
+	if (_transport_fsm->transport_speed() < 0) {
 		// we don't support rolling backwards
 		return 0;
 	}
@@ -510,63 +514,63 @@ Session::send_midi_time_code_for_cycle (framepos_t start_frame, framepos_t end_f
 	double const quarter_frame_duration = _samples_per_timecode_frame / 4.0;
 
 	DEBUG_TRACE (DEBUG::MTC, string_compose ("TF %1 SF %2 MT %3 QF %4 QD %5\n",
-				_transport_frame, start_frame, outbound_mtc_timecode_frame,
-				next_quarter_frame_to_send, quarter_frame_duration));
+	                                         _transport_sample, start_sample, outbound_mtc_timecode_frame,
+	                                         next_quarter_frame_to_send, quarter_frame_duration));
 
-	if (rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration)) < _transport_frame) {
+	if (rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration)) < _transport_sample) {
 		// send full timecode and set outbound_mtc_timecode_frame, next_quarter_frame_to_send
-		send_full_time_code (_transport_frame, nframes);
+		send_full_time_code (_transport_sample, nframes);
 	}
 
-	if (rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration)) < start_frame) {
+	if (rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration)) < start_sample) {
 		// no QF for this cycle
 		return 0;
 	}
 
 	/* Send quarter frames for this cycle */
-	while (end_frame > rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration))) {
+	while (end_sample > rint(outbound_mtc_timecode_frame + (next_quarter_frame_to_send * quarter_frame_duration))) {
 
-		DEBUG_TRACE (DEBUG::MTC, string_compose ("next frame to send: %1\n", next_quarter_frame_to_send));
+		DEBUG_TRACE (DEBUG::MTC, string_compose ("next sample to send: %1\n", next_quarter_frame_to_send));
 
 		switch (next_quarter_frame_to_send) {
-			case 0:
-				mtc_msg[1] = 0x00 | (transmitting_timecode_time.frames & 0xf);
-				break;
-			case 1:
-				mtc_msg[1] = 0x10 | ((transmitting_timecode_time.frames & 0xf0) >> 4);
-				break;
-			case 2:
-				mtc_msg[1] = 0x20 | (transmitting_timecode_time.seconds & 0xf);
-				break;
-			case 3:
-				mtc_msg[1] = 0x30 | ((transmitting_timecode_time.seconds & 0xf0) >> 4);
-				break;
-			case 4:
-				mtc_msg[1] = 0x40 | (transmitting_timecode_time.minutes & 0xf);
-				break;
-			case 5:
-				mtc_msg[1] = 0x50 | ((transmitting_timecode_time.minutes & 0xf0) >> 4);
-				break;
-			case 6:
-				mtc_msg[1] = 0x60 | ((mtc_timecode_bits | transmitting_timecode_time.hours) & 0xf);
-				break;
-			case 7:
-				mtc_msg[1] = 0x70 | (((mtc_timecode_bits | transmitting_timecode_time.hours) & 0xf0) >> 4);
-				break;
+		case 0:
+			mtc_msg[1] = 0x00 | (transmitting_timecode_time.frames & 0xf);
+			break;
+		case 1:
+			mtc_msg[1] = 0x10 | ((transmitting_timecode_time.frames & 0xf0) >> 4);
+			break;
+		case 2:
+			mtc_msg[1] = 0x20 | (transmitting_timecode_time.seconds & 0xf);
+			break;
+		case 3:
+			mtc_msg[1] = 0x30 | ((transmitting_timecode_time.seconds & 0xf0) >> 4);
+			break;
+		case 4:
+			mtc_msg[1] = 0x40 | (transmitting_timecode_time.minutes & 0xf);
+			break;
+		case 5:
+			mtc_msg[1] = 0x50 | ((transmitting_timecode_time.minutes & 0xf0) >> 4);
+			break;
+		case 6:
+			mtc_msg[1] = 0x60 | ((mtc_timecode_bits | transmitting_timecode_time.hours) & 0xf);
+			break;
+		case 7:
+			mtc_msg[1] = 0x70 | (((mtc_timecode_bits | transmitting_timecode_time.hours) & 0xf0) >> 4);
+			break;
 		}
 
-		const framepos_t msg_time = rint(outbound_mtc_timecode_frame	+ (quarter_frame_duration * next_quarter_frame_to_send));
+		const samplepos_t msg_time = rint (outbound_mtc_timecode_frame + (quarter_frame_duration * next_quarter_frame_to_send));
 
 		// This message must fall within this block or something is broken
-		assert (msg_time >= start_frame);
-		assert (msg_time < end_frame);
+		assert (msg_time >= start_sample);
+		assert (msg_time < end_sample);
 
-		/* convert from session frames back to JACK frames using the transport speed */
-		ARDOUR::pframes_t const out_stamp = (msg_time - start_frame) / _transport_speed;
+		/* convert from session samples back to JACK samples using the transport speed */
+		ARDOUR::pframes_t const out_stamp = (msg_time - start_sample) / _transport_fsm->transport_speed();
 		assert (out_stamp < nframes);
 
 		MidiBuffer& mb (_midi_ports->mtc_output_port()->get_midi_buffer(nframes));
-		if (!mb.push_back (out_stamp, 2, mtc_msg)) {
+		if (!mb.push_back (out_stamp, Evoral::MIDI_EVENT, 2, mtc_msg)) {
 			error << string_compose(_("Session: cannot send quarter-frame MTC message (%1)"), strerror (errno))
 			      << endmsg;
 			return -1;
@@ -574,11 +578,11 @@ Session::send_midi_time_code_for_cycle (framepos_t start_frame, framepos_t end_f
 
 #ifndef NDEBUG
 		if (DEBUG_ENABLED(DEBUG::MTC)) {
-			DEBUG_STR_DECL(foo)
+			DEBUG_STR_DECL(foo);
 			DEBUG_STR_APPEND(foo,"sending ");
 			DEBUG_STR_APPEND(foo, transmitting_timecode_time);
 			DEBUG_TRACE (DEBUG::MTC, string_compose ("%1 qfm = %2, stamp = %3\n", DEBUG_STR(foo).str(), next_quarter_frame_to_send,
-								 out_stamp));
+			                                         out_stamp));
 		}
 #endif
 
@@ -606,12 +610,7 @@ Session::send_midi_time_code_for_cycle (framepos_t start_frame, framepos_t end_f
 void
 Session::send_immediate_mmc (MachineControlCommand c)
 {
-	if (AudioEngine::instance()->in_process_thread()) {
-		_mmc->send (c, Port::port_offset());
-	} else {
-		_mmc->send (c, 0);
-	}
-
+	_mmc->send (c, 0);
 }
 
 bool
@@ -625,9 +624,9 @@ Session::mmc_step_timeout ()
 	timersub (&now, &last_mmc_step, &diff);
 	diff_usecs = diff.tv_sec * 1000000 + diff.tv_usec;
 
-	if (diff_usecs > 1000000.0 || fabs (_transport_speed) < 0.0000001) {
+	if (diff_usecs > 1000000.0 || fabs (_transport_fsm->transport_speed()) < 0.0000001) {
 		/* too long or too slow, stop transport */
-		request_transport_speed (0.0);
+		request_stop ();
 		step_queued = false;
 		return false;
 	}
@@ -639,17 +638,16 @@ Session::mmc_step_timeout ()
 
 	/* slow it down */
 
-	request_transport_speed_nonzero (_transport_speed * 0.75);
+	request_transport_speed_nonzero (actual_speed() * 0.75);
 	return true;
 }
 
-/***********************************************************************
- OUTBOUND SYSTEM COMMON STUFF
-**********************************************************************/
-
+/* *********************************************************************
+ * OUTBOUND SYSTEM COMMON STUFF
+ **********************************************************************/
 
 void
-Session::send_song_position_pointer (framepos_t)
+Session::send_song_position_pointer (samplepos_t)
 {
 	if (midi_clock) {
 		/* Do nothing for the moment */
@@ -665,66 +663,56 @@ Session::start_midi_thread ()
 	return 0;
 }
 
-boost::shared_ptr<ARDOUR::Port>
-Session::midi_input_port () const
-{
-	return _midi_ports->midi_input_port ();
-}
-
-boost::shared_ptr<ARDOUR::Port>
-Session::midi_output_port () const
-{
-	return _midi_ports->midi_output_port ();
-}
-
-boost::shared_ptr<ARDOUR::Port>
+std::shared_ptr<ARDOUR::Port>
 Session::mmc_output_port () const
 {
 	return _midi_ports->mmc_output_port ();
 }
 
-boost::shared_ptr<ARDOUR::Port>
+std::shared_ptr<ARDOUR::Port>
 Session::mmc_input_port () const
 {
 	return _midi_ports->mmc_input_port ();
 }
 
-boost::shared_ptr<ARDOUR::Port>
+std::shared_ptr<ARDOUR::Port>
 Session::scene_output_port () const
 {
 	return _midi_ports->scene_output_port ();
 }
 
-boost::shared_ptr<ARDOUR::Port>
+std::shared_ptr<ARDOUR::Port>
 Session::scene_input_port () const
 {
 	return _midi_ports->scene_input_port ();
 }
 
-boost::shared_ptr<MidiPort>
+std::shared_ptr<ARDOUR::Port>
+Session::trigger_input_port () const
+{
+	return _midi_ports->trigger_input_port ();
+}
+
+std::shared_ptr<AsyncMIDIPort>
+Session::vkbd_output_port () const
+{
+	return _midi_ports->vkbd_output_port ();
+}
+
+std::shared_ptr<MidiPort>
 Session::midi_clock_output_port () const
 {
 	return _midi_ports->midi_clock_output_port ();
 }
 
-boost::shared_ptr<MidiPort>
-Session::midi_clock_input_port () const
-{
-	return _midi_ports->midi_clock_input_port ();
-}
-boost::shared_ptr<MidiPort>
+std::shared_ptr<MidiPort>
 Session::mtc_output_port () const
 {
 	return _midi_ports->mtc_output_port ();
 }
-boost::shared_ptr<MidiPort>
-Session::mtc_input_port () const
-{
-	return _midi_ports->mtc_input_port ();
-}
 
 void
-Session::midi_track_presentation_info_changed (PropertyChange const& what_changed, boost::weak_ptr<MidiTrack> mt)
+Session::midi_track_presentation_info_changed (PropertyChange const& what_changed, std::weak_ptr<MidiTrack> mt)
 {
 	if (!Config->get_midi_input_follows_selection()) {
 		return;
@@ -734,40 +722,79 @@ Session::midi_track_presentation_info_changed (PropertyChange const& what_change
 		return;
 	}
 
-	boost::shared_ptr<MidiTrack> new_midi_target (mt.lock ());
+	std::shared_ptr<MidiTrack> new_midi_target (mt.lock ());
 
 	if (new_midi_target->is_selected()) {
 		rewire_selected_midi (new_midi_target);
 	}
 }
 
+
 void
-Session::rewire_selected_midi (boost::shared_ptr<MidiTrack> new_midi_target)
+Session::disconnect_port_for_rewire (std::string const& port) const
+{
+	MidiPortFlags mpf = AudioEngine::instance()->midi_port_metadata (port);
+
+	/* if a port is marked for control data, do not
+	 * disconnect it from everything since it may also be
+	 * used via a control surface or some other
+	 * functionality.
+	 */
+	bool keep_ctrl = mpf & MidiPortControl;
+
+	vector<string> port_connections;
+	AudioEngine::instance()->get_connections (port, port_connections);
+	for (vector<string>::iterator i = port_connections.begin(); i != port_connections.end(); ++i) {
+
+		/* test if (*i) is a control-surface input port */
+		if (keep_ctrl && AudioEngine::instance()->port_is_control_only (*i)) {
+			continue;
+		}
+		/* retain connection to "physical_midi_input_monitor_enable" */
+		if (AudioEngine::instance()->port_is_physical_input_monitor_enable (*i)) {
+			continue;
+		}
+
+		AudioEngine::instance()->disconnect (port, *i);
+	}
+}
+
+void
+Session::rewire_selected_midi (std::shared_ptr<MidiTrack> new_midi_target)
 {
 	if (!new_midi_target) {
 		return;
 	}
 
-	boost::shared_ptr<MidiTrack> old_midi_target = current_midi_target.lock ();
+	std::shared_ptr<MidiTrack> old_midi_target = current_midi_target.lock ();
 
 	if (new_midi_target == old_midi_target) {
 		return;
 	}
 
+	/* TODO: return, if this is triggered multiple times due to
+	 * group selection. Only handle this for the first route
+	 * in a group.
+	 */
+
 	vector<string> msp;
 	AudioEngine::instance()->get_midi_selection_ports (msp);
 
 	if (!msp.empty()) {
-
-		if (old_midi_target) {
-			old_midi_target->input()->disconnect (this);
-		}
-
 		for (vector<string>::const_iterator p = msp.begin(); p != msp.end(); ++p) {
-			/* disconnect the port from everything */
-			AudioEngine::instance()->disconnect (*p);
+			/* disconnect port */
+			disconnect_port_for_rewire (*p);
 			/* connect it to the new target */
 			new_midi_target->input()->connect (new_midi_target->input()->nth(0), (*p), this);
+			/* and grouped tracks */
+			RouteGroup* group = new_midi_target->route_group ();
+			if (group && group->is_active () && group->is_select ()) {
+				for (auto const& r : *group->route_list ()) {
+					if (dynamic_pointer_cast<MidiTrack> (r)) {
+						r->input()->connect (r->input()->nth(0), (*p), this);
+					}
+				}
+			}
 		}
 	}
 
@@ -781,7 +808,7 @@ Session::rewire_midi_selection_ports ()
 		return;
 	}
 
-	boost::shared_ptr<MidiTrack> target = current_midi_target.lock();
+	std::shared_ptr<MidiTrack> target = current_midi_target.lock();
 
 	if (!target) {
 		return;
@@ -797,7 +824,16 @@ Session::rewire_midi_selection_ports ()
 	target->input()->disconnect (this);
 
 	for (vector<string>::const_iterator p = msp.begin(); p != msp.end(); ++p) {
-		AudioEngine::instance()->disconnect (*p);
+		disconnect_port_for_rewire (*p);
 		target->input()->connect (target->input()->nth (0), (*p), this);
+
+		RouteGroup* group = target->route_group ();
+		if (group && group->is_active () && group->is_select ()) {
+			for (auto const& r : *group->route_list ()) {
+				if (dynamic_pointer_cast<MidiTrack> (r)) {
+					r->input()->connect (r->input()->nth(0), (*p), this);
+				}
+			}
+		}
 	}
 }
