@@ -1,20 +1,21 @@
 /*
-  Copyright (C) 2016 Paul Davis
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the Free
-  Software Foundation; either version 2 of the License, or (at your option)
-  any later version.
-
-  This program is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-  for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2016-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2017 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include "pbd/convert.h"
 
@@ -71,15 +72,15 @@ VCA::VCA (Session& s, int32_t num, const string& name)
 	: Stripable (s, name, PresentationInfo (num, PresentationInfo::VCA))
 	, Muteable (s, name)
 	, _number (num)
-	, _gain_control (new GainControl (s, Evoral::Parameter (GainAutomation), boost::shared_ptr<AutomationList> ()))
+	, _gain_control (new GainControl (s, Evoral::Parameter (GainAutomation), std::shared_ptr<AutomationList> ()))
 {
 }
 
 int
 VCA::init ()
 {
-	_solo_control.reset (new SoloControl (_session, X_("solo"), *this, *this));
-	_mute_control.reset (new MuteControl (_session, X_("mute"), *this));
+	_solo_control.reset (new SoloControl (_session, X_("solo"), *this, *this, time_domain()));
+	_mute_control.reset (new MuteControl (_session, X_("mute"), *this, time_domain()));
 
 	add_control (_gain_control);
 	add_control (_solo_control);
@@ -94,7 +95,7 @@ VCA::~VCA ()
 	{
 		Glib::Threads::Mutex::Lock lm (_control_lock);
 		for (Controls::const_iterator li = _controls.begin(); li != _controls.end(); ++li) {
-			boost::dynamic_pointer_cast<AutomationControl>(li->second)->drop_references ();
+			std::dynamic_pointer_cast<AutomationControl>(li->second)->drop_references ();
 		}
 	}
 	{
@@ -116,7 +117,7 @@ VCA::full_name() const
 }
 
 XMLNode&
-VCA::get_state ()
+VCA::get_state () const
 {
 	XMLNode* node = new XMLNode (xml_node_name);
 	node->set_property (X_("name"), name());
@@ -197,7 +198,7 @@ VCA::slaved () const
 }
 
 bool
-VCA::slaved_to (boost::shared_ptr<VCA> vca) const
+VCA::slaved_to (std::shared_ptr<VCA> vca) const
 {
 	if (!vca || !_gain_control) {
 		return false;
@@ -208,6 +209,17 @@ VCA::slaved_to (boost::shared_ptr<VCA> vca) const
 	return _gain_control->slaved_to (vca->gain_control());
 }
 
+void
+VCA::assign (std::shared_ptr<VCA> v)
+{
+	/* prevent recursive assignments */
+	if (assigned_to (_session.vca_manager_ptr (), v)) {
+		warning << _("Master assignment ignored to prevent recursion") << endmsg;
+		return;
+	}
+	Slavable::assign (v);
+}
+
 SlavableControlList
 VCA::slavables () const
 {
