@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2014 Robin Gareus <robin@gareus.org>
- * Copyright (C) 2013 Paul Davis
+ * Copyright (C) 2015-2018 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2016-2018 Paul Davis <paul@linuxaudiosystems.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,46 +12,45 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef __libbackend_coreaudio_backend_h__
 #define __libbackend_coreaudio_backend_h__
 
+#include <cstdint>
+#include <map>
+#include <memory>
+#include <set>
 #include <string>
 #include <vector>
-#include <map>
-#include <set>
 
-#include <stdint.h>
 #include <pthread.h>
 
-#include <boost/shared_ptr.hpp>
-
 #include "pbd/natsort.h"
 #include "ardour/audio_backend.h"
 #include "ardour/dsp_load_calculator.h"
+#include "ardour/port_engine_shared.h"
 #include "ardour/types.h"
 
 #include "coreaudio_pcmio.h"
 #include "coremidi_io.h"
 
-#define MaxCoreMidiEventSize 128 // matches CoreMidi's MIDIPacket
+#define MaxCoreMidiEventSize 256 // matches CoreMidi's MIDIPacket (https://developer.apple.com/documentation/coremidi/midipacket)
 
 namespace ARDOUR {
 
 class CoreAudioBackend;
 
-class CoreMidiEvent {
+class CoreMidiEvent : public BackendMIDIEvent {
   public:
 	CoreMidiEvent (const pframes_t timestamp, const uint8_t* data, size_t size);
 	CoreMidiEvent (const CoreMidiEvent& other);
 	size_t size () const { return _size; };
 	pframes_t timestamp () const { return _timestamp; };
 	const uint8_t* data () const { return _data; };
-	bool operator< (const CoreMidiEvent &other) const { return timestamp () < other.timestamp (); };
   private:
 	size_t _size;
 	pframes_t _timestamp;
@@ -60,69 +59,7 @@ class CoreMidiEvent {
 
 typedef std::vector<CoreMidiEvent> CoreMidiBuffer;
 
-class CoreBackendPort {
-  protected:
-	CoreBackendPort (CoreAudioBackend &b, const std::string&, PortFlags);
-  public:
-	virtual ~CoreBackendPort ();
-
-	const std::string& name () const { return _name; }
-	const std::string& pretty_name () const { return _pretty_name; }
-	PortFlags flags () const { return _flags; }
-
-	int set_name (const std::string &name) { _name = name; return 0; }
-	int set_pretty_name (const std::string &name) { _pretty_name = name; return 0; }
-
-	virtual DataType type () const = 0;
-
-	bool is_input ()     const { return flags () & IsInput; }
-	bool is_output ()    const { return flags () & IsOutput; }
-	bool is_physical ()  const { return flags () & IsPhysical; }
-	bool is_terminal ()  const { return flags () & IsTerminal; }
-	bool is_connected () const { return _connections.size () != 0; }
-	bool is_connected (const CoreBackendPort *port) const;
-	bool is_physically_connected () const;
-
-	const std::set<CoreBackendPort *>& get_connections () const { return _connections; }
-
-	int connect (CoreBackendPort *port);
-	int disconnect (CoreBackendPort *port);
-	void disconnect_all ();
-
-	virtual void* get_buffer (pframes_t nframes) = 0;
-
-	const LatencyRange latency_range (bool for_playback) const
-	{
-		return for_playback ? _playback_latency_range : _capture_latency_range;
-	}
-
-	void set_latency_range (const LatencyRange &latency_range, bool for_playback)
-	{
-		if (for_playback)
-		{
-			_playback_latency_range = latency_range;
-		}
-		else
-		{
-			_capture_latency_range = latency_range;
-		}
-	}
-
-  private:
-	CoreAudioBackend &_osx_backend;
-	std::string _name;
-	std::string _pretty_name;
-	const PortFlags _flags;
-	LatencyRange _capture_latency_range;
-	LatencyRange _playback_latency_range;
-	std::set<CoreBackendPort*> _connections;
-
-	void _connect (CoreBackendPort* , bool);
-	void _disconnect (CoreBackendPort* , bool);
-
-}; // class CoreBackendPort
-
-class CoreAudioPort : public CoreBackendPort {
+class CoreAudioPort : public BackendPort {
   public:
 	CoreAudioPort (CoreAudioBackend &b, const std::string&, PortFlags);
 	~CoreAudioPort ();
@@ -137,7 +74,7 @@ class CoreAudioPort : public CoreBackendPort {
 	Sample _buffer[8192];
 }; // class CoreAudioPort
 
-class CoreMidiPort : public CoreBackendPort {
+class CoreMidiPort : public BackendPort {
   public:
 	CoreMidiPort (CoreAudioBackend &b, const std::string&, PortFlags);
 	~CoreMidiPort ();
@@ -215,8 +152,7 @@ class CoreMidiPort : public CoreBackendPort {
 
 }; // class CoreMidiPort
 
-class CoreAudioBackend : public AudioBackend {
-	friend class CoreBackendPort;
+class CoreAudioBackend : public AudioBackend, public PortEngineSharedImpl {
   public:
 	CoreAudioBackend (AudioEngine& e, AudioBackendInfo& info);
 	~CoreAudioBackend ();
@@ -240,6 +176,7 @@ class CoreAudioBackend : public AudioBackend {
 
 	bool can_change_sample_rate_when_running () const;
 	bool can_change_buffer_size_when_running () const;
+	bool can_measure_systemic_latency () const { return true; }
 
 	int set_device_name (const std::string&);
 	int set_input_device_name (const std::string&);
@@ -270,6 +207,9 @@ class CoreAudioBackend : public AudioBackend {
 	uint32_t     systemic_midi_input_latency (std::string const) const { return 0; }
 	uint32_t     systemic_midi_output_latency (std::string const) const { return 0; }
 
+	uint32_t systemic_hw_input_latency () const;
+	uint32_t systemic_hw_output_latency () const;
+
 	bool can_set_systemic_midi_latencies () const { return false; /* XXX */}
 
 	/* External control app */
@@ -294,6 +234,7 @@ class CoreAudioBackend : public AudioBackend {
 	// really private, but needing static access:
 	int process_callback(uint32_t, uint64_t);
 	void error_callback();
+	void halted_callback();
 	void xrun_callback();
 	void buffer_size_callback();
 	void sample_rate_callback();
@@ -309,8 +250,8 @@ class CoreAudioBackend : public AudioBackend {
 	size_t raw_buffer_size (DataType t);
 
 	/* Process time */
-	framepos_t sample_time ();
-	framepos_t sample_time_at_cycle_start ();
+	samplepos_t sample_time ();
+	samplepos_t sample_time_at_cycle_start ();
 	pframes_t samples_since_cycle_start ();
 
 	int create_process_thread (boost::function<void()> func);
@@ -324,32 +265,34 @@ class CoreAudioBackend : public AudioBackend {
 
 	void* private_handle () const;
 	const std::string& my_name () const;
-	bool available () const;
-	uint32_t port_name_size () const;
 
-	int         set_port_name (PortHandle, const std::string&);
-	std::string get_port_name (PortHandle) const;
-	PortHandle  get_port_by_name (const std::string&) const;
-	int get_port_property (PortHandle, const std::string& key, std::string& value, std::string& type) const;
-	int set_port_property (PortHandle, const std::string& key, const std::string& value, const std::string& type);
+	/* PortEngine API - forwarded to PortEngineSharedImpl */
 
-	int get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>&) const;
-
-	DataType port_data_type (PortHandle) const;
-
-	PortHandle register_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
-	void unregister_port (PortHandle);
-
-	int  connect (const std::string& src, const std::string& dst);
-	int  disconnect (const std::string& src, const std::string& dst);
-	int  connect (PortHandle, const std::string&);
-	int  disconnect (PortHandle, const std::string&);
-	int  disconnect_all (PortHandle);
-
-	bool connected (PortHandle, bool process_callback_safe);
-	bool connected_to (PortHandle, const std::string&, bool process_callback_safe);
-	bool physically_connected (PortHandle, bool process_callback_safe);
-	int  get_connections (PortHandle, std::vector<std::string>&, bool process_callback_safe);
+	bool        port_is_physical (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_is_physical (ph); }
+	void        get_physical_outputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_outputs (type, results); }
+	void        get_physical_inputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_inputs (type, results); }
+	ChanCount   n_physical_outputs () const { return PortEngineSharedImpl::n_physical_outputs (); }
+	ChanCount   n_physical_inputs () const { return PortEngineSharedImpl::n_physical_inputs (); }
+	uint32_t    port_name_size () const { return PortEngineSharedImpl::port_name_size(); }
+	int         set_port_name (PortEngine::PortHandle ph, const std::string& name) { return PortEngineSharedImpl::set_port_name (ph, name); }
+	std::string get_port_name (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_name (ph); }
+	PortFlags   get_port_flags (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_flags (ph); }
+	PortEngine::PortPtr  get_port_by_name (std::string const & name) const { return PortEngineSharedImpl::get_port_by_name (name); }
+	int         get_port_property (PortEngine::PortHandle ph, const std::string& key, std::string& value, std::string& type) const { return PortEngineSharedImpl::get_port_property (ph, key, value, type); }
+	int         set_port_property (PortEngine::PortHandle ph, const std::string& key, const std::string& value, const std::string& type) { return PortEngineSharedImpl::set_port_property (ph, key, value, type); }
+	int         get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>& results) const { return PortEngineSharedImpl::get_ports (port_name_pattern, type, flags, results); }
+	DataType    port_data_type (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_data_type (ph); }
+	PortEngine::PortPtr register_port (const std::string& shortname, ARDOUR::DataType type, ARDOUR::PortFlags flags) { return PortEngineSharedImpl::register_port (shortname, type, flags); }
+	void        unregister_port (PortHandle ph) { if (!_run) return; PortEngineSharedImpl::unregister_port (ph); }
+	int         connect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::connect (src, dst); }
+	int         disconnect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::disconnect (src, dst); }
+	int         connect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::connect (ph, other); }
+	int         disconnect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::disconnect (ph, other); }
+	int         disconnect_all (PortEngine::PortHandle ph) { return PortEngineSharedImpl::disconnect_all (ph); }
+	bool        connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::connected (ph, process_callback_safe); }
+	bool        connected_to (PortEngine::PortHandle ph, const std::string& other, bool process_callback_safe) { return PortEngineSharedImpl::connected_to (ph, other, process_callback_safe); }
+	bool        physically_connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::physically_connected (ph, process_callback_safe); }
+	int         get_connections (PortEngine::PortHandle ph, std::vector<std::string>& results, bool process_callback_safe) { return PortEngineSharedImpl::get_connections (ph, results, process_callback_safe); }
 
 	/* MIDI */
 	int midi_event_get (pframes_t& timestamp, size_t& size, uint8_t const** buf, void* port_buffer, uint32_t event_index);
@@ -372,14 +315,6 @@ class CoreAudioBackend : public AudioBackend {
 	void         set_latency_range (PortHandle, bool for_playback, LatencyRange);
 	LatencyRange get_latency_range (PortHandle, bool for_playback);
 
-	/* Discovering physical ports */
-
-	bool      port_is_physical (PortHandle) const;
-	void      get_physical_outputs (DataType type, std::vector<std::string>&);
-	void      get_physical_inputs (DataType type, std::vector<std::string>&);
-	ChanCount n_physical_outputs () const;
-	ChanCount n_physical_inputs () const;
-
 	/* Getting access to the data buffer for a port */
 
 	void* get_buffer (PortHandle, pframes_t);
@@ -437,6 +372,8 @@ class CoreAudioBackend : public AudioBackend {
 	enum DeviceFilter { All, Input, Output, Duplex };
 	uint32_t name_to_id(std::string, DeviceFilter filter = All) const;
 
+	void unset_callbacks ();
+
 	/* processing */
 	float  _dsp_load;
 	ARDOUR::DSPLoadCalculator  _dsp_load_calc;
@@ -447,87 +384,38 @@ class CoreAudioBackend : public AudioBackend {
 
 	/* process threads */
 	static void* coreaudio_process_thread (void *);
-	bool coreaudio_set_realtime_policy (pthread_t) const;
 	std::vector<pthread_t> _threads;
 
 	struct ThreadData {
 		CoreAudioBackend* engine;
 		boost::function<void ()> f;
 		size_t stacksize;
+		double period_ns;
 
-		ThreadData (CoreAudioBackend* e, boost::function<void ()> fp, size_t stacksz)
-			: engine (e) , f (fp) , stacksize (stacksz) {}
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 110000
+		bool                      _joined_workgroup;
+		os_workgroup_t            _workgroup;
+		os_workgroup_join_token_s _join_token;
+#endif
+
+		ThreadData (CoreAudioBackend* e, boost::function<void ()> fp, size_t stacksz, double period)
+			: engine (e) , f (fp) , stacksize (stacksz), period_ns (period) {}
 	};
 
 	/* port engine */
-	PortHandle add_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
 	int register_system_audio_ports ();
-	void unregister_ports (bool system_only = false);
 
-	std::vector<CoreBackendPort *> _system_inputs;
-	std::vector<CoreBackendPort *> _system_outputs;
-	std::vector<CoreBackendPort *> _system_midi_in;
-	std::vector<CoreBackendPort *> _system_midi_out;
-
-	struct SortByPortName
-	{
-		bool operator ()(const CoreBackendPort* lhs, const CoreBackendPort* rhs) const
-		{
-			return PBD::naturally_less (lhs->name ().c_str (), rhs->name ().c_str ());
-		}
-	};
-
-	typedef std::map<std::string, CoreBackendPort *> PortMap; // fast lookup in _ports
-	typedef std::set<CoreBackendPort *, SortByPortName> PortIndex; // fast lookup in _ports
-	PortMap _portmap;
-	PortIndex _ports;
-
-	struct PortConnectData {
-		std::string a;
-		std::string b;
-		bool c;
-
-		PortConnectData (const std::string& a, const std::string& b, bool c)
-			: a (a) , b (b) , c (c) {}
-	};
-
-	std::vector<PortConnectData *> _port_connection_queue;
-	pthread_mutex_t _port_callback_mutex;
-	bool _port_change_flag;
-
-	void port_connect_callback (const std::string& a, const std::string& b, bool conn) {
-		pthread_mutex_lock (&_port_callback_mutex);
-		_port_connection_queue.push_back(new PortConnectData(a, b, conn));
-		pthread_mutex_unlock (&_port_callback_mutex);
-	}
-
-	void port_connect_add_remove_callback () {
-		pthread_mutex_lock (&_port_callback_mutex);
-		_port_change_flag = true;
-		pthread_mutex_unlock (&_port_callback_mutex);
-	}
-
-	bool valid_port (PortHandle port) const {
-		return std::find (_ports.begin(), _ports.end(), static_cast<CoreBackendPort*>(port)) != _ports.end ();
-	}
-
-	CoreBackendPort* find_port (const std::string& port_name) const {
-		PortMap::const_iterator it = _portmap.find (port_name);
-		if (it == _portmap.end()) {
-			return NULL;
-		}
-		return (*it).second;
-	}
-
-	CoreBackendPort * find_port_in (std::vector<CoreBackendPort *> plist, const std::string& port_name) const {
-		for (std::vector<CoreBackendPort*>::const_iterator it = plist.begin (); it != plist.end (); ++it) {
+	BackendPortPtr find_port_in (std::vector<BackendPortPtr> const & plist, const std::string& port_name) const {
+		for (std::vector<BackendPortPtr>::const_iterator it = plist.begin (); it != plist.end (); ++it) {
 			if ((*it)->name () == port_name) {
 				return *it;
 			}
 		}
-		return NULL;
+		return BackendPortPtr();
 	}
 
+	BackendPort* port_factory (std::string const & name, ARDOUR::DataType type, ARDOUR::PortFlags);
+
 	void reset_midi_parsers ();
 
 }; // class CoreAudioBackend
