@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015-2019 Robin Gareus <robin@gareus.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,13 +11,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <sstream>
 #include <CoreAudio/HostTime.h>
+#undef nil
 
 #include "coremidi_io.h"
 #include "coreaudio_backend.h"
@@ -82,7 +83,7 @@ static void midiInputCallback(const MIDIPacketList *list, void *procRef, void *s
 #endif
 		return;
 	}
-	RingBuffer<uint8_t> * rb  = static_cast<RingBuffer < uint8_t > *> (srcRef);
+	PBD::RingBuffer<uint8_t> * rb  = static_cast<PBD::RingBuffer < uint8_t > *> (srcRef);
 	if (!rb) {
 #ifndef NDEBUG
 		if (_debug_mode & 4) {
@@ -153,7 +154,7 @@ CoreMidiIo::CoreMidiIo()
 	pthread_mutex_init (&_discovery_lock, 0);
 
 #ifndef NDEBUG
-	const char *p = getenv ("COREMIDIDEBUG");
+	const char *p = getenv ("ARDOUR_COREMIDI_DEBUG");
 	if (p && *p) _debug_mode = atoi (p);
 #endif
 }
@@ -258,7 +259,7 @@ CoreMidiIo::recv_event (uint32_t port, double cycle_time_us, uint64_t &time, uin
 		assert(rv == sizeof(uint32_t));
 		rv = _rb[port]->read((uint8_t*)&packet, s);
 		assert(rv == s);
-		_input_queue[port].push_back(boost::shared_ptr<CoreMIDIPacket>(new _CoreMIDIPacket (&packet)));
+		_input_queue[port].push_back(std::shared_ptr<CoreMIDIPacket>(new _CoreMIDIPacket (&packet)));
 	}
 
 	UInt64 start = _time_at_cycle_start;
@@ -268,7 +269,11 @@ CoreMidiIo::recv_event (uint32_t port, double cycle_time_us, uint64_t &time, uin
 		if ((*it)->timeStamp < end) {
 			if ((*it)->timeStamp < start) {
 				uint64_t dt = AudioConvertHostTimeToNanos(start - (*it)->timeStamp);
-				if (dt > 1e7 && (*it)->timeStamp != 0) { // 10ms slack and a timestamp is given
+				/* note: it used to be 10ms (insert handwavy explanation about percievable latency)
+				 * turns out some midi-keyboads connected via bluetooth can be late by as much as 50ms
+				 * https://discourse.ardour.org/t/ardour-not-getting-all-messages-from-midi-keyboard/107618/13?
+				 */
+				if (dt > 6e7 && (*it)->timeStamp != 0) { // 60ms slack and a timestamp is given
 #ifndef NDEBUG
 					printf("Dropped Stale Midi Event. dt:%.2fms\n", dt * 1e-6);
 #endif
@@ -297,6 +302,11 @@ CoreMidiIo::recv_event (uint32_t port, double cycle_time_us, uint64_t &time, uin
 			}
 			_input_queue[port].erase(it);
 			return s;
+		} else {
+#ifndef NDEBUG
+			uint64_t dt = AudioConvertHostTimeToNanos((*it)->timeStamp - end);
+			printf("Postponed future Midi Event. dt:%.2fms\n", dt * 1e-6);
+#endif
 		}
 		++it;
 
@@ -450,7 +460,7 @@ CoreMidiIo::discover()
 		_input_ports = (MIDIPortRef *) malloc (srcCount * sizeof(MIDIPortRef));
 		_input_endpoints = (MIDIEndpointRef*) malloc (srcCount * sizeof(MIDIEndpointRef));
 		_input_queue = (CoreMIDIQueue*) calloc (srcCount, sizeof(CoreMIDIQueue));
-		_rb = (RingBuffer<uint8_t> **) malloc (srcCount * sizeof(RingBuffer<uint8_t>*));
+		_rb = (PBD::RingBuffer<uint8_t> **) malloc (srcCount * sizeof(PBD::RingBuffer<uint8_t>*));
 	}
 	if (dstCount > 0) {
 		_output_ports = (MIDIPortRef *) malloc (dstCount * sizeof(MIDIPortRef));
@@ -473,7 +483,7 @@ CoreMidiIo::discover()
 			fprintf(stderr, "Cannot create Midi Output\n");
 			continue;
 		}
-		_rb[_n_midi_in] = new RingBuffer<uint8_t>(32768);
+		_rb[_n_midi_in] = new PBD::RingBuffer<uint8_t>(32768);
 		_input_queue[_n_midi_in] = CoreMIDIQueue();
 		MIDIPortConnectSource(_input_ports[_n_midi_in], src, (void*) _rb[_n_midi_in]);
 		CFRelease(port_name);
