@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2014 Robin Gareus <robin@gareus.org>
- * Copyright (C) 2013 Paul Davis
+ * Copyright (C) 2014-2018 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2014-2018 Robin Gareus <robin@gareus.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,31 +12,31 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef __libbackend_dummy_audiobackend_h__
 #define __libbackend_dummy_audiobackend_h__
 
+#include <cstdint>
 #include <string>
 #include <vector>
 #include <map>
+#include <memory>
 #include <set>
 
-#include <stdint.h>
 #include <pthread.h>
 
 #include <ltc.h>
 
-#include <boost/shared_ptr.hpp>
-
 #include "pbd/natsort.h"
 #include "pbd/ringbuffer.h"
 #include "ardour/types.h"
 #include "ardour/audio_backend.h"
 #include "ardour/dsp_load_calculator.h"
+#include "ardour/port_engine_shared.h"
 
 namespace ARDOUR {
 
@@ -46,101 +46,51 @@ namespace DummyMidiData {
 	typedef struct _MIDISequence {
 		float   beat_time;
 		uint8_t size;
-		uint8_t event[10];
+		uint8_t event[3];
 	} MIDISequence;
 };
 
 
-class DummyMidiEvent {
+class DummyMidiEvent : public BackendMIDIEvent {
 	public:
 		DummyMidiEvent (const pframes_t timestamp, const uint8_t* data, size_t size);
 		DummyMidiEvent (const DummyMidiEvent& other);
 		~DummyMidiEvent ();
 		size_t size () const { return _size; };
 		pframes_t timestamp () const { return _timestamp; };
-		const unsigned char* const_data () const { return _data; };
-		unsigned char* data () { return _data; };
-		bool operator< (const DummyMidiEvent &other) const { return timestamp () < other.timestamp (); };
+		const uint8_t* data () const { return _data; };
 	private:
 		size_t _size;
 		pframes_t _timestamp;
 		uint8_t *_data;
 };
 
-typedef std::vector<boost::shared_ptr<DummyMidiEvent> > DummyMidiBuffer;
+typedef std::vector<std::shared_ptr<DummyMidiEvent> > DummyMidiBuffer;
 
-class DummyPort {
+class DummyPort : public BackendPort {
 	protected:
 		DummyPort (DummyAudioBackend &b, const std::string&, PortFlags);
 	public:
 		virtual ~DummyPort ();
 
-		const std::string& name () const { return _name; }
-		const std::string& pretty_name () const { return _pretty_name; }
-		PortFlags flags () const { return _flags; }
-
-		int set_name (const std::string &name) { _name = name; return 0; }
-		int set_pretty_name (const std::string &name) { _pretty_name = name; return 0; }
-
-		virtual DataType type () const = 0;
-
-		bool is_input ()     const { return flags () & IsInput; }
-		bool is_output ()    const { return flags () & IsOutput; }
-		bool is_physical ()  const { return flags () & IsPhysical; }
-		bool is_terminal ()  const { return flags () & IsTerminal; }
-		bool is_connected () const { return _connections.size () != 0; }
-		bool is_connected (const DummyPort *port) const;
-		bool is_physically_connected () const;
-
-		const std::set<DummyPort *>& get_connections () const { return _connections; }
-
-		int connect (DummyPort *port);
-		int disconnect (DummyPort *port);
-		void disconnect_all ();
-
-		virtual void* get_buffer (pframes_t nframes) = 0;
 		void next_period () { _gen_cycle = false; }
 
-		const LatencyRange latency_range (bool for_playback) const
-		{
-			return for_playback ? _playback_latency_range : _capture_latency_range;
-		}
-
-		void set_latency_range (const LatencyRange &latency_range, bool for_playback)
-		{
-			if (for_playback)
-			{
-				_playback_latency_range = latency_range;
-			}
-			else
-			{
-				_capture_latency_range = latency_range;
-			}
-		}
-
-	private:
-		DummyAudioBackend &_dummy_backend;
-		std::string _name;
-		std::string _pretty_name;
-		const PortFlags _flags;
-		LatencyRange _capture_latency_range;
-		LatencyRange _playback_latency_range;
-		std::set<DummyPort*> _connections;
-
-		void _connect (DummyPort* , bool);
-		void _disconnect (DummyPort* , bool);
-
 	protected:
-		// random number generator
+		/* random number generator */
 		void setup_random_number_generator ();
 		inline float    randf ();
 		inline uint32_t randi ();
 		uint32_t _rseed;
+		/* engine time */
+		pframes_t pulse_position () const;
 
 		// signal generator
 		volatile bool _gen_cycle;
 		Glib::Threads::Mutex generator_lock;
 
+        private:
+		AudioBackend& _engine;
+
 }; // class DummyPort
 
 class DummyAudioPort : public DummyPort {
@@ -170,6 +120,7 @@ class DummyAudioPort : public DummyPort {
 			SineSweepSwell,
 			SquareSweep,
 			SquareSweepSwell,
+			OneHz,
 			LTC,
 			Loopback,
 		};
@@ -200,7 +151,7 @@ class DummyAudioPort : public DummyPort {
 		float _rn1;
 		// LTC generator
 		LTCEncoder* _ltc;
-		RingBuffer<Sample>* _ltcbuf;
+		PBD::RingBuffer<Sample>* _ltcbuf;
 		float _ltc_spd;
 		float _ltc_rand;
 
@@ -227,13 +178,12 @@ class DummyMidiPort : public DummyPort {
 		// midi event generator ('fake' physical inputs)
 		void midi_generate (const pframes_t n_samples);
 		float   _midi_seq_spb; // samples per beat
-		int32_t _midi_seq_time;
+		int64_t _midi_seq_time;
 		uint32_t _midi_seq_pos;
 		DummyMidiData::MIDISequence const * _midi_seq_dat;
 }; // class DummyMidiPort
 
-class DummyAudioBackend : public AudioBackend {
-	friend class DummyPort;
+class DummyAudioBackend : public AudioBackend, public PortEngineSharedImpl {
 	public:
 	         DummyAudioBackend (AudioEngine& e, AudioBackendInfo& info);
 		~DummyAudioBackend ();
@@ -258,6 +208,7 @@ class DummyAudioBackend : public AudioBackend {
 
 		bool can_change_sample_rate_when_running () const;
 		bool can_change_buffer_size_when_running () const;
+		bool can_measure_systemic_latency () const { return true; }
 
 		int set_device_name (const std::string&);
 		int set_sample_rate (float);
@@ -316,8 +267,8 @@ class DummyAudioBackend : public AudioBackend {
 		size_t raw_buffer_size (DataType t);
 
 		/* Process time */
-		framepos_t sample_time ();
-		framepos_t sample_time_at_cycle_start ();
+		samplepos_t sample_time ();
+		samplepos_t sample_time_at_cycle_start ();
 		pframes_t samples_since_cycle_start ();
 
 		int create_process_thread (boost::function<void()> func);
@@ -331,33 +282,35 @@ class DummyAudioBackend : public AudioBackend {
 
 		void* private_handle () const;
 		const std::string& my_name () const;
-		bool available () const;
-		uint32_t port_name_size () const;
 
-		int         set_port_name (PortHandle, const std::string&);
-		std::string get_port_name (PortHandle) const;
-		PortHandle  get_port_by_name (const std::string&) const;
+		/* PortEngine API - forwarded to PortEngineSharedImpl */
 
-		int get_port_property (PortHandle, const std::string& key, std::string& value, std::string& type) const;
-		int set_port_property (PortHandle, const std::string& key, const std::string& value, const std::string& type);
+	bool        port_is_physical (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_is_physical (ph); }
+	void        get_physical_outputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_outputs (type, results); }
+	void        get_physical_inputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_inputs (type, results); }
+	ChanCount   n_physical_outputs () const { return PortEngineSharedImpl::n_physical_outputs (); }
+	ChanCount   n_physical_inputs () const { return PortEngineSharedImpl::n_physical_inputs (); }
+	uint32_t    port_name_size () const { return PortEngineSharedImpl::port_name_size(); }
+	int         set_port_name (PortEngine::PortHandle ph, const std::string& name) { return PortEngineSharedImpl::set_port_name (ph, name); }
+	std::string get_port_name (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_name (ph); }
+	PortFlags   get_port_flags (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_flags (ph); }
+	PortEngine::PortPtr  get_port_by_name (std::string const & name) const { return PortEngineSharedImpl::get_port_by_name (name); }
+	int         get_port_property (PortEngine::PortHandle ph, const std::string& key, std::string& value, std::string& type) const { return PortEngineSharedImpl::get_port_property (ph, key, value, type); }
+	int         set_port_property (PortEngine::PortHandle ph, const std::string& key, const std::string& value, const std::string& type) { return PortEngineSharedImpl::set_port_property (ph, key, value, type); }
+	int         get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>& results) const { return PortEngineSharedImpl::get_ports (port_name_pattern, type, flags, results); }
+	DataType    port_data_type (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_data_type (ph); }
+	PortEngine::PortPtr register_port (const std::string& shortname, ARDOUR::DataType type, ARDOUR::PortFlags flags) { return PortEngineSharedImpl::register_port (shortname, type, flags); }
+	void        unregister_port (PortHandle ph) { if (!_running) return; PortEngineSharedImpl::unregister_port (ph); }
+	int         connect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::connect (src, dst); }
+	int         disconnect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::disconnect (src, dst); }
+	int         connect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::connect (ph, other); }
+	int         disconnect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::disconnect (ph, other); }
+	int         disconnect_all (PortEngine::PortHandle ph) { return PortEngineSharedImpl::disconnect_all (ph); }
+	bool        connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::connected (ph, process_callback_safe); }
+	bool        connected_to (PortEngine::PortHandle ph, const std::string& other, bool process_callback_safe) { return PortEngineSharedImpl::connected_to (ph, other, process_callback_safe); }
+	bool        physically_connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::physically_connected (ph, process_callback_safe); }
+	int         get_connections (PortEngine::PortHandle ph, std::vector<std::string>& results, bool process_callback_safe) { return PortEngineSharedImpl::get_connections (ph, results, process_callback_safe); }
 
-		int get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>&) const;
-
-		DataType port_data_type (PortHandle) const;
-
-		PortHandle register_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
-		void unregister_port (PortHandle);
-
-		int  connect (const std::string& src, const std::string& dst);
-		int  disconnect (const std::string& src, const std::string& dst);
-		int  connect (PortHandle, const std::string&);
-		int  disconnect (PortHandle, const std::string&);
-		int  disconnect_all (PortHandle);
-
-		bool connected (PortHandle, bool process_callback_safe);
-		bool connected_to (PortHandle, const std::string&, bool process_callback_safe);
-		bool physically_connected (PortHandle, bool process_callback_safe);
-		int  get_connections (PortHandle, std::vector<std::string>&, bool process_callback_safe);
 
 		/* MIDI */
 		int midi_event_get (pframes_t& timestamp, size_t& size, uint8_t const** buf, void* port_buffer, uint32_t event_index);
@@ -377,14 +330,6 @@ class DummyAudioBackend : public AudioBackend {
 		void         set_latency_range (PortHandle, bool for_playback, LatencyRange);
 		LatencyRange get_latency_range (PortHandle, bool for_playback);
 
-		/* Discovering physical ports */
-
-		bool      port_is_physical (PortHandle) const;
-		void      get_physical_outputs (DataType type, std::vector<std::string>&);
-		void      get_physical_inputs (DataType type, std::vector<std::string>&);
-		ChanCount n_physical_outputs () const;
-		ChanCount n_physical_inputs () const;
-
 		/* Getting access to the data buffer for a port */
 
 		void* get_buffer (PortHandle, pframes_t);
@@ -397,6 +342,7 @@ class DummyAudioBackend : public AudioBackend {
 		enum MidiPortMode {
 			MidiNoEvents,
 			MidiGenerator,
+			MidiOneHz,
 			MidiLoopback,
 			MidiToAudio,
 		};
@@ -435,7 +381,7 @@ class DummyAudioBackend : public AudioBackend {
 		uint32_t _systemic_input_latency;
 		uint32_t _systemic_output_latency;
 
-		framecnt_t _processed_samples;
+		samplecnt_t _processed_samples;
 
 		pthread_t _main_thread;
 
@@ -453,64 +399,9 @@ class DummyAudioBackend : public AudioBackend {
 		};
 
 		/* port engine */
-		PortHandle add_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
 		int register_system_ports ();
-		void unregister_ports (bool system_only = false);
 
-		std::vector<DummyAudioPort *> _system_inputs;
-		std::vector<DummyAudioPort *> _system_outputs;
-		std::vector<DummyMidiPort *> _system_midi_in;
-		std::vector<DummyMidiPort *> _system_midi_out;
-
-		struct SortByPortName
-		{
-			bool operator ()(const DummyPort* lhs, const DummyPort* rhs) const
-			{
-				return PBD::naturally_less (lhs->name ().c_str (), rhs->name ().c_str ());
-			}
-		};
-
-		typedef std::map<std::string, DummyPort *> PortMap; // fast lookup in _ports
-		typedef std::set<DummyPort *, SortByPortName> PortIndex; // fast lookup in _ports
-		PortMap _portmap;
-		PortIndex _ports;
-
-		struct PortConnectData {
-			std::string a;
-			std::string b;
-			bool c;
-
-			PortConnectData (const std::string& a, const std::string& b, bool c)
-				: a (a) , b (b) , c (c) {}
-		};
-
-		std::vector<PortConnectData *> _port_connection_queue;
-		pthread_mutex_t _port_callback_mutex;
-		bool _port_change_flag;
-
-		void port_connect_callback (const std::string& a, const std::string& b, bool conn) {
-			pthread_mutex_lock (&_port_callback_mutex);
-			_port_connection_queue.push_back(new PortConnectData(a, b, conn));
-			pthread_mutex_unlock (&_port_callback_mutex);
-		}
-
-		void port_connect_add_remove_callback () {
-			pthread_mutex_lock (&_port_callback_mutex);
-			_port_change_flag = true;
-			pthread_mutex_unlock (&_port_callback_mutex);
-		}
-
-		bool valid_port (PortHandle port) const {
-			return std::find (_ports.begin(), _ports.end(), static_cast<DummyPort*>(port)) != _ports.end ();
-		}
-
-		DummyPort* find_port (const std::string& port_name) const {
-			PortMap::const_iterator it = _portmap.find (port_name);
-			if (it == _portmap.end()) {
-				return NULL;
-			}
-			return (*it).second;
-		}
+	BackendPort* port_factory (std::string const & name, ARDOUR::DataType type, ARDOUR::PortFlags);
 
 }; // class DummyAudioBackend
 
