@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2015-2015 Robin Gareus <robin@gareus.org>
- * Copyright (C) 2013 Paul Davis
+ * Copyright (C) 2015-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2015-2018 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2016-2018 Paul Davis <paul@linuxaudiosystems.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,9 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <regex.h>
@@ -36,6 +37,7 @@
 #include "pbd/error.h"
 #include "pbd/file_utils.h"
 #include "pbd/pthread_utils.h"
+#include "pbd/microseconds.h"
 #include "pbd/windows_timer_utils.h"
 #include "pbd/windows_mmcss.h"
 
@@ -63,6 +65,7 @@ std::vector<AudioBackend::DeviceStatus> PortAudioBackend::_output_audio_device_s
 
 PortAudioBackend::PortAudioBackend (AudioEngine& e, AudioBackendInfo& info)
 	: AudioBackend (e, info)
+	, PortEngineSharedImpl (e, s_instance_name)
 	, _pcmio (0)
 	, _run (false)
 	, _active (false)
@@ -77,7 +80,7 @@ PortAudioBackend::PortAudioBackend (AudioEngine& e, AudioBackendInfo& info)
 	, _max_deviation_us(0)
 	, _input_audio_device("")
 	, _output_audio_device("")
-	, _midi_driver_option(get_standard_device_name(DeviceNone))
+	, _midi_driver_option(winmme_driver_name)
 	, _samplerate (48000)
 	, _samples_per_period (1024)
 	, _n_inputs (0)
@@ -86,10 +89,8 @@ PortAudioBackend::PortAudioBackend (AudioEngine& e, AudioBackendInfo& info)
 	, _systemic_audio_output_latency (0)
 	, _dsp_load (0)
 	, _processed_samples (0)
-	, _port_change_flag (false)
 {
 	_instance_name = s_instance_name;
-	pthread_mutex_init (&_port_callback_mutex, 0);
 	pthread_mutex_init (&_freewheel_mutex, 0);
 	pthread_cond_init (&_freewheel_signal, 0);
 
@@ -104,7 +105,8 @@ PortAudioBackend::~PortAudioBackend ()
 	delete _pcmio; _pcmio = 0;
 	delete _midiio; _midiio = 0;
 
-	pthread_mutex_destroy (&_port_callback_mutex);
+	clear_ports ();
+
 	pthread_mutex_destroy (&_freewheel_mutex);
 	pthread_cond_destroy (&_freewheel_signal);
 }
@@ -570,15 +572,7 @@ PortAudioBackend::_start (bool for_latency_measurement)
 		return BackendReinitializationError;
 	}
 
-	if (_ports.size()) {
-		DEBUG_AUDIO(
-		    "Recovering from unclean shutdown, port registry is not empty.\n");
-		_system_inputs.clear();
-		_system_outputs.clear();
-		_system_midi_in.clear();
-		_system_midi_out.clear();
-		_ports.clear();
-	}
+	clear_ports ();
 
 	/* reset internal state */
 	assert (_run == false);
@@ -643,8 +637,6 @@ PortAudioBackend::_start (bool for_latency_measurement)
 
 	_measure_latency = for_latency_measurement;
 
-	_port_change_flag = false;
-
 	if (_midi_driver_option == winmme_driver_name) {
 		_midiio->set_enabled(true);
 		//_midiio->set_port_changed_callback(midi_port_change, this);
@@ -681,7 +673,9 @@ PortAudioBackend::_start (bool for_latency_measurement)
 	_run = true;
 
 	engine.reconnect_ports ();
-	_port_change_flag = false;
+	_port_change_flag.store (0);
+
+	_dsp_calc.reset ();
 
 	if (_use_blocking_api) {
 		if (!start_blocking_process_thread()) {
@@ -716,7 +710,7 @@ PortAudioBackend::_start (bool for_latency_measurement)
 int
 PortAudioBackend::portaudio_callback(const void* input,
                                      void* output,
-                                     unsigned long frame_count,
+                                     unsigned long sample_count,
                                      const PaStreamCallbackTimeInfo* time_info,
                                      PaStreamCallbackFlags status_flags,
                                      void* user_data)
@@ -725,7 +719,7 @@ PortAudioBackend::portaudio_callback(const void* input,
 
 	if (!pa_backend->process_callback((const float*)input,
 	                                  (float*)output,
-	                                  frame_count,
+	                                  sample_count,
 	                                  time_info,
 	                                  status_flags)) {
 		return paAbort;
@@ -737,10 +731,13 @@ PortAudioBackend::portaudio_callback(const void* input,
 bool
 PortAudioBackend::process_callback(const float* input,
                                    float* output,
-                                   uint32_t frame_count,
+                                   uint32_t sample_count,
                                    const PaStreamCallbackTimeInfo* timeInfo,
                                    PaStreamCallbackFlags statusFlags)
 {
+	PBD::WaitTimerRAII tr (dsp_stats[DeviceWait]);
+	PBD::TimerRAII tr2 (dsp_stats[RunLoop]);
+
 	_active = true;
 
 	_dsp_calc.set_start_timestamp_us (PBD::get_microseconds());
@@ -767,7 +764,7 @@ PortAudioBackend::process_callback(const float* input,
 	}
 
 	if (!_run || _freewheel) {
-		memset(output, 0, frame_count * sizeof(float) * _system_outputs.size());
+		memset(output, 0, sample_count * sizeof(float) * _system_outputs.size());
 		return true;
 	}
 
@@ -787,10 +784,10 @@ PortAudioBackend::process_callback(const float* input,
 bool
 PortAudioBackend::start_blocking_process_thread ()
 {
-	if (pbd_realtime_pthread_create (PBD_SCHED_FIFO, -20, 100000,
+	if (pbd_realtime_pthread_create (PBD_SCHED_FIFO, PBD_RT_PRI_MAIN, PBD_RT_STACKSIZE_PROC,
 				&_main_blocking_thread, blocking_thread_func, this))
 	{
-		if (pthread_create (&_main_blocking_thread, NULL, blocking_thread_func, this))
+		if (pbd_pthread_create (PBD_RT_STACKSIZE_PROC, &_main_blocking_thread, blocking_thread_func, this))
 		{
 			DEBUG_AUDIO("Failed to create main audio thread\n");
 			_run = false;
@@ -924,6 +921,7 @@ PortAudioBackend::freewheel_process_thread()
 				_freewheel_ack = false; // prepare next handshake
 				_midiio->set_enabled(true);
 				engine.freewheel_callback (_freewheeling);
+				_dsp_calc.reset ();
 			} else {
 				first_run = true;
 				_freewheel = true;
@@ -1004,13 +1002,13 @@ PortAudioBackend::raw_buffer_size (DataType t)
 }
 
 /* Process time */
-framepos_t
+samplepos_t
 PortAudioBackend::sample_time ()
 {
 	return _processed_samples;
 }
 
-framepos_t
+samplepos_t
 PortAudioBackend::sample_time_at_cycle_start ()
 {
 	return _processed_samples;
@@ -1109,22 +1107,15 @@ PortAudioBackend::portaudio_process_thread (void *arg)
 int
 PortAudioBackend::create_process_thread (boost::function<void()> func)
 {
-	pthread_t thread_id;
-	pthread_attr_t attr;
-	size_t stacksize = 100000;
+	pthread_t   thread_id;
+	ThreadData* td = new ThreadData (this, func, PBD_RT_STACKSIZE_PROC);
 
-	ThreadData* td = new ThreadData (this, func, stacksize);
-
-	if (pbd_realtime_pthread_create (PBD_SCHED_FIFO, -22, stacksize,
+	if (pbd_realtime_pthread_create (PBD_SCHED_FIFO, PBD_RT_PRI_PROC, PBD_RT_STACKSIZE_PROC,
 				&thread_id, portaudio_process_thread, td)) {
-		pthread_attr_init (&attr);
-		pthread_attr_setstacksize (&attr, stacksize);
-		if (pthread_create (&thread_id, &attr, portaudio_process_thread, td)) {
+		if (pbd_pthread_create (PBD_RT_STACKSIZE_PROC, &thread_id, portaudio_process_thread, td)) {
 			DEBUG_AUDIO("Cannot create process thread.");
-			pthread_attr_destroy (&attr);
 			return -1;
 		}
-		pthread_attr_destroy (&attr);
 	}
 
 	_threads.push_back (thread_id);
@@ -1196,180 +1187,6 @@ PortAudioBackend::my_name () const
 	return _instance_name;
 }
 
-bool
-PortAudioBackend::available () const
-{
-	return _run && _active;
-}
-
-uint32_t
-PortAudioBackend::port_name_size () const
-{
-	return 256;
-}
-
-int
-PortAudioBackend::set_port_name (PortEngine::PortHandle port, const std::string& name)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("set_port_name: Invalid Port(s)\n");
-		return -1;
-	}
-	return static_cast<PamPort*>(port)->set_name (_instance_name + ":" + name);
-}
-
-std::string
-PortAudioBackend::get_port_name (PortEngine::PortHandle port) const
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("get_port_name: Invalid Port(s)\n");
-		return std::string ();
-	}
-	return static_cast<PamPort*>(port)->name ();
-}
-
-int
-PortAudioBackend::get_port_property (PortHandle port,
-                                     const std::string& key,
-                                     std::string& value,
-                                     std::string& type) const
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("get_port_name: Invalid Port(s)\n");
-		return -1;
-	}
-
-	if (key == "http://jackaudio.org/metadata/pretty-name") {
-		type = "";
-		value = static_cast<PamPort*>(port)->pretty_name ();
-		if (!value.empty()) {
-			return 0;
-		}
-	}
-	return -1;
-}
-
-int
-PortAudioBackend::set_port_property (PortHandle port,
-                                     const std::string& key,
-                                     const std::string& value,
-                                     const std::string& type)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("get_port_name: Invalid Port(s)\n");
-		return -1;
-	}
-
-	if (key == "http://jackaudio.org/metadata/pretty-name" && type.empty ()) {
-		static_cast<PamPort*>(port)->set_pretty_name (value);
-		return 0;
-	}
-	return -1;
-}
-
-PortEngine::PortHandle
-PortAudioBackend::get_port_by_name (const std::string& name) const
-{
-	PortHandle port = (PortHandle) find_port (name);
-	return port;
-}
-
-int
-PortAudioBackend::get_ports (
-		const std::string& port_name_pattern,
-		DataType type, PortFlags flags,
-		std::vector<std::string>& port_names) const
-{
-	int rv = 0;
-	regex_t port_regex;
-	bool use_regexp = false;
-	if (port_name_pattern.size () > 0) {
-		if (!regcomp (&port_regex, port_name_pattern.c_str (), REG_EXTENDED|REG_NOSUB)) {
-			use_regexp = true;
-		}
-	}
-	for (size_t i = 0; i < _ports.size (); ++i) {
-		PamPort* port = _ports[i];
-		if ((port->type () == type) && flags == (port->flags () & flags)) {
-			if (!use_regexp || !regexec (&port_regex, port->name ().c_str (), 0, NULL, 0)) {
-				port_names.push_back (port->name ());
-				++rv;
-			}
-		}
-	}
-	if (use_regexp) {
-		regfree (&port_regex);
-	}
-	return rv;
-}
-
-DataType
-PortAudioBackend::port_data_type (PortEngine::PortHandle port) const
-{
-	if (!valid_port (port)) {
-		return DataType::NIL;
-	}
-	return static_cast<PamPort*>(port)->type ();
-}
-
-PortEngine::PortHandle
-PortAudioBackend::register_port (
-		const std::string& name,
-		ARDOUR::DataType type,
-		ARDOUR::PortFlags flags)
-{
-	if (name.size () == 0) { return 0; }
-	if (flags & IsPhysical) { return 0; }
-	return add_port (_instance_name + ":" + name, type, flags);
-}
-
-PortEngine::PortHandle
-PortAudioBackend::add_port (
-		const std::string& name,
-		ARDOUR::DataType type,
-		ARDOUR::PortFlags flags)
-{
-	assert(name.size ());
-	if (find_port (name)) {
-		DEBUG_PORTS(
-		    string_compose("register_port: Port already exists: (%1)\n", name));
-		return 0;
-	}
-	PamPort* port = NULL;
-	switch (type) {
-	case DataType::AUDIO:
-		port = new PortAudioPort(*this, name, flags);
-		break;
-	case DataType::MIDI:
-		port = new PortMidiPort(*this, name, flags);
-		break;
-	default:
-		DEBUG_PORTS("register_port: Invalid Data Type.\n");
-		return 0;
-	}
-
-	_ports.push_back (port);
-
-	return port;
-}
-
-void
-PortAudioBackend::unregister_port (PortEngine::PortHandle port_handle)
-{
-	if (!_run) {
-		return;
-	}
-	PamPort* port = static_cast<PamPort*>(port_handle);
-	std::vector<PamPort*>::iterator i = std::find (_ports.begin (), _ports.end (), static_cast<PamPort*>(port_handle));
-	if (i == _ports.end ()) {
-		DEBUG_PORTS("unregister_port: Failed to find port\n");
-		return;
-	}
-	disconnect_all(port_handle);
-	_ports.erase (i);
-	delete port;
-}
-
 int
 PortAudioBackend::register_system_audio_ports()
 {
@@ -1394,11 +1211,11 @@ PortAudioBackend::register_system_audio_ports()
 	for (uint32_t i = 0; i < a_ins; ++i) {
 		char tmp[64];
 		snprintf(tmp, sizeof(tmp), "system:capture_%d", i+1);
-		PortHandle p = add_port(std::string(tmp), DataType::AUDIO, static_cast<PortFlags>(IsOutput | IsPhysical | IsTerminal));
+		PortPtr p = add_port(std::string(tmp), DataType::AUDIO, static_cast<PortFlags>(IsOutput | IsPhysical | IsTerminal));
 		if (!p) return -1;
 		set_latency_range (p, false, lr);
-		PortAudioPort* audio_port = static_cast<PortAudioPort*>(p);
-		audio_port->set_pretty_name (
+		std::shared_ptr<PortAudioPort> audio_port = std::dynamic_pointer_cast<PortAudioPort>(p);
+		audio_port->set_hw_port_name (
 		    _pcmio->get_input_channel_name (name_to_id (_input_audio_device), i));
 		_system_inputs.push_back (audio_port);
 	}
@@ -1407,11 +1224,11 @@ PortAudioBackend::register_system_audio_ports()
 	for (uint32_t i = 0; i < a_out; ++i) {
 		char tmp[64];
 		snprintf(tmp, sizeof(tmp), "system:playback_%d", i+1);
-		PortHandle p = add_port(std::string(tmp), DataType::AUDIO, static_cast<PortFlags>(IsInput | IsPhysical | IsTerminal));
+		PortPtr p = add_port(std::string(tmp), DataType::AUDIO, static_cast<PortFlags>(IsInput | IsPhysical | IsTerminal));
 		if (!p) return -1;
 		set_latency_range (p, true, lr);
-		PortAudioPort* audio_port = static_cast<PortAudioPort*>(p);
-		audio_port->set_pretty_name (
+		std::shared_ptr<PortAudioPort> audio_port = std::dynamic_pointer_cast<PortAudioPort>(p);
+		audio_port->set_hw_port_name (
 		    _pcmio->get_output_channel_name (name_to_id (_output_audio_device), i));
 		_system_outputs.push_back(audio_port);
 	}
@@ -1435,11 +1252,11 @@ PortAudioBackend::register_system_midi_ports()
 	     i != inputs.end ();
 	     ++i) {
 		std::string port_name = "system:midi_capture_" + (*i)->name();
-		PortHandle p =
-		    add_port (port_name,
-		              DataType::MIDI,
-		              static_cast<PortFlags>(IsOutput | IsPhysical | IsTerminal));
-		if (!p) return -1;
+		PortPtr p = add_port (port_name, DataType::MIDI, static_cast<PortFlags>(IsOutput | IsPhysical | IsTerminal));
+
+		if (!p) {
+			return -1;
+		}
 
 		MidiDeviceInfo* info = _midiio->get_device_info((*i)->name());
 		if (info) { // assert?
@@ -1447,8 +1264,8 @@ PortAudioBackend::register_system_midi_ports()
 		}
 		set_latency_range (p, false, lr);
 
-		PortMidiPort* midi_port = static_cast<PortMidiPort*>(p);
-		midi_port->set_pretty_name ((*i)->name());
+		std::shared_ptr<PortMidiPort> midi_port = std::dynamic_pointer_cast<PortMidiPort>(p);
+		midi_port->set_hw_port_name ((*i)->name());
 		_system_midi_in.push_back (midi_port);
 		DEBUG_MIDI (string_compose ("Registered MIDI input port: %1\n", port_name));
 	}
@@ -1459,11 +1276,11 @@ PortAudioBackend::register_system_midi_ports()
 	     i != outputs.end ();
 	     ++i) {
 		std::string port_name = "system:midi_playback_" + (*i)->name();
-		PortHandle p =
-		    add_port (port_name,
-		              DataType::MIDI,
-		              static_cast<PortFlags>(IsInput | IsPhysical | IsTerminal));
-		if (!p) return -1;
+		PortPtr p = add_port (port_name, DataType::MIDI, static_cast<PortFlags>(IsInput | IsPhysical | IsTerminal));
+
+		if (!p) {
+			return -1;
+		}
 
 		MidiDeviceInfo* info = _midiio->get_device_info((*i)->name());
 		if (info) { // assert?
@@ -1471,150 +1288,33 @@ PortAudioBackend::register_system_midi_ports()
 		}
 		set_latency_range (p, false, lr);
 
-		PortMidiPort* midi_port = static_cast<PortMidiPort*>(p);
+		std::shared_ptr<PortMidiPort> midi_port = std::dynamic_pointer_cast<PortMidiPort>(p);
 		midi_port->set_n_periods(2);
-		midi_port->set_pretty_name ((*i)->name());
+		midi_port->set_hw_port_name ((*i)->name());
 		_system_midi_out.push_back (midi_port);
 		DEBUG_MIDI (string_compose ("Registered MIDI output port: %1\n", port_name));
 	}
 	return 0;
 }
 
-void
-PortAudioBackend::unregister_ports (bool system_only)
+BackendPort*
+PortAudioBackend::port_factory (std::string const & name, ARDOUR::DataType type, ARDOUR::PortFlags flags)
 {
-	size_t i = 0;
-	_system_inputs.clear();
-	_system_outputs.clear();
-	_system_midi_in.clear();
-	_system_midi_out.clear();
-	while (i <  _ports.size ()) {
-		PamPort* port = _ports[i];
-		if (! system_only || (port->is_physical () && port->is_terminal ())) {
-			port->disconnect_all ();
-			delete port;
-			_ports.erase (_ports.begin() + i);
-		} else {
-			++i;
-		}
-	}
-}
+	BackendPort* port = 0;
 
-int
-PortAudioBackend::connect (const std::string& src, const std::string& dst)
-{
-	PamPort* src_port = find_port (src);
-	PamPort* dst_port = find_port (dst);
-
-	if (!src_port) {
-		DEBUG_PORTS(string_compose("connect: Invalid Source port: (%1)\n", src));
-		return -1;
-	}
-	if (!dst_port) {
-		DEBUG_PORTS(string_compose("connect: Invalid Destination port: (%1)\n", dst));
-		return -1;
-	}
-	return src_port->connect (dst_port);
-}
-
-int
-PortAudioBackend::disconnect (const std::string& src, const std::string& dst)
-{
-	PamPort* src_port = find_port (src);
-	PamPort* dst_port = find_port (dst);
-
-	if (!src_port || !dst_port) {
-		DEBUG_PORTS("disconnect: Invalid Port(s)\n");
-		return -1;
-	}
-	return src_port->disconnect (dst_port);
-}
-
-int
-PortAudioBackend::connect (PortEngine::PortHandle src, const std::string& dst)
-{
-	PamPort* dst_port = find_port (dst);
-	if (!valid_port (src)) {
-		DEBUG_PORTS("connect: Invalid Source Port Handle\n");
-		return -1;
-	}
-	if (!dst_port) {
-		DEBUG_PORTS(string_compose("connect: Invalid Destination Port (%1)\n", dst));
-		return -1;
-	}
-	return static_cast<PamPort*>(src)->connect (dst_port);
-}
-
-int
-PortAudioBackend::disconnect (PortEngine::PortHandle src, const std::string& dst)
-{
-	PamPort* dst_port = find_port (dst);
-	if (!valid_port (src) || !dst_port) {
-		DEBUG_PORTS("disconnect: Invalid Port(s)\n");
-		return -1;
-	}
-	return static_cast<PamPort*>(src)->disconnect (dst_port);
-}
-
-int
-PortAudioBackend::disconnect_all (PortEngine::PortHandle port)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("disconnect_all: Invalid Port\n");
-		return -1;
-	}
-	static_cast<PamPort*>(port)->disconnect_all ();
-	return 0;
-}
-
-bool
-PortAudioBackend::connected (PortEngine::PortHandle port, bool /* process_callback_safe*/)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("disconnect_all: Invalid Port\n");
-		return false;
-	}
-	return static_cast<PamPort*>(port)->is_connected ();
-}
-
-bool
-PortAudioBackend::connected_to (PortEngine::PortHandle src, const std::string& dst, bool /*process_callback_safe*/)
-{
-	PamPort* dst_port = find_port (dst);
-	if (!valid_port (src) || !dst_port) {
-		DEBUG_PORTS("connected_to: Invalid Port\n");
-		return false;
-	}
-	return static_cast<PamPort*>(src)->is_connected (dst_port);
-}
-
-bool
-PortAudioBackend::physically_connected (PortEngine::PortHandle port, bool /*process_callback_safe*/)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("physically_connected: Invalid Port\n");
-		return false;
-	}
-	return static_cast<PamPort*>(port)->is_physically_connected ();
-}
-
-int
-PortAudioBackend::get_connections (PortEngine::PortHandle port, std::vector<std::string>& names, bool /*process_callback_safe*/)
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("get_connections: Invalid Port\n");
-		return -1;
+	switch (type) {
+		case DataType::AUDIO:
+			port = new PortAudioPort (*this, name, flags);
+			break;
+		case DataType::MIDI:
+			port = new PortMidiPort (*this, name, flags);
+			break;
+		default:
+			PBD::error << string_compose (_("%1::register_port: Invalid Data Type."), _instance_name) << endmsg;
+			return 0;
 	}
 
-	assert (0 == names.size ());
-
-	const std::vector<PamPort*>& connected_ports = static_cast<PamPort*>(port)->get_connections ();
-
-	for (std::vector<PamPort*>::const_iterator i = connected_ports.begin (); i != connected_ports.end (); ++i) {
-		names.push_back ((*i)->name ());
-	}
-
-	return (int)names.size ();
+	return port;
 }
 
 /* MIDI */
@@ -1702,35 +1402,36 @@ PortAudioBackend::monitoring_input (PortEngine::PortHandle)
 /* Latency management */
 
 void
-PortAudioBackend::set_latency_range (PortEngine::PortHandle port, bool for_playback, LatencyRange latency_range)
+PortAudioBackend::set_latency_range (PortEngine::PortHandle port_handle, bool for_playback, LatencyRange latency_range)
 {
+	std::shared_ptr<BackendPort> port = std::dynamic_pointer_cast<BackendPort>(port_handle);
 	if (!valid_port (port)) {
-		DEBUG_PORTS("PamPort::set_latency_range (): invalid port.\n");
+		DEBUG_PORTS("BackendPort::set_latency_range (): invalid port.\n");
+		return;
 	}
-	static_cast<PamPort*>(port)->set_latency_range (latency_range, for_playback);
+	port->set_latency_range (latency_range, for_playback);
 }
 
 LatencyRange
-PortAudioBackend::get_latency_range (PortEngine::PortHandle port, bool for_playback)
+PortAudioBackend::get_latency_range (PortEngine::PortHandle port_handle, bool for_playback)
 {
+	std::shared_ptr<BackendPort> port = std::dynamic_pointer_cast<BackendPort>(port_handle);
 	LatencyRange r;
 	if (!valid_port (port)) {
-		DEBUG_PORTS("PamPort::get_latency_range (): invalid port.\n");
+		DEBUG_PORTS("BackendPort::get_latency_range (): invalid port.\n");
 		r.min = 0;
 		r.max = 0;
 		return r;
 	}
-	PamPort* p = static_cast<PamPort*>(port);
-	assert(p);
 
-	r = p->latency_range (for_playback);
+	r = port->latency_range (for_playback);
 	// TODO MIDI
-	if (p->is_physical() && p->is_terminal() && p->type() == DataType::AUDIO) {
-		if (p->is_input() && for_playback) {
+	if (port->is_physical() && port->is_terminal() && port->type() == DataType::AUDIO) {
+		if (port->is_input() && for_playback) {
 			r.min += _samples_per_period;
 			r.max += _samples_per_period;
 		}
-		if (p->is_output() && !for_playback) {
+		if (port->is_output() && !for_playback) {
 			r.min += _samples_per_period;
 			r.max += _samples_per_period;
 		}
@@ -1738,101 +1439,15 @@ PortAudioBackend::get_latency_range (PortEngine::PortHandle port, bool for_playb
 	return r;
 }
 
-/* Discovering physical ports */
-
-bool
-PortAudioBackend::port_is_physical (PortEngine::PortHandle port) const
-{
-	if (!valid_port (port)) {
-		DEBUG_PORTS("PamPort::port_is_physical (): invalid port.\n");
-		return false;
-	}
-	return static_cast<PamPort*>(port)->is_physical ();
-}
-
-void
-PortAudioBackend::get_physical_outputs (DataType type, std::vector<std::string>& port_names)
-{
-	for (size_t i = 0; i < _ports.size (); ++i) {
-		PamPort* port = _ports[i];
-		if ((port->type () == type) && port->is_input () && port->is_physical ()) {
-			port_names.push_back (port->name ());
-		}
-	}
-}
-
-void
-PortAudioBackend::get_physical_inputs (DataType type, std::vector<std::string>& port_names)
-{
-	for (size_t i = 0; i < _ports.size (); ++i) {
-		PamPort* port = _ports[i];
-		if ((port->type () == type) && port->is_output () && port->is_physical ()) {
-			port_names.push_back (port->name ());
-		}
-	}
-}
-
-ChanCount
-PortAudioBackend::n_physical_outputs () const
-{
-	int n_midi = 0;
-	int n_audio = 0;
-	for (size_t i = 0; i < _ports.size (); ++i) {
-		PamPort* port = _ports[i];
-		if (port->is_output () && port->is_physical ()) {
-			switch (port->type ()) {
-			case DataType::AUDIO:
-				++n_audio;
-				break;
-			case DataType::MIDI:
-				++n_midi;
-				break;
-			default:
-				break;
-			}
-		}
-	}
-	ChanCount cc;
-	cc.set (DataType::AUDIO, n_audio);
-	cc.set (DataType::MIDI, n_midi);
-	return cc;
-}
-
-ChanCount
-PortAudioBackend::n_physical_inputs () const
-{
-	int n_midi = 0;
-	int n_audio = 0;
-	for (size_t i = 0; i < _ports.size (); ++i) {
-		PamPort* port = _ports[i];
-		if (port->is_input () && port->is_physical ()) {
-			switch (port->type ()) {
-			case DataType::AUDIO:
-				++n_audio;
-				break;
-			case DataType::MIDI:
-				++n_midi;
-				break;
-			default:
-				break;
-			}
-		}
-	}
-	ChanCount cc;
-	cc.set (DataType::AUDIO, n_audio);
-	cc.set (DataType::MIDI, n_midi);
-	return cc;
-}
 
 /* Getting access to the data buffer for a port */
 
 void*
-PortAudioBackend::get_buffer (PortEngine::PortHandle port, pframes_t nframes)
+PortAudioBackend::get_buffer (PortEngine::PortHandle port_handle, pframes_t nframes)
 {
+	std::shared_ptr<BackendPort> port = std::dynamic_pointer_cast<BackendPort>(port_handle);
 	assert (port);
-	assert (valid_port (port));
-	if (!port || !valid_port (port)) return NULL; // XXX remove me
-	return static_cast<PamPort*>(port)->get_buffer (nframes);
+	return port->get_buffer (nframes);
 }
 
 
@@ -1860,16 +1475,23 @@ PortAudioBackend::blocking_process_thread ()
 	DWORD tid = GetCurrentThreadId ();
 	DEBUG_THREADS (string_compose ("Process Thread Master ID: %1\n", tid));
 
+	_dsp_calc.reset ();
 	while (_run) {
 
 		if (_freewheeling != _freewheel) {
 			_freewheel = _freewheeling;
 			engine.freewheel_callback (_freewheel);
+			if (!_freewheel) {
+				_dsp_calc.reset ();
+			}
 		}
 
 		if (!_freewheel) {
 
-			switch (_pcmio->next_cycle (_samples_per_period)) {
+			dsp_stats[DeviceWait].start();
+			int r = _pcmio->next_cycle (_samples_per_period);
+			dsp_stats[DeviceWait].update();
+			switch (r) {
 			case 0: // OK
 				break;
 			case 1:
@@ -1913,6 +1535,7 @@ bool
 PortAudioBackend::blocking_process_main(const float* interleaved_input_data,
                                         float* interleaved_output_data)
 {
+	PBD::TimerRAII tr (dsp_stats[RunLoop]);
 	uint32_t i = 0;
 	int64_t min_elapsed_us = 1000000;
 	int64_t max_elapsed_us = 0;
@@ -1921,7 +1544,7 @@ PortAudioBackend::blocking_process_main(const float* interleaved_input_data,
 
 	i = 0;
 	/* Copy input audio data into input port buffers */
-	for (std::vector<PamPort*>::const_iterator it = _system_inputs.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_inputs.begin();
 	     it != _system_inputs.end();
 	     ++it, ++i) {
 		assert(_system_inputs.size() == _pcmio->n_capture_channels());
@@ -1934,7 +1557,7 @@ PortAudioBackend::blocking_process_main(const float* interleaved_input_data,
 	process_incoming_midi ();
 
 	/* clear output buffers */
-	for (std::vector<PamPort*>::const_iterator it = _system_outputs.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_outputs.begin();
 	     it != _system_outputs.end();
 	     ++it) {
 		memset((*it)->get_buffer(_samples_per_period),
@@ -1978,7 +1601,7 @@ PortAudioBackend::blocking_process_main(const float* interleaved_input_data,
 
 	/* write back audio */
 	i = 0;
-	for (std::vector<PamPort*>::const_iterator it = _system_outputs.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_outputs.begin();
 	     it != _system_outputs.end();
 	     ++it, ++i) {
 		assert(_system_outputs.size() == _pcmio->n_playback_channels());
@@ -2011,7 +1634,7 @@ bool
 PortAudioBackend::blocking_process_freewheel()
 {
 	// zero audio input buffers
-	for (std::vector<PamPort*>::const_iterator it = _system_inputs.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_inputs.begin();
 	     it != _system_inputs.end();
 	     ++it) {
 		memset((*it)->get_buffer(_samples_per_period),
@@ -2028,7 +1651,7 @@ PortAudioBackend::blocking_process_freewheel()
 	}
 
 	// drop all outgoing MIDI messages
-	for (std::vector<PamPort*>::const_iterator it = _system_midi_out.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_midi_out.begin();
 	     it != _system_midi_out.end();
 	     ++it) {
 		void* bptr = (*it)->get_buffer(0);
@@ -2044,7 +1667,7 @@ void
 PortAudioBackend::process_incoming_midi ()
 {
 	uint32_t i = 0;
-	for (std::vector<PamPort*>::const_iterator it = _system_midi_in.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_midi_in.begin();
 	     it != _system_midi_in.end();
 	     ++it, ++i) {
 		PortMidiBuffer* mbuf = static_cast<PortMidiBuffer*>((*it)->get_buffer(0));
@@ -2076,18 +1699,18 @@ void
 PortAudioBackend::process_outgoing_midi ()
 {
 	/* mixdown midi */
-	for (std::vector<PamPort*>::iterator it = _system_midi_out.begin();
+	for (std::vector<BackendPortPtr>::iterator it = _system_midi_out.begin();
 	     it != _system_midi_out.end();
 	     ++it) {
-		static_cast<PortMidiPort*>(*it)->next_period();
+		std::dynamic_pointer_cast<PortMidiPort>(*it)->next_period();
 	}
 	/* queue outgoing midi */
 	uint32_t i = 0;
-	for (std::vector<PamPort*>::const_iterator it = _system_midi_out.begin();
+	for (std::vector<BackendPortPtr>::const_iterator it = _system_midi_out.begin();
 	     it != _system_midi_out.end();
 	     ++it, ++i) {
 		const PortMidiBuffer* src =
-		    static_cast<const PortMidiPort*>(*it)->const_buffer();
+			std::dynamic_pointer_cast<const PortMidiPort>(*it)->const_buffer();
 
 		for (PortMidiBuffer::const_iterator mit = src->begin(); mit != src->end();
 		     ++mit) {
@@ -2110,9 +1733,9 @@ PortAudioBackend::process_port_connection_changes ()
 	bool connections_changed = false;
 	bool ports_changed = false;
 	if (!pthread_mutex_trylock (&_port_callback_mutex)) {
-		if (_port_change_flag) {
+		int canderef (1);
+		if (_port_change_flag.compare_exchange_strong (canderef, 0)) {
 			ports_changed = true;
-			_port_change_flag = false;
 		}
 		if (!_port_connection_queue.empty ()) {
 			connections_changed = true;
@@ -2132,6 +1755,7 @@ PortAudioBackend::process_port_connection_changes ()
 		manager.graph_order_callback();
 	}
 	if (connections_changed || ports_changed) {
+		update_system_port_latencies ();
 		engine.latency_callback(false);
 		engine.latency_callback(true);
 	}
@@ -2139,9 +1763,9 @@ PortAudioBackend::process_port_connection_changes ()
 
 /******************************************************************************/
 
-static boost::shared_ptr<PortAudioBackend> _instance;
+static std::shared_ptr<PortAudioBackend> _instance;
 
-static boost::shared_ptr<AudioBackend> backend_factory (AudioEngine& e);
+static std::shared_ptr<AudioBackend> backend_factory (AudioEngine& e);
 static int instantiate (const std::string& arg1, const std::string& /* arg2 */);
 static int deinstantiate ();
 static bool already_configured ();
@@ -2156,7 +1780,7 @@ static ARDOUR::AudioBackendInfo _descriptor = {
 	available
 };
 
-static boost::shared_ptr<AudioBackend>
+static std::shared_ptr<AudioBackend>
 backend_factory (AudioEngine& e)
 {
 	if (!_instance) {
@@ -2198,134 +1822,11 @@ extern "C" ARDOURBACKEND_API ARDOUR::AudioBackendInfo* descriptor ()
 
 
 /******************************************************************************/
-PamPort::PamPort (PortAudioBackend &b, const std::string& name, PortFlags flags)
-	: _osx_backend (b)
-	, _name  (name)
-	, _flags (flags)
-{
-	_capture_latency_range.min = 0;
-	_capture_latency_range.max = 0;
-	_playback_latency_range.min = 0;
-	_playback_latency_range.max = 0;
-}
-
-PamPort::~PamPort () {
-	disconnect_all ();
-}
-
-
-int PamPort::connect (PamPort *port)
-{
-	if (!port) {
-		DEBUG_PORTS("PamPort::connect (): invalid (null) port\n");
-		return -1;
-	}
-
-	if (type () != port->type ()) {
-		DEBUG_PORTS("PamPort::connect (): wrong port-type\n");
-		return -1;
-	}
-
-	if (is_output () && port->is_output ()) {
-		DEBUG_PORTS("PamPort::connect (): cannot inter-connect output ports.\n");
-		return -1;
-	}
-
-	if (is_input () && port->is_input ()) {
-		DEBUG_PORTS("PamPort::connect (): cannot inter-connect input ports.\n");
-		return -1;
-	}
-
-	if (this == port) {
-		DEBUG_PORTS("PamPort::connect (): cannot self-connect ports.\n");
-		return -1;
-	}
-
-	if (is_connected (port)) {
-#if 0 // don't bother to warn about this for now. just ignore it
-		PBD::error << _("PamPort::connect (): ports are already connected:")
-			<< " (" << name () << ") -> (" << port->name () << ")"
-			<< endmsg;
-#endif
-		return -1;
-	}
-
-	_connect (port, true);
-	return 0;
-}
-
-
-void PamPort::_connect (PamPort *port, bool callback)
-{
-	_connections.push_back (port);
-	if (callback) {
-		port->_connect (this, false);
-		_osx_backend.port_connect_callback (name(),  port->name(), true);
-	}
-}
-
-int PamPort::disconnect (PamPort *port)
-{
-	if (!port) {
-		DEBUG_PORTS("PamPort::disconnect (): invalid (null) port\n");
-		return -1;
-	}
-
-	if (!is_connected (port)) {
-		DEBUG_PORTS(string_compose(
-		    "PamPort::disconnect (): ports are not connected: (%1) -> (%2)\n",
-		    name(),
-		    port->name()));
-		return -1;
-	}
-	_disconnect (port, true);
-	return 0;
-}
-
-void PamPort::_disconnect (PamPort *port, bool callback)
-{
-	std::vector<PamPort*>::iterator it = std::find (_connections.begin (), _connections.end (), port);
-
-	assert (it != _connections.end ());
-
-	_connections.erase (it);
-
-	if (callback) {
-		port->_disconnect (this, false);
-		_osx_backend.port_connect_callback (name(),  port->name(), false);
-	}
-}
-
-
-void PamPort::disconnect_all ()
-{
-	while (!_connections.empty ()) {
-		_connections.back ()->_disconnect (this, false);
-		_osx_backend.port_connect_callback (name(),  _connections.back ()->name(), false);
-		_connections.pop_back ();
-	}
-}
-
-bool
-PamPort::is_connected (const PamPort *port) const
-{
-	return std::find (_connections.begin (), _connections.end (), port) != _connections.end ();
-}
-
-bool PamPort::is_physically_connected () const
-{
-	for (std::vector<PamPort*>::const_iterator it = _connections.begin (); it != _connections.end (); ++it) {
-		if ((*it)->is_physical ()) {
-			return true;
-		}
-	}
-	return false;
-}
 
 /******************************************************************************/
 
 PortAudioPort::PortAudioPort (PortAudioBackend &b, const std::string& name, PortFlags flags)
-	: PamPort (b, name, flags)
+	: BackendPort (b, name, flags)
 {
 	memset (_buffer, 0, sizeof (_buffer));
 #ifndef PLATFORM_WINDOWS
@@ -2338,15 +1839,15 @@ PortAudioPort::~PortAudioPort () { }
 void* PortAudioPort::get_buffer (pframes_t n_samples)
 {
 	if (is_input ()) {
-		std::vector<PamPort*>::const_iterator it = get_connections ().begin ();
+		std::set<BackendPortPtr>::const_iterator it = get_connections ().begin ();
 		if (it == get_connections ().end ()) {
 			memset (_buffer, 0, n_samples * sizeof (Sample));
 		} else {
-			PortAudioPort const * source = static_cast<const PortAudioPort*>(*it);
+			std::shared_ptr<const PortAudioPort> source = std::dynamic_pointer_cast<const PortAudioPort>(*it);
 			assert (source && source->is_output ());
 			memcpy (_buffer, source->const_buffer (), n_samples * sizeof (Sample));
 			while (++it != get_connections ().end ()) {
-				source = static_cast<const PortAudioPort*>(*it);
+				source = std::dynamic_pointer_cast<const PortAudioPort>(*it);
 				assert (source && source->is_output ());
 				Sample* dst = buffer ();
 				const Sample* src = source->const_buffer ();
@@ -2361,7 +1862,7 @@ void* PortAudioPort::get_buffer (pframes_t n_samples)
 
 
 PortMidiPort::PortMidiPort (PortAudioBackend &b, const std::string& name, PortFlags flags)
-	: PamPort (b, name, flags)
+	: BackendPort (b, name, flags)
 	, _n_periods (1)
 	, _bufperiod (0)
 {
@@ -2384,10 +1885,10 @@ void* PortMidiPort::get_buffer (pframes_t /* nframes */)
 {
 	if (is_input ()) {
 		(_buffer[_bufperiod]).clear ();
-		for (std::vector<PamPort*>::const_iterator i = get_connections ().begin ();
+		for (std::set<BackendPortPtr>::const_iterator i = get_connections ().begin ();
 				i != get_connections ().end ();
 				++i) {
-			const PortMidiBuffer * src = static_cast<const PortMidiPort*>(*i)->const_buffer ();
+			const PortMidiBuffer * src = std::dynamic_pointer_cast<const PortMidiPort>(*i)->const_buffer ();
 			for (PortMidiBuffer::const_iterator it = src->begin (); it != src->end (); ++it) {
 				(_buffer[_bufperiod]).push_back (*it);
 			}
