@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2014-2015 Robin Gareus <robin@gareus.org>
- * Copyright (C) 2013 Paul Davis
+ * Copyright (C) 2015-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2015-2018 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2017-2018 Paul Davis <paul@linuxaudiosystems.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,25 +13,25 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef __libbackend_portaudio_backend_h__
 #define __libbackend_portaudio_backend_h__
 
+#include <cstdint>
+#include <memory>
 #include <string>
 #include <vector>
 #include <set>
 
-#include <stdint.h>
 #include <pthread.h>
 
-#include <boost/shared_ptr.hpp>
-
 #include "ardour/audio_backend.h"
 #include "ardour/dsp_load_calculator.h"
+#include "ardour/port_engine_shared.h"
 #include "ardour/types.h"
 
 #include "portaudio_io.h"
@@ -41,14 +42,13 @@ namespace ARDOUR {
 
 class PortAudioBackend;
 
-class PortMidiEvent {
+class PortMidiEvent : public BackendMIDIEvent {
 	public:
 		PortMidiEvent (const pframes_t timestamp, const uint8_t* data, size_t size);
 		PortMidiEvent (const PortMidiEvent& other);
 		size_t size () const { return _size; };
 		pframes_t timestamp () const { return _timestamp; };
 		const uint8_t* data () const { return _data; };
-		bool operator< (const PortMidiEvent &other) const { return timestamp () < other.timestamp (); };
 	private:
 		size_t _size;
 		pframes_t _timestamp;
@@ -57,69 +57,7 @@ class PortMidiEvent {
 
 typedef std::vector<PortMidiEvent> PortMidiBuffer;
 
-class PamPort { // PortAudio / PortMidi Backend Port
-	protected:
-		PamPort (PortAudioBackend &b, const std::string&, PortFlags);
-	public:
-		virtual ~PamPort ();
-
-		const std::string& name () const { return _name; }
-		const std::string& pretty_name () const { return _pretty_name; }
-		PortFlags flags () const { return _flags; }
-
-		int set_name (const std::string &name) { _name = name; return 0; }
-		int set_pretty_name (const std::string& name) { _pretty_name = name; return 0;}
-
-		virtual DataType type () const = 0;
-
-		bool is_input ()     const { return flags () & IsInput; }
-		bool is_output ()    const { return flags () & IsOutput; }
-		bool is_physical ()  const { return flags () & IsPhysical; }
-		bool is_terminal ()  const { return flags () & IsTerminal; }
-		bool is_connected () const { return _connections.size () != 0; }
-		bool is_connected (const PamPort *port) const;
-		bool is_physically_connected () const;
-
-		const std::vector<PamPort *>& get_connections () const { return _connections; }
-
-		int connect (PamPort *port);
-		int disconnect (PamPort *port);
-		void disconnect_all ();
-
-		virtual void* get_buffer (pframes_t nframes) = 0;
-
-		const LatencyRange latency_range (bool for_playback) const
-		{
-			return for_playback ? _playback_latency_range : _capture_latency_range;
-		}
-
-		void set_latency_range (const LatencyRange &latency_range, bool for_playback)
-		{
-			if (for_playback)
-			{
-				_playback_latency_range = latency_range;
-			}
-			else
-			{
-				_capture_latency_range = latency_range;
-			}
-		}
-
-	private:
-		PortAudioBackend &_osx_backend;
-		std::string _name;
-		std::string _pretty_name;
-		const PortFlags _flags;
-		LatencyRange _capture_latency_range;
-		LatencyRange _playback_latency_range;
-		std::vector<PamPort*> _connections;
-
-		void _connect (PamPort* , bool);
-		void _disconnect (PamPort* , bool);
-
-}; // class PamPort
-
-class PortAudioPort : public PamPort {
+class PortAudioPort : public BackendPort {
 	public:
 		PortAudioPort (PortAudioBackend &b, const std::string&, PortFlags);
 		~PortAudioPort ();
@@ -134,7 +72,7 @@ class PortAudioPort : public PamPort {
 		Sample _buffer[8192];
 }; // class PortAudioPort
 
-class PortMidiPort : public PamPort {
+class PortMidiPort : public BackendPort {
 	public:
 		PortMidiPort (PortAudioBackend &b, const std::string&, PortFlags);
 		~PortMidiPort ();
@@ -153,8 +91,7 @@ class PortMidiPort : public PamPort {
 		int _bufperiod;
 }; // class PortMidiPort
 
-class PortAudioBackend : public AudioBackend {
-	friend class PamPort;
+class PortAudioBackend : public AudioBackend, public PortEngineSharedImpl {
 	public:
 		PortAudioBackend (AudioEngine& e, AudioBackendInfo& info);
 		~PortAudioBackend ();
@@ -218,6 +155,7 @@ class PortAudioBackend : public AudioBackend {
 		uint32_t     systemic_midi_input_latency (std::string const) const;
 		uint32_t     systemic_midi_output_latency (std::string const) const;
 
+		bool can_measure_systemic_latency () const { return true; }
 		bool can_set_systemic_midi_latencies () const { return true; }
 
 		/* External control app */
@@ -243,8 +181,8 @@ class PortAudioBackend : public AudioBackend {
 		size_t raw_buffer_size (DataType t);
 
 		/* Process time */
-		framepos_t sample_time ();
-		framepos_t sample_time_at_cycle_start ();
+		samplepos_t sample_time ();
+		samplepos_t sample_time_at_cycle_start ();
 		pframes_t samples_since_cycle_start ();
 
 		int create_process_thread (boost::function<void()> func);
@@ -258,32 +196,35 @@ class PortAudioBackend : public AudioBackend {
 
 		void* private_handle () const;
 		const std::string& my_name () const;
-		bool available () const;
-		uint32_t port_name_size () const;
 
-		int         set_port_name (PortHandle, const std::string&);
-		std::string get_port_name (PortHandle) const;
-		PortHandle  get_port_by_name (const std::string&) const;
-		int get_port_property (PortHandle, const std::string& key, std::string& value, std::string& type) const;
-		int set_port_property (PortHandle, const std::string& key, const std::string& value, const std::string& type);
 
-		int get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>&) const;
+		/* PortEngine API - forwarded to PortEngineSharedImpl */
 
-		DataType port_data_type (PortHandle) const;
-
-		PortHandle register_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
-		void unregister_port (PortHandle);
-
-		int  connect (const std::string& src, const std::string& dst);
-		int  disconnect (const std::string& src, const std::string& dst);
-		int  connect (PortHandle, const std::string&);
-		int  disconnect (PortHandle, const std::string&);
-		int  disconnect_all (PortHandle);
-
-		bool connected (PortHandle, bool process_callback_safe);
-		bool connected_to (PortHandle, const std::string&, bool process_callback_safe);
-		bool physically_connected (PortHandle, bool process_callback_safe);
-		int  get_connections (PortHandle, std::vector<std::string>&, bool process_callback_safe);
+	bool        port_is_physical (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_is_physical (ph); }
+	void        get_physical_outputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_outputs (type, results); }
+	void        get_physical_inputs (DataType type, std::vector<std::string>& results) { PortEngineSharedImpl::get_physical_inputs (type, results); }
+	ChanCount   n_physical_outputs () const { return PortEngineSharedImpl::n_physical_outputs (); }
+	ChanCount   n_physical_inputs () const { return PortEngineSharedImpl::n_physical_inputs (); }
+	uint32_t    port_name_size () const { return PortEngineSharedImpl::port_name_size(); }
+	int         set_port_name (PortEngine::PortHandle ph, const std::string& name) { return PortEngineSharedImpl::set_port_name (ph, name); }
+	std::string get_port_name (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_name (ph); }
+	PortFlags   get_port_flags (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::get_port_flags (ph); }
+	PortEngine::PortPtr  get_port_by_name (std::string const & name) const { return PortEngineSharedImpl::get_port_by_name (name); }
+	int         get_port_property (PortEngine::PortHandle ph, const std::string& key, std::string& value, std::string& type) const { return PortEngineSharedImpl::get_port_property (ph, key, value, type); }
+	int         set_port_property (PortEngine::PortHandle ph, const std::string& key, const std::string& value, const std::string& type) { return PortEngineSharedImpl::set_port_property (ph, key, value, type); }
+	int         get_ports (const std::string& port_name_pattern, DataType type, PortFlags flags, std::vector<std::string>& results) const { return PortEngineSharedImpl::get_ports (port_name_pattern, type, flags, results); }
+	DataType    port_data_type (PortEngine::PortHandle ph) const { return PortEngineSharedImpl::port_data_type (ph); }
+	PortEngine::PortPtr register_port (const std::string& shortname, ARDOUR::DataType type, ARDOUR::PortFlags flags) { return PortEngineSharedImpl::register_port (shortname, type, flags); }
+	void        unregister_port (PortHandle ph) { if (!_run) return; PortEngineSharedImpl::unregister_port (ph); }
+	int         connect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::connect (src, dst); }
+	int         disconnect (const std::string& src, const std::string& dst) { return PortEngineSharedImpl::disconnect (src, dst); }
+	int         connect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::connect (ph, other); }
+	int         disconnect (PortEngine::PortHandle ph, const std::string& other) { return PortEngineSharedImpl::disconnect (ph, other); }
+	int         disconnect_all (PortEngine::PortHandle ph) { return PortEngineSharedImpl::disconnect_all (ph); }
+	bool        connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::connected (ph, process_callback_safe); }
+	bool        connected_to (PortEngine::PortHandle ph, const std::string& other, bool process_callback_safe) { return PortEngineSharedImpl::connected_to (ph, other, process_callback_safe); }
+	bool        physically_connected (PortEngine::PortHandle ph, bool process_callback_safe) { return PortEngineSharedImpl::physically_connected (ph, process_callback_safe); }
+	int         get_connections (PortEngine::PortHandle ph, std::vector<std::string>& results, bool process_callback_safe) { return PortEngineSharedImpl::get_connections (ph, results, process_callback_safe); }
 
 		/* MIDI */
 		int midi_event_get (pframes_t& timestamp, size_t& size, uint8_t const** buf, void* port_buffer, uint32_t event_index);
@@ -303,14 +244,6 @@ class PortAudioBackend : public AudioBackend {
 		void         set_latency_range (PortHandle, bool for_playback, LatencyRange);
 		LatencyRange get_latency_range (PortHandle, bool for_playback);
 
-		/* Discovering physical ports */
-
-		bool      port_is_physical (PortHandle) const;
-		void      get_physical_outputs (DataType type, std::vector<std::string>&);
-		void      get_physical_inputs (DataType type, std::vector<std::string>&);
-		ChanCount n_physical_outputs () const;
-		ChanCount n_physical_inputs () const;
-
 		/* Getting access to the data buffer for a port */
 
 		void* get_buffer (PortHandle, pframes_t);
@@ -342,7 +275,7 @@ class PortAudioBackend : public AudioBackend {
 
 		bool process_callback(const float* input,
 	                          float* output,
-	                          uint32_t frame_count,
+	                          uint32_t sample_count,
 	                          const PaStreamCallbackTimeInfo* timeInfo,
 	                          PaStreamCallbackFlags statusFlags);
 
@@ -407,7 +340,7 @@ class PortAudioBackend : public AudioBackend {
 
 		/* processing */
 		float  _dsp_load;
-		framecnt_t _processed_samples;
+		samplecnt_t _processed_samples;
 
 		/* blocking thread */
 		pthread_t _main_blocking_thread;
@@ -432,63 +365,10 @@ class PortAudioBackend : public AudioBackend {
 		};
 
 		/* port engine */
-		PortHandle add_port (const std::string& shortname, ARDOUR::DataType, ARDOUR::PortFlags);
+		BackendPort* port_factory (std::string const & name, ARDOUR::DataType dt, ARDOUR::PortFlags flags);
+
 		int register_system_audio_ports ();
 		int register_system_midi_ports ();
-		void unregister_ports (bool system_only = false);
-
-		std::vector<PamPort *> _ports;
-		std::vector<PamPort *> _system_inputs;
-		std::vector<PamPort *> _system_outputs;
-		std::vector<PamPort *> _system_midi_in;
-		std::vector<PamPort *> _system_midi_out;
-
-		struct PortConnectData {
-			std::string a;
-			std::string b;
-			bool c;
-
-			PortConnectData (const std::string& a, const std::string& b, bool c)
-				: a (a) , b (b) , c (c) {}
-		};
-
-		std::vector<PortConnectData *> _port_connection_queue;
-		pthread_mutex_t _port_callback_mutex;
-		bool _port_change_flag;
-
-		void port_connect_callback (const std::string& a, const std::string& b, bool conn) {
-			pthread_mutex_lock (&_port_callback_mutex);
-			_port_connection_queue.push_back(new PortConnectData(a, b, conn));
-			pthread_mutex_unlock (&_port_callback_mutex);
-		}
-
-		void port_connect_add_remove_callback () {
-			pthread_mutex_lock (&_port_callback_mutex);
-			_port_change_flag = true;
-			pthread_mutex_unlock (&_port_callback_mutex);
-		}
-
-		bool valid_port (PortHandle port) const {
-			return std::find (_ports.begin (), _ports.end (), (PamPort*)port) != _ports.end ();
-		}
-
-		PamPort * find_port (const std::string& port_name) const {
-			for (std::vector<PamPort*>::const_iterator it = _ports.begin (); it != _ports.end (); ++it) {
-				if ((*it)->name () == port_name) {
-					return *it;
-				}
-			}
-			return NULL;
-		}
-
-		PamPort * find_port_in (std::vector<PamPort *> plist, const std::string& port_name) const {
-			for (std::vector<PamPort*>::const_iterator it = plist.begin (); it != plist.end (); ++it) {
-				if ((*it)->name () == port_name) {
-					return *it;
-				}
-			}
-			return NULL;
-		}
 
 }; // class PortAudioBackend
 
