@@ -1,21 +1,23 @@
 /*
-    Copyright (C) 2006 Paul Davis
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * Copyright (C) 2006-2009 David Robillard <d@drobilla.net>
+ * Copyright (C) 2006-2018 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2012 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2015-2019 Robin Gareus <robin@gareus.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef ardour_generic_midi_control_protocol_h
 #define ardour_generic_midi_control_protocol_h
@@ -23,6 +25,9 @@
 #include <list>
 #include <glibmm/threads.h>
 
+#define ABSTRACT_UI_EXPORTS
+#include "pbd/abstract_ui.h"
+
 #include "ardour/types.h"
 #include "ardour/port.h"
 
@@ -34,44 +39,55 @@ namespace PBD {
 
 namespace ARDOUR {
 	class AsyncMIDIPort;
-	class ControllableDescriptor;
 	class MidiPort;
 	class Session;
 }
 
 namespace MIDI {
-    class Port;
+	class Port;
 }
 
 class MIDIControllable;
 class MIDIFunction;
 class MIDIAction;
 
-class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
-  public:
+struct GenericMIDIRequest : public BaseUI::BaseRequestObject {
+public:
+	GenericMIDIRequest () {}
+	~GenericMIDIRequest () {}
+};
+
+
+class GenericMidiControlProtocol : public ARDOUR::ControlProtocol, public AbstractUI<GenericMIDIRequest> {
+public:
 	GenericMidiControlProtocol (ARDOUR::Session&);
 	virtual ~GenericMidiControlProtocol();
 
+	void do_request (GenericMIDIRequest*);
+	int stop ();
+
+	void thread_init ();
+
 	int set_active (bool yn);
 	static bool probe() { return true; }
 
 	void stripable_selection_changed () {}
 
-	std::list<boost::shared_ptr<ARDOUR::Bundle> > bundles ();
+	std::list<std::shared_ptr<ARDOUR::Bundle> > bundles ();
 
-	boost::shared_ptr<ARDOUR::Port> input_port () const;
-	boost::shared_ptr<ARDOUR::Port> output_port () const;
+	std::shared_ptr<ARDOUR::Port> input_port () const;
+	std::shared_ptr<ARDOUR::Port> output_port () const;
 
-	void set_feedback_interval (ARDOUR::microseconds_t);
+	void set_feedback_interval (PBD::microseconds_t);
 
 	int set_feedback (bool yn);
 	bool get_feedback () const;
 
-        boost::shared_ptr<PBD::Controllable> lookup_controllable (const ARDOUR::ControllableDescriptor&) const;
+	std::shared_ptr<PBD::Controllable> lookup_controllable (std::string const &) const;
 
-	void maybe_start_touch (PBD::Controllable*);
+	void maybe_start_touch (std::shared_ptr<PBD::Controllable>);
 
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int version);
 
 	bool has_editor () const { return true; }
@@ -80,6 +96,7 @@ class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
 
 	int load_bindings (const std::string&);
 	void drop_bindings ();
+	void drop_all ();
 
 	void check_used_event (int, int);
 
@@ -109,16 +126,23 @@ class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
 		return _threshold;
 	}
 
+	void add_rid_to_selection (int rid);
+	void set_rid_selection (int rid);
+	void toggle_rid_selection (int rid);
+	void remove_rid_from_selection (int rid);
+
 	PBD::Signal0<void> ConnectionChange;
 
-  private:
-	boost::shared_ptr<ARDOUR::Bundle> _input_bundle;
-	boost::shared_ptr<ARDOUR::Bundle> _output_bundle;
-	boost::shared_ptr<ARDOUR::AsyncMIDIPort> _input_port;
-	boost::shared_ptr<ARDOUR::AsyncMIDIPort> _output_port;
+	CONTROL_PROTOCOL_THREADS_NEED_TEMPO_MAP_DECL();
 
-	ARDOUR::microseconds_t _feedback_interval;
-	ARDOUR::microseconds_t last_feedback_time;
+private:
+	std::shared_ptr<ARDOUR::Bundle> _input_bundle;
+	std::shared_ptr<ARDOUR::Bundle> _output_bundle;
+	std::shared_ptr<ARDOUR::AsyncMIDIPort> _input_port;
+	std::shared_ptr<ARDOUR::AsyncMIDIPort> _output_port;
+
+	PBD::microseconds_t _feedback_interval;
+	PBD::microseconds_t last_feedback_time;
 
 	bool  do_feedback;
 	void _send_feedback ();
@@ -145,23 +169,19 @@ class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
 	};
 	typedef std::list<MIDIPendingControllable* > MIDIPendingControllables;
 	MIDIPendingControllables pending_controllables;
-        Glib::Threads::Mutex controllables_lock;
-        Glib::Threads::Mutex pending_lock;
+	mutable Glib::Threads::Mutex controllables_lock;
+	Glib::Threads::Mutex pending_lock;
 
-	bool start_learning (PBD::Controllable*);
-	void stop_learning (PBD::Controllable*);
+	bool start_learning (std::weak_ptr<PBD::Controllable>);
+	void stop_learning (std::weak_ptr<PBD::Controllable>);
 
 	void learning_stopped (MIDIControllable*);
 
-	void create_binding (PBD::Controllable*, int, int);
-	void delete_binding (PBD::Controllable*);
-
 	MIDIControllable* create_binding (const XMLNode&);
 	MIDIFunction* create_function (const XMLNode&);
 	MIDIAction* create_action (const XMLNode&);
 
 	void reset_controllables ();
-	void drop_all ();
 
 	enum ConnectionState {
 		InputConnected = 0x1,
@@ -169,9 +189,8 @@ class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
 	};
 
 	int connection_state;
-	bool connection_handler (boost::weak_ptr<ARDOUR::Port>, std::string name1, boost::weak_ptr<ARDOUR::Port>, std::string name2, bool yn);
-	PBD::ScopedConnection port_connection;
-	void connected();
+	bool connection_handler (std::weak_ptr<ARDOUR::Port>, std::string name1, std::weak_ptr<ARDOUR::Port>, std::string name2, bool yn);
+	PBD::ScopedConnection _port_connection;
 
 	std::string _current_binding;
 	uint32_t _bank_size;
@@ -187,7 +206,11 @@ class GenericMidiControlProtocol : public ARDOUR::ControlProtocol {
 	mutable void *gui;
 	void build_gui ();
 
+	PBD::ScopedConnectionList midi_connections;
 
+	bool midi_input_handler (Glib::IOCondition ioc, std::weak_ptr<ARDOUR::AsyncMIDIPort> port);
+	void start_midi_handling ();
+	void stop_midi_handling ();
 };
 
 #endif /* ardour_generic_midi_control_protocol_h */
