@@ -1,21 +1,24 @@
 /*
-	Copyright (C) 2006,2007 John Anderson
-	Copyright (C) 2012 Paul Davis
-
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2007 John Anderson
+ * Copyright (C) 2012-2016 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2012-2016 Tim Mayberry <mojofunk@gmail.com>
+ * Copyright (C) 2014-2015 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2015 Len Ovens <len@ovenwerks.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <cstdlib>
 #include <cstring>
@@ -56,8 +59,15 @@ DeviceInfo::DeviceInfo()
 	, _uses_logic_control_buttons (false)
 	, _uses_ipmidi (false)
 	, _no_handshake (false)
+	, _is_qcon(false)
+	, _is_platformMp(false)
+	, _is_proG2(false)
+	, _is_xtouch(false)
+	, _has_qcon_second_lcd(false)
+	, _has_qcon_master_meters(false)
 	, _has_meters (true)
 	, _has_separate_meters (false)
+	, _single_fader_follows_selection (false)
 	, _device_type (MCU)
 	, _name (X_("Mackie Control Universal Pro"))
 {
@@ -254,6 +264,12 @@ DeviceInfo::set_state (const XMLNode& node, int /* version */)
 		return -1;
 	}
 
+	if ((child = node.child ("SingleFaderFollowsSelection")) != 0) {
+		child->get_property ("value", _single_fader_follows_selection);
+	} else {
+		_single_fader_follows_selection = false;
+	}
+
 	if ((child = node.child ("Extenders")) != 0) {
 		if (!child->get_property ("value", _extenders)) {
 			_extenders = 0;
@@ -320,6 +336,42 @@ DeviceInfo::set_state (const XMLNode& node, int /* version */)
 		_has_meters = true;
 	}
 
+	if ((child = node.child ("IsQCon")) != 0) {
+		child->get_property ("value", _is_qcon);
+	} else {
+		_is_qcon = false;
+	}
+	
+	if ((child = node.child ("IsXTouch")) != 0) {
+		child->get_property ("value", _is_xtouch);
+	} else {
+		_is_xtouch = false;
+	}
+
+	if ((child = node.child ("IsPlatformMp")) != 0) {
+		child->get_property ("value", _is_platformMp);
+	} else {
+		_is_platformMp = false;
+	}
+
+	if ((child = node.child ("IsProG2")) != 0) {
+		child->get_property ("value", _is_proG2);
+	} else {
+		_is_proG2 = false;
+	}
+
+	if ((child = node.child ("HasQConSecondLCD")) != 0) {
+		child->get_property ("value", _has_qcon_second_lcd);
+	} else {
+		_has_qcon_second_lcd = false;
+	}
+
+	if ((child = node.child ("HasQConMasterMeters")) != 0) {
+		child->get_property ("value", _has_qcon_master_meters);
+	} else {
+		_has_qcon_master_meters = false;
+	}
+
 	if ((child = node.child ("HasSeparateMeters")) != 0) {
 		child->get_property ("value", _has_separate_meters);
 	} else {
@@ -414,6 +466,12 @@ DeviceInfo::has_meters() const
 	return _has_meters;
 }
 
+bool
+DeviceInfo::single_fader_follows_selection() const
+{
+	return _single_fader_follows_selection;
+}
+
 bool
 DeviceInfo::has_separate_meters() const
 {
@@ -456,6 +514,39 @@ DeviceInfo::no_handshake () const
 	return _no_handshake;
 }
 
+bool
+DeviceInfo::is_qcon () const
+{
+	return _is_qcon;
+}
+
+bool DeviceInfo::is_platformMp () const
+{
+	return _is_platformMp;
+}
+
+bool DeviceInfo::is_proG2 () const
+{
+	return _is_proG2;
+}
+
+bool DeviceInfo::is_xtouch () const
+{
+	return _is_xtouch;
+}
+
+bool
+DeviceInfo::has_qcon_second_lcd () const
+{
+	return _has_qcon_second_lcd;
+}
+
+bool
+DeviceInfo::has_qcon_master_meters () const
+{
+	return _has_qcon_master_meters;
+}
+
 bool
 DeviceInfo::has_touch_sense_faders () const
 {
