@@ -1,20 +1,25 @@
 /*
-    Copyright (C) 2006,2007 John Anderson
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Copyright (C) 2006-2007 John Anderson
+ * Copyright (C) 2007-2009 David Robillard <d@drobilla.net>
+ * Copyright (C) 2007-2017 Paul Davis <paul@linuxaudiosystems.com>
+ * Copyright (C) 2009-2011 Carl Hetherington <carl@carlh.net>
+ * Copyright (C) 2015-2016 Len Ovens <len@ovenwerks.net>
+ * Copyright (C) 2016-2018 Ben Loftis <ben@harrisonconsoles.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef ardour_mackie_control_protocol_h
 #define ardour_mackie_control_protocol_h
@@ -41,6 +46,7 @@
 #include "timer.h"
 #include "device_info.h"
 #include "device_profile.h"
+#include "subview.h"
 
 namespace ARDOUR {
 	class AutomationControl;
@@ -86,21 +92,14 @@ class MackieControlProtocol
 
 	enum ViewMode {
 		Mixer,
-		AudioTracks,
 		MidiTracks,
-		Busses,
+		Inputs,
+		AudioTracks,
+		AudioInstr,
 		Auxes,
+		Busses,
+		Outputs,
 		Selected,
-		Hidden,
-		Plugins,
-	};
-
-	enum SubViewMode {
-		None,
-		EQ,
-		Dynamics,
-		Sends,
-		TrackView,
 	};
 
 	enum FlipMode {
@@ -119,7 +118,7 @@ class MackieControlProtocol
 	Mackie::DeviceProfile& device_profile() { return _device_profile; }
 
 	PBD::Signal0<void> DeviceChanged;
-	PBD::Signal1<void,boost::shared_ptr<Mackie::Surface> > ConnectionChange;
+	PBD::Signal1<void,std::shared_ptr<Mackie::Surface> > ConnectionChange;
 
         void device_ready ();
 
@@ -129,28 +128,31 @@ class MackieControlProtocol
 
 	FlipMode flip_mode () const { return _flip_mode; }
 	ViewMode view_mode () const { return _view_mode; }
-	SubViewMode subview_mode () const { return _subview_mode; }
-	static bool subview_mode_would_be_ok (SubViewMode, boost::shared_ptr<ARDOUR::Stripable>);
-	boost::shared_ptr<ARDOUR::Stripable> subview_stripable() const;
+	std::shared_ptr<Mackie::Subview> subview() { return _subview; }
 	bool zoom_mode () const { return modifier_state() & MODIFIER_ZOOM; }
 	bool     metering_active () const { return _metering_active; }
 
-	bool is_track (boost::shared_ptr<ARDOUR::Stripable>) const;
-	bool is_audio_track (boost::shared_ptr<ARDOUR::Stripable>) const;
-	bool is_midi_track (boost::shared_ptr<ARDOUR::Stripable>) const;
-	bool is_mapped (boost::shared_ptr<ARDOUR::Stripable>) const;
-	boost::shared_ptr<ARDOUR::Stripable> first_selected_stripable () const;
+	bool is_track (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_audio_track (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_midi_track (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_trigger_track (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_foldback_bus (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_vca (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool has_instrument (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool is_mapped (std::shared_ptr<ARDOUR::Stripable>) const;
+	std::shared_ptr<ARDOUR::Stripable> first_selected_stripable () const;
 
 	void check_fader_automation_state ();
 	void update_fader_automation_state ();
 	void set_automation_state (ARDOUR::AutoState);
 
 	void set_view_mode (ViewMode);
-	int set_subview_mode (SubViewMode, boost::shared_ptr<ARDOUR::Stripable>);
+	bool set_subview_mode (Mackie::Subview::Mode, std::shared_ptr<ARDOUR::Stripable>);
+	bool redisplay_subview_mode ();
 	void set_flip_mode (FlipMode);
 	void display_view_mode ();
 
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int version);
 
 	/* Note: because Mackie control is inherently a duplex protocol,
@@ -159,19 +161,18 @@ class MackieControlProtocol
 	*/
 
 	static bool probe();
-	static void* request_factory (uint32_t);
 
 	mutable Glib::Threads::Mutex surfaces_lock;
-	typedef std::list<boost::shared_ptr<Mackie::Surface> > Surfaces;
+	typedef std::list<std::shared_ptr<Mackie::Surface> > Surfaces;
 	Surfaces surfaces;
 
-	boost::shared_ptr<Mackie::Surface> get_surface_by_raw_pointer (void*) const;
-	boost::shared_ptr<Mackie::Surface> nth_surface (uint32_t) const;
+	std::shared_ptr<Mackie::Surface> get_surface_by_raw_pointer (void*) const;
+	std::shared_ptr<Mackie::Surface> nth_surface (uint32_t) const;
 
 	uint32_t global_index (Mackie::Strip&);
 	uint32_t global_index_locked (Mackie::Strip&);
 
-	std::list<boost::shared_ptr<ARDOUR::Bundle> > bundles ();
+	std::list<std::shared_ptr<ARDOUR::Bundle> > bundles ();
 
 	void set_master_on_surface_strip (uint32_t surface, uint32_t strip);
 	void set_monitor_on_surface_strip (uint32_t surface, uint32_t strip);
@@ -185,9 +186,9 @@ class MackieControlProtocol
 	void handle_button_event (Mackie::Surface&, Mackie::Button& button, Mackie::ButtonState);
 
 	void notify_subview_stripable_deleted ();
-	void notify_stripable_removed ();
 	void notify_routes_added (ARDOUR::RouteList &);
 	void notify_vca_added (ARDOUR::VCAList &);
+	void notify_monitor_added_or_removed ();
 
 	void notify_presentation_info_changed(PBD::PropertyChange const &);
 
@@ -219,12 +220,12 @@ class MackieControlProtocol
 	void update_global_led (int id, Mackie::LedState);
 
 	ARDOUR::Session & get_session() { return *session; }
-	framepos_t transport_frame() const;
+	samplepos_t transport_sample() const;
 
 	int modifier_state() const { return _modifier_state; }
 	int main_modifier_state() const { return _modifier_state & MAIN_MODIFIER_MASK; }
 
-	typedef std::list<boost::shared_ptr<ARDOUR::AutomationControl> > ControlList;
+	typedef std::list<std::shared_ptr<ARDOUR::AutomationControl> > ControlList;
 
 	void add_down_button (ARDOUR::AutomationType, int surface, int strip);
 	void remove_down_button (ARDOUR::AutomationType, int surface, int strip);
@@ -258,7 +259,7 @@ class MackieControlProtocol
 	   Fetch the set of Stripables to be considered for control by the
 	   surface. Excluding master, hidden and control routes, and inactive routes
 	*/
-	typedef std::vector<boost::shared_ptr<ARDOUR::Stripable> > Sorted;
+	typedef std::vector<std::shared_ptr<ARDOUR::Stripable> > Sorted;
 	Sorted get_sorted_stripables();
 
 	// bank switching
@@ -269,15 +270,17 @@ class MackieControlProtocol
 	// also called from poll_automation to update timecode display
 	void update_timecode_display();
 
-	std::string format_bbt_timecode (ARDOUR::framepos_t now_frame);
-	std::string format_timecode_timecode (ARDOUR::framepos_t now_frame);
+	std::string format_bbt_timecode (ARDOUR::samplepos_t now_sample);
+	std::string format_timecode_timecode (ARDOUR::samplepos_t now_sample);
 
 	void do_request (MackieControlUIRequest*);
 	int stop ();
 
 	void thread_init ();
 
-	bool stripable_is_locked_to_strip (boost::shared_ptr<ARDOUR::Stripable>) const;
+	bool stripable_is_locked_to_strip (std::shared_ptr<ARDOUR::Stripable>) const;
+
+	CONTROL_PROTOCOL_THREADS_NEED_TEMPO_MAP_DECL();
 
   private:
 
@@ -306,26 +309,24 @@ class MackieControlProtocol
 	PBD::ScopedConnectionList audio_engine_connections;
 	PBD::ScopedConnectionList session_connections;
 	PBD::ScopedConnectionList stripable_connections;
-	PBD::ScopedConnectionList subview_stripable_connections;
 	PBD::ScopedConnectionList gui_connections;
 	PBD::ScopedConnectionList fader_automation_connections;
 	// timer for two quick marker left presses
 	Mackie::Timer            _frm_left_last;
 	// last written timecode string
 	std::string              _timecode_last;
-	framepos_t				 _frame_last;
+	samplepos_t				 _sample_last;
 	// Which timecode are we displaying? BBT or Timecode
 	ARDOUR::AnyTime::Type    _timecode_type;
 	// Bundle to represent our input ports
-	boost::shared_ptr<ARDOUR::Bundle> _input_bundle;
+	std::shared_ptr<ARDOUR::Bundle> _input_bundle;
 	// Bundle to represent our output ports
-	boost::shared_ptr<ARDOUR::Bundle> _output_bundle;
+	std::shared_ptr<ARDOUR::Bundle> _output_bundle;
 	void*                    _gui;
 	bool                     _scrub_mode;
 	FlipMode                 _flip_mode;
 	ViewMode                 _view_mode;
-	SubViewMode              _subview_mode;
-	boost::shared_ptr<ARDOUR::Stripable> _subview_stripable;
+	std::shared_ptr<Mackie::Subview> _subview;
 	int                      _current_selected_track;
 	int                      _modifier_state;
 	ButtonMap                 button_map;
@@ -333,13 +334,13 @@ class MackieControlProtocol
 	bool                      needs_ipmidi_restart;
 	bool                     _metering_active;
 	bool                     _initialized;
-	XMLNode*                 configuration_state;
+	mutable XMLNode*         configuration_state;
 	int                      state_version;
 	int                      _last_bank[9];
 	bool                     marker_modifier_consumed_by_button;
 	bool                     nudge_modifier_consumed_by_button;
 
-	boost::shared_ptr<ArdourSurface::Mackie::Surface>	_master_surface;
+	std::shared_ptr<ArdourSurface::Mackie::Surface>	_master_surface;
 
         struct ipMIDIHandler {
                 MackieControlProtocol* mcp;
@@ -351,24 +352,24 @@ class MackieControlProtocol
 	int create_surfaces ();
 	bool periodic();
 	bool redisplay();
-	bool redisplay_subview_mode ();
 	bool hui_heartbeat ();
 	void build_gui ();
 	bool midi_input_handler (Glib::IOCondition ioc, MIDI::Port* port);
 	void clear_ports ();
 	void clear_surfaces ();
-	void force_special_stripable_to_strip (boost::shared_ptr<ARDOUR::Stripable> r, uint32_t surface, uint32_t strip_number);
+	void force_special_stripable_to_strip (std::shared_ptr<ARDOUR::Stripable> r, uint32_t surface, uint32_t strip_number);
 	void build_button_map ();
+	void build_device_specific_button_map ();
 	void stripable_selection_changed ();
 	int ipmidi_restart ();
         void initialize ();
         int set_device_info (const std::string& device_name);
-	void update_configuration_state ();
+	void update_configuration_state () const;
 
 	/* MIDI port connection management */
 
 	PBD::ScopedConnection port_connection;
-	void connection_handler (boost::weak_ptr<ARDOUR::Port>, std::string name1, boost::weak_ptr<ARDOUR::Port>, std::string name2, bool);
+	void connection_handler (std::weak_ptr<ARDOUR::Port>, std::string name1, std::weak_ptr<ARDOUR::Port>, std::string name2, bool);
 
 	/* BUTTON HANDLING */
 
@@ -516,6 +517,32 @@ class MackieControlProtocol
 	Mackie::LedState view_release (Mackie::Button&);
 
 	Mackie::LedState bank_release (Mackie::Button&, uint32_t bank_num);
+	Mackie::LedState master_press(Mackie::Button &);
+	Mackie::LedState master_release(Mackie::Button &);
+	Mackie::LedState redo_press(Mackie::Button &);
+	Mackie::LedState redo_release(Mackie::Button &);
+	Mackie::LedState prev_marker_press(Mackie::Button &);
+	Mackie::LedState prev_marker_release(Mackie::Button &);
+	Mackie::LedState next_marker_press(Mackie::Button &);
+	Mackie::LedState next_marker_release(Mackie::Button &);
+	Mackie::LedState flip_window_press (Mackie::Button&);
+	Mackie::LedState flip_window_release (Mackie::Button&);
+	Mackie::LedState open_press(Mackie::Button &);
+	Mackie::LedState open_release(Mackie::Button &);
+	Mackie::LedState prog2_clear_solo_press(Mackie::Button &);
+	Mackie::LedState prog2_clear_solo_release(Mackie::Button &);
+	Mackie::LedState prog2_save_press(Mackie::Button &);
+	Mackie::LedState prog2_save_release(Mackie::Button &);
+	Mackie::LedState prog2_vst_press(Mackie::Button &);
+	Mackie::LedState prog2_vst_release(Mackie::Button &);
+	Mackie::LedState prog2_left_press(Mackie::Button &);
+	Mackie::LedState prog2_left_release(Mackie::Button &);
+	Mackie::LedState prog2_right_press(Mackie::Button &);
+	Mackie::LedState prog2_right_release(Mackie::Button &);
+	Mackie::LedState prog2_marker_press(Mackie::Button &);
+	Mackie::LedState prog2_marker_release(Mackie::Button &);
+	Mackie::LedState prog2_undo_press(Mackie::Button &);
+	Mackie::LedState prog2_undo_release(Mackie::Button &);
 };
 
 } // namespace
