@@ -1,3 +1,22 @@
+/*
+ * Copyright (C) 2006-2007 John Anderson
+ * Copyright (C) 2012-2015 Paul Davis <paul@linuxaudiosystems.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #ifndef mackie_surface_h
 #define mackie_surface_h
 
@@ -5,6 +24,7 @@
 
 #include <sigc++/trackable.h>
 
+#include "pbd/property_basics.h"
 #include "pbd/signals.h"
 #include "pbd/xml++.h"
 #include "midi++/types.h"
@@ -48,6 +68,17 @@ class Led;
 class Surface : public PBD::ScopedConnectionList, public sigc::trackable
 {
 public:
+	enum XTouchColors {
+		Off,
+		Red,
+		Green,
+		Yellow,
+		Blue,
+		Purple,
+		Cyan,
+		White,
+	};
+	
 	Surface (MackieControlProtocol&, const std::string& name, uint32_t number, surface_type_t stype);
 	virtual ~Surface();
 
@@ -79,8 +110,8 @@ public:
 	uint32_t n_strips (bool with_locked_strips = true) const;
 	Strip* nth_strip (uint32_t n) const;
 
-	bool stripable_is_locked_to_strip (boost::shared_ptr<ARDOUR::Stripable>) const;
-	bool stripable_is_mapped (boost::shared_ptr<ARDOUR::Stripable>) const;
+	bool stripable_is_locked_to_strip (std::shared_ptr<ARDOUR::Stripable>) const;
+	bool stripable_is_mapped (std::shared_ptr<ARDOUR::Stripable>) const;
 
 	/// This collection owns the groups
 	typedef std::map<std::string,Group*> Groups;
@@ -88,14 +119,14 @@ public:
 
 	SurfacePort& port() const { return *_port; }
 
-	void map_stripables (const std::vector<boost::shared_ptr<ARDOUR::Stripable> >&);
+	void map_stripables (const std::vector<std::shared_ptr<ARDOUR::Stripable> >&);
 
 	void update_strip_selection ();
 
 	const MidiByteArray& sysex_hdr() const;
 
-	void periodic (ARDOUR::microseconds_t now_usecs);
-	void redisplay (ARDOUR::microseconds_t now_usecs, bool force);
+	void periodic (PBD::microseconds_t now_usecs);
+	void redisplay (PBD::microseconds_t now_usecs, bool force);
 	void hui_heartbeat ();
 
 	void handle_midi_pitchbend_message (MIDI::Parser&, MIDI::pitchbend_t, uint32_t channel_id);
@@ -169,13 +200,18 @@ public:
 
 	void display_message_for (std::string const& msg, uint64_t msecs);
 
-	bool connection_handler (boost::weak_ptr<ARDOUR::Port>, std::string name1, boost::weak_ptr<ARDOUR::Port>, std::string name2, bool);
+	bool connection_handler (std::weak_ptr<ARDOUR::Port>, std::string name1, std::weak_ptr<ARDOUR::Port>, std::string name2, bool);
 
 	void master_monitor_may_have_changed ();
 
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int version);
 
+	bool get_qcon_flag() { return is_qcon; }
+
+	void toggle_master_monitor ();
+	bool master_stripable_is_master_monitor ();
+
   private:
 	MackieControlProtocol& _mcp;
 	SurfacePort*           _port;
@@ -188,6 +224,11 @@ public:
 	Fader*                 _master_fader;
 	float                  _last_master_gain_written;
 	PBD::ScopedConnection   master_connection;
+	bool                   _has_master_display;
+	bool                   _has_master_meter;
+	std::shared_ptr<ARDOUR::Stripable> _master_stripable;
+	std::string pending_display[2];
+	std::string current_display[2];
 
 	void handle_midi_sysex (MIDI::Parser&, MIDI::byte *, size_t count);
 	MidiByteArray host_connection_query (MidiByteArray& bytes);
@@ -198,6 +239,11 @@ public:
 	void init_strips (uint32_t n);
 	void setup_master ();
 	void master_gain_changed ();
+	void master_property_changed (const PBD::PropertyChange&);
+	void master_meter_changed ();
+	void show_master_name();
+	MidiByteArray master_display (uint32_t line_number, const std::string&);		// QCon ProX 2nd LCD master label
+	MidiByteArray blank_master_display (uint32_t line_number);
 
 	enum ConnectionState {
 		InputConnected = 0x1,
@@ -206,7 +252,12 @@ public:
 
 	int connection_state;
 
+	// QCon Flag
+	bool is_qcon;
+
 	MidiByteArray display_line (std::string const& msg, int line_num);
+	MidiByteArray display_colors_on_xtouch (const XTouchColors color_values[]) const;
+	uint8_t convert_color_to_xtouch_value (uint32_t color) const;
 
   public:
 	/* IP MIDI devices need to keep a handle on this and destroy it */
