@@ -19,7 +19,7 @@ test -f gtk2_ardour/wscript || exit 1
 
 : ${HARRISONCHANNELSTRIP=harrison_channelstrip}
 : ${HARRISONLV2=harrison_lv2s-n}
-: ${HARRISONDSPURL=http://www.harrisonconsoles.com/plugins/releases/public}
+: ${HARRISONDSPURL=https://rsrc.harrisonconsoles.com/plugins/releases/public}
 
 # see also wscript, video_tool_paths.cc, bundle_env_mingw.cc
 # registry keys based on this are used there
@@ -31,7 +31,8 @@ PRODUCT_NAME=Ardour
 PRODUCT_VERSION=${major_version}
 
 WITH_HARRISON_LV2=1 ;
-WITH_X42_LV2=1 ;
+WITH_COMMERCIAL_X42_LV2=
+WITH_GRATIS_X42_LV2=
 
 # TODO: grep from build/config.log instead
 while [ $# -gt 0 ] ; do
@@ -40,21 +41,23 @@ while [ $# -gt 0 ] ; do
 		--mixbus)
 			MIXBUS=1
 			WITH_HARRISON_LV2=1 ;
-			WITH_X42_LV2=1 ;
+			WITH_COMMERCIAL_X42_LV2=1
+			WITH_GRATIS_X42_LV2=1
 			PROGRAM_NAME=Mixbus
 			PROGRAM_KEY=Mixbus
 			PRODUCT_NAME=Mixbus
-			MANUAL_NAME="mixbus${major_version}-live-manual"
+			MANUAL_NAME="mixbus-live-manual"
 			shift ;;
 		--mixbus32c)
 			MIXBUS=1
 			WITH_HARRISON_LV2=1 ;
-			WITH_X42_LV2=1 ;
+			WITH_COMMERCIAL_X42_LV2=1
+			WITH_GRATIS_X42_LV2=1
 			PRODUCT_NAME=Mixbus32C
 			PROGRAM_KEY=Mixbus32C
 			PROGRAM_NAME=Mixbus32C-${PROGRAM_VERSION}
 			PROGRAM_VERSION=""
-			MANUAL_NAME="mixbus32c-${major_version}-live-manual"
+			MANUAL_NAME="mixbus32c-live-manual"
 			shift ;;
 		--chanstrip) HARRISONCHANNELSTRIP=$2 ; shift; shift ;;
 	esac
@@ -88,7 +91,7 @@ fi
 export SRCCACHE
 
 if [ "$(id -u)" = "0" ]; then
-	apt-get -y install nsis curl
+	apt-get -y install nsis curl wget
 fi
 
 
@@ -116,13 +119,13 @@ echo " === bundle to $DESTDIR"
 if test -z "$DESTDIR"; then
 	DESTDIR=`mktemp -d`
 	trap 'rm -rf $DESTDIR' exit SIGINT SIGTERM
+	rm -rf $DESTDIR
 fi
 
 echo " === bundle to $DESTDIR"
 
 ALIBDIR=$DESTDIR/lib/${LOWERCASE_DIRNAME}
 
-rm -rf $DESTDIR
 mkdir -p $DESTDIR/bin
 mkdir -p $DESTDIR/share/
 mkdir -p $ALIBDIR/surfaces
@@ -135,25 +138,38 @@ cp build/libs/gtkmm2ext/gtkmm2ext-*.dll $DESTDIR/bin/
 cp build/libs/midi++2/midipp-*.dll $DESTDIR/bin/
 cp build/libs/evoral/evoral-*.dll $DESTDIR/bin/
 cp build/libs/ardour/ardour-*.dll $DESTDIR/bin/
-cp build/libs/timecode/timecode.dll $DESTDIR/bin/
+cp build/libs/temporal/temporal-*.dll $DESTDIR/bin/
 cp build/libs/canvas/canvas-*.dll $DESTDIR/bin/
 cp build/libs/widgets/widgets-*.dll $DESTDIR/bin/
 cp build/libs/waveview/waveview-*.dll $DESTDIR/bin/
 cp build/libs/pbd/pbd-*.dll $DESTDIR/bin/
+cp build/libs/ctrl-interface/midi_surface/ardour*.dll $DESTDIR/bin/
+cp build/libs/ctrl-interface/control_protocol/ardour*.dll $DESTDIR/bin/
 cp build/libs/ptformat/ptformat-*.dll $DESTDIR/bin/
 cp build/libs/audiographer/audiographer-*.dll $DESTDIR/bin/
 cp build/libs/fst/ardour-vst-scanner.exe $DESTDIR/bin/ || true
+cp build/libs/fst/ardour-vst3-scanner.exe $DESTDIR/bin/ || true
 cp build/session_utils/*-*.exe $DESTDIR/bin/ || true
+cp build/luasession/ardour*-lua.exe $DESTDIR/bin/ || true
 cp `ls -t build/gtk2_ardour/ardour-*.exe | head -n1` $DESTDIR/bin/${PRODUCT_EXE}
 
 mkdir -p $DESTDIR/lib/gtk-2.0/engines
 cp build/libs/clearlooks-newer/clearlooks.dll $DESTDIR/lib/gtk-2.0/engines/libclearlooks.la
 
-cp $PREFIX/bin/*dll $DESTDIR/bin/
-cp $PREFIX/lib/*dll $DESTDIR/bin/
+cp $PREFIX/bin/*.dll $DESTDIR/bin/
+cp $PREFIX/bin/*.yes $DESTDIR/bin/ || true
+cp $PREFIX/lib/*.dll $DESTDIR/bin/
 # special case libportaudio (wasapi), old stack has no wasapi and hence no .xp
 cp $PREFIX/bin/libportaudio-2.xp $DESTDIR/bin/ || cp $PREFIX/bin/libportaudio-2.dll $DESTDIR/bin/libportaudio-2.xp
+
+# prefer system-wide DLL
 rm -rf $DESTDIR/bin/libjack*.dll
+# Also for these (even though M$ recommends to bundle these [1],
+# there is no single set that works on all target systems, particularly
+# since some plugins also rely on it.
+# [1] https://docs.microsoft.com/en-us/windows/win32/debug/calling-the-dbghelp-library
+rm -rf $DESTDIR/bin/dbghelp*.dll
+rm -rf $DESTDIR/bin/dbgcore*.dll
 
 cp `find build/libs/surfaces/ -iname "*.dll"` $ALIBDIR/surfaces/
 cp `find build/libs/backends/ -iname "*.dll"` $ALIBDIR/backends/
@@ -161,14 +177,15 @@ cp `find build/libs/panners/ -iname "*.dll"` $ALIBDIR/panners/
 
 cp -r build/libs/LV2 $ALIBDIR/
 cp -r build/libs/vamp-plugins/*ardourvampplugins*.dll $ALIBDIR/vamp/libardourvampplugins.dll
+cp -r build/libs/vamp-pyin/*ardourvamppyin*.dll $ALIBDIR/vamp/libardourvamppyin.dll
 cp $PREFIX/lib/suil-*/*.dll $ALIBDIR/suil/ || true
 
-# lv2 core, classifications etc - TODO check if we need the complete LV2 ontology
-if test -d $PREFIX/lib/lv2/lv2core.lv2 ; then
-	cp -R $PREFIX/lib/lv2/lv2core.lv2 $ALIBDIR/LV2/
-fi
-
-mv $ALIBDIR/surfaces/ardourcp*.dll $DESTDIR/bin/
+# lv2 core, classifications
+for file in $PREFIX/lib/lv2/*.lv2; do
+	BN=$(basename $file)
+	mkdir -p $ALIBDIR/LV2/$BN
+	cp $PREFIX/lib/lv2/${BN}/*.ttl $ALIBDIR/LV2/${BN}/
+done
 
 # TODO use -static-libgcc -static-libstdc++ -- but for .exe files only
 if update-alternatives --query ${XPREFIX}-gcc | grep Value: | grep -q win32; then
@@ -200,6 +217,7 @@ cp gtk2_ardour/icons/cursor_square/* $DESTDIR/share/${LOWERCASE_DIRNAME}/icons/
 # clean build-dir after depoyment
 echo " === bundle completed, cleaning up"
 ./waf uninstall
+find $DESTDIR -name "*.dll.a" | xargs rm
 echo " === complete"
 du -sh $DESTDIR
 
@@ -211,16 +229,16 @@ if test -z "$NOVIDEOTOOLS"; then
 	XJADEO_VERSION=$(curl -s -S http://ardour.org/files/video-tools/xjadeo_version.txt)
 
 	rsync -a -q --partial \
-		rsync://ardour.org/video-tools/harvid_win-${HARVID_VERSION}.tar.xz \
-		"${SRCCACHE}/harvid_win-${HARVID_VERSION}.tar.xz"
+		rsync://ardour.org/video-tools/harvid_${WARCH}-${HARVID_VERSION}.tar.xz \
+		"${SRCCACHE}/harvid_${WARCH}-${HARVID_VERSION}.tar.xz"
 
 	rsync -a -q --partial \
-		rsync://ardour.org/video-tools/xjadeo_win-${XJADEO_VERSION}.tar.xz \
-		"${SRCCACHE}/xjadeo_win-${XJADEO_VERSION}.tar.xz"
+		rsync://ardour.org/video-tools/xjadeo_${WARCH}-${XJADEO_VERSION}.tar.xz \
+		"${SRCCACHE}/xjadeo_${WARCH}-${XJADEO_VERSION}.tar.xz"
 
 	mkdir $DESTDIR/video
-	tar -xf "${SRCCACHE}/harvid_win-${HARVID_VERSION}.tar.xz" -C "$DESTDIR/video/"
-	tar -xf "${SRCCACHE}/xjadeo_win-${XJADEO_VERSION}.tar.xz" -C "$DESTDIR/video/"
+	tar -xf "${SRCCACHE}/harvid_${WARCH}-${HARVID_VERSION}.tar.xz" -C "$DESTDIR/video/"
+	tar -xf "${SRCCACHE}/xjadeo_${WARCH}-${XJADEO_VERSION}.tar.xz" -C "$DESTDIR/video/"
 
 	echo " === unzipped"
 	du -sh $DESTDIR/video
@@ -231,7 +249,28 @@ fi
 ### include static gdb - re-zipped binaries from
 ### http://sourceforge.net/projects/mingw/files/MinGW/Extension/gdb/gdb-7.6.1-1/gdb-7.6.1-1-mingw32-bin.tar.lzma
 ### http://sourceforge.net/projects/mingw-w64/files/Toolchains%20targetting%20Win64/Personal%20Builds/mingw-builds/4.9.1/threads-win32/sjlj/x86_64-4.9.1-release-win32-sjlj-rt_v3-rev1.7z
-if ! grep " using ./waf configure" build/config.log | grep -q -- "--optimize"; then
+
+BUILDTYPE=""
+VERSIONINFO="Optimized Version."
+
+if [ "$DEBUG" = "T" ]; then
+  BUILDTYPE="-dbg"
+	PACKAGE_GDB=1
+	VERSIONINFO="Debug Build."
+fi
+if [ "$FREEBI" = "T" ]; then
+  BUILDTYPE="-demo"
+	VERSIONINFO="Optimized Demo Version."
+fi
+if [ "$DEBUG$FREEBI" = "TT" ]; then
+  BUILDTYPE="-demo-dbg"
+	PACKAGE_GDB=1
+	VERSIONINFO="Demo Version."
+fi
+
+OUTFILE="${TMPDIR}/${PRODUCT_NAME}-${ARDOURVERSION}${BUILDTYPE}-${WARCH}-Setup.exe"
+
+if test -n "$PACKAGE_GDB"; then
 	download gdb-static-win3264.tar.xz http://robin.linuxaudio.org/gdb-static-win3264.tar.xz
 	cd ${SRCCACHE}
 	tar xf gdb-static-win3264.tar.xz
@@ -243,11 +282,6 @@ if ! grep " using ./waf configure" build/config.log | grep -q -- "--optimize"; t
 cd bin
 START ..\\gdb\\bin\\gdb.exe -iex "set logging overwrite on" -iex "set height 0" -iex "set logging on %UserProfile%\\${PRODUCT_NAME}-debug.log" -iex "target exec ${PRODUCT_EXE}" -iex "run"
 EOF
-	OUTFILE="${TMPDIR}/${PRODUCT_NAME}-${ARDOURVERSION}-dbg-${WARCH}-Setup.exe"
-	VERSIONINFO="Debug Version."
-else
-	OUTFILE="${TMPDIR}/${PRODUCT_NAME}-${ARDOURVERSION}-${WARCH}-Setup.exe"
-	VERSIONINFO="Optimized Version."
 fi
 
 ################################################################################
@@ -266,12 +300,27 @@ if true ; then
 	done
 fi
 
-if test x$WITH_X42_LV2 != x ; then
+if test x$WITH_COMMERCIAL_X42_LV2 != x ; then
 	mkdir -p $ALIBDIR/LV2
 
-	echo "Adding x42 Plugins"
+	echo "Adding commercial x42 Plugins"
 
-	for proj in x42-meters x42-midifilter x42-midimap x42-stereoroute x42-eq setBfree x42-avldrums; do
+	for proj in x42-meters x42-eq x42-whirl; do
+		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
+		rsync -a -q --partial \
+			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
+			"${SRCCACHE}/${proj}-lv2-${WARCH}-${X42_VERSION}.zip"
+		unzip -q -d "$ALIBDIR/LV2/" "${SRCCACHE}/${proj}-lv2-${WARCH}-${X42_VERSION}.zip"
+	done
+fi
+
+
+if test x$WITH_GRATIS_X42_LV2 != x ; then
+	mkdir -p $ALIBDIR/LV2
+
+	echo "Adding gratis x42 Plugins"
+
+	for proj in x42-autotune x42-midifilter x42-stereoroute setBfree x42-avldrums x42-limiter x42-tuner; do
 		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
 		rsync -a -q --partial \
 			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
@@ -281,7 +330,7 @@ if test x$WITH_X42_LV2 != x ; then
 fi
 
 if test x$WITH_HARRISON_LV2 != x ; then
-	mkdir -p $ALIBDIR/LV2
+	mkdir -p $DESTDIR/LV2
 
 	echo "Including Harrison LV2s"
 
@@ -289,7 +338,7 @@ if test x$WITH_HARRISON_LV2 != x ; then
 		-z "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip" \
 		-o "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip" \
 		"${HARRISONDSPURL}/${HARRISONLV2}.${WARCH}.zip"
-	unzip -q -d "$ALIBDIR/LV2/" "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip"
+	unzip -q -d "$DESTDIR/LV2/" "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip"
 fi
 
 if test -n "$MIXBUS"; then
@@ -313,15 +362,39 @@ if test -n "$MIXBUS"; then
 
 	cp "${SRCCACHE}/harrison_vamp.${WARCH}.dll" \
 		"$ALIBDIR/vamp/harrison_vamp.dll"
+
+	# Mixbus Bundled Media Content
+	curl -s -S --fail -#  \
+		-z "${SRCCACHE}/MixbusBundledMedia.zip" \
+		-o "${SRCCACHE}/MixbusBundledMedia.zip" \
+		"http://builder.harrisonconsoles.com/pub/share/MixbusBundledMedia.zip"
+
+	if test -f "${SRCCACHE}/MixbusBundledMedia.zip"; then
+		echo "Adding Mixbus Bundled Content"
+		rm -f $DESTDIR/share/${LOWERCASE_DIRNAME}/media/*.*
+		unzip -q -o -d "$DESTDIR/share/${LOWERCASE_DIRNAME}/media/" "${SRCCACHE}/MixbusBundledMedia.zip"
+	fi
+else
+        echo "Fetching Ardour bundled media"
+	curl -s -S --fail -#  \
+		-z "${SRCCACHE}/ArdourBundledMedia.zip" \
+		-o "${SRCCACHE}/ArdourBundledMedia.zip" \
+		"http://stuff.ardour.org/loops/ArdourBundledMedia.zip"
+
+	if test -f "${SRCCACHE}/ArdourBundledMedia.zip"; then
+		echo "Adding Ardour Bundled Content"
+		rm -f $DESTDIR/share/${LOWERCASE_DIRNAME}/media/*.*
+		unzip -q -o -d "$DESTDIR/share/${LOWERCASE_DIRNAME}/media/" "${SRCCACHE}/ArdourBundledMedia.zip"
+	fi
 fi
 
 ################################################################################
 
 if test x$DEMO_SESSION_URL != x ; then
-	mkdir -p $Shared/sessions
+	mkdir -p $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions
 	DEMO_SESSIONS=$(curl -s -S --fail $DEMO_SESSION_URL/index.txt)
 	for demo in $DEMO_SESSIONS; do
-		curl -s -S --fail -# -o $Shared/sessions/$demo $DEMO_SESSION_URL/$demo
+		curl -s -S --fail -# -o $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions/$demo $DEMO_SESSION_URL/$demo
 	done
 fi
 
@@ -365,6 +438,7 @@ RequestExecutionLevel admin
 InstallDir "\$${PGF}\\${PRODUCT_ID}"
 InstallDirRegKey HKLM "Software\\${PRODUCT_NAME}\\${PRODUCT_ID}\\$WARCH" "Install_Dir"
 !define MUI_ICON "share\\${PRODUCT_ICON}"
+!define MUI_UNICON "share\\${PRODUCT_ICON}"
 
 EOF
 
@@ -372,10 +446,10 @@ if test -n "$MIXBUS"; then
 
 # TODO: proper welcome/finish text.
 	cat >> $NSISFILE << EOF
-!define MUI_FINISHPAGE_TITLE "Welcome to Harrison Mixbus"
-!define MUI_FINISHPAGE_TEXT "Thanks for your purchase of Mixbus!\$\\r\$\\nYou will find the Mixbus application in the Start Menu (or the All Apps panel for Windows 8) \$\\r\$\\nClick the link below to view the Mixbus manual, and learn ways to get involved with the Mixbus community."
-!define MUI_FINISHPAGE_LINK "Mixbus Manual"
-!define MUI_FINISHPAGE_LINK_LOCATION "http://www.harrisonconsoles.com/mixbus/${MANUAL_NAME}/"
+!define MUI_FINISHPAGE_TITLE "Welcome to Harrison ${PROGRAM_NAME}"
+!define MUI_FINISHPAGE_TEXT "Thanks for your purchase of ${PROGRAM_NAME}!\$\\r\$\\nYou will find the ${PROGRAM_NAME} application in the Start Menu (or the All Apps panel for Windows 8) \$\\r\$\\nClick the link below to view the ${PROGRAM_NAME} manual, and learn ways to get involved with the Mixbus community."
+!define MUI_FINISHPAGE_LINK "${PROGRAM_NAME} Manual"
+!define MUI_FINISHPAGE_LINK_LOCATION "https://rsrc.harrisonconsoles.com/mixbus/${MANUAL_NAME}/"
 !define MUI_FINISHPAGE_NOREBOOTSUPPORT
 EOF
 
@@ -383,7 +457,7 @@ else
 
 	cat >> $NSISFILE << EOF
 !define MUI_FINISHPAGE_TITLE "Welcome to Ardour"
-!define MUI_FINISHPAGE_TEXT "This windows versions or Ardour is provided as-is.\$\\r\$\\nThe Ardour community currently has no expertise in supporting windows users, and there are no developers focusing on windows specific issues either.\$\\r\$\\nIf you like Ardour, please consider helping out."
+!define MUI_FINISHPAGE_TEXT "This Windows version of Ardour is provided as-is.\$\\r\$\\nThe Ardour community currently has no expertise in supporting Windows users, and there are no developers focusing on Windows-specific issues either.\$\\r\$\\nIf you like Ardour, please consider helping out."
 !define MUI_FINISHPAGE_LINK "Ardour Manual"
 !define MUI_FINISHPAGE_LINK_LOCATION "http://manual.ardour.org/"
 #this would run as admin - see http://forums.winamp.com/showthread.php?t=353366
@@ -421,16 +495,14 @@ Section "${PROGRAM_NAME}${PROGRAM_VERSION} (required)" SecMainProg
   CreateShortCut "\$INSTDIR\\${PROGRAM_NAME}${PROGRAM_VERSION}.lnk" "\$INSTDIR\\bin\\${PRODUCT_EXE}" "" "\$INSTDIR\\bin\\${PRODUCT_EXE}" 0
   \${registerExtension} "\$INSTDIR\\bin\\${STATEFILE_SUFFIX}" ".${PRODUCT_NAME}" "${PROGRAM_NAME} Session"
 SectionEnd
-
-Section "WASAPI sound driver" SecWASAPI
-SectionEnd
 EOF
 
 if test -z "$NOVIDEOTOOLS"; then
 
 	cat >> $NSISFILE << EOF
-Section "Videotimeline Tools" SecVideo
+Section "Videotimeline Tools (required)" SecVideo
   WriteRegStr HKLM "Software\\${PROGRAM_KEY}\\v${major_version}\\video" "Install_Dir" "\$INSTDIR\\video"
+  SectionIn RO
   SetOutPath \$INSTDIR
   File /r video
 SectionEnd
@@ -438,9 +510,35 @@ EOF
 
 fi
 
+if test x$WITH_HARRISON_LV2 != x ; then
+if test -n "$MIXBUS"; then
+	cat >> $NSISFILE << EOF
+Section "Harrison XT plugins (required)" SecXT
+  SectionIn RO
+  SetOutPath \$INSTDIR\\lib\\${LOWERCASE_DIRNAME}\\LV2
+  File LV2\\.harrison_version.txt
+  File /r LV2\\*.lv2
+SectionEnd
+EOF
+else
+	cat >> $NSISFILE << EOF
+Section "Harrison XT plugins and ACE plugin GUIs" SecXT
+  SetOutPath \$INSTDIR\\lib\\${LOWERCASE_DIRNAME}\\LV2
+  File LV2\\.harrison_version.txt
+  File /r LV2\\*.lv2
+SectionEnd
+EOF
+fi
+fi
+
 cat >> $NSISFILE << EOF
+
+Section "WASAPI sound driver" SecWASAPI
+SectionEnd
+
 Section "Start Menu Shortcuts" SecMenu
   SetShellVarContext all
+  SetOutPath \$INSTDIR
   CreateDirectory "\$SMPROGRAMS\\${PRODUCT_ID}${SFX}"
   CreateShortCut "\$SMPROGRAMS\\${PRODUCT_ID}${SFX}\\${PROGRAM_NAME}${PROGRAM_VERSION}.lnk" "\$INSTDIR\\bin\\${PRODUCT_EXE}" "" "\$INSTDIR\\bin\\${PRODUCT_EXE}" 0
 EOF
@@ -470,6 +568,11 @@ if test -z "$NOVIDEOTOOLS"; then
 LangString DESC_SecVideo \${LANG_ENGLISH} "Video Tools\$\\r\$\\nxjadeo-${XJADEO_VERSION}\$\\r\$\\nharvid-${HARVID_VERSION}"
 EOF
 fi
+if test x$WITH_HARRISON_LV2 != x ; then
+	cat >> $NSISFILE << EOF
+LangString DESC_SecXT \${LANG_ENGLISH} "These are proprietary additions, but the DSP is not license encumbered. XT-plugin GUIs are commercial, the additional a-*/ACE plugin GUIs are free."
+EOF
+fi
 
 cat >> $NSISFILE << EOF
 LangString DESC_SecMenu \${LANG_ENGLISH} "Create Start-Menu Shortcuts (recommended)."
@@ -484,6 +587,12 @@ if test -z "$NOVIDEOTOOLS"; then
 EOF
 fi
 
+if test x$WITH_HARRISON_LV2 != x ; then
+	cat >> $NSISFILE << EOF
+!insertmacro MUI_DESCRIPTION_TEXT \${SecXT} \$(DESC_SecXT)
+EOF
+fi
+
 cat >> $NSISFILE << EOF
 !insertmacro MUI_DESCRIPTION_TEXT \${SecMenu} \$(DESC_SecMenu)
 !insertmacro MUI_FUNCTION_DESCRIPTION_END
@@ -514,11 +623,25 @@ Function .onInit
     "UninstallString"
   StrCmp \$R0 "" done
 
+  IfSilent silentuninst
+
   MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION \
     "${PROGRAM_NAME} is already installed. Click 'OK' to remove the previous version or 'Cancel' to cancel this upgrade." \
     IDOK uninst
     Abort
 
+  silentuninst:
+    ExecWait '\$R0 /S _?=\$INSTDIR'
+
+    ReadRegStr \$R1 HKLM \
+      "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\${PRODUCT_ID}-${WARCH}" \
+      "UninstallString"
+    StrCmp \$R1 "" 0 done
+
+    Delete "\$INSTDIR\\uninstall.exe"
+    RMDir "\$INSTDIR"
+    goto done
+
   uninst:
     ClearErrors
     ExecWait '\$R0 _?=\$INSTDIR'
